@@ -1,10 +1,10 @@
 ;*=====================================================================*/
-;*    serrano/prgm/project/bdk/kbdb/src/Gdb/gread.scm                  */
+;*    serrano/prgm/project/bigloo/bdb/bdb/Gdb/gread.scm                */
 ;*    -------------------------------------------------------------    */
 ;*    Author      :  Manuel Serrano                                    */
 ;*    Creation    :  Wed Aug  9 08:43:03 2000                          */
-;*    Last change :  Thu Aug 10 10:44:23 2000 (serrano)                */
-;*    Copyright   :  2000 Manuel Serrano                               */
+;*    Last change :  Fri Dec 14 13:37:58 2012 (serrano)                */
+;*    Copyright   :  2000-12 Manuel Serrano                            */
 ;*    -------------------------------------------------------------    */
 ;*    Read GDB outputs.                                                */
 ;*=====================================================================*/
@@ -41,7 +41,7 @@
 ;*    input-port->file ...                                             */
 ;*---------------------------------------------------------------------*/
 (define (input-port->file::file iport::input-port)
-   (pragma::file "INPUT_PORT( $1 ).file" iport))
+   (pragma::file "PORT_FILE( $1 )" iport))
 
 ;*---------------------------------------------------------------------*/
 ;*    getc ...                                                         */
@@ -49,8 +49,8 @@
 (define (getc::char file::file block?)
    (pragma "extern int (*bdb_getc)(FILE *)")
    (if block?
-       (pragma::char "getc( $1 )" file)
-       (pragma::char "bdb_getc( $1 )" file)))
+       (free-pragma::char "getc( $1 )" file)
+       (free-pragma::char "bdb_getc( $1 )" file)))
 
 ;*---------------------------------------------------------------------*/
 ;*    read-unbuffered-line ...                                         */
