@@ -3,7 +3,7 @@
 ;*    -------------------------------------------------------------    */
 ;*    Author      :  Manuel Serrano                                    */
 ;*    Creation    :  Wed Jul 28 11:04:39 1999                          */
-;*    Last change :  Fri Feb 16 15:30:41 2001 (serrano)                */
+;*    Last change :  Fri Nov 22 14:26:08 2002 (serrano)                */
 ;*    -------------------------------------------------------------    */
 ;*    The handling of the demangling environment inside the patient.   */
 ;*    This makes use of hash tables but to get simplicity it does not  */
@@ -227,9 +227,9 @@
 			   (match-global pattern mod-info))
 			globals)))
 	  (else
-	   (error "bdb" "Illegal module table format" list))))
+	   (error "bdb" "Illegal module table format" module))))
       (else
-       (error "bdb" "Illegal module format" list))))
+       (error "bdb" "Illegal module format" (bdb-table->list table)))))
 
 ;*---------------------------------------------------------------------*/
 ;*    *module-env* ...                                                 */
