@@ -3,7 +3,7 @@
 ;*    -------------------------------------------------------------    */
 ;*    Author      :  Manuel Serrano                                    */
 ;*    Creation    :  Sun May 31 07:37:29 1998                          */
-;*    Last change :  Thu Feb  8 12:05:05 2001 (serrano)                */
+;*    Last change :  Thu Dec 15 18:31:04 2011 (serrano)                */
 ;*    -------------------------------------------------------------    */
 ;*    The Bigloo depend utility.                                       */
 ;*=====================================================================*/
@@ -17,8 +17,8 @@
 ;*---------------------------------------------------------------------*/
 ;*    Global parameters                                                */
 ;*---------------------------------------------------------------------*/
-(define *bdepend-version*          "0.1")
-(define *bdepend-path*             '())
+(define *bdepend-version*          "0.4")
+(define *bdepend-path*             '("."))
 (define *bdepend-exclude-path*     '())
 (define *bdepend-source-files*     '())
 (define *bdepend-makefile*         #f)
@@ -33,7 +33,13 @@
 (define *bdepend-mco?*             #t)
 (define *bdepend-jvm?*             #t)
 (define *bdepend-obj/mco-dir*      "")
+(define *bdepend-obj-suffix*       ".o")
+(define *bdepend-strict-obj-dir*   #f)
+(define *bdepend-strict-mco-dir*   #f)
+(define *bdepend-strict-class-dir* #f)
 (define *bdepend-include-path*     '("."))
+(define *bdepend-append*           #f)
+(define *bdepend-sentinel-found*   #f)
 
 ;*---------------------------------------------------------------------*/
 ;*    main ...                                                         */
@@ -66,15 +72,17 @@
 ;*---------------------------------------------------------------------*/
 (define (parse-args cmd-args)
    (define (usage args-parse-usage level)
-      (print "usage: bdepend [options] file ...")
+      (print "usage: bgldepend [options] file ...")
       (newline)
       (args-parse-usage #f)
       (newline))
    (args-parse (cdr cmd-args)
       (("?")
-       (usage args-parse-usage 1))
-      (("-help" (help "This help message"))
-       (usage args-parse-usage 1))
+       (usage args-parse-usage 1)
+       (exit 0))
+      ((("-help" "--help") (help "This help message"))
+       (usage args-parse-usage 1)
+       (exit 0))
       (("-search-path" ?path (help "Add search path"))
        (if (directory? path)
 	   (set! *bdepend-path* (cons path *bdepend-path*))
@@ -87,7 +95,7 @@
        (set! *bdepend-include-path* (cons path *bdepend-include-path*)))
       (("-suffix" ?suf (help "Add Bigloo source suffixes"))
        (set! *bdepend-suffixes* (cons suf *bdepend-suffixes*)))
-      (("-gui-suffix" ?suf (help "Set GUI suffixe"))
+      (("-gui-suffix" ?suf (help "Set GUI suffix"))
        (set! *bdepend-gui-suffix* suf))
       (("-o" ?name (help "Set makefile name"))
        (set! *bdepend-makefile* name))
@@ -103,8 +111,19 @@
        (set! *bdepend-jvm?* #t))
       (("-obj-dir" ?pref (help "directory prefix for .o/.mco files"))
        (set! *bdepend-obj/mco-dir* (string-append pref "/")))
+      (("-obj-suffix" ?suf (help "object file suffix (defaults to .o)"))
+       (set! *bdepend-obj-suffix* suf))
+      (("-strict-obj-dir" ?pref (help "directory for .o"))
+       (set! *bdepend-strict-obj-dir* (string-append pref "/")))
+      (("-strict-mco-dir" ?pref (help "directory for .mco"))
+       (set! *bdepend-strict-mco-dir* (string-append pref "/")))
+      (("-strict-class-dir" ?pref (help "directory for .class"))
+       (set! *bdepend-strict-class-dir* (string-append pref "/")))
+      (("-append" (help "append depencies instead of replacing existing ones"))
+       (set! *bdepend-append* #t))
       (else
-       (set! *bdepend-source-files* (cons else *bdepend-source-files*)))))
+       (if (source? else)
+	   (set! *bdepend-source-files* (cons else *bdepend-source-files*))))))
 
 ;*---------------------------------------------------------------------*/
 ;*    duplicate-makefile-prolog ...                                    */
@@ -115,13 +134,15 @@
       (start-writer! *bdepend-makefile*)
       (let ((iport (open-input-file svg-name)))
 	 (if (not (input-port? iport))
-	     (error "bdepend" "Can't open file for input" svg-name)
+	     (error "bgldepend" "Can't open file for input" svg-name)
 	     (begin
 		(let loop ((line (read-line iport)))
 		   (cond
 		      ((eof-object? line)
 		       (close-input-port iport))
 		      ((string=? line *bdepend-start-sentinel*)
+		       (set! *bdepend-sentinel-found* #t)
+		       (when *bdepend-append* (wprint line))
 		       (let loop ((line (read-line iport)))
 			  (cond
 			     ((eof-object? line)
@@ -129,6 +150,7 @@
 			     ((string=? line *bdepend-stop-sentinel*)
 			      (set! *bdepend-iport* iport))
 			     (else
+			      (when *bdepend-append* (wprint line))
 			      (loop (read-line iport))))))
 		      (else
 		       (wprint line)
@@ -250,8 +272,31 @@
 ;*    generate-depends ...                                             */
 ;*---------------------------------------------------------------------*/
 (define (generate-depends)
+   (define (generate-include-dependency inc)
+      (wprin inc ": "))
+   (define (generate-module-dependency mod)
+      (wprin (if (source? mod) (source->object mod) mod))
+      (if (and (source? mod) *bdepend-jvm?*)
+	  (wprin " " (source->class mod)))
+      (wprin ": "))
+   (define (generate-dependency objects)
+      (for-each (lambda (object)
+		   (let ((len (+fx 1 (string-length object))))
+		      (if (>fx len 74)
+			  (wprint object " \\")
+			  (begin
+			     (if (> (+fx *wcol* len) 74)
+				 (begin
+				    (wfill-to-column 74 #\space)
+				    (if (=fx *wcol* 1)
+					(wprin "      "))
+				    (wprin #"\\\n      ")))
+			     (wprin object " ")))))
+		objects)
+      (wnewline))
    ;; The object files
-   (wprint *bdepend-start-sentinel*)
+   (unless (and *bdepend-append* *bdepend-sentinel-found*)
+      (wprint *bdepend-start-sentinel*))
    ;; the dependencies entry
    (wprin "#*") (wfill-to-column 72 #\-) (wprint "*/")
    (wcomment "Dependencies ...")
@@ -259,26 +304,15 @@
    ;; we load all source files
    (find-all-bigloo-sources)
    ;; and we start emitting objects
-   (let ((objects (find-all-dependences *bdepend-source-files*)))
+   (multiple-value-bind (objects includes)
+      (find-all-dependences *bdepend-source-files*)
+      (for-each (lambda (include)
+		   (generate-include-dependency (car include))
+		   (generate-dependency (cdr include)))
+		includes)
       (for-each (lambda (module)
-		   (wprin (if (source? (car module))
-			      (source->object (car module))
-			      (car module)))
-		   (if (and (source? (car module)) *bdepend-jvm?*)
-		       (wprin " " (source->class (car module))))
-		   (wprin ": ")
-		   (for-each (lambda (object)
-				(let ((len (+fx 1 (string-length object))))
-				   (if (>fx len 74)
-				       (wprint object #" \\\n      ")
-				       (begin
-					  (if (> (+fx *wcol* len) 74)
-					      (begin
-						 (wfill-to-column 74 #\space)
-						 (wprin #"\\\n      ")))
-					  (wprin object " ")))))
-			     (cdr module))
-		   (wnewline))
+		   (generate-module-dependency (car module))
+		   (generate-dependency (cdr module)))
 		objects)
       (wnewline))
    ;; we are done
@@ -318,14 +352,15 @@
 		  (lambda (escape obj proc msg)
 		     (escape '())))
 	     (close-input-port iport))
-	  (error "bdepend" "Can't open file for input" fname))))
+	  (error "bgldepend" "Can't open file for input" fname))))
 
 ;*---------------------------------------------------------------------*/
 ;*    find-all-bigloo-sources ...                                      */
 ;*---------------------------------------------------------------------*/
 (define (find-all-bigloo-sources)
-   (if *bdepend-verbose*
-       (print "Scanning for source files..."))
+   (if *bdepend-verbose* (print "Scanning for source files..."))
+   (define (dir->list dir)
+      (sort (directory->list dir) string<?))
    (define (find-all-bigloo-sources/basename basename files)
       (for-each (lambda (file)
 		   (let ((fname (string-append basename file)))
@@ -335,56 +370,73 @@
 			      (if (not (memq fname *bdepend-exclude-path*))
 				  (find-all-bigloo-sources/basename
 				   (string-append fname "/")
-				   (directory->list fname))))
+				   (dir->list fname))))
 			     ((source? fname)
 			      (let ((module (source->module fname)))
 				 (if (symbol? module)
 				     (putprop! module 'source fname))))))))
 		files))
-   (find-all-bigloo-sources/basename "" (directory->list "."))
+   (find-all-bigloo-sources/basename "" (dir->list "."))
    (for-each (lambda (dir)
 		(find-all-bigloo-sources/basename (string-append dir "/")
-						  (directory->list dir)))
+						  (dir->list dir)))
 	     *bdepend-path*))
 
 ;*---------------------------------------------------------------------*/
 ;*    *object-env* ...                                                 */
 ;*---------------------------------------------------------------------*/
 (define *object-env* (make-hashtable))
-
 (define *include-env* (make-hashtable))
+(define *extern-include-env* (make-hashtable))
 
 ;*---------------------------------------------------------------------*/
 ;*    find-all-dependences ...                                         */
 ;*---------------------------------------------------------------------*/
 (define (find-all-dependences source-files)
    (if *bdepend-verbose*
-       (print "Generating object list..."))
+       (print "Generating object list...: " source-files))
    (for-each add-one-source! source-files)
-   (let ((res '()))
+   (let ((reso '())
+	 (resi '()))
       (hashtable-for-each *object-env*
 			  (lambda (key src)
 			     (if (pair? (cdr src))
-				 (set! res (cons src res)))))
-      res))
+				 (set! reso (cons src reso)))))
+      (hashtable-for-each *include-env*
+			  (lambda (key src)
+			     (if (pair? (cdr src))
+				 (set! resi (cons src resi)))))
+      (values reso resi)))
+
+;*---------------------------------------------------------------------*/
+;*    add-one-file! ...                                                */
+;*---------------------------------------------------------------------*/
+(define (add-one-file! source env)
+   (if (not (hashtable-get env source))
+       (let ((object (list source)))
+	  (if *bdepend-verbose* (print source ":"))
+	  (hashtable-put! env source object)
+	  (set-cdr! object (find-dependences source))
+	  (let ((include-files (hashtable-get *extern-include-env* source)))
+	     (when (and include-files *bdepend-mco?*)
+		(let ((key (source->mco source)))
+		   (hashtable-put! env
+				   key
+				   (cons key (cdr include-files))))))
+	  source)
+       source))
 
 ;*---------------------------------------------------------------------*/
 ;*    add-one-source! ...                                              */
 ;*---------------------------------------------------------------------*/
 (define (add-one-source! source)
-   (if (not (hashtable-get *object-env* source))
-       (let ((object (list source)))
-	  (if *bdepend-verbose* (print source ":"))
-	  (hashtable-put! *object-env* source object)
-	  (set-cdr! object (find-dependences source))
-	  (let ((include-files (hashtable-get *include-env* source)))
-	     (when (and include-files *bdepend-mco?*)
-		(let ((key (source->mco source)))
-		   (hashtable-put! *object-env*
-				   key
-				   (cons  key (cdr include-files))))))
-	  source)
-       source))
+   (add-one-file! source *object-env*))
+
+;*---------------------------------------------------------------------*/
+;*    add-one-include! ...                                             */
+;*---------------------------------------------------------------------*/
+(define (add-one-include! source)
+   (add-one-file! source *include-env*))
 
 ;*---------------------------------------------------------------------*/
 ;*    find-dependences ...                                             */
@@ -401,7 +453,7 @@
    (if (file-exists? fname)
        (let ((port (open-input-file fname)))
 	  (if (not (input-port? port))
-	      (error "bdepend" "Can't open file for input" fname)
+	      (error "bgldepend" "Can't open file for input" fname)
 	      (unwind-protect
 		 (try (let ((module (read port)))
 			 (match-case module
@@ -420,19 +472,27 @@
 ;*    source->object ...                                               */
 ;*---------------------------------------------------------------------*/
 (define (source->object src)
-   (string-append *bdepend-obj/mco-dir* (prefix src) ".o"))
+   (if (string? *bdepend-strict-obj-dir*)
+       (string-append *bdepend-strict-obj-dir* (basename (prefix src))
+                      *bdepend-obj-suffix*)
+       (string-append *bdepend-obj/mco-dir* (prefix src)
+                      *bdepend-obj-suffix*)))
 
 ;*---------------------------------------------------------------------*/
 ;*    source->mco ...                                                  */
 ;*---------------------------------------------------------------------*/
 (define (source->mco src)
-   (string-append *bdepend-obj/mco-dir* (prefix src) ".mco"))
+   (if (string? *bdepend-strict-mco-dir*)
+       (string-append *bdepend-strict-mco-dir* (basename (prefix src)) ".mco")
+       (string-append *bdepend-obj/mco-dir* (prefix src) ".mco")))
 
 ;*---------------------------------------------------------------------*/
 ;*    source->class ...                                                */
 ;*---------------------------------------------------------------------*/
 (define (source->class src)
-   (string-append *bdepend-obj/mco-dir* (prefix src) ".class"))
+   (if (string? *bdepend-strict-class-dir*)
+       (string-append *bdepend-strict-class-dir* (basename (prefix src)) ".class")
+       (string-append *bdepend-obj/mco-dir* (prefix src) ".class")))
 
 ;*---------------------------------------------------------------------*/
 ;*    find-imported-modules/clauses ...                                */
@@ -442,10 +502,7 @@
       (let find ((search-dirs *bdepend-path*))
 	 (if (null? search-dirs)
 	     file
-	     (let ((find-file (string-append
-			       (car search-dirs)
-			       "/"
-			       file)))
+	     (let ((find-file (make-file-name (car search-dirs) file)))
 		(if (file-exists? find-file)
 		    find-file
 		    (find (cdr search-dirs)))))))
@@ -494,6 +551,11 @@
 			  (if (pair? aux)
 			      (append aux res)
 			      res))))))
+	 ((cond-expand . ?eclauses)
+	  (append-map
+	     (lambda (c)
+		(find-imported-modules/clause (cadr c)))
+	     eclauses))
  	 ((extern . ?eclauses)
 	  ;; we only fetch external include clauses
  	  (let loop ((clauses eclauses)
@@ -502,7 +564,8 @@
 	     (cond
 		((null? clauses)
 		 (if (pair? all)
-		     (hashtable-put! *include-env* source (cons source all)))
+		     (hashtable-put! *extern-include-env*
+				     source (cons source all)))
 		 res)
 		(else
 		 (match-case (car clauses)
@@ -536,7 +599,7 @@
 		 (loop (cdr tag) fnames)))))
 	 ((include . ?fnames)
  	  (let ((actual-includes (map find-imported-file fnames)))
- 	     (for-each add-one-source! actual-includes)
+ 	     (for-each add-one-include! actual-includes)
  	     actual-includes))
 	 (else
 	  '())))
