@@ -1,10 +1,10 @@
 ;*=====================================================================*/
-;*    serrano/prgm/project/bigloo2.2a/bde/btags/btags.scm              */
+;*    serrano/prgm/project/bigloo/bde/btags/btags.scm                  */
 ;*    -------------------------------------------------------------    */
 ;*    Author      :  Manuel Serrano                                    */
 ;*    Creation    :  Thu May 28 07:24:34 1998                          */
-;*    Last change :  Sat Jul  8 08:35:00 2000 (serrano)                */
-;*    Copyright   :  2000 Manuel Serrano                               */
+;*    Last change :  Wed Jul 24 07:42:08 2013 (serrano)                */
+;*    Copyright   :  2000-13 Manuel Serrano                            */
 ;*    -------------------------------------------------------------    */
 ;*    The Bigloo tag generator.                                        */
 ;*=====================================================================*/
@@ -18,7 +18,7 @@
 ;*---------------------------------------------------------------------*/
 ;*    Global parameters                                                */
 ;*---------------------------------------------------------------------*/
-(define *btags-version*         "0.0")
+(define *btags-version*         "0.3")
 (define *btags-append*          #f)
 (define *btags-language*        #f)
 (define *btags-table-name*      "TAGS")
@@ -27,9 +27,12 @@
 (define *btags-modules*         #f)
 (define *btags-extern?*         #f)
 (define *oport*                 #f)
-(define *btags-define-var-list* '(define define-struct))
+(define *btags-define-var-list* '(define define-struct define-parameter))
 (define *btags-define-fun-list* '(define define-inline define-generic
 				    define-macro define-expander))
+(define *btags-module-list*     '(module))
+(define *btags-suffixes*        '("scm" "sch" "bgl"))
+(define *btags-ignore-errors*   #f)
 
 ;*---------------------------------------------------------------------*/
 ;*    main ...                                                         */
@@ -55,17 +58,21 @@
 ;*---------------------------------------------------------------------*/
 (define (parse-args cmd-args)
    (define (usage args-parse-usage level)
-      (print "usage: btags [options] file ...")
+      (print "usage: bgltags [options] file ...")
       (newline)
       (args-parse-usage #f)
       (newline))
    (args-parse (cdr cmd-args)
       (("?")
-       (usage args-parse-usage 1))
-      (("-help" (help "This help message"))
-       (usage args-parse-usage 1))
+       (usage args-parse-usage 1)
+       (exit 1))
+      ((("-help" "--help") (help "This help message"))
+       (usage args-parse-usage 1)
+       (exit 0))
       (("-a" (help "-a, --append" "Append to existing tag file."))
        (set! *btags-append* #t))
+      ((("-s" "-suffix" "--suffix") ?suffix (help "Add source suffix"))
+       (set! *btags-suffixes* (cons suffix *btags-suffixes*)))
       (("--append")
        (set! *btags-append* #t))
       (("-e" (help "-e, --extern" "Parse Bigloo extern clauses"))
@@ -101,12 +108,15 @@
        (exit 0))
       (("--define-var" ?k (synopsis "Add define keyword `(KWD VAR VAL)'"))
        (set! *btags-define-var-list*
-	     (cons (string->symbol (string-upcase k))
-		   *btags-define-var-list*)))
+	     (cons (string->symbol k) *btags-define-var-list*)))
       (("--define-fun" ?k (synopsis "Add define keyword `(KWD (ID ...) VAL)'"))
        (set! *btags-define-fun-list*
-	     (cons (string->symbol (string-upcase k))
-		   *btags-define-fun-list*)))
+	     (cons (string->symbol k) *btags-define-fun-list*)))
+      (("--module" ?k (synopsis "Add module keyword `(KWD ID ...)'"))
+       (set! *btags-module-list*
+	     (cons (string->symbol k) *btags-module-list*)))
+      (("--ignore-error" (synopsis "Ignore errors"))
+       (set! *btags-ignore-errors* #t))
       (else
        (if (string? *btags-language*)
 	   (set! *btags-files* (cons (cons else *btags-language*)
@@ -125,7 +135,7 @@
 	      (other-btags fname language)))
        (let ((suffix (suffix file)))
 	  (cond
-	     ((member suffix '("scm" "sch" "bgl"))
+	     ((member suffix *btags-suffixes*)
 	      (bigloo-btags file))
 	     (else
 	      (other-btags file #f))))))
@@ -165,19 +175,22 @@
        (begin
 	  (set! *oport* (append-output-file *btags-table-name*))
 	  (if (not (output-port? *oport*))
-	      (error "btags"
+	      (error 'bgltags
 		     "Can't open file for output"
 		     *btags-table-name*))))
    ;; then we do the real job
    (if (or (pair? *btags-define-var-list*)
-	   (pair? *btags-define-fun-list*))
+	   (pair? *btags-define-fun-list*)
+	   (pair? *btags-module-list*))
        (begin
 	  (fprint *oport* #a012)
 	  (fprint *oport* "-,0")
 	  (if (pair? *btags-define-var-list*)
 	      (btags-list *btags-define-var-list*))
 	  (if (pair? *btags-define-fun-list*)
-	      (btags-list *btags-define-fun-list*)))))
+	      (btags-list *btags-define-fun-list*))
+	  (if (pair? *btags-module-list*)
+	      (btags-list *btags-module-list*)))))
 
 ;*---------------------------------------------------------------------*/
 ;*    bigloo-btags ...                                                 */
@@ -190,11 +203,16 @@
        (begin
 	  (set! *oport* (append-output-file *btags-table-name*))
 	  (if (not (output-port? *oport*))
-	      (error "btags"
+	      (error 'bgltags
 		     "Can't open file for output"
 		     *btags-table-name*))))
    ;; the real btags jobs
-   (bigloo-btags-file *oport* file))
+   (if *btags-ignore-errors*
+       (with-handler
+	  (lambda (e)
+	     #f)
+	  (bigloo-btags-file *oport* file))
+       (bigloo-btags-file *oport* file)))
 
 ;*---------------------------------------------------------------------*/
 ;*    bigloo-btags-file ...                                            */
@@ -209,7 +227,7 @@
       (let ((iport (and (file-exists? file)
 			(open-input-file file))))
 	 (if (not (input-port? iport))
-	     (error "btags" "Can't open file for intput" file)
+	     (error 'bgltags "Can't open file for input" file)
 	     (unwind-protect
 		;; we have to reset the reader other line numbering is
 		;; erroneous
@@ -223,24 +241,29 @@
 		   (if (and (pair? lines) (not (eof-object? line)))
 		       (if (=fx lnum (car (car lines)))
 			   ;; this is a declaration line
-			   (let ((tag (tags-substring line
-						      (cdr (car lines)))))
-			      (loop (read-line iport)
-				    (+fx lnum 1)
-				    (+fx pos (string-length line))
+			   (let ((tag (tags-substring line (cdr (car lines)))))
+			      (loop line
+				    lnum
+				    pos
 				    (cdr lines)
 				    (if (string? tag)
-					(string-append tags
-						       tag
-						       (make-string 1 #a127)
-						       (number->string lnum)
-						       ","
-						       (number->string pos)
-						       #"\n")
+					(string-append
+					 tags
+					 tag
+					 (make-string 1 #a127)
+					 (number->string lnum)
+					 ","
+					 ;; we have to remove 1 to the position
+					 ;; otherwise etags (at least the one
+					 ;; comes with gnu-emacs 21.2) gets
+					 ;; wrong and search the next tag
+					 ;; match
+					 (number->string (-fx pos 1))
+					 #"\n")
 					tags)))
 			   (loop (read-line iport)
 				 (+fx lnum 1)
-				 (+fx pos (string-length line))
+				 (+fx pos (+fx 1 (string-length line)))
 				 lines
 				 tags))
 		       (begin
@@ -255,16 +278,28 @@
 ;*    tags-substring ...                                               */
 ;*---------------------------------------------------------------------*/
 (define (tags-substring str1 str2)
+   (define (substring-at? start str1 str2)
+      (let ((len2 (string-length str2)))
+	 (let loop ((i1 start)
+		    (i2 0))
+	    (cond
+	       ((=fx i2 len2)
+		#t)
+	       ((char-ci=? (string-ref str1 i1) (string-ref str2 i2))
+		(loop (+fx i1 1) (+fx i2 1)))
+	       (else
+		#f)))))
    (let ((len1 (string-length str1)))
-      (if (pair? str2)
+      (cond
+	 ((pair? str2)
 	  ;; if the argument is a pair instead of a string, it means that
 	  ;; it is a typed identifier (such as a class or a method) that
 	  ;; must be written with its type information. Nevertheless we
 	  ;; have to take care that we must strip off all the extra
-	  ;; information found a the line. For instead for a class declaration
+	  ;; information found at the line. For instead for a class declaration
 	  ;; such as: (class point-3d::point (z (default 0)))
 	  ;; we have to write:
-	  ;; (class point-3d::point^?...
+	  ;; (class point-3d::point^?...)
 	  (let loop ((r 0))
 	     (cond
 		((=fx r len1)
@@ -273,38 +308,33 @@
 		     (char=? (string-ref str1 r) #\;))
 		 (substring str1 0 r))
 		(else
-		 (loop (+fx r 1)))))
+		 (loop (+fx r 1))))))
+	 ((vector? str2)
+	  (string-append "(define (" (vector-ref str2 1)))
+	 ((cell? str2)
+	  (string-append "(define-parameter " (cell-ref str2) "-set!"))
+	 (else
 	  ;; this is neither a class nor a method...
 	  (let* ((len2 (string-length str2))
-		 (stop (- len1 len2)))
-	     (define (substring-at? start)
-		(let loop ((i1 start)
-			   (i2 0))
-		   (cond
-		      ((=fx i2 len2)
-		       #t)
-		      ((char-ci=? (string-ref str1 i1) (string-ref str2 i2))
-		       (loop (+ i1 1) (+ i2 1)))
-		      (else
-		       #f))))
-	     (let ((start (let loop ((j 0))
-			     (cond
-				((=fx j len1)
-				 #f)
-				((char=? (string-ref str1 j) #\space)
-				 (+fx j 1))
-				(else
-				 (loop (+fx j 1)))))))
-		(if (integer? start)
-		    (let loop ((i start))
-		       (cond
-			  ((> i stop)
-			   #f)
-			  ((substring-at? i)
-			   (substring str1 0 (+fx i len2)))
-			  (else
-			   (loop (+ i 1)))))
-		    #f))))))
+		 (stop (-fx len1 len2))
+		 (start (let loop ((j 0))
+			   (cond
+			      ((=fx j len1)
+			       #f)
+			      ((char=? (string-ref str1 j) #\space)
+			       (+fx j 1))
+			      (else
+			       (loop (+fx j 1)))))))
+	     (if (fixnum? start)
+		 (let loop ((i start))
+		    (cond
+		       ((>fx i stop)
+			#f)
+		       ((substring-at? i str1 str2)
+			(substring str1 0 (+fx i len2)))
+		       (else
+			(loop (+fx i 1)))))
+		 #f))))))
  
 ;*---------------------------------------------------------------------*/
 ;*    find-declaration-lines ...                                       */
@@ -317,12 +347,13 @@
 (define (find-declaration-lines file)
    (let ((iport (and (file-exists? file) (open-input-file file))))
       (if (not (input-port? iport))
-	  (error "btags" "Can't open file for intput" file)
+	  (error 'bgltags "Can't open file for input" file)
 	  (unwind-protect
 	     (let ((exp (read iport #t)))
 		(if (not (eof-object? exp))
 		    (let ((lines (if (module? exp)
-				     (btags-module exp)
+				     (append (btags-module exp)
+					     (btags-expression exp))
 				     (btags-expression exp))))
 		       (let loop ((exp   (read iport #t))
 				  (lines lines))
@@ -341,9 +372,9 @@
 ;*---------------------------------------------------------------------*/
 (define (module? expr)
    (match-case expr
-      ((module (? symbol?) . ?-)
+      (((? (lambda (x) (memq x *btags-module-list*))) (? symbol?) . ?-)
        #t)
-      ((?directives . ?-)
+      ((directives . ?-)
        #t)
       (else
        #f)))
@@ -354,8 +385,10 @@
 (define (line-number expr)
    (and (epair? expr)
 	(match-case (cer expr)
-	   ((at ?- ?pos ?line)
-	    line))))
+	   ((at ?fname ?pos)
+	    (file-position->line (+fx pos 1) fname))
+	   (else
+	    #f))))
 
 ;*---------------------------------------------------------------------*/
 ;*    id->string ...                                                   */
@@ -367,7 +400,7 @@
       ((string? id)
        id)
       ((not (symbol? id))
-       (error "btags" "Illegal identifier" decl))
+       (error 'bgltags "Illegal identifier" decl))
       (else
        (let* ((string (symbol->string id))
 	      (len    (string-length string)))
@@ -425,10 +458,10 @@
    (let loop ((decls clause)
 	      (lines '()))
       (if (pair? decls)
-	  (let ((line (parser (car decls))))
+	  (let ((nlines (parser (car decls))))
 	     (loop (cdr decls)
-		   (if (pair? line)
-		       (cons line lines)
+		   (if (list? nlines)
+		       (append nlines lines)
 		       lines)))
 	  lines)))
    
@@ -441,7 +474,7 @@
 	 ((import (and (? symbol?) ?var))
 	  (let ((lnum (line-number decl)))
 	     (if (number? lnum)
-		 (cons lnum (symbol->string var)))))))
+		 (list (cons lnum (symbol->string var))))))))
    (btags-clause-walker eval-parse clause))
 
 ;*---------------------------------------------------------------------*/
@@ -450,12 +483,65 @@
 (define (btags-static+export-clause clause)
    (define (static+export-parse decl)
       (match-case decl
-	 (((and ?class (or class final-class wide-class)) ?name . ?-)
-	  (let ((lnum (line-number decl)))
-	     (if (number? lnum)
-		 (cons lnum (list (id->string name decl))))))))
+	 (((and ?class (or class final-class wide-class)) ?name . ?clauses)
+	  (btags-class-clause decl class name clauses))))
    (btags-clause-walker static+export-parse clause))
 
+;*---------------------------------------------------------------------*/
+;*    btags-class-clause ...                                           */
+;*---------------------------------------------------------------------*/
+(define (btags-class-clause decl class id slots)
+   (let ((name (id->string id decl)))
+      (define (clines l cid ro cs)
+	 (let ((sid (symbol->string cid)))
+	    (if (number? l)
+		(if ro
+		    (cons (cons l (vector cid
+					  (string-append name "-" sid)))
+			  cs)
+		    (cons* (cons l (vector cid
+					   (string-append name "-" sid)))
+			   (cons l (vector cid
+					   (string-append name "-" sid "-set!")))
+			   cs))
+		cs)))
+      (let loop ((slots slots)
+		 (lines '()))
+	 (if (null? slots)
+	     (let ((l (line-number decl)))
+		(if (number? l)
+		    (reverse!
+		     (cons* (cons l (vector name
+					    (string-append name "?")))
+			    (cons l (vector name
+					    (string-append "make-" name)))
+			    (cons l (vector name
+					    (string-append "fill-" name "!")))
+			    (cons l name)
+			    (reverse! lines)))
+		    (reverse! lines)))
+	     (match-case (car slots)
+		((? symbol?)
+		 (let ((l (line-number slots)))
+		    (loop (cdr slots)
+			  (clines l (car slots) #f lines))))
+		((* (and ?id (? symbol?)) . ?att)
+		 (let ((l (or (line-number (cdr (car slots)))
+			      (line-number (car slots))
+			      (line-number slots)))
+		       (ro (and (list? att) (memq 'read-only att))))
+		    (loop (cdr slots)
+			  (clines l id ro lines))))
+		(((and ?id (? symbol?)) . ?att)
+		 (let ((l (or (line-number (cdr (car slots)))
+			      (line-number (car slots))
+			      (line-number slots)))
+		       (ro (and (list? att) (memq 'read-only att))))
+		    (loop (cdr slots)
+			  (clines l id ro lines))))
+		(else
+		 (loop (cdr slots) lines)))))))
+
 ;*---------------------------------------------------------------------*/
 ;*    btags-extern-clause ...                                          */
 ;*---------------------------------------------------------------------*/
@@ -466,25 +552,25 @@
 	     ((export (and (? symbol?) ?bname) (and (? string?) ?cname))
 	      (let ((lnum (line-number decl)))
 		 (if (number? lnum)
-		     (cons lnum (string-append "\"" cname "\"")))))
+		     (list (cons lnum (string-append "\"" cname "\""))))))
 	     ((or (macro ?l-name ?proto ?c-name)
 		  (infix macro ?l-name ?proto ?c-name))
 	      (let ((lnum (line-number decl)))
 		 (if (number? lnum)
-		     (cons lnum (id->string l-name decl)))))
+		     (list (cons lnum (id->string l-name decl))))))
 	     ((macro ?l-name ?c-name)
 	      (let ((lnum (line-number decl)))
 		 (if (number? lnum)
-		     (cons lnum (id->string l-name decl)))))
+		     (list (cons lnum (id->string l-name decl))))))
 	     ((?l-name ?proto ?c-name)
 	      (let ((lnum (line-number decl)))
 		 (if (number? lnum)
-		     (cons lnum (id->string l-name decl)))))
+		     (list (cons lnum (id->string l-name decl))))))
 	     ((?l-name ?c-name)
 	      (let ((lnum (line-number decl)))
 		 (if (number? lnum)
-		     (cons lnum (id->string l-name decl))))))))
-   (btags-clause-walker extern-parse clause))
+		     (list (cons lnum (id->string l-name decl)))))))))
+   (btags-clause-walker extern-parse (cdr clause)))
 
 ;*---------------------------------------------------------------------*/
 ;*    btags-expression ...                                             */
@@ -494,7 +580,36 @@
       (and (symbol? id) (memq id *btags-define-var-list*)))
    (define (fun-keyword? id)
       (and (symbol? id) (memq id *btags-define-fun-list*)))
+   (define (module-keyword? id)
+      (and (symbol? id) (memq id *btags-module-list*)))
    (match-case expr
+      (((kwote cond-expand) . ?clauses)
+       (letrec ((match-cond (lambda (requirement)
+                              (match-case requirement
+                                (((kwote and) . ?rest)
+                                 (every match-cond rest))
+                                (((kwote or) . ?rest)
+                                 (any match-cond rest))
+                                (((kwote not) ?req)
+                                 (not (match-cond req)))
+                                ((or srfi-0 srfi-1 srfi-2 srfi-6 srfi-8 srfi-9
+                                     srfi-22 srfi-28 srfi-30 srfi-33
+                                     bigloo bigloo-c bigloo-compile bigloo-debug
+                                     bigloo3 bigloo38 debug)
+                                 #t)
+                                ((kwote else)
+				 ;; cond-expand's else-clause
+                                 #t)
+                                (else
+				 ;; no feature match
+                                 #f)))))
+         (let ((first-match (find (lambda (clause) (match-cond (car clause)))
+			       clauses)))
+           (if (and (pair? first-match) (pair? (cdr first-match)))
+	       ;; definition might be any expression
+	       (btags-expression (cadr first-match))
+	       ;; requirement, but empty definition
+	       '()))))
       ((begin . ?rest)
        (apply append (map btags-expression rest)))
       ((define-method (?fun . ?-) . ?-)
@@ -510,7 +625,16 @@
 	  (if (number? lnum)
 	      (list (cons lnum (id->string fun expr)))
 	      '())))
-      (((? var-keyword?) (and (? symbol?) ?var) . ?-)
+      (((and ?def (? var-keyword?)) (and (? symbol?) ?var) . ?-)
+       (let ((lnum (line-number expr)))
+	  (if (number? lnum)
+	      (if (eq? def 'define-parameter)
+		  (let ((s (id->string var expr)))
+		     (list (cons lnum s)
+			   (cons lnum (make-cell s))))
+		  (list (cons lnum (id->string var expr))))
+	      '())))
+      (((? module-keyword?) (and (? symbol?) ?var) . ?-)
        (let ((lnum (line-number expr)))
 	  (if (number? lnum)
 	      (list (cons lnum (id->string var expr)))
