@@ -3,13 +3,15 @@
    (main jas) )
 
 (define (jas a)
-   (if (and (pair? (cdr a)) (string=? (cadr a) "-help"))
+   (if (and (pair? (cdr a))
+	    (or (string=? (cadr a) "-help")
+		(string=? (cadr a) "--help")))
        (print "usage: jas <filein> [-nopeep] <fileout>")
        (match-case (cdr a)
 	  ((?filein ?fileout)
 	   (doit filein fileout) )
 	  ((?filein ?- ?fileout)
-	   (set! **aspeephole** #f)
+	   (set! *jas-peephole* #f)
 	   (doit filein fileout) )
 	  (else (error 'jas "bad command line" a)) )))
 
