@@ -3,7 +3,7 @@
 ;*    -------------------------------------------------------------    */
 ;*    Author      :  Manuel Serrano                                    */
 ;*    Creation    :  Fri Nov 24 11:17:34 1995                          */
-;*    Last change :  Mon Dec  3 18:26:09 2001 (serrano)                */
+;*    Last change :  Mon Jul 31 10:11:01 2006 (serrano)                */
 ;*    -------------------------------------------------------------    */
 ;*    The C syntax                                                     */
 ;*=====================================================================*/
@@ -38,7 +38,10 @@
        < > asm auto break case char const __const continue default do double
        else enum extern float for fortran goto if int long register FILE
        return short signed sizeof static struct switch typedef union unsigned
-       void volatile while __attribute__ inline __inline__ __extension__ obj_t)
+       void volatile while __attribute__ inline __inline__ __inline
+       __extension__ obj_t
+       restrict __restrict__ __restrict
+       __gnuc_va_list)
  
       ;; we build the overall list in a inversed way in order to
       ;; remove a shift/reduce in the grammar.
@@ -57,6 +60,12 @@
 	declaration))
 
       (c++-definition
+       ((extern CONSTANT BRA-OPEN c++-declarations BRA-CLO)
+	(if (string=? (cadr CONSTANT) "\"C\"")
+	    c++-declarations
+	    (error "cigloo"
+		   "illegal syntax"
+		   (list 'extern (car extern) "Illegal construction"))))
        ((extern CONSTANT BRA-OPEN c++-declarations BRA-CLO SEMI-COMMA)
 	(if (string=? (cadr CONSTANT) "\"C\"")
 	    c++-declarations
@@ -127,14 +136,26 @@
 		     init-declarator-list)))
       
       (declaration-specifiers
+       ((gcc-attribute declaration-specifiers)
+         declaration-specifiers)
        ((storage-class-specifier)
 	`(,storage-class-specifier))
+       ((storage-class-specifier gcc-attributes)
+	`(,storage-class-specifier))
        ((storage-class-specifier declaration-specifiers)
 	`(,storage-class-specifier ,@declaration-specifiers))
        ((type-specifier)
 	`(,type-specifier))
+       ((type-specifier gcc-attributes)
+	`(,type-specifier))
        ((type-specifier declaration-specifiers)
-	`(,type-specifier ,@declaration-specifiers)))
+	`(,type-specifier ,@declaration-specifiers))
+       ((type-qualifier-specifier)
+	`(,type-qualifier-specifier))
+       ((type-qualifier-specifier gcc-attributes)
+	`(,type-qualifier-specifier))
+       ((type-qualifier-specifier declaration-specifiers)
+	`(,type-qualifier-specifier ,@declaration-specifiers)))
       
       (init-declarator-list
        ((init-declarator)
@@ -145,7 +166,11 @@
       (init-declarator
        ((declarator)
 	declarator)
+       ((declarator gcc-attributes)
+	declarator)
        ((declarator = initializer)
+	`(,declarator ,initializer))
+       ((declarator = initializer gcc-attributes)
 	`(,declarator ,initializer)))
       
       (storage-class-specifier
@@ -155,12 +180,30 @@
 	(ast-storage-class-spec (car typedef) 'typedef))
        ((extern)
 	(ast-storage-class-spec (car extern) 'extern))
+       ((__extension__ extern)
+	(ast-storage-class-spec (car extern) 'extern))
        ((static)
 	(ast-storage-class-spec (car static) 'static))
        ((auto)
 	(ast-storage-class-spec (car auto) 'auto))
        ((register)
-	(ast-storage-class-spec (car register) 'register)))
+	(ast-storage-class-spec (car register) 'register))
+       ((__inline)
+	(ast-storage-class-spec (car __inline) 'inline)))
+
+      (type-qualifier-specifier
+       ((const)
+	(ast-type-qualifier-spec (car const) 'const))
+       ((__const)
+	(ast-type-qualifier-spec (car __const) 'const))
+       ((volatile)
+	(ast-type-qualifier-spec (car volatile) 'volatile))
+       ((restrict)
+	(ast-type-qualifier-spec (car restrict) 'restrict))
+       ((__restrict)
+	(ast-type-qualifier-spec (car __restrict) 'restrict))
+       ((__restrict__)
+	(ast-type-qualifier-spec (car __restrict__) 'restrict)))
       
       (type-specifier
        ((FILE)
@@ -181,12 +224,6 @@
 	(ast-type-spec (car float) 'float "float" #unspecified))
        ((double)
 	(ast-type-spec (car double) 'double "double" #unspecified))
-       ((const)
-	(ast-type-spec (car const) 'const "const" #unspecified))
-       ((__const)
-	(ast-type-spec (car __const) 'const "const" #unspecified))
-       ((volatile)
-	(ast-type-spec (car volatile) 'volatile "volatile" #unspecified))
        ((void)
 	(ast-type-spec (car void) 'void "void" #unspecified))
        ((obj_t)
@@ -201,6 +238,8 @@
 		       'enum
 		       #unspecified
 		       enum-specifier))
+       ((__gnuc_va_list)
+        (ast-type-spec (car __gnuc_va_list) '__gnuc_va_list "__gnuc_va_list" #unspecified))
        ((TYPE-ID)
 	(ast-type-spec (car TYPE-ID) 'TYPE-ID (cadr TYPE-ID) #unspecified)))
       
@@ -290,44 +329,50 @@
       (declarator
        ((declarator2)
 	(ast-decl #f #f declarator2))
+;       ((declarator2 gcc-attributes)
+;	(ast-decl #f #f declarator2))
        ((pointer declarator2)
 	(ast-decl #f pointer declarator2)))
        
       (declarator2
        ((identifier) 
 	(ast-decl2 #f identifier #f #f #f #f #f))
-       ((identifier gcc-attribute) 
-	(ast-decl2 #f identifier #f #f #f #f #f))
+;       ((identifier gcc-attribute)
+;	(ast-decl2 #f identifier #f #f #f #f #f))
        ((PAR-OPEN declarator PAR-CLO)
-	(ast-decl2 #f #f declarator #f #f #f #f))
-       ((PAR-OPEN declarator PAR-CLO gcc-attribute)
-	(ast-decl2 #f #f declarator #f #f #f #f))
+        (ast-decl2 #f #f declarator #f #f #f #f))
+;       ((PAR-OPEN declarator PAR-CLO gcc-attribute)
+;	(ast-decl2 #f #f declarator #f #f #f #f))
        ((declarator2 ANGLE-OPEN ANGLE-CLO)
-	(ast-decl2 #f #f #f declarator2 '() #f #f))
-       ((declarator2 ANGLE-OPEN ANGLE-CLO gcc-attribute)
-	(ast-decl2 #f #f #f declarator2 '() #f #f))
+        (ast-decl2 #f #f #f declarator2 '() #f #f))
+;       ((declarator2 ANGLE-OPEN ANGLE-CLO gcc-attribute)
+;	(ast-decl2 #f #f #f declarator2 '() #f #f))
        ((declarator2 ANGLE-OPEN constant-expr ANGLE-CLO)
-	(ast-decl2 #f #f #f declarator2 constant-expr #f #f))
-       ((declarator2 ANGLE-OPEN constant-expr ANGLE-CLO gcc-attribute)
-	(ast-decl2 #f #f #f declarator2 constant-expr #f #f))
+        (ast-decl2 #f #f #f declarator2 constant-expr #f #f))
+;       ((declarator2 ANGLE-OPEN constant-expr ANGLE-CLO gcc-attribute)
+;	(ast-decl2 #f #f #f declarator2 constant-expr #f #f))
        ((declarator2 PAR-OPEN PAR-CLO)
-	(ast-decl2 #f #f #f declarator2 #f '() #f))
-       ((declarator2 PAR-OPEN PAR-CLO gcc-attribute)
-	(ast-decl2 #f #f #f declarator2 #f '() #f))
+        (ast-decl2 #f #f #f declarator2 #f '() #f))
+;       ((declarator2 PAR-OPEN PAR-CLO gcc-attribute)
+;	(ast-decl2 #f #f #f declarator2 #f '() #f))
        ((declarator2 PAR-OPEN parameter-type-list PAR-CLO)
-	(ast-decl2 #f #f #f declarator2 #f parameter-type-list #f))
-       ((declarator2 PAR-OPEN parameter-type-list PAR-CLO gcc-attribute)
-	(ast-decl2 #f #f #f declarator2 #f parameter-type-list #f))
+        (ast-decl2 #f #f #f declarator2 #f parameter-type-list #f))
+;       ((declarator2 PAR-OPEN parameter-type-list PAR-CLO gcc-attribute)
+;	(ast-decl2 #f #f #f declarator2 #f parameter-type-list #f))
        ((declarator2 PAR-OPEN parameter-identifier-list PAR-CLO)
-	(ast-decl2 #f #f #f declarator2 #f #f parameter-identifier-list))
-       ((declarator2 PAR-OPEN parameter-identifier-list PAR-CLO gcc-attribute)
-	(ast-decl2 #f #f #f declarator2 #f #f parameter-identifier-list)))
+        (ast-decl2 #f #f #f declarator2 #f #f parameter-identifier-list)) )
+;       ((declarator2 PAR-OPEN parameter-identifier-list PAR-CLO gcc-attribute)
+;	(ast-decl2 #f #f #f declarator2 #f #f parameter-identifier-list)))
       
       (pointer
        ((*)
 	(ast-ptr (car *) #f #f))
+       ((* type-qualifier-specifier)
+	(ast-ptr (car *) #f #f))
        ((* type-specifier-list)
 	(ast-ptr (car *) type-specifier-list #f))
+       ((* type-qualifier-specifier pointer)
+	(ast-ptr (car *) #f pointer))
        ((* pointer)
 	(ast-ptr (car *) #f pointer))
        ((* type-specifier-list pointer)
@@ -336,6 +381,10 @@
       (type-specifier-list
        ((type-specifier)
 	`(,type-specifier))
+       ((type-qualifier-specifier type-specifier-list)
+	`(,type-qualifier-specifier ,@type-specifier-list))
+       ((type-specifier type-qualifier-specifier)
+	`(,type-specifier ,type-qualifier-specifier))
        ((type-specifier type-specifier-list)
 	`(,type-specifier ,@type-specifier-list)))
       
@@ -727,6 +776,12 @@
 
       ;; This is not an ANSI syntax. This is a Gcc specific syntax in order
       ;; to be able to parse the __attribute__ expression. 
+      (gcc-attributes
+        ((gcc-attribute)
+          #unspecified)
+        ((gcc-attribute gcc-attributes)
+          #unspecified))
+
       (gcc-attribute
        ((__attribute__ PAR-OPEN gcc-attribute-values PAR-CLO)
 	#unspecified))
