@@ -3,7 +3,7 @@
 ;*    -------------------------------------------------------------    */
 ;*    Author      :  Manuel Serrano                                    */
 ;*    Creation    :  Tue Nov 28 09:49:40 1995                          */
-;*    Last change :  Sat Nov 20 14:19:44 1999 (serrano)                */
+;*    Last change :  Sun Dec 30 09:11:05 2007 (serrano)                */
 ;*    -------------------------------------------------------------    */
 ;*    The function definition and declaration translation.             */
 ;*=====================================================================*/
@@ -21,6 +21,7 @@
 	    translate_decl
 	    translate_tspec
 	    translate_eval
+	    translate_ident
 	    tools_speek
 	    tools_error)
    (export  (translate-function-definition  <ast>)
@@ -60,8 +61,7 @@
 	 (verbose 2
 		  "do-translate-function-declaration: " #\Newline
 		  "   decl: " decl #\Newline
-		  "   spec: " spec #\Newline
-		  "   para: " para-list #\Newline)
+		  "   spec: " spec #\Newline)
 	 [assert check (type) (function-t? type)]
 	 (let ((sf-id (string->symbol (string-upcase f-id))))
 	    (if (not (getprop sf-id 'fun-processed))
@@ -70,7 +70,7 @@
 		   (verbose 1 "   " f-id #\Newline)
 		   (fprin *oport* "   ("
 			  (if *macro-function* "macro " "")
-			  f-id "::" 
+			  (ident->ident f-id) "::" 
 			  (type-id (function-t-to type))
 			  " ")
 		   (add-eval-function!
@@ -134,7 +134,7 @@
 						    fun-decl)))
 			  [assert check (type) (function-t? type)]
 			  (begin
-			     (display fun-id *oport*)
+			     (display (ident->ident fun-id) *oport*)
 			     (display "::" *oport*)
 			     (display (type-id (function-t-to type)) *oport*)
 			     (display #\space *oport*)
