@@ -1,41 +1,9 @@
-;*---------------------------------------------------------------------*/
-;*   A pratical implementation for the Scheme programming language     */
-;*                                                                     */
-;*                                    ,--^,                            */
-;*                              _ ___/ /|/                             */
-;*                          ,;'( )__, ) '                              */
-;*                         ;;  //   L__.                               */
-;*                         '   \\   /  '                               */
-;*                              ^   ^                                  */
-;*                                                                     */
-;*   Copyright (c) 1992-1999 Manuel Serrano                            */
-;*                                                                     */
-;*     Bug descriptions, use reports, comments or suggestions are      */
-;*     welcome. Send them to                                           */
-;*       bigloo-request@kaolin.unice.fr                                */
-;*       http://kaolin.unice.fr/bigloo                                 */
-;*                                                                     */
-;*   This program is free software; you can redistribute it            */
-;*   and/or modify it under the terms of the GNU General Public        */
-;*   License as published by the Free Software Foundation; either      */
-;*   version 2 of the License, or (at your option) any later version.  */
-;*                                                                     */
-;*   This program is distributed in the hope that it will be useful,   */
-;*   but WITHOUT ANY WARRANTY; without even the implied warranty of    */
-;*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     */
-;*   GNU General Public License for more details.                      */
-;*                                                                     */
-;*   You should have received a copy of the GNU General Public         */
-;*   License along with this program; if not, write to the Free        */
-;*   Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,   */
-;*   MA 02111-1307, USA.                                               */
-;*---------------------------------------------------------------------*/
 ;*=====================================================================*/
-;*    serrano/prgm/project/bigloo/cigloo0.4/Translate/type.scm         */
+;*    serrano/prgm/project/bigloo/cigloo/Translate/type.scm            */
 ;*    -------------------------------------------------------------    */
 ;*    Author      :  Manuel Serrano                                    */
 ;*    Creation    :  Tue Nov 28 10:33:46 1995                          */
-;*    Last change :  Fri Sep  6 11:04:23 1996 (serrano)                */
+;*    Last change :  Fri Nov  4 12:10:46 2011 (serrano)                */
 ;*    -------------------------------------------------------------    */
 ;*    The type translation                                             */
 ;*=====================================================================*/
@@ -63,16 +31,10 @@
 	    (replace-$        <string> <string>)
 	    ($-in-name?       <string>)))
 
-;*---------------------------------------------------------------------*/
-;*    get-hash-number ...                                              */
-;*---------------------------------------------------------------------*/
-(define (get-hash-number o::string)
-   (string->0..2^x-1 o 12))
-
 ;*---------------------------------------------------------------------*/
 ;*    *Tenv*                                                           */
 ;*---------------------------------------------------------------------*/
-(define *Tenv* (make-hash-table 4096 get-hash-number type-id string=? 1024))
+(define *Tenv* (make-hashtable))
 
 ;*---------------------------------------------------------------------*/
 ;*    *Tlist* ...                                                      */
@@ -86,14 +48,14 @@
 ;*    find-type ...                                                    */
 ;*---------------------------------------------------------------------*/
 (define (find-type id)
-   (get-hash id *Tenv*))
+   (hashtable-get *Tenv* id))
 
 ;*---------------------------------------------------------------------*/
 ;*    bind-type! ...                                                   */
 ;*---------------------------------------------------------------------*/
 (define (bind-type! type)
    (set! *Tlist* (cons type *Tlist*))
-   (put-hash! type *Tenv*))
+   (hashtable-put! *Tenv* (type-id type) type))
       
 ;*---------------------------------------------------------------------*/
 ;*    reset-type-list! ...                                             */
@@ -227,7 +189,7 @@
 ;*    make-function-type-name ...                                      */
 ;*---------------------------------------------------------------------*/
 (define (make-function-type-name to from-name)
-   (compose-type-name to (string-append "$(" from-name ")")))
+   (compose-type-name to (string-append "($(" from-name "))")))
 
 ;*---------------------------------------------------------------------*/
 ;*    make-... ...                                                     */
@@ -266,6 +228,8 @@
    (cond
       ((and (simple-t? type) (string=? (simple-t-id type) "char"))
        (simple->type 'char* "char *"))
+      ((and (simple-t? type) (string=? (simple-t-id type) "FILE"))
+       (simple->type 'file* "FILE *"))
       ((function-t? type)
        (make-ptr-to-function type))
       (else
@@ -300,8 +264,11 @@
 ;*    make-array ...                                                   */
 ;*---------------------------------------------------------------------*/
 (define (make-array type constant-expr)
-   (let* ((expr-string (expr->scheme-id constant-expr))
-	  (type-id     (string-append (type-id type) "-array-" expr-string))
+   (let* ((expr-string
+	   (if (null? constant-expr) "" (expr->scheme-id constant-expr)))
+	  (type-id     (string-append (type-id type) "-array"
+				      (if (null? constant-expr) ""
+					  (string-append "-" expr-string))))
 	  (array-type  (find-type type-id)))
       (if (type? array-type)
 	  array-type
@@ -320,7 +287,7 @@
 ;*    make-function ...                                                */
 ;*---------------------------------------------------------------------*/
 (define (make-function to from)
-   (let* ((from-id.name (cond
+   (let* ((from-id-name (cond
 			   ((null? from)
 			    (cons "" ""))
 			   ((null? (cdr from))
@@ -353,8 +320,8 @@
 							   (car from)))
 						      ","
 						      (cdr rec)))))))))
-	  (from-id      (car from-id.name))
-	  (from-name    (cdr from-id.name))
+	  (from-id      (car from-id-name))
+	  (from-name    (cdr from-id-name))
 	  (type-id      (string-append from-id "->" (type-id to))))
       (let ((old-type (find-type type-id)))
 	 (if (type? old-type)
@@ -474,24 +441,52 @@
 	  (warning "cigloo"
 		   "incomplete enum type -- "
 		   (enum-t-c-name enum)))
-      (if (member (type-c-name enum) *no-type*)
-	  'nothing
-	  (begin
-	     (fprin *oport*
-		    "   (type "
-		    (type-id enum)
-		    " (enum")
-	     (for-each (lambda (field)
-			  (fprin *oport*
-				 " ("
-				 (ident-id field)
-				 " \""
-				 (ident-id field)
-				 "\")"))
-		       (enum-t-fields enum))
-	     (fprint *oport* ") \""
-		     (type-c-name enum)
-		     "\")"))))
+      (cond
+	 ((member (type-c-name enum) *no-type*)
+	  'nothing)
+	 (*int-enum*
+	  (fprint *oport*
+		  "   (type "
+		  (type-id enum)
+		  " int "
+		  "\""
+		  (type-c-name enum)
+		  "\")")
+	  (for-each (lambda (field)
+		       (fprint *oport*
+			       "   (macro "
+			       (ident-id field)
+			       "::int \""
+			       (ident-id field)
+			       "\")"))
+		    (enum-t-fields enum)))
+	 (else
+	  (fprin *oport*
+		 "   (type "
+		 (type-id enum)
+		 " (enum")
+	  (for-each (lambda (field)
+		       (fprin *oport*
+			      " ("
+			      (ident-id field)
+			      " \""
+			      (ident-id field)
+			      "\")"))
+		    (enum-t-fields enum))
+	  (fprint *oport* ") \""
+		  (type-c-name enum)
+		  "\")")
+	  (if *enum-macros*
+	      (for-each (lambda (field)
+			   (fprint *oport*
+				   "   (macro "
+				   (ident-id field)
+				   "::"
+				   (type-id enum)
+				   " \""
+				   (ident-id field)
+				   "\")"))
+			(enum-t-fields enum))))))
    (define (translate-array-t array)
       (if (member (type-c-name array) *no-type*)
 	  'nothing
