@@ -1,9 +1,9 @@
 ;*=====================================================================*/
-;*    serrano/prgm/project/bigloo/comptime/Ast/apply.scm               */
+;*    serrano/prgm/project/bigloo/bigloo/comptime/Ast/apply.scm        */
 ;*    -------------------------------------------------------------    */
 ;*    Author      :  Manuel Serrano                                    */
 ;*    Creation    :  Fri Jun 21 09:34:48 1996                          */
-;*    Last change :  Sun Apr 26 19:52:51 1998 (serrano)                */
+;*    Last change :  Thu Jul  8 11:26:41 2021 (serrano)                */
 ;*    -------------------------------------------------------------    */
 ;*    The apply compilation                                            */
 ;*=====================================================================*/
@@ -40,7 +40,11 @@
 				stack
 				(find-location/loc arg loc)
 				'value)))
-	  (if (and (var? proc) (fun? (variable-value (var-variable proc))))
+	  (if (and (var? proc)
+		   (fun? (variable-value (var-variable proc)))
+		   (or (not (global? (var-variable proc)))
+		       (and (not (global-optional? (var-variable proc)))
+			    (not (global-key? (var-variable proc))))))
 	      (known-app-ly->node stack loc proc arg site)
 	      (instantiate::app-ly
 		 (loc loc)
@@ -48,9 +52,8 @@
 		 (fun proc)
 		 (arg arg)))))
       (else
-       (error-sexp->node "Illegal `apply' form"
-			 exp
-			 (find-location/loc exp loc)))))
+       (error-sexp->node
+	"Illegal `apply' form" exp (find-location/loc exp loc)))))
 
 ;*---------------------------------------------------------------------*/
 ;*    make-fun-frame ...                                               */
@@ -97,11 +100,17 @@
 
 ;*---------------------------------------------------------------------*/
 ;*    known-app-ly->node ...                                           */
+;*    -------------------------------------------------------------    */
+;*    This function assumes that proc is not an opt- or key-           */
+;*    function.                                                        */
 ;*---------------------------------------------------------------------*/
 (define (known-app-ly->node stack loc proc arg site)
    (let* ((fun   (variable-value (var-variable proc)))
 	  (arity (fun-arity fun))
 	  (frame (make-fun-frame fun)))
+      (assert (arity fun) (or (<=fx arity 0)
+			      (and (not (sfun-optional? fun))
+				   (not (sfun-key? fun)))))
       (cond
 	 ((>fx arity 0)
 	  (fx-known-app-ly->node stack loc proc arg frame site))
@@ -138,20 +147,20 @@
 (define (fx-known-app-ly->node stack loc proc arg frame site)
    (let ((runner (make-local-svar (mark-symbol-non-user! (gensym 'runner))
 				  *_*))
-	 (type   (node-type proc)))
+	 (type (node-type proc)))
       (local-access-set! runner 'write)
       (instantiate::let-var
 	 (loc loc)
-	 (type type)
+	 (type (strict-node-type *_* type))
 	 (bindings (list (cons runner arg)))
 	 (body (let loop ((locals frame))
 		  (if (null? locals)
 		      (let ((app (application->node
 				  `(,proc
 				    ,@(map (lambda (local)
-					      (instantiate::var
+					      (instantiate::ref
 						 (loc loc)
-						 (type (local-type local))
+						 (type (strict-node-type *_* (local-type local)))
 						 (variable local)))
 					   frame))
 				  stack
@@ -161,9 +170,9 @@
 			     app
 			     (sexp->node
 			      `(if (null? (cdr
-					   ,(instantiate::var
+					   ,(instantiate::ref
 					       (loc loc)
-					       (type (local-type runner))
+					       (type (strict-node-type *_* (local-type runner)))
 					       (variable runner))))
 				   ,app
 				   (failure "apply"
@@ -174,14 +183,14 @@
 			      site)))
 		      (instantiate::let-var
 			 (loc loc)
-			 (type type)
+			 (type (strict-node-type *_* type))
 			 (bindings (list (cons
 					  (car locals)
 					  (sexp->node
 					   `(car
-					     ,(instantiate::var
+					     ,(instantiate::ref
 						 (loc loc)
-						 (type (local-type runner))
+						 (type (strict-node-type *_* (local-type runner)))
 						 (variable runner)))
 					   stack
 					   loc
@@ -190,14 +199,14 @@
 				   (loop (cdr locals))
 				   (sexp->node
 				    `(begin
-					(set! ,(instantiate::var
+					(set! ,(instantiate::ref
 						  (loc loc)
-						  (type (local-type runner))
+						  (type (strict-node-type *_* (local-type runner)))
 						  (variable runner))
 					      (cdr
-					       ,(instantiate::var
+					       ,(instantiate::ref
 						   (loc loc)
-						   (type (local-type runner))
+						   (type (strict-node-type *_* (local-type runner)))
 						   (variable runner))))
 					,(loop (cdr locals)))
 				    stack
@@ -221,14 +230,13 @@
 ;*             (f a0 ... runner))))                                    */
 ;*---------------------------------------------------------------------*/
 (define (va-known-app-ly->node stack loc proc arg frame site)
-   (let ((runner (make-local-svar (mark-symbol-non-user! (gensym 'runner))
-				  *_*))
-	 (type   (node-type proc)))
-      (if (pair? frame)
-	  (local-access-set! runner 'write))
+   (let ((runner (make-local-svar (mark-symbol-non-user! (gensym 'runner)) *_*))
+	 (type   (strict-node-type *_* (node-type proc))))
+      (when (pair? frame)
+	 (local-access-set! runner 'write))
       (instantiate::let-var
 	 (loc loc)
-	 (type type)
+	 (type (strict-node-type *_* type))
 	 (bindings (list (cons runner arg)))
 	 (body (let loop ((locals frame)
 			  (old    '()))
@@ -239,37 +247,37 @@
 			     (set-cdr! old (cons runner '())))
 			 (instantiate::app
 			    (loc loc)
-			    (type (variable-type (var-variable proc)))
-			    (fun (duplicate::var proc))
+			    (type (strict-node-type *_* (variable-type (var-variable proc))))
+			    (fun (duplicate::ref proc))
 			    (args (map (lambda (local)
-					  (instantiate::var
+					  (instantiate::ref
 					     (loc loc)
-					     (type (local-type local))
+					     (type (strict-node-type *_* (local-type local)))
 					     (variable local)))
 				       frame))))
 		      (instantiate::let-var
 			 (loc loc)
-			 (type type)
+			 (type (strict-node-type *_* type))
 			 (bindings (list (cons
 					  (car locals)
 					  (sexp->node
 					   `(car
-					     ,(instantiate::var
+					     ,(instantiate::ref
 						 (loc loc)
-						 (type (local-type runner))
+						 (type (strict-node-type *_* (local-type runner)))
 						 (variable runner)))
 					   stack
 					   loc
 					   'value))))
 			 (body (sexp->node
 				`(begin
-				    (set! ,(instantiate::var
+				    (set! ,(instantiate::ref
 					      (loc loc)
-					      (type (local-type runner))
+					      (type (strict-node-type *_* (local-type runner)))
 					      (variable runner))
-					  (cdr ,(instantiate::var
+					  (cdr ,(instantiate::ref
 						   (loc loc)
-						   (type (local-type runner))
+						   (type (strict-node-type *_* (local-type runner)))
 						   (variable runner))))
 				    ,(loop (cdr locals) locals))
 				stack
