@@ -1,12 +1,12 @@
 ;*=====================================================================*/
-;*    serrano/prgm/project/bigloo/comptime/Ast/remove.scm              */
+;*    serrano/prgm/project/bigloo/bigloo/comptime/Ast/remove.scm       */
 ;*    -------------------------------------------------------------    */
 ;*    Author      :  Manuel Serrano                                    */
 ;*    Creation    :  Mon Jun  3 08:46:28 1996                          */
-;*    Last change :  Tue Dec 26 09:40:12 2000 (serrano)                */
+;*    Last change :  Wed Jun 16 15:53:32 2021 (serrano)                */
 ;*    -------------------------------------------------------------    */
 ;*    This module implements a function which takes a list of          */
-;*    global variable and remove all globals which are not             */
+;*    global variables and remove all globals which are not            */
 ;*    reachable.                                                       */
 ;*=====================================================================*/
 
@@ -20,9 +20,11 @@
 	    tools_shape
 	    effect_effect
 	    engine_param
-	    ast_occur)
+	    ast_occur
+	    ast_env)
    (export  (remove-var-from! ::symbol ::variable)
-	    (remove-var::obj ::obj ::obj)))
+	    (remove-var::obj ::obj ::obj)
+	    (generic node-remove! ::node)))
 
 ;*---------------------------------------------------------------------*/
 ;*    remove-var-from! ...                                             */
@@ -43,7 +45,12 @@
 			  (string-append (symbol->string old)
 					 "/"
 					 (symbol->string pass)))))))
-   
+
+;*---------------------------------------------------------------------*/
+;*    *removed-count* ...                                              */
+;*---------------------------------------------------------------------*/
+(define *removed-count* 0)
+
 ;*---------------------------------------------------------------------*/
 ;*    remove-var ...                                                   */
 ;*---------------------------------------------------------------------*/
@@ -52,17 +59,22 @@
    (occur-var globals)
    ;; then, we do the removal
    (let loop ((globals globals)
-	      (res     '()))
+	      (res     '())
+	      (fixp    #t))
       (if (null? globals)
-	  ;; it is finished ...
-	  (reverse! res)
+	  ;; it is finished, recompute the occur prop (for read/write) ...
+	  (let ((nglobals (reverse! res)))
+	     (occur-var nglobals)
+	     ;; have we reached the fix point?
+	     (if fixp
+		 nglobals
+		 (loop nglobals '() #t)))
 	  (let ((global (car globals)))
 	     (cond
 		;; the function is exported, it hence can't be removed
 		((eq? (global-import global) 'export)
-		 (let ((sfun (global-value global)))
-		    (sfun-body-set! sfun (node-remove! (sfun-body sfun))))
-		 (loop (cdr globals) (cons global res)))
+		 (global-remove-sfun! global)
+		 (loop (cdr globals) (cons global res) fixp))
 		;; we can remove this global from now, 
 		;; we mark it or remove it.
 		((or (eq? (global-removable global) pass)
@@ -73,22 +85,37 @@
 			;; we remove this function
 			(verbose 3 "         removing "
 				 (shape global) #\Newline)
-			(loop (cdr globals) res))
-		     (let ((sfun (global-value global)))
+			(loop (cdr globals) res #f))
+		     (begin
 			(global-removable-set! global 'now)
-			(sfun-body-set! sfun (node-remove! (sfun-body sfun)))
-			(loop (cdr globals) (cons global res)))))
+			(global-remove-sfun! global)
+			(loop (cdr globals) (cons global res) fixp))))
 		((and (<=fx (global-occurrence global) 0)
 		      (eq? (global-removable global) 'now)
 		      (not (eq? pass 'no-remove)))
 		 ;; this function is never used, we skip it
-		 (verbose 3 "        " (shape global) " removed" #\Newline)
-		 (loop (cdr globals) res))
+		 (verbose 3 "        " (shape global) " removed "
+			  " (import: " (global-import global) ")"
+			  #\Newline)
+		 (loop (cdr globals) res #f))
 		(else
-		 (let ((sfun (global-value global)))
-		    (sfun-body-set! sfun (node-remove! (sfun-body sfun)))
-		    ;; we keep the function
-		    (loop (cdr globals) (cons global res)))))))))
+		 (global-remove-sfun! global)
+		 ;; we keep the function
+		 (loop (cdr globals) (cons global res) fixp)))))))
+
+;*---------------------------------------------------------------------*/
+;*    global-remove-sfun! ...                                          */
+;*---------------------------------------------------------------------*/
+(define (global-remove-sfun! global)
+   (let ((sfun (global-value global)))
+      (set! *removed-count* 0)
+      (sfun-body-set! sfun (node-remove! (sfun-body sfun)))
+      (if (and (>=fx *optim* 2) (>fx *removed-count* 0))
+	  ;; execute the fix point iteration for optmized mode only
+	  (begin
+	     (occur-node-sfun! (global-value global) global)
+	     (global-remove-sfun! global))
+	  global)))
    
 ;*---------------------------------------------------------------------*/
 ;*    node-remove! ...                                                 */
@@ -114,12 +141,21 @@
    node)
        
 ;*---------------------------------------------------------------------*/
-;*    node-remove*! ::sequence ...                                     */
+;*    node-remove! ::sequence ...                                      */
 ;*---------------------------------------------------------------------*/
 (define-method (node-remove! node::sequence)
    (node-remove*! (sequence-nodes node))
    node)
 
+;*---------------------------------------------------------------------*/
+;*    node-remove! ::sync ...                                          */
+;*---------------------------------------------------------------------*/
+(define-method (node-remove! node::sync)
+   (sync-mutex-set! node (node-remove! (sync-mutex node)))
+   (sync-prelock-set! node (node-remove! (sync-prelock node)))
+   (sync-body-set! node (node-remove! (sync-body node)))
+   node)
+
 ;*---------------------------------------------------------------------*/
 ;*    node-remove! ::app ...                                           */
 ;*---------------------------------------------------------------------*/
@@ -155,7 +191,7 @@
 ;*    node-remove! ::cast ...                                          */
 ;*---------------------------------------------------------------------*/
 (define-method (node-remove! node::cast)
-   (node-remove! (cast-arg node))
+   (cast-arg-set! node (node-remove! (cast-arg node)))
    node)
 
 ;*---------------------------------------------------------------------*/
@@ -184,13 +220,13 @@
    node)
 
 ;*---------------------------------------------------------------------*/
-;*    node-remove! ::select ...                                        */
+;*    node-remove! ::switch ...                                        */
 ;*---------------------------------------------------------------------*/
-(define-method (node-remove! node::select)
-   (select-test-set! node (node-remove! (select-test node)))
+(define-method (node-remove! node::switch)
+   (switch-test-set! node (node-remove! (switch-test node)))
    (for-each (lambda (clause)
 		(set-cdr! clause (node-remove! (cdr clause))))
-	     (select-clauses node))
+	     (switch-clauses node))
    node)
 
 ;*---------------------------------------------------------------------*/
@@ -231,7 +267,9 @@
 		    node))
 	     (let ((local (car old-locals)))
 		(if (<=fx (local-occurrence local) 0)
-		    (loop (cdr old-locals) new-locals)
+		    (begin
+		       (set! *removed-count* (+fx *removed-count* 1))
+		       (loop (cdr old-locals) new-locals))
 		    (let ((sfun (local-value local)))
 		       (sfun-body-set! sfun (node-remove! (sfun-body sfun)))
 		       (loop (cdr old-locals) (cons local new-locals)))))))))
@@ -255,10 +293,11 @@
 		    (set! bindings (reverse! new-bindings))
 		    node))
 	     (let* ((binding (car old-bindings))
-		    (var     (car binding))
-		    (val     (node-remove! (cdr binding))))
+		    (var (car binding))
+		    (val (node-remove! (cdr binding))))
 		(if (and (<=fx (local-occurrence var) 0)
 			 (not (side-effect? val))
+			 (eq? (local-access var) 'read)
 			 (or (not (local-user? var))
 			     (=fx *bdb-debug* 0)))
 		    (begin
@@ -267,6 +306,7 @@
 			      (variable-occurrence-set!
 			       v
 			       (-fx (variable-occurrence v) 1))))
+		       (set! *removed-count* (+fx *removed-count* 1))
 		       (loop (cdr old-bindings) new-bindings))
 		    (begin
 		       (set-cdr! binding val)
@@ -278,6 +318,7 @@
 ;*---------------------------------------------------------------------*/
 (define-method (node-remove! node::set-ex-it)
    (set-ex-it-body-set! node (node-remove! (set-ex-it-body node)))
+   (set-ex-it-onexit-set! node (node-remove! (set-ex-it-onexit node)))
    (if (<=fx (local-occurrence (var-variable (set-ex-it-var node))) 0)
        (set-ex-it-body node)
        node))
@@ -290,6 +331,20 @@
    (jump-ex-it-value-set! node (node-remove! (jump-ex-it-value node)))
    node)
 
+;*---------------------------------------------------------------------*/
+;*    node-remove! ::retblock ...                                      */
+;*---------------------------------------------------------------------*/
+(define-method (node-remove! node::retblock)
+   (retblock-body-set! node (node-remove! (retblock-body node)))
+   node)
+
+;*---------------------------------------------------------------------*/
+;*    node-remove! ::return ...                                        */
+;*---------------------------------------------------------------------*/
+(define-method (node-remove! node::return)
+   (return-value-set! node (node-remove! (return-value node)))
+   node)
+
 ;*---------------------------------------------------------------------*/
 ;*    node-remove*! ...                                                */
 ;*---------------------------------------------------------------------*/
