@@ -1,41 +1,9 @@
-;*---------------------------------------------------------------------*/
-;*   A pratical implementation for the Scheme programming language     */
-;*                                                                     */
-;*                                    ,--^,                            */
-;*                              _ ___/ /|/                             */
-;*                          ,;'( )__, ) '                              */
-;*                         ;;  //   L__.                               */
-;*                         '   \\   /  '                               */
-;*                              ^   ^                                  */
-;*                                                                     */
-;*   Copyright (c) 1992-1999 Manuel Serrano                            */
-;*                                                                     */
-;*     Bug descriptions, use reports, comments or suggestions are      */
-;*     welcome. Send them to                                           */
-;*       bigloo-request@kaolin.unice.fr                                */
-;*       http://kaolin.unice.fr/bigloo                                 */
-;*                                                                     */
-;*   This program is free software; you can redistribute it            */
-;*   and/or modify it under the terms of the GNU General Public        */
-;*   License as published by the Free Software Foundation; either      */
-;*   version 2 of the License, or (at your option) any later version.  */
-;*                                                                     */
-;*   This program is distributed in the hope that it will be useful,   */
-;*   but WITHOUT ANY WARRANTY; without even the implied warranty of    */
-;*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     */
-;*   GNU General Public License for more details.                      */
-;*                                                                     */
-;*   You should have received a copy of the GNU General Public         */
-;*   License along with this program; if not, write to the Free        */
-;*   Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,   */
-;*   MA 02111-1307, USA.                                               */
-;*---------------------------------------------------------------------*/
 ;*=====================================================================*/
-;*    serrano/prgm/project/bigloo/comptime1.9/Ast/shrinkify.scm        */
+;*    .../prgm/project/bigloo/bigloo/comptime/Ast/shrinkify.scm        */
 ;*    -------------------------------------------------------------    */
 ;*    Author      :  Manuel Serrano                                    */
 ;*    Creation    :  Fri Jul  5 11:09:52 1996                          */
-;*    Last change :  Mon Jul 15 14:39:17 1996 (serrano)                */
+;*    Last change :  Wed Jun 16 15:53:47 2021 (serrano)                */
 ;*    -------------------------------------------------------------    */
 ;*    We shrink all the ast to get rid off all the pass info for the   */
 ;*    following passes.                                                */
@@ -102,6 +70,15 @@
    (shrink-node*! (sequence-nodes node))
    #unspecified)
 
+;*---------------------------------------------------------------------*/
+;*    shrink-node*! ::sync ...                                         */
+;*---------------------------------------------------------------------*/
+(define-method (shrink-node! node::sync)
+   (shrink-node! (sync-mutex node))
+   (shrink-node! (sync-prelock node))
+   (shrink-node! (sync-body node))
+   #unspecified)
+
 ;*---------------------------------------------------------------------*/
 ;*    shrink-node! ::app ...                                           */
 ;*---------------------------------------------------------------------*/
@@ -112,7 +89,7 @@
    #unspecified)
 
 ;*---------------------------------------------------------------------*/
-;*    shrink-node! ::app-ly ...                                         */
+;*    shrink-node! ::app-ly ...                                        */
 ;*---------------------------------------------------------------------*/
 (define-method (shrink-node! node::app-ly)
    (if (wide-object? node) (shrink! node))
@@ -130,19 +107,19 @@
    #unspecified)
 
 ;*---------------------------------------------------------------------*/
-;*    shrink-node! ::pragma ...                                        */
+;*    shrink-node! ::extern ...                                        */
 ;*---------------------------------------------------------------------*/
-(define-method (shrink-node! node::pragma)
+(define-method (shrink-node! node::extern)
    (if (wide-object? node) (shrink! node))
-   (shrink-node*! (pragma-args node))
-   node)
+   (shrink-node*! (extern-expr* node))
+   #unspecified)
 
 ;*---------------------------------------------------------------------*/
 ;*    shrink-node! ::cast ...                                          */
 ;*---------------------------------------------------------------------*/
 (define-method (shrink-node! node::cast)
    (shrink-node! (cast-arg node))
-   node)
+   #unspecified)
 
 ;*---------------------------------------------------------------------*/
 ;*    shrink-node! ::setq ...                                          */
@@ -170,13 +147,13 @@
    #unspecified)
 
 ;*---------------------------------------------------------------------*/
-;*    shrink-node! ::select ...                                        */
+;*    shrink-node! ::switch ...                                        */
 ;*---------------------------------------------------------------------*/
-(define-method (shrink-node! node::select)
-   (shrink-node! (select-test node))
+(define-method (shrink-node! node::switch)
+   (shrink-node! (switch-test node))
    (for-each (lambda (clause)
 		(shrink-node! (cdr clause)))
-	     (select-clauses node))
+	     (switch-clauses node))
    #unspecified)
 
 ;*---------------------------------------------------------------------*/
@@ -234,9 +211,10 @@
 ;*    shrink-node! ::set-ex-it ...                                     */
 ;*---------------------------------------------------------------------*/
 (define-method (shrink-node! node::set-ex-it)
-   (with-access::set-ex-it node (var body)
+   (with-access::set-ex-it node (var body onexit)
       (shrink-node! var)
       (shrink-node! body)
+      (shrink-node! onexit)
       #unspecified)) 
 
 ;*---------------------------------------------------------------------*/
