@@ -1,9 +1,9 @@
 ;*=====================================================================*/
-;*    serrano/prgm/project/bigloo/comptime.case1.3/Ast/venv.scm        */
+;*    serrano/prgm/project/bigloo/bigloo/comptime/Ast/venv.scm         */
 ;*    -------------------------------------------------------------    */
 ;*    Author      :  Manuel Serrano                                    */
 ;*    Creation    :  Sun Dec 25 11:32:49 1994                          */
-;*    Last change :  Thu Jul  5 15:21:19 2001 (serrano)                */
+;*    Last change :  Sun Apr 14 08:04:18 2019 (serrano)                */
 ;*    -------------------------------------------------------------    */
 ;*    The global environment manipulation                              */
 ;*=====================================================================*/
@@ -21,17 +21,22 @@
 	    ast_var
 	    ast_node
 	    ast_hrtype
+	    ast_dump
 	    read_jvm
-	    module_module)
+	    module_module
+	    tools_location
+	    backend_backend
+	    inline_inline)
    (export  (initialize-Genv!)
-	    (set-genv!              <Genv>)
-	    (add-genv!              <Genv>)
+	    (set-genv! <Genv>)
+	    (add-genv! <Genv>)
 	    (get-genv) 
-	    (find-global            ::symbol . <symbol>)
-	    (find-global/module     ::symbol ::symbol)
-	    (bind-global!::global   ::symbol ::symbol ::value ::symbol ::obj)
-	    (unbind-global!         ::symbol ::symbol)
-	    (for-each-global!       ::procedure)
+	    (find-global ::symbol . <symbol>)
+	    (find-global/module ::symbol ::symbol)
+	    (get-global/module ::symbol ::symbol)
+	    (bind-global!::global ::symbol ::obj ::symbol ::value ::symbol ::obj)
+	    (unbind-global! ::symbol ::symbol)
+	    (for-each-global! ::procedure . env)
 	    (global-bucket-position ::symbol ::symbol)
 	    (restore-global! new)
 	    (additional-heap-restore-globals!)
@@ -68,7 +73,14 @@
 		       (cond
 			  ((not (pair? bucket))
 			   (hashtable-put! *Genv* id (list id new)))
-			  ((eq? module *module*)
+			  ((or (eq? module *module*)
+			       (not (eq? *module*
+					 (global-module (cadr bucket)))))
+			   ;; we add the new global in first position if:
+			   ;;   - we are binding a variable of the current
+			   ;;     module
+			   ;;   - the first global already bound is not owned
+			   ;;     by the current module
 			   (let ((new-bucket (cons new (cdr bucket))))
 			      (set-cdr! bucket new-bucket)))
 			  (else
@@ -103,17 +115,17 @@
    (let* ((id      (global-id new))
 	  (type    (global-type new))
 	  (value   (global-value new))
-	  (type-id (type-id type)))
-      ;; we mark the global variable
-      (global-library?-set! new #t)
+	  (typeid  (type-id type)))
       ;; we restore type result
-      (global-type-set! new (find-type type-id))
+      (global-type-set! new (find-type typeid))
       ;; the parameters type
-      (restore-value-types! value)
+      (restore-value-types! value id)
       ;; we restore the jvm qualified type name
-      (if (eq? *target-language* 'jvm)
-	  (add-qualified-type! (global-module new)
-			       (global-jvm-type-name new)))))
+      (when (and (backend-qualified-types (the-backend))
+		 (not (eq? (global-module new) 'foreign)))
+	 (add-qualified-type! (global-module new)
+	    (global-jvm-type-name new)
+	    (shape new)))))
 
 ;*---------------------------------------------------------------------*/
 ;*    *restored* ...                                                   */
@@ -138,13 +150,22 @@
 ;*---------------------------------------------------------------------*/
 ;*    restore-value-types! ...                                         */
 ;*---------------------------------------------------------------------*/
-(define-generic (restore-value-types! value::value)
+(define-generic (restore-value-types! value::value id)
    #unspecified)
 
+;*---------------------------------------------------------------------*/
+;*    restore-value-types! ::fun ...                                   */
+;*---------------------------------------------------------------------*/
+(define-method (restore-value-types! value::fun id)
+   (with-access::fun value (predicate-of)
+      (when (type? predicate-of)
+         (set! predicate-of (find-type (type-id predicate-of))))))
+
 ;*---------------------------------------------------------------------*/
 ;*    restore-value-types! ::sfun ...                                  */
 ;*---------------------------------------------------------------------*/
-(define-method (restore-value-types! value::sfun)
+(define-method (restore-value-types! value::sfun id)
+   (call-next-method)
    (with-access::sfun value (args)
       (let loop ((args args))
 	 (cond
@@ -166,25 +187,37 @@
 		;; we still have to restore the body types
 		(if (node? body)
 		    (let ((tres (node-type body)))
-		       (if (type? tres)
-			   (begin
-			      (hrtype-node! body)
-			      (node-type-set! body
-					      (find-type (type-id tres)))))))))
+		       (hrtype-node! body)
+		       (when (type? tres)
+			  (node-type-set! body (find-type (type-id tres))))))))
 	    (else
 	     (error "restore-value-types"
 		    "Illegal non pair argument"
 		    (shape args)))))))
 
+;*---------------------------------------------------------------------*/
+;*    restore-value-types! ::isfun ...                                 */
+;*---------------------------------------------------------------------*/
+(define-method (restore-value-types! value::isfun id)
+   (call-next-method)
+   (hrtype-node! (isfun-original-body value)))
+
 ;*---------------------------------------------------------------------*/
 ;*    restore-value-types! ::cfun ...                                  */
 ;*---------------------------------------------------------------------*/
-(define-method (restore-value-types! value::cfun)
+(define-method (restore-value-types! value::cfun id)
+   (call-next-method)
    (with-access::cfun value (args-type)
       (let loop ((args args-type))
 	 (if (pair? args)
 	     (begin
-		(set-car! args (find-type (type-id (car args))))
+		(if (eq? (type-id (car args)) '_)
+		    (begin
+		       (user-warning "head-restore"
+			  "Illegal restored type for foreign function"
+			  id)
+		       (set-car! args *obj*))
+		    (set-car! args (find-type (type-id (car args)))))
 		(loop (cdr args)))))))
    
 ;*---------------------------------------------------------------------*/
@@ -203,7 +236,7 @@
 ;*    find-global ...                                                  */
 ;*---------------------------------------------------------------------*/
 (define (find-global id::symbol . module)
-   [assert (module) (or (null? module) (symbol? (car module)))]
+   (assert (module) (or (null? module) (symbol? (car module))))
    (let ((bucket (hashtable-get *Genv* id))
 	 (module (if (null? module) '() (car module))))
       (cond
@@ -245,61 +278,118 @@
 		(else
 		 (loop (cdr globals)))))))))
 
+;*---------------------------------------------------------------------*/
+;*    get-global/module ...                                            */
+;*---------------------------------------------------------------------*/
+(define (get-global/module id::symbol module)
+   (let ((global (find-global/module id module)))
+      (when (and (not (global? global)) (not *lib-mode*))
+	  (internal-error 'get-global/module
+			  "Cannot find global variable"
+			  (format "~a::~a" id module)))
+      global))
+
+;*---------------------------------------------------------------------*/
+;*    warning-override-global! ...                                     */
+;*---------------------------------------------------------------------*/
+(define (warning-override-global! n o srce)
+   (when (and (>fx (bigloo-warning) 0) *warning-overriden-variables*)
+      (with-access::global o (id module src)
+	 (let ((loc (or (find-location src) (find-location srce)))
+	       (old `(@ ,id ,module))
+	       (new `(@ ,(global-id n) ,(global-module n)))
+	       (msg "Variable overridden by"))
+	    (if loc
+		(user-warning/location loc old msg new)
+		(user-warning old msg new))))))
+
+;*---------------------------------------------------------------------*/
+;*    error-rebind-global! ...                                         */
+;*---------------------------------------------------------------------*/
+(define (error-rebind-global! o src)
+   (with-access::global o (id module)
+      (let ((loc (find-location src))
+	    (msg "Duplicate definition"))
+	 (if loc
+	     (user-error/location loc module msg (shape o))
+	     (user-error module msg (shape o))))))
+
+;*---------------------------------------------------------------------*/
+;*    warning-rebind-global! ...                                       */
+;*---------------------------------------------------------------------*/
+(define (warning-rebind-global! o src)
+   (with-access::global o (id module)
+      (let ((loc (find-location src))
+	    (msg "Duplicate definition"))
+	 (if loc
+	     (user-warning/location loc module msg (shape o))
+	     (user-warning module msg (shape o))))))
+
 ;*---------------------------------------------------------------------*/
 ;*    bind-global! ...                                                 */
 ;*    -------------------------------------------------------------    */
-;*    When binding a global, if a previous global with the same id     */
-;*    has already been bound, we follow the two rules:                 */
-;*       1- if module is the name of the current module, the global    */
-;*          is added at the head of the list.                          */
-;*       2- if module is not the name of the current module, the       */
-;*          global is not added at the head of the list (practically,  */
-;*          it is added in second position).                           */
-;*    Moreover, because we have add a lot of confusion because of this */
-;*    we always check if we are redefining a foreign function with a   */
-;*    Scheme function. In such a situation, we raise a warning.        */
+;*    This function binds a new global variable in the global table.   */
+;*    When several variables share the identifier the following rules  */
+;*    are observed:                                                    */
+;*       1- if the two declarations concerns the same module, an error */
+;*          is notified.                                               */
+;*       2- the variable defined in the current module has the highest */
+;*          priority (it override the others definitions).             */
+;*       3- imported variables have a higher priority than library     */
+;*          variables.                                                 */
+;*       4- user library variables have a higher priority than system  */
+;*          library variables.                                         */
 ;*---------------------------------------------------------------------*/
-(define (bind-global!::global id::symbol
-			      module::symbol
-			      value::value
-			      import::symbol
+(define (bind-global!::global id::symbol alias::obj module::symbol
+			      value::value import::symbol
 			      src::obj)
-   (let ((global (find-global id module)))
-      ;; If the current module if not foreign we make the foreign check
-      ;; descibed above
-      (if (not (eq? module 'foreign))
-	  (let ((old-foreign (find-global/module id 'foreign)))
-	     (if (global? old-foreign)
-		 (if (and (number? *warning*) (>=fx *warning* 2))
-		     (user-warning id
-				   "Scheme declaration overrides foreign declaration"
-				   src)))))
-      ;; Now we keep going we the other check.
-      (if (global? global)
-	  (if (not *lib-mode*)
-	      (user-error id "Illegal global redefinition" src)
-	      global)
-	  (let* ((jvm-qtype (if (eq? import 'eval)
-				"eval"
-				(module->qualified-type module)))
-		 (new (instantiate::global
-			 (module module)
-			 (jvm-type-name jvm-qtype)
-			 (id id)
-			 (value value)
-			 (src src) 
-			 (import import)))
-		 (bucket (hashtable-get *Genv* id)))
+   (let* ((ident (or alias id))
+	  (old (find-global ident module)))
+      (if (global? old)
+	  (cond
+	     (*lib-mode*
+	      old)
+	     ((eq? module (global-module old))
+	      (warning-rebind-global! old src)
+	      old)
+	     (else
+	      (error-rebind-global! old src)))
+	  (let ((bucket (hashtable-get *Genv* ident))
+		(new (instantiate::global
+			(type *_*)
+			(module module)
+			(jvm-type-name (if (eq? import 'eval)
+					   "eval"
+					   (module->qualified-type module)))
+			(id ident)
+			(alias (when alias id))
+			(value value)
+			(src src)
+			(user? #t)
+			(import import))))
 	     (cond
-		((not (pair? bucket))
-		 (hashtable-put! *Genv* id (list id new)))
-		((eq? module *module*)
-		 (let ((new-bucket (cons new (cdr bucket))))
-		    (set-cdr! bucket new-bucket)))
+		((or (not (pair? bucket)) (null? (cdr bucket)))
+		 ;; this is the firt time we see this identifier
+		 (hashtable-put! *Genv* ident (list ident new))
+		 new)
 		(else
-		 (set-cdr! (cdr bucket) (cons new (cddr bucket)))))
-	     new))))
- 
+		 (let* ((old* (cdr bucket))
+			(mid (module-initialization-id
+			      (global-module (car old*)))))
+		    (cond
+		       ((eq? (global-module (car old*)) *module*)
+			;; hidden by a local variable
+			(if (and (not (eq? ident mid)) (not *lib-mode*))
+			    (warning-override-global! (car old*) new src))
+			(set-cdr! (cdr bucket) (cons new (cddr bucket)))
+			new)
+		       (else
+			(let ((new* (cons new old*)))
+			   (if (and (not (eq? ident mid)) (not *lib-mode*))
+			       (warning-override-global! new (car old*) src))
+			   (set-cdr! bucket new*)
+			   new))))))))))
+
 ;*---------------------------------------------------------------------*/
 ;*    unbind-global! ...                                               */
 ;*---------------------------------------------------------------------*/
@@ -317,8 +407,8 @@
 ;*---------------------------------------------------------------------*/
 ;*    for-each-global! ...                                             */
 ;*---------------------------------------------------------------------*/
-(define (for-each-global! proc::procedure)
-   (hashtable-for-each *Genv*
+(define (for-each-global! proc::procedure . env)
+   (hashtable-for-each (if (null? env) *Genv* (car env))
 		       (lambda (k bucket) (for-each proc (cdr bucket)))))
    
 ;*---------------------------------------------------------------------*/
