@@ -1,10 +1,10 @@
 ;*=====================================================================*/
-;*    serrano/prgm/project/bigloo2.3/comptime/Callcc/walk.scm          */
+;*    serrano/prgm/project/bigloo/bigloo/comptime/Callcc/walk.scm      */
 ;*    -------------------------------------------------------------    */
 ;*    Author      :  Manuel Serrano                                    */
 ;*    Creation    :  Fri Apr 28 10:50:15 1995                          */
-;*    Last change :  Thu Jul 13 11:08:44 2000 (serrano)                */
-;*    Copyright   :  1995-2000 Manuel Serrano, see LICENSE file        */
+;*    Last change :  Thu Jul  8 11:31:37 2021 (serrano)                */
+;*    Copyright   :  1995-2021 Manuel Serrano, see LICENSE file        */
 ;*    -------------------------------------------------------------    */
 ;*    When compiling for call/cc we put all written local variables    */
 ;*    in cells.                                                        */
@@ -15,7 +15,8 @@
 ;*---------------------------------------------------------------------*/
 (module callcc_walk
    (include "Engine/pass.sch"
-	    "Ast/node.sch")
+	    "Ast/node.sch"
+	    "Callcc/walk.sch")
    (import  tools_error
 	    tools_shape
 	    type_cache
@@ -30,8 +31,8 @@
 (define (callcc-walk! globals)
    (pass-prelude "Callcc")
    ;; -g3 and -call/cc are incompatible
-   (if (>fx *compiler-debug* 2)
-       (set! *compiler-debug* 2))
+   (if (>=fx *compiler-debug* 2)
+       (set! *compiler-debug* 1))
    (for-each callcc-fun! globals)
    (pass-postlude globals))
 
@@ -43,13 +44,13 @@
 	  (body   (sfun-body fun))
 	  (celled (celled-bindings (sfun-args fun))))
       ;; we set alpha-fast slot
-      (for-each (lambda (w.b)
-		   (local-fast-alpha-set! (car w.b) (cdr w.b)))
+      (for-each (lambda (w-b)
+		   (local-fast-alpha-set! (car w-b) (cdr w-b)))
 		celled)
       (sfun-body-set! fun (cell-formals celled (callcc! body)))
       ;; we remove alpha-fast slots
-      (for-each (lambda (w.b)
-		   (local-fast-alpha-set! (car w.b) #unspecified))
+      (for-each (lambda (w-b)
+		   (local-fast-alpha-set! (car w-b) #unspecified))
 		celled)))
 
 ;*---------------------------------------------------------------------*/
@@ -64,10 +65,15 @@
 	 ((not (celled? (car formals)))
 	  (loop celled (cdr formals)))
 	 (else
-	  (let* ((var (make-local-svar (local-id (car formals)) *obj*))
-		 (o.n (cons (car formals) var)))
+	  (let* ((vtype (local-type (car formals)))
+		 (ntype (cond
+			   ((eq? vtype *_*) *obj*)
+			   ((bigloo-type? vtype) vtype)
+			   (else (get-bigloo-type vtype))))
+		 (var (make-local-svar (local-id (car formals)) *cell*))
+		 (o-n (cons (car formals) var)))
 	     (widen!::local/cell var)
-	     (loop (cons o.n celled) (cdr formals)))))))
+	     (loop (cons o-n celled) (cdr formals)))))))
 
 ;*---------------------------------------------------------------------*/
 ;*    cell-formals ...                                                 */
@@ -79,15 +85,16 @@
 	  (instantiate::let-var
 	     (loc loc)
 	     (body body)
-	     (type *_*)
-	     (bindings (map (lambda (o.n)
-			       (cons (cdr o.n)
-				     (a-make-cell (instantiate::var
-						     (type *_*)
-						     (loc loc)
-						     (variable (car o.n)))
-						  (car o.n))))
-			    celled))))))
+	     (type (node-type body))
+	     (bindings (map (lambda (o-n)
+			       (cons (cdr o-n)
+				  (a-make-cell
+				     (instantiate::ref
+					(type (variable-type (car o-n)))
+					(loc loc)
+					(variable (car o-n)))
+				     (car o-n))))
+			  celled))))))
 
 ;*---------------------------------------------------------------------*/
 ;*    a-make-cell ...                                                  */
@@ -96,13 +103,20 @@
    (with-access::node node (loc)
       (local-access-set! variable 'cell-callcc)
       (widen!::local/cell variable)
-      (instantiate::make-box (type *_*) (loc loc) (value node))))
+      (instantiate::make-box
+	 (type *cell*)
+	 (vtype (get-bigloo-defined-type (variable-type variable)))
+	 (loc loc)
+	 (value node))))
    
 ;*---------------------------------------------------------------------*/
 ;*    celled? ...                                                      */
 ;*---------------------------------------------------------------------*/
 (define (celled?::bool var::variable)
-   (or (local/cell? var) (eq? (local-access var) 'write)))
+   (or (local/cell? var)
+       (and (eq? (local-access var) 'write)
+	    (or (eq? (local-type var) *_*)
+		(bigloo-type? (local-type var))))))
 
 ;*---------------------------------------------------------------------*/
 ;*    callcc! ...                                                      */
@@ -125,7 +139,8 @@
 ;*    callcc! ::var ...                                                */
 ;*---------------------------------------------------------------------*/
 (define-method (callcc! node::var)
-   (let ((var (var-variable node)))
+   (let* ((var (var-variable node))
+	  (vtype (variable-type var)))
       (let loop ((var   var)
 		 (alpha (variable-fast-alpha var)))
 	 (if (local? alpha)
@@ -135,16 +150,20 @@
 	     (cond
 		((local? alpha)
 		 (var-variable-set! node alpha)
+		 (node-type-set! node (variable-type alpha))
 		 (callcc! node))
 		((global? var)
 		 node)
 		((not (celled? var))
 		 node)
 		(else
-		 (instantiate::box-ref
-		    (type *_*)
-		    (loc (node-loc node))
-		    (var node))))))))
+		 (let ((vtype (get-bigloo-defined-type vtype)))
+		    (node-type-set! node *cell*)
+		    (instantiate::box-ref
+		       (type vtype)
+		       (vtype vtype)
+		       (loc (node-loc node))
+		       (var node)))))))))
 
 ;*---------------------------------------------------------------------*/
 ;*    callcc! ::sequence ...                                           */
@@ -153,6 +172,16 @@
    (callcc*! (sequence-nodes node))
    node)
 
+;*---------------------------------------------------------------------*/
+;*    callcc! ::sync ...                                               */
+;*---------------------------------------------------------------------*/
+(define-method (callcc! node::sync)
+   (with-access::sync node (mutex prelock body)
+      (set! mutex (callcc! mutex))
+      (set! prelock (callcc! prelock))
+      (set! body (callcc! body))
+      node))
+
 ;*---------------------------------------------------------------------*/
 ;*    callcc! ::app ...                                                */
 ;*---------------------------------------------------------------------*/
@@ -162,7 +191,7 @@
       node))
  
 ;*---------------------------------------------------------------------*/
-;*    callcc! ::app-ly ...                                              */
+;*    callcc! ::app-ly ...                                             */
 ;*---------------------------------------------------------------------*/
 (define-method (callcc! node::app-ly)
    (with-access::app-ly node (fun arg)
@@ -190,7 +219,7 @@
 ;*    callcc! ::cast ...                                               */
 ;*---------------------------------------------------------------------*/
 (define-method (callcc! node::cast)
-   (callcc! (cast-arg node))
+   (cast-arg-set! node (callcc! (cast-arg node)))
    node)
 
 ;*---------------------------------------------------------------------*/
@@ -198,7 +227,8 @@
 ;*---------------------------------------------------------------------*/
 (define-method (callcc! node::setq)
    (setq-value-set! node (callcc! (setq-value node)))
-   (let ((var (var-variable (setq-var node))))
+   (let* ((var (var-variable (setq-var node)))
+	  (vtype (variable-type var)))
       (let loop ((var   var)
 		 (alpha (variable-fast-alpha var)))
 	 (if (local? alpha)
@@ -214,15 +244,16 @@
 		 (let ((a-var (make-local-svar 'aux *obj*))
 		       (loc   (node-loc node)))
 		    (instantiate::let-var
-		       (type     *_*)
+		       (type     *unspec*)
 		       (loc      loc)
 		       (bindings (list (cons a-var (setq-value node))))
 		       (body     (instantiate::box-set!
-				    (type *_*)
+				    (type *unspec*)
+				    (vtype (get-bigloo-defined-type vtype))
 				    (loc loc)
 				    (var (setq-var node))
-				    (value (instantiate::var
-					      (type *_*)
+				    (value (instantiate::ref
+					      (type (variable-type a-var))
 					      (loc loc)
 					      (variable a-var)))))))))))))
 
@@ -247,10 +278,10 @@
       node))
 
 ;*---------------------------------------------------------------------*/
-;*    callcc! ::select ...                                             */
+;*    callcc! ::switch ...                                             */
 ;*---------------------------------------------------------------------*/
-(define-method (callcc! node::select)
-   (with-access::select node (clauses test)
+(define-method (callcc! node::switch)
+   (with-access::switch node (clauses test)
       (set! test (callcc! test))
       (for-each (lambda (clause)
 		   (set-cdr! clause (callcc! (cdr clause))))
@@ -278,9 +309,9 @@
 		      (set-cdr! binding (callcc! val))
 		      (if (celled? var)
 			  (begin
-			     (local-type-set! var *obj*)
 			     (set-cdr! binding
-				       (a-make-cell (cdr binding) var))))))
+				(a-make-cell (cdr binding) var))
+			     (local-type-set! var *cell*)))))
 		bindings)
       node))
 
@@ -288,6 +319,7 @@
 ;*    callcc! ::set-ex-it ...                                          */
 ;*---------------------------------------------------------------------*/
 (define-method (callcc! node::set-ex-it)
+   (set-ex-it-onexit-set! node (callcc! (set-ex-it-onexit node)))
    (set-ex-it-body-set! node (callcc! (set-ex-it-body node)))
    node)
 
