@@ -3,8 +3,8 @@
 ;*    -------------------------------------------------------------    */
 ;*    Author      :  Manuel Serrano                                    */
 ;*    Creation    :  Mon Jun 24 17:36:29 1996                          */
-;*    Last change :  Sun Dec  9 15:20:46 2001 (serrano)                */
-;*    Copyright   :  1996-2001 Manuel Serrano, see LICENSE file        */
+;*    Last change :  Sat Jun  4 06:51:29 2016 (serrano)                */
+;*    Copyright   :  1996-2016 Manuel Serrano, see LICENSE file        */
 ;*    -------------------------------------------------------------    */
 ;*    The cfa on `app' node                                            */
 ;*=====================================================================*/
@@ -17,6 +17,7 @@
    (import  tools_shape
 	    tools_error
 	    type_type
+	    type_cache
 	    ast_var
 	    ast_node
 	    cfa_info
@@ -24,66 +25,105 @@
 	    cfa_cfa
 	    cfa_iterate
 	    cfa_loose
-	    cfa_approx)
+	    cfa_approx
+	    cfa_procedure)
    (export  (generic app!::approx ::fun ::var approx)))
 
 ;*---------------------------------------------------------------------*/
 ;*    cfa! ::app ...                                                   */
 ;*---------------------------------------------------------------------*/
 (define-method (cfa!::approx node::app)
-   (trace (cfa 2) " app: " (shape node) #\Newline)
    (with-access::app node (fun args)
-      (app! (variable-value (var-variable fun)) fun (map cfa! args))))
+      (trace (cfa 2) (cfa-current)
+	 ": >>> app " (shape fun) ": " (shape node) #\Newline)
+      (let ((args-approx (map cfa! args)))
+	 (trace (cfa 3) (cfa-current)
+	    ":     args=" (map shape args-approx) #\Newline)
+	 (app! (variable-value (var-variable fun)) fun args-approx))))
 
 ;*---------------------------------------------------------------------*/
 ;*    app! ...                                                         */
 ;*---------------------------------------------------------------------*/
 (define-generic (app!::approx fun::fun var::var args-approx)
-   (internal-error "app!"
-		   "No method for this function"
-		   (cons fun (shape var))))
+   (internal-error "app!" "No method for this function" (cons fun (shape var))))
 
 ;*---------------------------------------------------------------------*/
 ;*    app! ::intern-sfun/Cinfo ...                                     */
 ;*---------------------------------------------------------------------*/
 (define-method (app! fun::intern-sfun/Cinfo var::var args-approx)
-   (trace (cfa 3) "  app(intern)!: " (shape var) " " (shape args-approx)
-	  #\Newline)
-   (with-access::intern-sfun/Cinfo fun (args)
+   (with-access::intern-sfun/Cinfo fun (args polymorphic?)
+      (trace (cfa 3)
+	 (cfa-current)
+	 ": >>> app(intern-sfun/Cinfo)!" " polymorphic?=" polymorphic?
+	 #\Newline)
+      (trace (cfa 3) (cfa-current) ": ~~~   app, formals="
+	 (map shape args) #\Newline)
+      (trace (cfa 3) (cfa-current) ": ~~~   app, approx="
+	 (map shape args-approx) #\Newline)
       ;; we set the new formals approximation
       (for-each (lambda (formal approx)
 		   (union-approx! (svar/Cinfo-approx (local-value formal))
-				  approx))
-		args
-		args-approx)
+		      approx))
+	 args
+	 args-approx)
+      (trace (cfa 3)
+	 (cfa-current)
+	 ": --- app(intern-sfun/Cinfo)!" " polymorphic?=" polymorphic?
+	 #\Newline)
+      (trace (cfa 3) (cfa-current) ": ~~~   app, formals="
+	 (map shape args) #\Newline)
       ;; and we jump to the function body
       (if (or (not (global? (var-variable var)))
 	      (and (global? (var-variable var))
 		   (eq? (global-import (var-variable var)) 'static)))
 	  ;; this is a unexported function 
-	  (cfa-intern-sfun! fun (var-variable var))
+	  (let ((a (cfa-intern-sfun! fun (var-variable var))))
+	     (trace (cfa 3) (cfa-current)
+		": <<< app " (shape var) " <- (intern) " (shape a) "\n")
+	     a)
 	  ;; this is an exported function
-	  (cfa-export-var! fun (var-variable var)))))
+	  (let ((a (cfa-export-var! fun (var-variable var))))
+	     (trace (cfa 3) (cfa-current)
+		": <<< app " (shape var) " <- (export) " (shape a) "\n")
+	     a))))
 
 ;*---------------------------------------------------------------------*/
 ;*    app! ::extern-sfun ...                                           */
 ;*---------------------------------------------------------------------*/
 (define-method (app! fun::extern-sfun/Cinfo var::var args-approx)
-   (trace (cfa 3) "  app(extern)!: " (shape var) " " (shape args-approx)
-	  #\Newline)
-   (with-access::extern-sfun/Cinfo fun (top? approx)
+   (with-access::extern-sfun/Cinfo fun (top? approx polymorphic?)
+      (trace (cfa 3) (cfa-current) ": >>>   app(extern-sfun/Cinfo)!"
+	 " polymorphic?=" polymorphic? #\Newline)
       ;; we set the new formals approximation
-      (if top? (for-each (lambda (a) (loose! a 'all)) args-approx))
+      (if top?
+	  ;; calling a random extern function, loose everyting
+	  (for-each (lambda (a) (loose! a 'all)) args-approx)
+	  ;; don't loose but mark functions as not candidate to X/T optim
+	  (for-each (lambda (a) (disable-X-T! a "extern call")) args-approx))
+      (when polymorphic?
+	 (with-access::approx approx (type)
+	    (set! type (get-bigloo-type (approx-type approx)))))
       ;; and we return the global approximation
+      (trace (cfa 3) (cfa-current)
+	 ": <<< app " (shape var) " <- (extern) " (shape approx)
+	 #\Newline)
       approx))
    
 ;*---------------------------------------------------------------------*/
 ;*    app! ::cfun/Cinfo ...                                            */
 ;*---------------------------------------------------------------------*/
 (define-method (app! fun::cfun/Cinfo var::var args-approx)
-   (trace (cfa 3) "  app(cfun)!: " (shape var) #\Newline)
+   (trace (cfa 3) (cfa-current) ": >>>   app(cfun)!"
+      " args=" (shape args-approx)
+      #\Newline)
    (with-access::cfun/Cinfo fun (top? approx)
       ;; we set the new formals approximation
-      (if top? (for-each (lambda (a) (loose! a 'all)) args-approx))
+      (if top?
+	  (for-each (lambda (a) (loose! a 'all)) args-approx)
+	  ;; don't loose but mark functions as not candidate to X/T optim
+	  (for-each (lambda (a) (disable-X-T! a "extern call")) args-approx))
+      (trace (cfa 3) (cfa-current)
+	 ": <<< app " (shape var) " <- (foreign) " (shape approx)
+	 #\Newline)
       ;; and we return the global approximation
       approx))
