@@ -1,10 +1,10 @@
 ;*=====================================================================*/
-;*    serrano/prgm/project/bigloo2.3/comptime/Cfa/cfa.scm              */
+;*    serrano/prgm/project/bigloo/bigloo/comptime/Cfa/cfa.scm          */
 ;*    -------------------------------------------------------------    */
 ;*    Author      :  Manuel Serrano                                    */
 ;*    Creation    :  Thu Feb 23 14:21:20 1995                          */
-;*    Last change :  Mon Jul 17 13:13:47 2000 (serrano)                */
-;*    Copyright   :  1995-2000 Manuel Serrano, see LICENSE file        */
+;*    Last change :  Fri Nov  4 09:52:04 2022 (serrano)                */
+;*    Copyright   :  1995-2022 Manuel Serrano, see LICENSE file        */
 ;*    -------------------------------------------------------------    */
 ;*    The `control flow analysis': the walk down the ast               */
 ;*=====================================================================*/
@@ -25,8 +25,10 @@
 	    cfa_info3
 	    cfa_loose
 	    cfa_approx
+	    cfa_iterate
 	    cfa_app
-	    cfa_funcall)
+	    cfa_funcall
+	    cfa_type)
    (export  (generic cfa!::approx ::node)
 	    (generic cfa-variable-value-approx value::value)))
 
@@ -37,10 +39,17 @@
    (internal-error "cfa!:no method for this ast" node (shape node)))
    
 ;*---------------------------------------------------------------------*/
-;*    cfa! ::atom/Cinfo ...                                            */
+;*    cfa! ::literal/Cinfo ...                                         */
 ;*---------------------------------------------------------------------*/
-(define-method (cfa! node::atom/Cinfo)
-   (with-access::atom/Cinfo node (approx)
+(define-method (cfa! node::literal/Cinfo)
+   (with-access::literal/Cinfo node (approx)
+      approx))
+
+;*---------------------------------------------------------------------*/
+;*    cfa! ::patch/Cinfo ...                                           */
+;*---------------------------------------------------------------------*/
+(define-method (cfa! node::patch/Cinfo)
+   (with-access::patch/Cinfo node (approx)
       approx))
 
 ;*---------------------------------------------------------------------*/
@@ -55,16 +64,22 @@
 ;*---------------------------------------------------------------------*/
 (define-method (cfa! knode::kwote/node)
    (with-access::kwote/node knode (node)
-      (trace (cfa 3) "kwote/node: " (shape knode) " -> " (shape node)
-	     #\Newline)
+      (trace (cfa 3) (shape (cfa-current)) ": kwote/node, "
+	 (shape knode) " -> " (shape node)
+	 #\Newline)
       (cfa! node)))
 		    
 ;*---------------------------------------------------------------------*/
 ;*    cfa! ::var ...                                                   */
 ;*---------------------------------------------------------------------*/
 (define-method (cfa! node::var)
-   (with-access::var node (variable)
-      (cfa-variable-value-approx (variable-value variable))))
+   (with-access::var node (variable type)
+      (let ((approx (cfa-variable-value-approx (variable-value variable))))
+	 (if (or (eq? (variable-type variable) type) (eq? type *_*))
+	     approx
+	     (duplicate::approx approx
+		(dup approx)
+		(type type))))))
 
 ;*---------------------------------------------------------------------*/
 ;*    cfa! ::closure ...                                               */
@@ -121,12 +136,21 @@
 ;*---------------------------------------------------------------------*/
 (define-method (cfa! node::sequence)
    (with-access::sequence node (nodes)
-      (let loop ((n      nodes)
+      (let loop ((n nodes)
 		 (approx #unspecified))
 	 (if (null? n)
 	     approx
 	     (loop (cdr n) (cfa! (car n)))))))
 
+;*---------------------------------------------------------------------*/
+;*    cfa! ::sync ...                                                  */
+;*---------------------------------------------------------------------*/
+(define-method (cfa! node::sync)
+   (with-access::sync node (body mutex prelock)
+      (cfa! prelock)
+      (cfa! mutex)
+      (cfa! body)))
+
 ;*---------------------------------------------------------------------*/
 ;*    cfa! ::app-ly ...                                                */
 ;*---------------------------------------------------------------------*/
@@ -144,6 +168,14 @@
       (for-each (lambda (a) (loose! (cfa! a) 'all)) expr*)
       approx))
 
+;*---------------------------------------------------------------------*/
+;*    cfa! ::genpatchid/Cinfo ...                                      */
+;*---------------------------------------------------------------------*/
+(define-method (cfa! node::genpatchid/Cinfo)
+   (with-access::genpatchid/Cinfo node (approx expr*)
+      (for-each (lambda (a) (loose! (cfa! a) 'all)) expr*)
+      approx))
+
 ;*---------------------------------------------------------------------*/
 ;*    cfa! ::getfield/Cinfo ...                                        */
 ;*---------------------------------------------------------------------*/
@@ -164,14 +196,22 @@
 ;*    cfa! ::new/Cinfo ...                                             */
 ;*---------------------------------------------------------------------*/
 (define-method (cfa! node::new/Cinfo)
-   (with-access::new/Cinfo node (approx)
+   (with-access::new/Cinfo node (approx expr*)
+      (for-each (lambda (a) (loose! (cfa! a) 'all)) expr*)
       approx))
 
 ;*---------------------------------------------------------------------*/
-;*    cfa! ::isa/Cinfo ...                                             */
+;*    cfa! ::instanceof/Cinfo ...                                      */
 ;*---------------------------------------------------------------------*/
-(define-method (cfa! node::isa/Cinfo)
-   (with-access::isa/Cinfo node (approx expr*)
+(define-method (cfa! node::instanceof/Cinfo)
+   (with-access::instanceof/Cinfo node (approx expr*)
+      approx))
+
+;*---------------------------------------------------------------------*/
+;*    cfa! ::cast-null/Cinfo ...                                       */
+;*---------------------------------------------------------------------*/
+(define-method (cfa! node::cast-null/Cinfo)
+   (with-access::cast-null/Cinfo node (approx expr*)
       approx))
 
 ;*---------------------------------------------------------------------*/
@@ -188,11 +228,15 @@
    (with-access::setq/Cinfo node (approx var value)
       (let* ((var-approx (cfa! var))
 	     (val-approx (cfa! value))
-	     (v          (var-variable var)))
-	 (trace (cfa 3) "*** " (shape node) " <- " (shape val-approx)
-		#\Newline)
+	     (v (var-variable var)))
+	 (trace (cfa 3) (shape (cfa-current)) ": setq, " (shape node)
+	    " <- " (shape val-approx)
+	    #\Newline)
 	 (union-approx! var-approx val-approx)
-	 (if (global? v) (global-loose! v var-approx))
+	 (when (and (global? v)
+		    (or (not (global-init v))
+			(not (eq? (global-import v) 'static))))
+	    (global-loose! v var-approx))
 	 approx)))
 
 ;*---------------------------------------------------------------------*/
@@ -218,10 +262,10 @@
       approx))
 
 ;*---------------------------------------------------------------------*/
-;*    cfa! ::select/Cinfo ...                                          */
+;*    cfa! ::switch/Cinfo ...                                          */
 ;*---------------------------------------------------------------------*/
-(define-method (cfa! node::select/Cinfo)
-   (with-access::select/Cinfo node (approx clauses test)
+(define-method (cfa! node::switch/Cinfo)
+   (with-access::switch/Cinfo node (approx clauses test)
       (cfa! test)
       (let ((res-approx approx))
 	 (let loop ((cls clauses))
@@ -243,38 +287,49 @@
 ;*---------------------------------------------------------------------*/
 (define-method (cfa! node::let-var)
    (with-access::let-var node (body bindings)
-      (trace (cfa 3) "cfa! ::let-var: " (shape node) #\Newline)
+      (trace (cfa 4) (shape (cfa-current)) ": let-var, " (shape node) #\Newline)
       (for-each (lambda (binding)
-		   (let* ((var        (car binding))
+		   (let* ((var (car binding))
 			  (var-approx (svar/Cinfo-approx (variable-value var)))
 			  (val-approx (cfa! (cdr binding))))
 		      (let ((vtype (variable-type var))
 			    (atype (approx-type val-approx)))
-			 (union-approx! var-approx val-approx)
 			 (trace (cfa 4)
-				"   binding: " (shape binding) #\Newline
-				"       var: " (shape var-approx) #\Newline
-				"       val: " (shape val-approx) #\Newline
-				"     vtype: " (shape vtype) #\Newline
-				"     atype: " (shape atype) #\Newline)
-			 ;; Here we explicitly check for type errors.
+			    (shape (cfa-current)) ": let-var, var=" (shape var) 
+			    ", v-type=" (shape vtype)
+			    ", va-type=" (shape (approx-type var-approx))
+			    ", atype=" (shape atype) "\n")
+			 (union-approx-filter! var-approx val-approx)
+			 (trace (cfa 4)
+			    (shape (cfa-current)) ": -> type="
+			    (shape (approx-type var-approx)) "\n")
+			 ;; Here we explicitly check for type downcasts.
 			 ;; When one is encountered, top is propagated
-			 (if (and (not (eq? vtype *_*))
-				  (not (eq? vtype *obj*))
-				  (not (eq? atype *_*))
-				  (not (eq? atype vtype)))
-			     (begin
-				(approx-set-top! val-approx)
-				(loose! val-approx 'all))))))
-		bindings)
-      (cfa! body)))
+			 ;; Before version 3.5b (27 Nov 2010), this was
+			 ;; propagating a top as soon as atype was not eq
+			 ;; to vtype (not (eq? atype vtype))
+			 (when (and (not (eq? vtype *_*))
+				    (not (eq? vtype *obj*))
+				    (not (eq? atype *_*))
+				    (eq? atype *obj*))
+			    (trace (cfa 4) (shape (cfa-current))
+			       ": -----> setting top "
+			       (shape val-approx) "\n")
+			    (approx-set-top! val-approx)
+			    (loose! val-approx 'all)))))
+	 bindings)
+      (let ((approx (cfa! body)))
+	 (trace (cfa 3) (shape (cfa-current)) ":let-var <- "
+	    (shape node) #\Newline)
+	 approx)))
 
 ;*---------------------------------------------------------------------*/
 ;*    cfa! ::set-ex-it/Cinfo ...                                       */
 ;*---------------------------------------------------------------------*/
 (define-method (cfa! node::set-ex-it/Cinfo)
-   (with-access::set-ex-it/Cinfo node (approx body)
+   (with-access::set-ex-it/Cinfo node (approx body onexit)
       (loose! (cfa! body) 'all)
+      (loose! (cfa! onexit) 'all)
       approx))
 
 ;*---------------------------------------------------------------------*/
@@ -292,7 +347,8 @@
 ;*---------------------------------------------------------------------*/
 (define-method (cfa! node::make-box/Cinfo)
    (with-access::make-box/Cinfo node (approx value)
-      (trace (cfa 3) "cfa!(make-box): " (shape value) #\Newline)
+      (trace (cfa 3) (shape (cfa-current)) ": make-box), "
+	 (shape value) #\Newline)
       (loose! (cfa! value) 'all)
       approx))
 
@@ -302,6 +358,8 @@
 (define-method (cfa! node::box-set!/Cinfo)
    (with-access::box-set!/Cinfo node (approx var value)
       (cfa! var)
+      (let ((val-approx (cfa! value)))
+	 (union-approx! approx val-approx))
       (loose! (cfa! value) 'all)
       approx))
 
