@@ -1,10 +1,10 @@
 ;*=====================================================================*/
-;*    serrano/prgm/project/bigloo/comptime/Cfa/cinfo.scm               */
+;*    serrano/prgm/project/bigloo/bigloo/comptime/Cfa/cinfo.scm        */
 ;*    -------------------------------------------------------------    */
 ;*    Author      :  Manuel Serrano                                    */
 ;*    Creation    :  Mon Jun 24 15:46:49 1996                          */
-;*    Last change :  Sat Dec 30 11:18:15 2000 (serrano)                */
-;*    Copyright   :  1996-2000 Manuel Serrano, see LICENSE file        */
+;*    Last change :  Fri Nov  4 15:02:39 2022 (serrano)                */
+;*    Copyright   :  1996-2022 Manuel Serrano, see LICENSE file        */
 ;*    -------------------------------------------------------------    */
 ;*    The cfa's information structures                                 */
 ;*=====================================================================*/
@@ -18,6 +18,8 @@
 	   type_cache
 	   ast_var
 	   ast_node)
+
+   (include "Cfa/cinfo.sch")
    
    (export ;; the approximations
     (class approx
@@ -34,14 +36,21 @@
        ;; or top
        (top?::bool (default #f))
        ;; a stamp to avoid useless multiple loose.
-       (lost-stamp::long (default -1)))
+       (lost-stamp::long (default -1))
+       ;; dup (variables duplicate their approx), this points to
+       ;; the initial approx
+       (dup (default #unspecified))
+       ;; does the approximation contains at least one make-procedure-app?
+       (has-procedure?::bool (default #f)))
     
     ;; function extensions
     (wide-class cfun/Cinfo::cfun
        (approx::approx read-only))
     (wide-class extern-sfun/Cinfo::sfun
+       (polymorphic?::bool (default #f))
        (approx::approx read-only))
     (wide-class intern-sfun/Cinfo::sfun
+       (polymorphic?::bool (default #f))
        (approx::approx read-only)
        (stamp::long (default -1)))
     
@@ -56,7 +65,8 @@
        ;; is this variable holding a closure environement
        ;; (if it is it won't be lost when loosing the
        ;; function whose's this variable owner).
-       (clo-env?::bool (default #f)))
+       (clo-env?::bool (default #f))
+       (stamp::long (default -1)))
     (wide-class cvar/Cinfo::cvar
        (approx::approx read-only))
     
@@ -70,7 +80,11 @@
     (wide-class reshaped-global::global)
     
     ;; node extension
-    (wide-class atom/Cinfo::atom
+    (wide-class literal/Cinfo::literal
+       (approx::approx read-only))
+    (wide-class genpatchid/Cinfo::genpatchid
+       (approx::approx read-only))
+    (wide-class patch/Cinfo::patch
        (approx::approx read-only))
     (wide-class kwote/node::kwote
        (node::node read-only))
@@ -81,15 +95,15 @@
     (wide-class funcall/Cinfo::funcall
        (approx::approx read-only)
        (va-approx::approx read-only)
-       (arity-error-noticed?::bool (default (not *warning*)))
-       (type-error-noticed?::bool (default (not *warning*))))
+       (arity-error-noticed?::bool (default (=fx (bigloo-warning) 0)))
+       (type-error-noticed?::bool (default (=fx (bigloo-warning) 0))))
     (wide-class setq/Cinfo::setq
        (approx::approx read-only))
     (wide-class conditional/Cinfo::conditional
        (approx::approx read-only))
     (wide-class fail/Cinfo::fail
        (approx::approx read-only))
-    (wide-class select/Cinfo::select
+    (wide-class switch/Cinfo::switch
        (approx::approx read-only))
     (wide-class set-ex-it/Cinfo::set-ex-it
        (approx::approx read-only))
