@@ -1,12 +1,12 @@
 ;*=====================================================================*/
-;*    serrano/prgm/project/bigloo/comptime/Cfa/collect.scm             */
+;*    serrano/prgm/project/bigloo/bigloo/comptime/Cfa/collect.scm      */
 ;*    -------------------------------------------------------------    */
 ;*    Author      :  Manuel Serrano                                    */
 ;*    Creation    :  Wed Apr  5 09:06:26 1995                          */
-;*    Last change :  Mon Nov  6 17:39:41 2000 (serrano)                */
-;*    Copyright   :  1995-2000 Manuel Serrano, see LICENSE file        */
+;*    Last change :  Wed Jun 16 15:56:42 2021 (serrano)                */
+;*    Copyright   :  1995-2021 Manuel Serrano, see LICENSE file        */
 ;*    -------------------------------------------------------------    */
-;*    We collect all type and alloc approximations                     */
+;*    Collect all types and allocs approximations                      */
 ;*=====================================================================*/
 
 ;*---------------------------------------------------------------------*/
@@ -18,8 +18,8 @@
 	    tools_error
 	    type_type
 	    type_cache
-	    type_typeof
 	    module_module
+	    backend_backend
 	    engine_param
 	    ast_var
 	    ast_node
@@ -33,6 +33,7 @@
 	    cfa_specialize
 	    cfa_procedure
 	    cfa_vector
+	    cfa_pair
 	    cfa_struct
 	    cfa_box
 	    cfa_closure)
@@ -43,6 +44,7 @@
 ;*    collect-all-approx! ...                                          */
 ;*---------------------------------------------------------------------*/
 (define (collect-all-approx! globals)
+   (trace cfa "================== collect ===========================\n")
    (for-each (lambda (global) (collect-sfun! (global-value global) global))
 	     globals))
 
@@ -67,50 +69,81 @@
 ;*    node-collect! ::kwote ...                                        */
 ;*---------------------------------------------------------------------*/
 (define-method (node-collect! node::kwote owner)
+   
+   (define (monomorphic-vector? vector)
+      (define (get-atype value)
+	 (cond
+	    ((fixnum? value) 'integer)
+	    ((char? value) 'char)
+	    ((boolean? value) 'boolean)
+	    ((string? value) 'string)
+	    ((real? value) 'real)
+	    (else #f)))
+      (let ((len (vector-length vector)))
+	 (if (=fx len 0)
+	     #f 
+	     (let ((atype (get-atype (vector-ref vector 0))))
+		(let loop ((i 1))
+		   (cond
+		      ((not atype)
+		       #f)
+		      ((=fx i len)
+		       #t)
+		      ((eq? (get-atype (vector-ref vector i)) atype)
+		       (loop (+fx i 1)))
+		      (else
+		       #f)))))))
+
+   (define (monomorphic-list? list)
+      (monomorphic-vector? (list->vector list)))
+
+   (define (node-collect-vector! value)
+      (let* ((warning (let ((wan (bigloo-warning)))
+			 (bigloo-warning-set! 0)
+			 wan))
+	     (backend (the-backend))
+	     (pragma? (let ((tgt (backend-pragma-support backend)))
+			 (backend-pragma-support-set! backend #t)
+			 tgt))
+	     (dummy (top-level-sexp->node
+		       `($make-vector ,(vector-length value)
+			   ,(if (monomorphic-vector? value)
+				(vector-ref value 0)
+				'(pragma::obj "")))
+		       #f)))
+	 (backend-pragma-support-set! backend pragma?)
+	 (bigloo-warning-set! warning)
+	 (widen!::kwote/node node (node dummy))
+	 (node-collect! dummy owner)))
+
+   (define (node-collect-pair! value)
+      (let* ((warning (let ((wan (bigloo-warning)))
+			 (bigloo-warning-set! 0)
+			 wan))
+	     (backend (the-backend))
+	     (pragma? (let ((tgt (backend-pragma-support backend)))
+			 (backend-pragma-support-set! backend #t)
+			 tgt))
+	     (exp (let loop ((v value))
+		     (if (null? v)
+			 ''()
+			 `($cons ',(car v) ,(loop (cdr v))))))
+	     (dummy (top-level-sexp->node exp #f)))
+	 (backend-pragma-support-set! backend pragma?)
+	 (bigloo-warning-set! warning)
+	 (widen!::kwote/node node (node dummy))
+	 (node-collect! dummy owner)))      
+   
    (let ((value (kwote-value node)))
-      (if (and (vector-optim?) (vector? value))
-	  (let ((dummy (top-level-sexp->node
-			`(c-make-vector ,(vector-length value)
-					,(if (monomorphic-vector? value)
-					     (vector-ref value 0)
-					     '(pragma::obj "")))
-			#f)))
-	     (widen!::kwote/node node (node dummy))
-	     (node-collect! dummy owner)))))
-		    
-;*---------------------------------------------------------------------*/
-;*    monomorphic-vector? ...                                          */
-;*---------------------------------------------------------------------*/
-(define (monomorphic-vector? vector)
-   (define (get-atype value)
       (cond
-	 ((integer? value)
-	  'integer)
-	 ((char? value)
-	  'char)
-	 ((boolean? value)
-	  'boolean)
-	 ((string? value)
-	  'string)
-	 ((real? value)
-	  'real)
-	 (else
-	  #f)))
-   (let ((len (vector-length vector)))
-      (if (=fx len 0)
-	  #f 
-	  (let ((atype (get-atype (vector-ref vector 0))))
-	     (let loop ((i 1))
-		(cond
-		   ((not atype)
-		    #f)
-		   ((=fx i len)
-		    #t)
-		   ((eq? (get-atype (vector-ref vector i)) atype)
-		    (loop (+fx i 1)))
-		   (else
-		    #f)))))))
-  
+	 ((and (vector? value) (vector-optim?))
+	  ;; vector tracking
+	  (node-collect-vector! value))
+	 ((and (pair? value) (pair-optim?) (list? value)
+	       (<fx (length value) (pair-optim-quote-maxlen)))
+	  ;; pair tracking
+	  (node-collect-pair! value)))))
+		    
 ;*---------------------------------------------------------------------*/
 ;*    node-collect! ::var ...                                          */
 ;*---------------------------------------------------------------------*/
@@ -134,6 +167,14 @@
 (define-method (node-collect! node::sequence owner)
    (node-collect*! (sequence-nodes node) owner))
 
+;*---------------------------------------------------------------------*/
+;*    node-collect! ::sync ...                                         */
+;*---------------------------------------------------------------------*/
+(define-method (node-collect! node::sync owner)
+   (node-collect! (sync-mutex node) owner)
+   (node-collect! (sync-prelock node) owner)
+   (node-collect! (sync-body node) owner))
+
 ;*---------------------------------------------------------------------*/
 ;*    node-collect! ::app ...                                          */
 ;*---------------------------------------------------------------------*/
@@ -151,7 +192,8 @@
 		    ;; some functions).
 		    (case (global-id v)
 		       ((c-eq?)
-			(if *optim-cfa-arithmetic?*
+			(if (or *optim-cfa-fixnum-arithmetic?*
+				*optim-cfa-flonum-arithmetic?*)
 			    (widen!::pre-arithmetic-app node
 			       (spec-types (arithmetic-spec-types v)))))
 		       ((make-fx-procedure)
@@ -163,32 +205,49 @@
 		       ((procedure-ref)
 			(widen!::pre-procedure-ref-app node))
 		       ((procedure-set!)
-			(widen!::pre-procedure-set!-app node)))
-		    (if (vector-optim?)
-			(case (global-id v)
-			   ((c-make-vector)
-			    (use-alloc! node)
-			    (widen!::pre-make-vector-app node (owner owner)))
-			   ((c-vector-ref c-vector-set!
-					  c-vector-length
-					  c-create-vector)
-			    (internal-error "node-collect!"
-					    "Illegal foreign application"
-					    (global-id v)))))
-		    (if (>=fx *optim* 2)
-			(case (global-id v)
-			   ((c-make-struct)
-			    (use-alloc! node)
-			    (widen!::pre-make-struct-app node (owner owner)))
-			   ((c-struct-ref)
-			    (widen!::pre-struct-ref-app node))
-			   ((c-struct-set!)
-			    (widen!::pre-struct-set!-app node)))))
+			(widen!::pre-procedure-set!-app node))
+		       (($make-vector)
+			(when (vector-optim?)
+			   (use-alloc! node)
+			   (widen!::pre-make-vector-app node (owner owner))))
+		       (($make-struct)
+			(when (>=fx *optim* 2)
+			   (use-alloc! node)
+			   (widen!::pre-make-struct-app node (owner owner))))
+		       (($struct-ref)
+			(when (>=fx *optim* 2)
+			   (widen!::pre-struct-ref-app node)))
+		       (($struct-set!)
+			(when (>=fx *optim* 2)
+			   (widen!::pre-struct-set!-app node)))
+		       (($cons)
+			(when (pair-optim?)
+			   (use-alloc! node)
+			   (widen!::pre-cons-app node (owner owner))))
+		       (($car)
+			(when (pair-optim?)
+			   (widen!::pre-cons-ref-app node
+			      (get car))))
+		       (($cdr)
+			(when (pair-optim?)
+			   (widen!::pre-cons-ref-app node
+			      (get cdr))))
+		       (($set-car!)
+			(when (pair-optim?)
+			   (widen!::pre-cons-set!-app node
+			      (get car))))
+		       (($set-cdr!)
+			(when (pair-optim?)
+			   (widen!::pre-cons-set!-app node
+			      (get cdr))))))
 		 ;; non C function 
-		 (if *optim-cfa-arithmetic?*
+		 (if (or *optim-cfa-fixnum-arithmetic?*
+			 *optim-cfa-flonum-arithmetic?*)
 		     (if (arithmetic-operator? v)
-			 (widen!::pre-arithmetic-app node
-			    (spec-types (arithmetic-spec-types v))))))))))
+			 (let ((types (arithmetic-spec-types v)))
+			    (when (pair? types)
+			       (widen!::pre-arithmetic-app node
+				  (spec-types types)))))))))))
 
 ;*---------------------------------------------------------------------*/
 ;*    node-collect! ::valloc ...                                       */
@@ -196,10 +255,9 @@
 (define-method (node-collect! node::valloc owner)
    (with-access::valloc node (expr*)
       (node-collect*! expr* owner)
-      (if (vector-optim?)
-	  (begin
-	     (use-alloc! node)
-	     (widen!::pre-valloc/Cinfo node (owner owner)))))) 
+      (when (vector-optim?)
+	 (use-alloc! node)
+	 (widen!::pre-valloc/Cinfo node (owner owner)))))
    
 ;*---------------------------------------------------------------------*/
 ;*    node-collect! ::app-ly ...                                       */
@@ -236,7 +294,7 @@
 ;*    node-collect! ::setq ...                                         */
 ;*---------------------------------------------------------------------*/
 (define-method (node-collect! node::setq owner)
-   (with-access::setq node (type value)
+   (with-access::setq node (value)
       (node-collect! value owner)))
 
 ;*---------------------------------------------------------------------*/
@@ -258,10 +316,10 @@
       (node-collect! obj owner)))
 
 ;*---------------------------------------------------------------------*/
-;*    node-collect! ::select ...                                       */
+;*    node-collect! ::switch ...                                       */
 ;*---------------------------------------------------------------------*/
-(define-method (node-collect! node::select owner)
-   (with-access::select node (clauses test)
+(define-method (node-collect! node::switch owner)
+   (with-access::switch node (clauses test)
       (node-collect! test owner)
       (for-each (lambda (clause)
 		   (node-collect! (cdr clause) owner))
@@ -292,7 +350,8 @@
 ;*    node-collect! ::set-ex-it ...                                    */
 ;*---------------------------------------------------------------------*/
 (define-method (node-collect! node::set-ex-it owner)
-   (with-access::set-ex-it node (var body)
+   (with-access::set-ex-it node (var body onexit)
+      (node-collect! onexit owner)
       (node-collect! body owner)))
 
 ;*---------------------------------------------------------------------*/
