@@ -1,10 +1,10 @@
 ;*=====================================================================*/
-;*    serrano/prgm/project/bigloo2.3/comptime/Cfa/setup.scm            */
+;*    serrano/prgm/project/bigloo/bigloo/comptime/Cfa/setup.scm        */
 ;*    -------------------------------------------------------------    */
 ;*    Author      :  Manuel Serrano                                    */
 ;*    Creation    :  Tue Jun 25 14:08:53 1996                          */
-;*    Last change :  Thu Jul 20 07:34:06 2000 (serrano)                */
-;*    Copyright   :  1996-2000 Manuel Serrano, see LICENSE file        */
+;*    Last change :  Wed Jun 16 15:57:11 2021 (serrano)                */
+;*    Copyright   :  1996-2021 Manuel Serrano, see LICENSE file        */
 ;*    -------------------------------------------------------------    */
 ;*    We setup the ast for the Cfa.                                    */
 ;*=====================================================================*/
@@ -14,7 +14,8 @@
 ;*---------------------------------------------------------------------*/
 (module cfa_setup
    (include "Tools/trace.sch")
-   (import  type_type
+   (import  engine_param
+	    type_type
 	    type_cache
 	    type_typeof
 	    module_module
@@ -25,7 +26,8 @@
 	    cfa_info
 	    cfa_info2
 	    cfa_info3
-	    cfa_approx)
+	    cfa_approx
+	    cfa_pair)
    (export  (set-initial-approx! globals)
 	    (generic node-setup! ::node)
 	    (node-setup*! node*)))
@@ -34,8 +36,9 @@
 ;*    set-initial-approx! ...                                          */
 ;*---------------------------------------------------------------------*/
 (define (set-initial-approx! globals)
+   (trace cfa "================== initial ===========================\n")   
    (for-each (lambda (global)
-		(trace (cfa 2) "set-initial-approx!: " (shape global)
+		(trace (cfa 5) "set-initial-approx!: " (shape global)
 		       #\Newline)
 		(let ((fun (global-value global)))
 		   (fun-setup! fun global)
@@ -53,20 +56,31 @@
 (define-generic (node-setup! node::node))
 
 ;*---------------------------------------------------------------------*/
-;*    node-setup! ::atom ...                                           */
+;*    node-setup! ::literal ...                                        */
 ;*---------------------------------------------------------------------*/
-(define-method (node-setup! node::atom)
-   (with-access::atom node (value)
-      (widen!::atom/Cinfo node
-	 (approx (make-type-approx (typeof-atom value))))))
+(define-method (node-setup! node::literal)
+   (with-access::literal node (value)
+      (widen!::literal/Cinfo node
+	 (approx (make-type-approx (get-type-atom value))))))
+
+;*---------------------------------------------------------------------*/
+;*    node-setup! ::patch ...                                          */
+;*---------------------------------------------------------------------*/
+(define-method (node-setup! node::patch)
+   (with-access::patch node (value type)
+      (node-setup! value)
+      (widen!::patch/Cinfo node
+	 (approx (make-type-approx type)))))
 
 ;*---------------------------------------------------------------------*/
 ;*    node-setup! ::kwote ...                                          */
 ;*---------------------------------------------------------------------*/
 (define-method (node-setup! node::kwote)
    (with-access::kwote node (value)
-      (widen!::kwote/Cinfo node
-	 (approx (make-type-approx (typeof-kwote value))))))
+      (let ((approx (make-type-approx (get-type-kwote value))))
+	 (approx-top?-set! approx #t)
+	 (widen!::kwote/Cinfo node
+	    (approx approx)))))
    
 ;*---------------------------------------------------------------------*/
 ;*    node-setup! ::kwote/node ...                                     */
@@ -91,19 +105,16 @@
 ;*---------------------------------------------------------------------*/
 ;*    alloc-type? ...                                                  */
 ;*    -------------------------------------------------------------    */
-;*    This predicate returns #t for all type denoting data             */
+;*    This predicate returns #t for all types denoting data            */
 ;*    structures approximated by the cfa.                              */
 ;*---------------------------------------------------------------------*/
 (define (alloc-type? type)
    (cond
-      ((eq? type *vector*)
-       #t)
-      ((eq? type *procedure*)
-       #t)
-      ((eq? type *struct*)
-       #t)
-      (else
-       #f)))
+      ((eq? type *vector*) #t)
+      ((eq? type *procedure*) #t)
+      ((eq? type *struct*) #t)
+      ((eq? type *pair*) (pair-optim?))
+      (else #f)))
 
 ;*---------------------------------------------------------------------*/
 ;*    variable-value-setup! ...                                        */
@@ -121,21 +132,21 @@
 ;*---------------------------------------------------------------------*/
 ;*    variable-value-setup! ::svar ...                                 */
 ;*---------------------------------------------------------------------*/
-(define-method (variable-value-setup! value::svar var)
-   (if (global? var)
-       (let ((value (widen!::svar/Cinfo value
-		       (approx (make-type-approx (variable-type var))))))
-	  (if (and (not (eq? (global-import var) 'static))
-		   (alloc-type? (variable-type var)))
-	      (approx-set-top! (svar/Cinfo-approx value))))
-       (widen!::svar/Cinfo value
-	  (approx (make-type-approx (variable-type var))))))
+(define-method (variable-value-setup! value::svar var::variable)
+   (let ((typ (variable-type var)))
+      (if (global? var)
+	  (let ((value (widen!::svar/Cinfo value
+			  (approx (make-type-approx typ)))))
+	     (if (and (not (eq? (global-import var) 'static)) (alloc-type? typ))
+		 (approx-set-top! (svar/Cinfo-approx value))))
+	  (widen!::svar/Cinfo value
+	     (approx (make-type-approx typ))))))
 
 ;*---------------------------------------------------------------------*/
 ;*    variable-value-setup! ::pre-clo-env ...                          */
 ;*---------------------------------------------------------------------*/
 (define-method (variable-value-setup! value::pre-clo-env var)
-   (trace (cfa 2) "Je set un pre-clo-env..." (shape var) #\Newline)
+   (trace (cfa 5) "Je set un pre-clo-env..." (shape var) #\Newline)
    (call-next-method)
    (svar/Cinfo-clo-env?-set! (local-value var) #t))
 
@@ -156,7 +167,7 @@
 ;*    variable-value-setup! ::scnst ...                                */
 ;*---------------------------------------------------------------------*/
 (define-method (variable-value-setup! value::scnst var)
-   (trace (cfa 2) "Je setup une scnst: " (shape var) " "
+   (trace (cfa 5) "Je setup une scnst: " (shape var) " "
 	  (shape (scnst-node value)) #\Newline)
    (if (global? var)
        (if (and (eq? (global-module var) *module*)
@@ -164,7 +175,7 @@
 		(pre-make-procedure-app? (scnst-node value)))
 	   ;; this variable holds a closure
 	   (let ((node (scnst-node value)))
-	      (trace (cfa 2) "    et en plus, c'est une closure" #\Newline)
+	      (trace (cfa 5) "    et en plus, c'est une closure" #\Newline)
 	      (node-setup! node)
 	      (widen!::scnst/Cinfo value
 		 (approx (make-procedure-app-approx node))))
@@ -180,8 +191,8 @@
 (define-method (variable-value-setup! value::cvar var)
    (widen!::cvar/Cinfo value
       (approx (make-type-approx (variable-type var))))
-   (if (alloc-type? (variable-type var))
-       (approx-set-top! (cvar/Cinfo-approx value))))
+   (when (alloc-type? (variable-type var))
+      (approx-set-top! (cvar/Cinfo-approx value))))
 
 ;*---------------------------------------------------------------------*/
 ;*    node-setup! ::closure ...                                        */
@@ -196,11 +207,20 @@
    (with-access::sequence node (nodes)
       (node-setup*! nodes)))
 
+;*---------------------------------------------------------------------*/
+;*    node-setup! ::sync ...                                           */
+;*---------------------------------------------------------------------*/
+(define-method (node-setup! node::sync)
+   (with-access::sync node (body mutex prelock)
+      (node-setup! mutex)
+      (node-setup! prelock)
+      (node-setup! body)))
+
 ;*---------------------------------------------------------------------*/
 ;*    node-setup! ::app ...                                            */
 ;*---------------------------------------------------------------------*/
 (define-method (node-setup! node::app)
-   (trace (cfa 2) "Je setup une app: " (shape node) #\Newline)
+   (trace (cfa 5) "Je setup une app: " (shape node) #\Newline)
    (with-access::app node (fun args)
       (node-setup*! args)
       (let ((variable (var-variable fun)))
@@ -222,11 +242,14 @@
        (widen!::reshaped-global var))
    (if (and (global? var) (eq? (global-import var) 'import))
        (let ((approx (make-type-approx (global-type var))))
-	  (if (sfun-top? fun) (approx-set-top! approx))
+	  (if (or #t (sfun-top? fun)) (approx-set-top! approx))
 	  (widen!::extern-sfun/Cinfo fun
 	     (approx approx)))
-       (widen!::intern-sfun/Cinfo fun
-	  (approx (make-type-approx (variable-type var))))))
+       (let ((approx (make-type-approx (variable-type var))))
+	  (when *optim-cfa-force-loose-local-function?*
+	     (approx-set-top! approx))
+	  (widen!::intern-sfun/Cinfo fun
+	     (approx approx)))))
 
 ;*---------------------------------------------------------------------*/
 ;*    fun-setup! ::cfun ...                                            */
@@ -235,7 +258,7 @@
    (if (not (reshaped-global? var))
        (widen!::reshaped-global var))
    (let ((approx (make-type-approx (global-type var))))
-      (if (cfun-top? fun) (approx-set-top! approx))
+      (if (or #t (cfun-top? fun)) (approx-set-top! approx))
       (widen!::cfun/Cinfo fun
 	 (approx approx))))
  
@@ -246,9 +269,13 @@
    (with-access::app-ly node (fun arg)
       (node-setup! fun)
       (node-setup! arg)
-      (widen!::app-ly/Cinfo node
-	 (approx (make-type-approx *obj*)))
-      (approx-set-top! (app-ly/Cinfo-approx node))))
+      (if *optim-cfa-apply-tracking?*
+	  (widen!::app-ly/Cinfo node
+	     (approx (make-empty-approx)))
+	  (begin
+	     (widen!::app-ly/Cinfo node
+		(approx (make-type-approx *obj*)))
+	     (approx-set-top! (app-ly/Cinfo-approx node))))))
 
 ;*---------------------------------------------------------------------*/
 ;*    node-setup! ::funcall ...                                        */
@@ -257,13 +284,18 @@
    (with-access::funcall node (fun args)
       (node-setup! fun)
       (node-setup*! args)
-      (widen!::funcall/Cinfo node
-	 (approx (make-type-approx *obj*))
-	 (va-approx (make-type-approx *obj*)))
-      (approx-set-top! (funcall/Cinfo-va-approx node))))
+      (if *optim-cfa-unbox-closure-args*
+	  (widen!::funcall/Cinfo node
+	     (approx (make-empty-approx))
+	     (va-approx (make-empty-approx)))
+	  (begin
+	     (widen!::funcall/Cinfo node
+		(approx (make-type-approx *obj*))
+		(va-approx (make-type-approx *obj*)))
+	     (approx-set-top! (funcall/Cinfo-va-approx node))))))
 
 ;*---------------------------------------------------------------------*/
-;*    node-setup! ::extern ...                                         */
+;*    node-setup! ::pragma ...                                         */
 ;*---------------------------------------------------------------------*/
 (define-method (node-setup! node::pragma)
    (with-access::pragma node (expr* type)
@@ -272,6 +304,15 @@
 	 (approx (make-type-approx type)))
       (approx-set-top! (pragma/Cinfo-approx node))))
 
+;*---------------------------------------------------------------------*/
+;*    node-setup! ::genpatchid ...                                     */
+;*---------------------------------------------------------------------*/
+(define-method (node-setup! node::genpatchid)
+   (with-access::genpatchid node (expr*)
+      (node-setup*! expr*)
+      (widen!::genpatchid/Cinfo node 
+	 (approx (make-type-approx *long*)))))
+
 ;*---------------------------------------------------------------------*/
 ;*    node-setup! ::getfield ...                                       */
 ;*---------------------------------------------------------------------*/
@@ -296,20 +337,29 @@
 ;*    node-setup! ::new ...                                            */
 ;*---------------------------------------------------------------------*/
 (define-method (node-setup! node::new)
-   (with-access::new node (type)
+   (with-access::new node (expr* type)
+      (node-setup*! expr*)
       (widen!::new/Cinfo node
 	 (approx (make-type-approx type)))
       (approx-set-top! (new/Cinfo-approx node))))
 
 ;*---------------------------------------------------------------------*/
-;*    node-setup! ::isa ...                                            */
+;*    node-setup! ::instanceof ...                                     */
 ;*---------------------------------------------------------------------*/
-(define-method (node-setup! node::isa)
-   (with-access::isa node (type expr*)
+(define-method (node-setup! node::instanceof)
+   (with-access::instanceof node (type expr*)
       (node-setup*! expr*)
-      (widen!::isa/Cinfo node
+      (widen!::instanceof/Cinfo node
 	 (approx (make-type-approx type)))
-      (approx-set-top! (isa/Cinfo-approx node))))
+      (approx-set-top! (instanceof/Cinfo-approx node))))
+
+;*---------------------------------------------------------------------*/
+;*    node-setup! ::cast-null ...                                      */
+;*---------------------------------------------------------------------*/
+(define-method (node-setup! node::cast-null)
+   (with-access::cast-null node (type)
+      (widen!::cast-null/Cinfo node
+	 (approx (make-type-approx type)))))
 
 ;*---------------------------------------------------------------------*/
 ;*    node-setup! ::cast ...                                           */
@@ -322,7 +372,7 @@
 ;*    node-setup! ::setq ...                                           */
 ;*---------------------------------------------------------------------*/
 (define-method (node-setup! node::setq)
-   (trace (cfa 2) "Je setup un setq: " (shape node) #\Newline)
+   (trace (cfa 5) "Je setup un setq: " (shape node) #\Newline)
    (with-access::setq node (var value)
       (node-setup! value)
       (node-setup! var)
@@ -352,15 +402,15 @@
 	 (approx (make-type-approx *obj*)))))
 
 ;*---------------------------------------------------------------------*/
-;*    node-setup! ::select ...                                         */
+;*    node-setup! ::switch ...                                         */
 ;*---------------------------------------------------------------------*/
-(define-method (node-setup! node::select)
-   (with-access::select node (clauses test)
+(define-method (node-setup! node::switch)
+   (with-access::switch node (clauses test)
       (node-setup! test)
       (for-each (lambda (clause)
 		   (node-setup! (cdr clause)))
 		clauses)
-      (widen!::select/Cinfo node
+      (widen!::switch/Cinfo node
 	 (approx (make-empty-approx)))))
 
 ;*---------------------------------------------------------------------*/
@@ -385,7 +435,7 @@
 ;*---------------------------------------------------------------------*/
 (define-method (node-setup! node::let-var)
    (with-access::let-var node (body bindings)
-      (trace (cfa 3) "let-var settup: " (shape node) #\Newline)
+      (trace (cfa 5) "let-var setup: " (shape node) #\Newline)
       (for-each (lambda (binding)
 		   (let ((var (car binding))
 			 (val (cdr binding)))
@@ -398,19 +448,22 @@
 			 (binding-value (if (eq? (local-access var) 'read)
 					    val
 					    #f)))
-		      (trace (cfa 3) "Je vais descendre dans val: " (shape val)
+		      (trace (cfa 5) "~~~ let-var setup " (shape val)
 			     " " (find-runtime-type val)
 			     #\Newline)
-		      (trace (cfa 3) "  " (shape var) " ... " #\Newline)))
+		      (trace (cfa 5) "  " (shape var) " ... " #\Newline)))
 		bindings)
-      (node-setup! body)))
+      (trace (cfa 5) "let-var body: " (shape body) #\Newline)
+      (node-setup! body)
+      (trace (cfa 5) "<<< let-var setup...\n")))
  
 ;*---------------------------------------------------------------------*/
 ;*    node-setup! ::set-ex-it ...                                      */
 ;*---------------------------------------------------------------------*/
 (define-method (node-setup! node::set-ex-it)
-   (with-access::set-ex-it node (var body)
+   (with-access::set-ex-it node (var body onexit)
       (node-setup! body)
+      (node-setup! onexit)
       (node-setup! var)
       (widen!::reshaped-local (var-variable var))
       (widen!::set-ex-it/Cinfo node
