@@ -1,41 +1,10 @@
-;*---------------------------------------------------------------------*/
-;*   A pratical implementation for the Scheme programming language     */
-;*                                                                     */
-;*                                    ,--^,                            */
-;*                              _ ___/ /|/                             */
-;*                          ,;'( )__, ) '                              */
-;*                         ;;  //   L__.                               */
-;*                         '   \\   /  '                               */
-;*                              ^   ^                                  */
-;*                                                                     */
-;*   Copyright (c) 1992-1999 Manuel Serrano                            */
-;*                                                                     */
-;*     Bug descriptions, use reports, comments or suggestions are      */
-;*     welcome. Send them to                                           */
-;*       bigloo-request@kaolin.unice.fr                                */
-;*       http://kaolin.unice.fr/bigloo                                 */
-;*                                                                     */
-;*   This program is free software; you can redistribute it            */
-;*   and/or modify it under the terms of the GNU General Public        */
-;*   License as published by the Free Software Foundation; either      */
-;*   version 2 of the License, or (at your option) any later version.  */
-;*                                                                     */
-;*   This program is distributed in the hope that it will be useful,   */
-;*   but WITHOUT ANY WARRANTY; without even the implied warranty of    */
-;*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     */
-;*   GNU General Public License for more details.                      */
-;*                                                                     */
-;*   You should have received a copy of the GNU General Public         */
-;*   License along with this program; if not, write to the Free        */
-;*   Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,   */
-;*   MA 02111-1307, USA.                                               */
-;*---------------------------------------------------------------------*/
 ;*=====================================================================*/
-;*    serrano/prgm/project/bigloo/comptime1.9c/Cfa/tvector.scm         */
+;*    serrano/prgm/project/bigloo/bigloo/comptime/Cfa/tvector.scm      */
 ;*    -------------------------------------------------------------    */
 ;*    Author      :  Manuel Serrano                                    */
 ;*    Creation    :  Wed Apr  5 18:47:23 1995                          */
-;*    Last change :  Tue Aug  5 10:48:54 1997 (serrano)                */
+;*    Last change :  Fri Nov  4 09:21:53 2022 (serrano)                */
+;*    Copyright   :  1995-2022 Manuel Serrano, see LICENSE file        */
 ;*    -------------------------------------------------------------    */
 ;*    The `vector->tvector' optimization.                              */
 ;*=====================================================================*/
@@ -46,13 +15,15 @@
 (module cfa_tvector
    (include "Tools/trace.sch"
 	    "Ast/unit.sch"
-	    "Tvector/tvector.sch")
+	    "Tvector/tvector.sch"
+	    "Cfa/set.sch")
    (import  engine_param
 	    module_type
 	    module_pragma
 	    type_type
 	    type_cache
 	    type_env
+	    type_typeof
 	    tvector_tvector
 	    tools_shape
 	    tools_speek
@@ -62,13 +33,15 @@
 	    ast_build
 	    ast_sexp
 	    ast_env
+	    ast_lvtype
+	    ast_walk
 	    cfa_info
+	    cfa_info2
+	    cfa_info3
 	    cfa_cfa
 	    cfa_approx
 	    cfa_set
-	    cfa_info
 	    cfa_type
-	    cfa_stack
 	    globalize_walk
 	    inline_inline
 	    inline_walk)
@@ -77,7 +50,7 @@
 	    (vector->tvector! globals)
 	    (add-make-vector! ::node)
 	    (tvector-optimization?)
-	    (generic get-vector-item-type::type ::app)))
+	    (generic get-vector-item-type::type ::node)))
 
 ;*---------------------------------------------------------------------*/
 ;*    tvector-optimization? ...                                        */
@@ -97,59 +70,63 @@
 ;*    beginning of the compilation (just after the heap restoration).  */
 ;*---------------------------------------------------------------------*/
 (define (patch-vector-set!)
-   (if (tvector-optimization?)
-       (begin
-	  (for-each (lambda (set)
-		       (let ((g (find-global set)))
-			  (if (global? g)
-			      (let ((fun (global-value g)))
-				 (cond
-				    ((cfun? fun)
-				     (set-car! (cddr (cfun-args-type fun))
-					       (get-default-type)))
-				    ((sfun? fun)
-				     (local-type-set! (caddr (sfun-args fun))
-						      (get-default-type))))))))
-		    '(vector-set! c-vector-set! vector-set-ur!))
-	  (let ((g (find-global 'c-vector?)))
-	     (if (global? g)
-		 (let ((f (global-value g)))
-		    (set-car! (cfun-args-type f) (get-default-type)))))
-	  (let ((g (find-global 'vector?)))
-	     (if (global? g)
-		 (let ((f (global-value g)))
-		    (local-type-set! (car (sfun-args f))
-				     (get-default-type))))))))
+   (when (tvector-optimization?)
+      (for-each (lambda (set)
+		   (let ((g (find-global set)))
+		      (if (global? g)
+			  (let ((fun (global-value g)))
+			     (cond
+				((cfun? fun)
+				 (set-car! (cddr (cfun-args-type fun))
+				    (get-default-type)))
+				((sfun? fun)
+				 (local-type-set! (caddr (sfun-args fun))
+				    (get-default-type))))))))
+	 '(vector-set! vector-set-ur! c-vector-set!
+	   $vector-set! $vector-set-ur!))
+      (let ((g (find-global 'c-vector?)))
+	 (if (global? g)
+	     (let ((f (global-value g)))
+		(set-car! (cfun-args-type f) (get-default-type)))))
+      (let ((g (find-global '$vector?)))
+	 (if (global? g)
+	     (let ((f (global-value g)))
+		(set-car! (cfun-args-type f) (get-default-type)))))
+      (let ((g (find-global 'vector?)))
+	 (if (global? g)
+	     (let ((f (global-value g)))
+		(local-type-set! (car (sfun-args f)) (get-default-type)))))))
 	
 ;*---------------------------------------------------------------------*/
 ;*    unpatch-vector-set! ...                                          */
-;*    -------------------------------------------------------------    */
-;*    This function is called by (@ compiler engine) at the very       */
-;*    beginning of the compilation (just after the heap restoration).  */
 ;*---------------------------------------------------------------------*/
 (define (unpatch-vector-set!)
-   (if (tvector-optimization?)
-       (begin
-	  (for-each (lambda (set)
-		       (let ((g (find-global set)))
-			  (if (global? g)
-			      (let ((fun (global-value g)))
-				 (cond
-				    ((cfun? fun)
-				     (set-car! (cddr (cfun-args-type fun))
-					       *obj*))
-				    ((sfun? fun)
-				     (local-type-set! (caddr (sfun-args fun))
-						      *obj*)))))))
-		    '(vector-set! c-vector-set! vector-set-ur!))
-	  (let ((g (find-global 'c-vector?)))
-	     (if (global? g)
-		 (let ((f (global-value g)))
-		    (set-car! (cfun-args-type f) *obj*))))
-	  (let ((g (find-global 'vector?)))
-	     (if (global? g)
-		 (let ((f (global-value g)))
-		    (local-type-set! (car (sfun-args f)) *obj*)))))))
+   (when (tvector-optimization?)
+      (for-each (lambda (set)
+		   (let ((g (find-global set)))
+		      (if (global? g)
+			  (let ((fun (global-value g)))
+			     (cond
+				((cfun? fun)
+				 (set-car! (cddr (cfun-args-type fun))
+				    *obj*))
+				((sfun? fun)
+				 (local-type-set! (caddr (sfun-args fun))
+				    *obj*)))))))
+	 '(vector-set! c-vector-set! vector-set-ur!))
+      (let ((g (find-global 'c-vector?)))
+	 (if (global? g)
+	     (let ((f (global-value g)))
+		(set-car! (cfun-args-type f) *obj*))))
+      (let ((g (find-global '$vector?)))
+	 (if (global? g)
+	     (let ((f (global-value g)))
+		(set-car! (cfun-args-type f) *obj*))))
+      (let ((g (find-global 'vector?)))
+	 (if (global? g)
+	     (let ((f (global-value g)))
+		(local-type-set! (car (sfun-args f)) *obj*)))))
+   #unspecified)
     
 ;*---------------------------------------------------------------------*/
 ;*    vector->tvector! ...                                             */
@@ -158,22 +135,30 @@
    (if (tvector-optimization?)
        (begin
 	  (trace cfa
-		 "--------------------------------------"
-		 #\Newline "tvector-optimization! :" #\Newline
-		 (shape *make-vector-list*)
-		 #\Newline)
+	     "--------------------------------------"
+	     #\Newline "tvector-optimization! :" #\Newline
+	     (shape *make-vector-list*)
+	     #\Newline)
 	  ;; we setup the inlining 
 	  (inline-setup! 'all)
-	  (let ((tvectors (get-tvectors)))
+	  (multiple-value-bind (vectors tvectors)
+	     (get-tvectors)
+	     (for-each (lambda (v)
+			  (when (eq? (node-type v) *_*)
+			     (node-type-set! v *vector*)))
+		vectors)
 	     (show-tvector tvectors)
 	     (trace (cfa 2) "tvectors: " (shape tvectors) #\Newline)
 	     (if (pair? tvectors)
 		 (let ((add-tree (declare-tvectors tvectors)))
 		    (trace (cfa 2)
-			   "additional-body: " (shape add-tree) #\Newline)
+		       "additional-body: " (shape add-tree) #\Newline)
 		    (patch-tree! globals)
+		    (lvtype-ast! add-tree)
 		    add-tree)
-		 '())))
+		 (begin
+		    (patch-tree! globals)
+		    '()))))
        '()))
 
 ;*---------------------------------------------------------------------*/
@@ -191,28 +176,29 @@
 ;*---------------------------------------------------------------------*/
 ;*    get-tvectors ...                                                 */
 ;*    -------------------------------------------------------------    */
-;*    We scan all declared vector to find which of them can be         */
+;*    We scan all declared vectors to find which of them can be        */
 ;*    optimized.                                                       */
 ;*---------------------------------------------------------------------*/
 (define (get-tvectors)
-   (let loop ((apps      *make-vector-list*)
+   (let loop ((apps *make-vector-list*)
+	      (vectors '())
 	      (tvectors '()))
       (if (null? apps)
-	  tvectors
-	  (let* ((app  (car apps))
+	  (values vectors tvectors)
+	  (let* ((app (car apps))
 		 (type (get-vector-item-type app)))
 	     (trace (cfa 1)
-		    "vector: " (shape app) " type: " (shape type) " "
-		    (type-class type) #\Newline)
+		    "vector: " (shape app) " item-type: " (shape type)
+		    " < " (type-class type) #\Newline)
 	     (if (and (not (eq? type *_*))
 		      (not (sub-type? type *obj*)))
-		 (loop (cdr apps) (cons app tvectors))
-		 (loop (cdr apps) tvectors))))))
-			   
+		 (loop (cdr apps) vectors (cons app tvectors))
+		 (loop (cdr apps) (cons app vectors) tvectors))))))
+
 ;*---------------------------------------------------------------------*/
 ;*    get-vector-item-type ...                                         */
 ;*---------------------------------------------------------------------*/
-(define-generic (get-vector-item-type app::app))
+(define-generic (get-vector-item-type node::node))
  
 ;*---------------------------------------------------------------------*/
 ;*    get-vector-item-type ::make-vector-app ...                       */
@@ -224,14 +210,14 @@
 	  (approx-type value-approx))))
 
 ;*---------------------------------------------------------------------*/
-;*    get-vector-item-type ::create-vector-app ...                     */
+;*    get-vector-item-type ::valloc/Cinfo+optim ...                    */
 ;*---------------------------------------------------------------------*/
-(define-method (get-vector-item-type app::create-vector-app)
-   (with-access::create-vector-app app (value-approx seen?)
+(define-method (get-vector-item-type node::valloc/Cinfo+optim)
+   (with-access::valloc/Cinfo+optim node (value-approx seen?)
       (if (not seen?)
 	  *vector*
 	  (approx-type value-approx))))
-
+   
 ;*---------------------------------------------------------------------*/
 ;*    show-tvector ...                                                 */
 ;*---------------------------------------------------------------------*/
@@ -256,8 +242,7 @@
 		      (if (not (type? tvector))
 			  (let ((tv-id (symbol-append 'tv-of- id)))
 			     (set! tvector
-				   (type-parser `(tvector ,tv-id (,id))
-						'())))))))
+				   (module-tvector-clause tv-id id 'cfa #f)))))))
 	     tvector)
    ;; now we just make the new ast for the typed vectors.
    (let ((old-default-type (get-default-type)))
@@ -265,10 +250,10 @@
       (let ((tvector-unit (tvector-finalizer)))
 	 (pragma-finalizer)
 	 (let ((res (if (unit? tvector-unit)
-			(globalize-walk! (build-ast-sans-remove
-					  (list tvector-unit))
-					 'no-remove)
+			(let ((ast (build-ast-sans-remove (list tvector-unit))))
+			   (globalize-walk! ast 'no-remove))
 			'())))
+	    (lvtype-ast! res)
 	    (set-default-type! old-default-type)
 	    res))))
  
@@ -314,11 +299,16 @@
 (define-method (patch! knode::kwote/node)
    (with-access::kwote/node knode (node value)
       (let* ((approx (cfa! node))
-	     (tv     (get-approx-type approx)))
-	 (if (tvec? tv)
-	     (let ((knode (shrink! knode)))
-		(duplicate::kwote knode (value (a-tvector tv value))))
-	     (shrink! knode)))))
+	     (tv (get-approx-type approx node)))
+	 (if (tvec? tv) 
+	     (let* ((knode (shrink! knode))
+		    (n (duplicate::kwote knode
+			  (type (strict-node-type tv (node-type knode)))
+			  (value (a-tvector tv value)))))
+		(widen!::kwote/Cinfo n
+		   (approx approx)))
+	     (widen!::kwote/Cinfo (shrink! knode)
+		(approx approx))))))
 		    
 ;*---------------------------------------------------------------------*/
 ;*    patch! ::var ...                                                 */
@@ -340,6 +330,16 @@
       (patch*! nodes)
       node))
 
+;*---------------------------------------------------------------------*/
+;*    patch! ::sync ...                                                */
+;*---------------------------------------------------------------------*/
+(define-method (patch! node::sync)
+   (with-access::sync node (body mutex prelock)
+      (set! mutex (patch! mutex))
+      (set! prelock (patch! prelock))
+      (set! body (patch! body))
+      node))
+
 ;*---------------------------------------------------------------------*/
 ;*    patch! ::app-ly ...                                              */
 ;*---------------------------------------------------------------------*/
@@ -359,11 +359,11 @@
       node))
 
 ;*---------------------------------------------------------------------*/
-;*    patch! ::pragma ...                                              */
+;*    patch! ::extern ...                                              */
 ;*---------------------------------------------------------------------*/
-(define-method (patch! node::pragma)
-   (with-access::pragma node (args type)
-      (patch*! args)
+(define-method (patch! node::extern)
+   (with-access::extern node (expr* type)
+      (patch*! expr*)
       node))
 
 ;*---------------------------------------------------------------------*/
@@ -404,10 +404,10 @@
       node))
 
 ;*---------------------------------------------------------------------*/
-;*    patch! ::select ...                                              */
+;*    patch! ::switch ...                                              */
 ;*---------------------------------------------------------------------*/
-(define-method (patch! node::select)
-   (with-access::select node (clauses test)
+(define-method (patch! node::switch)
+   (with-access::switch node (clauses test)
       (set! test (patch! test))
       (for-each (lambda (clause)
 		   (set-cdr! clause (patch! (cdr clause))))
@@ -439,8 +439,9 @@
 ;*    patch! ::set-ex-it ...                                           */
 ;*---------------------------------------------------------------------*/
 (define-method (patch! node::set-ex-it)
-   (with-access::set-ex-it node (var body)
+   (with-access::set-ex-it node (var body onexit)
       (set! body (patch! body))
+      (set! onexit (patch! onexit))
       (patch! var)
       node))
 
@@ -493,19 +494,16 @@
 ;*    patch! ::app ...                                                 */
 ;*---------------------------------------------------------------------*/
 (define-method (patch! node::app)
-   (with-access::app node (fun args)
+   (with-access::app node (fun args loc)
       (patch*! args)
       (set! fun (patch! fun))
       (let ((v (var-variable fun)))
 	 (if (global? v)
 	     (if (cfun? (variable-value v))
 		 (case (global-id v)
-		    ((c-vector-length)
-		     (patch-vector-length! node))
-		    ((c-vector?)
-		     (patch-vector?! node))
-		    (else
-		     node))
+		    ((c-vector?) (patch-vector?! node))
+		    (($vector?) (patch-vector?! node))
+		    (else node))
 		 (if (and (eq? (global-id v) 'vector->list)
 			  (eq? (global-module v) '__r4_vectors_6_8))
 		     (patch-vector->list! node)
@@ -513,20 +511,20 @@
 	     node))))
 
 ;*---------------------------------------------------------------------*/
-;*    patch-vector-length! ...                                         */
+;*    patch! ::vlength ...                                             */
 ;*---------------------------------------------------------------------*/
-(define (patch-vector-length! node::app)
-   (with-access::vector-set!-app node (args loc)
-      (patch*! args)
-      (let* ((approx (cfa! (car args)))
-	     (tv     (get-approx-type approx)))
-	 (if (tvec? tv)
+(define-method (patch! node::vlength)
+   (with-access::vlength/Cinfo node (expr* loc tvector?)
+      (patch*! expr*)
+      (let* ((approx (cfa! (car expr*)))
+	     (tv (get-approx-type approx node)))
+	 (if (and (tvec? tv) (not tvector?))
 	     (let* ((length-tv (symbol-append (type-id tv) '-length))
-		    (new-node  (sexp->node `(,length-tv ,@args)
+		    (new-node  (sexp->node `(,length-tv ,(car expr*))
 					   '()
 					   loc
 					   'value)))
-		(node-type-set! new-node *long*)
+		(node-type-set! new-node *int*)
 		(inline-node new-node 1 '()))
 	     node))))
 
@@ -534,14 +532,14 @@
 ;*    patch-vector?! ...                                               */
 ;*---------------------------------------------------------------------*/
 (define (patch-vector?! node::app)
-   (with-access::make-vector-app node (args loc)
+   (with-access::app node (args loc)
       (patch*! args)
       (let* ((approx (cfa! (car args)))
-	     (type   (approx-type approx)))
-	 (if (eq? type *vector*)
-	     (instantiate::atom
+	     (type (get-approx-type approx (car args))))
+	 (if (or (eq? type *vector*) (isa? type tvec))
+	     (instantiate::literal
 		(loc loc)
-		(type *bool*)
+		(type (strict-node-type (get-type-atom #t) *bool*))
 		(value #t))
 	     node))))
 
@@ -549,10 +547,10 @@
 ;*    patch-vector->list! ...                                          */
 ;*---------------------------------------------------------------------*/
 (define (patch-vector->list! node::app)
-   (with-access::vector-set!-app node (args loc)
+   (with-access::app node (args loc)
       (patch*! args)
       (let* ((approx (cfa! (car args)))
-	     (tv     (get-approx-type approx)))
+	     (tv     (get-approx-type approx node)))
 	 (if (tvec? tv)
 	     (let* ((tv->list  (symbol-append (type-id tv) '->list))
 		    (new-node  (sexp->node `(,tv->list ,@args)
@@ -567,83 +565,77 @@
 ;*    patch! ::make-vector-app ...                                     */
 ;*---------------------------------------------------------------------*/
 (define-method (patch! node::make-vector-app)
-   (with-access::make-vector-app node (value-approx fun args loc stackable?)
+   (with-access::make-vector-app node (value-approx fun args loc tvector?)
       (patch*! args)
       (let* ((type (approx-type value-approx))
 	     (tv   (type-tvector type)))
-	 (if (type? tv)
+	 (if (and (type? tv) tvector?)
 	     (let* ((make-tv   (symbol-append 'make- (type-id tv)))
 		    (new-node  (sexp->node `(,make-tv ,@args)
-					   '()
-					   loc
-					   'value)))
+				  '()
+				  loc
+				  'value)))
 		(node-type-set! new-node tv)
-		(inline-node (if (stack-optimization?)
-				 (node-heap->stack! new-node stackable?)
-				 new-node)
-			     1
-			     '()))
+		(inline-node new-node 1 '()))
 	     node))))
 
 ;*---------------------------------------------------------------------*/
-;*    patch! ::create-vector-app ...                                   */
+;*    patch! ::valloc/Cinfo+optim ...                                  */
 ;*---------------------------------------------------------------------*/
-(define-method (patch! node::create-vector-app)
-   (with-access::create-vector-app node (value-approx fun args loc stackable?)
-      (patch*! args)
+(define-method (patch! node::valloc/Cinfo+optim)
+   (with-access::valloc/Cinfo+optim node (value-approx expr* loc ftype)
+      (patch*! expr*)
       (let* ((type (approx-type value-approx))
-	     (tv   (type-tvector type)))
-	 (if (type? tv)
+	     (tv (type-tvector type)))
+	 (if (and (type? tv) (eq? ftype *_*))
 	     (let* ((create-tv (symbol-append 'allocate- (type-id tv)))
-		    (new-node  (sexp->node `(,create-tv ,@args)
-					   '()
-					   loc
-					   'value)))
-		(node-type-set! new-node tv)
-		(inline-node (if (stack-optimization?)
-				 (node-heap->stack! new-node stackable?)
-				 new-node)
-			     1
-			     '()))
-	     node))))
+		    (new-node  (sexp->node `(,create-tv ,@expr*)
+				  '()
+				  loc
+				  'value)))
+		(let ((n (inline-node new-node 1 '())))
+		   (lvtype-node! n)
+		   n))
+	     (begin
+		(set! ftype (get-approx-type value-approx node))
+		node)))))
 	    
 ;*---------------------------------------------------------------------*/
-;*    patch! ::vector-ref-app ...                                      */
+;*    patch! ::vref/Cinfo ...                                          */
 ;*---------------------------------------------------------------------*/
-(define-method (patch! node::vector-ref-app)
-   (with-access::vector-ref-app node (fun args loc)
-      (patch*! args)
-      (let* ((vec-approx (cfa! (car args)))
-	     (tv         (get-approx-type vec-approx)))
-	 (if (not (tvec? tv))
-	     node
-	     (let* ((tv-ref   (symbol-append (type-id tv) '-ref))
-		    (new-node (sexp->node `(,tv-ref ,@args)
-					  '()
-					  loc
-					  'value)))
-		(inline-node new-node 1 '()))))))
+(define-method (patch! node::vref/Cinfo)
+   (with-access::vref/Cinfo node (expr* loc tvector? unsafe type ftype approx)
+      (patch*! expr*)
+      (if tvector?
+	  (let ((ty (vref-ftype node)))
+	     (set! type ty)
+	     node)
+	  (let* ((vec-approx (cfa! (car expr*)))
+		 (tv (get-approx-type vec-approx node)))
+	     (if (not (tvec? tv))
+		 (let ((ty (get-approx-type approx node)))
+		    (set! ftype ty)
+		    (set! type ty)
+		    node)
+		 (let* ((ty (get-approx-type approx node))
+			(tv-ref (symbol-append (type-id tv) '-ref))
+			(new-node (sexp->node `(,tv-ref ,@expr*) '() loc 'value)))
+		    (node-type-set! new-node (strict-node-type tv ty))
+		    (inline-node new-node 1 '())))))))
 
 ;*---------------------------------------------------------------------*/
-;*    patch! ::vector-set!-app ...                                     */
+;*    patch! ::vset!/Cinfo ...                                         */
 ;*---------------------------------------------------------------------*/
-(define-method (patch! node::vector-set!-app)
-   (with-access::vector-set!-app node (fun args loc)
-      (patch*! args)
-      (let* ((vec-approx (cfa! (car args)))
-	     (tv         (get-approx-type vec-approx)))
-	 (if (not (tvec? tv))
-	     node
-	     (let* ((tv-set!  (symbol-append (type-id tv) '-set!))
-		    (new-node (sexp->node `(,tv-set! ,@args)
-					  '()
-					  loc
-					  'value)))
-		(inline-node new-node 1 '()))))))
-	    
-
-
-
-
-
+(define-method (patch! node::vset!/Cinfo)
+   (with-access::vset!/Cinfo node (expr* loc tvector? ftype)
+      (patch*! expr*)
+      (if tvector?
+	  node
+	  (let* ((vec-approx (cfa! (car expr*)))
+		 (tv (get-approx-type vec-approx node)))
+	     (if (not (tvec? tv))
+		 node
+		 (let* ((tv-set!  (symbol-append (type-id tv) '-set!))
+			(new-node (sexp->node `(,tv-set! ,@expr*) '() loc 'value)))
+		    (inline-node new-node 1 '())))))))
 
