@@ -1,10 +1,10 @@
 ;*=====================================================================*/
-;*    serrano/prgm/project/bigloo/comptime/Cfa/walk.scm                */
+;*    serrano/prgm/project/bigloo/bigloo/comptime/Cfa/walk.scm         */
 ;*    -------------------------------------------------------------    */
 ;*    Author      :  Manuel Serrano                                    */
 ;*    Creation    :  Tue Feb 21 08:37:48 1995                          */
-;*    Last change :  Sat Jul  7 07:30:50 2001 (serrano)                */
-;*    Copyright   :  1995-2001 Manuel Serrano, see LICENSE file        */
+;*    Last change :  Tue Jul  4 10:57:52 2023 (serrano)                */
+;*    Copyright   :  1995-2023 Manuel Serrano, see LICENSE file        */
 ;*    -------------------------------------------------------------    */
 ;*    The `control flow analysis' and its optimizations described in:  */
 ;*                                                                     */
@@ -22,7 +22,8 @@
 ;*    The module                                                       */
 ;*---------------------------------------------------------------------*/
 (module cfa_walk
-   (include "Engine/pass.sch")
+   (include "Engine/pass.sch"
+	    "Tools/trace.sch")
    (import  tools_shape
 	    tools_error
 	    engine_param
@@ -31,6 +32,7 @@
 	    ast_var
 	    ast_node
 	    ast_shrinkify
+	    write_ast
 	    cfa_collect
 	    cfa_setup
 	    cfa_iterate
@@ -43,6 +45,7 @@
 	    cfa_closure
 	    cfa_specialize
 	    cfa_tvector
+	    cfa_pair
 	    cfa_arithmetic)
    (export  (cfa-walk! <global>*)))
 
@@ -58,36 +61,40 @@
 ;*---------------------------------------------------------------------*/
 (define (cfa-walk! globals)
    (pass-prelude "Cfa")
-   ;; first of all, we scan the global definitions in order
+   ;; first, the global definitions are scanned in order
    ;; to collect all used types and allocations.
    (collect-all-approx! globals)
-   ;; we have collected all the approximation, we can now declare
-   ;; the approximations sets.
+   ;; declare the approximations sets.
    (declare-approx-sets!)
-   ;; we have collect all the possible approximation, we now
-   ;; prepare the ast to the iteration process.
+   ;; all approximations being now known, prepare the ast for
+   ;; the main CFA iteration loop.
    (set-initial-approx! globals)
-   ;; ok, we can start now the control flow analysis
+   ;; start now the control flow analysis.
    (let ((iteration-roots (cfa-iterate-to-fixpoint! globals)))
-      ;; the number of iterations
+      ;; show the number of iterations.
       (show-cfa-nb-iterations)
       ;; dead code removal
       (let ((globals (remove-var '(cfa inline) globals)))
-	 ;; we show the result of the approximation (after dead-code-removal!)
+	 ;; show approximation results (after dead-code-removal!)
 	 (show-cfa-results globals)
 	 ;; tvector optimization
+;* 	 (let ((d *dest*))                                             */
+;* 	    (set! *dest* "/tmp/vec.ast")                               */
+;* 	    (write-ast globals)                                        */
+;* 	    (set! *dest* d))                                           */
 	 (let ((additional (profile tvect (vector->tvector! globals))))
-	    ;; type settings
+;* 	    (let ((d *dest*))                                          */
+;* 	    (set! *dest* "/tmp/tvec.ast")                              */
+;* 	    (write-ast globals)                                        */
+;* 	    (set! *dest* d))                                           */
+	    ;; closure allocations optimization
+	    (profile clo (closure-optimization! globals))
+	    ;; type setting
 	    (profile type (type-settings! globals))
-	    ;; we optimize closure allocations
-	    (if (not (and (number? *profile-mode*) (> *profile-mode* 0)))
-		(profile clo (closure-optimization!)))
-	    ;; we cleanup the arithmetic optimizer
-	    (cleanup-arithmetic-nodes!)
 	    ;; generic arithmetic specialization
 	    (specialize! globals)
-	    ;; and we are done
+	    ;; cleanup and statistics display.
 	    (pass-postlude (shrinkify! (append additional globals))
-			   unpatch-vector-set!)))))
- 
- 
+	       stop-closure-cache
+	       unpatch-vector-set!
+	       unpatch-pair-set!)))))
