@@ -1,10 +1,10 @@
 ;*=====================================================================*/
-;*    serrano/prgm/project/bigloo/comptime/Cnst/initialize.scm         */
+;*    .../prgm/project/bigloo/bigloo/comptime/Cnst/initialize.scm      */
 ;*    -------------------------------------------------------------    */
 ;*    Author      :  Manuel Serrano                                    */
 ;*    Creation    :  Mon Feb 20 15:50:19 1995                          */
-;*    Last change :  Mon May  7 19:36:07 2001 (serrano)                */
-;*    Copyright   :  1995-2001 Manuel Serrano, see LICENSE file        */
+;*    Last change :  Fri Jun  3 14:41:10 2022 (serrano)                */
+;*    Copyright   :  1995-2022 Manuel Serrano, see LICENSE file        */
 ;*    -------------------------------------------------------------    */
 ;*    The initialize function definition.                              */
 ;*=====================================================================*/
@@ -19,6 +19,7 @@
 	    tools_speek
 	    tools_error
 	    engine_param
+	    backend_backend
 	    type_type
 	    type_cache
 	    ast_var
@@ -50,12 +51,12 @@
 					 "Illegal init-mode"
 					 *init-mode*)))))
       (if (pair? body)
-	  (let ((unit (unit 'cnst 8 body #t)))
+	  (let ((unit (unit 'cnst 8 body #t (eq? *object-init-mode* 'staged))))
 	     (let ((ast (build-ast (list unit))))
 		(for-each (lambda (global)
-			     (coerce! (sfun-body (global-value global))
-				      global
-				      (global-type global)))
+			     (let ((body (sfun-body (global-value global))))
+				(lvtype-node! body)
+				(coerce! body global (global-type global) #f)))
 			  ast)
 		ast))
 	  '())))
@@ -64,11 +65,8 @@
 ;*    lib-initialize! ...                                              */
 ;*---------------------------------------------------------------------*/
 (define (lib-initialize!)
-   (case *target-language*
-      ((jvm)
-       (global-name-set! (get-cnst-table) (string-append "__cnst")))
-      (else
-       (global-name-set! (get-cnst-table) "*__cnst")))
+   (global-name-set! (get-cnst-table) (backend-cnst-table-name (the-backend) 0))
+   (global-user?-set! (get-cnst-table) #f)
    (get-cnst-sexp))
 
 ;*---------------------------------------------------------------------*/
@@ -83,42 +81,36 @@
 ;*    read-empty-cnst-initialize! ...                                  */
 ;*---------------------------------------------------------------------*/
 (define (read-empty-cnst-initialize!)
-   (case *target-language*
-      ((jvm)
-       (global-name-set! (get-cnst-table) (string-append "__cnst")))
-      (else
-       (global-name-set! (get-cnst-table) "*__cnst")))
+   (global-name-set! (get-cnst-table) (backend-cnst-table-name (the-backend) 0))
+   (global-user?-set! (get-cnst-table) #f)
    (get-cnst-sexp))
 
 ;*---------------------------------------------------------------------*/
 ;*    read-full-cnst-initialize! ...                                   */
 ;*---------------------------------------------------------------------*/
 (define (read-full-cnst-initialize!)
-   (case *target-language*
-      ((jvm)
-       (global-name-set! (get-cnst-table) (string-append "__cnst")))
-      (else
-       (global-name-set! (get-cnst-table)
-			 (string-append "__cnst[ "
-					(number->string (get-cnst-offset))
-					" ] "))))
+   
    (define (read-full-cnst-initialize/small-string cnst-string)
       (let* ((var-string  (cnst-alloc-string cnst-string #f))
 	     (sexp `(let ((cport::input-port
-			   (c-open-input-string ,(global-id
-						  (var-variable
-						   var-string)))))
+			   (open-input-string!
+			      ,(global-id (var-variable var-string)))))
 		       (labels ((loop (i::long)
-				      (if (c-=fx i -1)
+				      (if ($=fx i -1)
 					  #unspecified
 					  (begin
 					     (cnst-table-set!
 					      i
 					      ((@ read __reader) cport))
-					     (let ((aux::int (c--fx i 1)))
+					     (let ((aux::int ($-fx i 1)))
 						(loop aux))))))
 			  (loop ,(-fx (get-cnst-offset) 1))))))
 	 (cons sexp (get-cnst-sexp))))
+
+   (global-name-set! (get-cnst-table)
+      (backend-cnst-table-name (the-backend) (get-cnst-offset)))
+   (global-user?-set! (get-cnst-table) #f)
+   
    (let ((cnst-string (cnst-set->cnst-string (get-cnst-set))))
       (read-full-cnst-initialize/small-string cnst-string)))
 
@@ -132,13 +124,13 @@
    (let ((port (open-output-string)))
       (if (not (output-port? port))
 	  (internal-error "cnst-set->cnst-string"
-			  "Can't open output string port"
-			  port)
+	     "Can't open output string port"
+	     port)
 	  (begin
 	     (for-each (lambda (cnst)
 			  (write cnst port)
 			  (write-char #\space port))
-		       set)
+		set)
 	     (close-output-port port)))))
   
 ;*---------------------------------------------------------------------*/
@@ -153,35 +145,36 @@
 ;*    intern-empty-cnst-initialize! ...                                */
 ;*---------------------------------------------------------------------*/
 (define (intern-empty-cnst-initialize!)
-   (global-name-set! (get-cnst-table) "*__cnst")
+   (global-name-set! (get-cnst-table)
+		     (backend-cnst-table-name (the-backend) 0))
    (get-cnst-sexp))
 
 ;*---------------------------------------------------------------------*/
 ;*    intern-full-cnst-initialize! ...                                 */
 ;*---------------------------------------------------------------------*/
 (define (intern-full-cnst-initialize!)
-   (global-name-set! (get-cnst-table)
-		     (string-append "__cnst[ "
-				    (number->string (get-cnst-offset))
-				    " ] "))
+
    (define (intern-full-cnst-initialize/small-string cnst-string)
       (let* ((var-string  (cnst-alloc-string cnst-string #f))
 	     (sexp `(let ((cnst-tmp::vector
-				     ((@ string->obj __intext)
-				      ,(global-id
-					(var-variable var-string)))))
+			     ((@ string->obj __intext)
+			      ,(global-id
+				  (var-variable var-string)))))
 		       (labels ((loop (i::int)
-				      (if (c-=fx i -1)
-					  #unspecified
-					  (begin
-					     (cnst-table-set! i
-							      (c-vector-ref
-							       cnst-tmp
-							       i))
-					     (let ((aux::int (c--fx i 1)))
-						(loop aux))))))
+				   (if ($=fx i -1)
+				       #unspecified
+				       (begin
+					  (cnst-table-set!
+					     i
+					     ($vector-ref-ur cnst-tmp i))
+					  (let ((aux::int ($-fx i 1)))
+					     (loop aux))))))
 			  (loop ,(-fx (get-cnst-offset) 1))))))
 	 (cons sexp (get-cnst-sexp))))
+   
+   (global-name-set! (get-cnst-table)
+      (backend-cnst-table-name (the-backend) (get-cnst-offset)))
+   
    (let* ((cnst-string (obj->string (list->vector (reverse! (get-cnst-set))))))
       (intern-full-cnst-initialize/small-string cnst-string)))
 
