@@ -1,10 +1,10 @@
 ;*=====================================================================*/
-;*    serrano/prgm/project/bigloo/comptime/Cnst/node.scm               */
+;*    serrano/prgm/project/bigloo/bigloo/comptime/Cnst/node.scm        */
 ;*    -------------------------------------------------------------    */
 ;*    Author      :  Manuel Serrano                                    */
 ;*    Creation    :  Mon Feb  6 14:08:40 1995                          */
-;*    Last change :  Wed Jun  6 11:43:13 2001 (serrano)                */
-;*    Copyright   :  1995-2001 Manuel Serrano, see LICENSE file        */
+;*    Last change :  Thu Nov  3 11:44:41 2022 (serrano)                */
+;*    Copyright   :  1995-2022 Manuel Serrano, see LICENSE file        */
 ;*    -------------------------------------------------------------    */
 ;*    The constant compilation (of the kwoted forms and                */
 ;*    `make-??-procedure' calls).                                      */
@@ -15,12 +15,14 @@
 ;*---------------------------------------------------------------------*/
 (module cnst_node
    (include "Tools/trace.sch"
-	    "Tvector/tvector.sch")
+	    "Tvector/tvector.sch"
+	    "Cnst/node.sch")
    (import  tools_shape
 	    tools_speek
 	    tools_error
 	    type_type
 	    type_cache
+	    type_typeof
 	    ast_var
 	    ast_node
 	    ast_env
@@ -39,16 +41,23 @@
 ;*---------------------------------------------------------------------*/
 ;*    cnst! ...                                                        */
 ;*---------------------------------------------------------------------*/
-(define-method (cnst! node::atom)
-   (with-access::atom node (value loc)
+(define-method (cnst! node::literal)
+   (with-access::literal node (value loc)
       (cond
-	 ((keyword? value)
-	  (cnst-alloc-keyword value loc))
-	 ((ucs2-string? value)
-	  (cnst-alloc-ucs2-string value loc))
-	 (else
-	  node))))
- 
+	 ((keyword? value) (cnst-alloc-keyword value loc))
+	 ((ucs2-string? value) (cnst-alloc-ucs2-string value loc))
+	 ((string? value) (cnst-alloc-string value loc))
+	 ((bignum? value) (or (cnst-alloc-bignum value loc) node))
+	 (else node))))
+
+;*---------------------------------------------------------------------*/
+;*    cnst! ::patch ...                                                */
+;*---------------------------------------------------------------------*/
+(define-method (cnst! node::patch)
+   (with-access::patch node (value)
+      (cnst! value)
+      node))
+
 ;*---------------------------------------------------------------------*/
 ;*    cnst! ...                                                        */
 ;*---------------------------------------------------------------------*/
@@ -63,6 +72,8 @@
 	  (cnst-alloc-list value loc))
 	 ((vector? value)
 	  (cnst-alloc-vector value loc))
+	 ((homogeneous-vector? value)
+	  (cnst-alloc-homogenous-vector value loc))
 	 ((string? value)
 	  (cnst-alloc-string value loc))
 	 ((ucs2-string? value)
@@ -70,11 +81,32 @@
 	 ((a-tvector? value)
 	  (cnst-alloc-tvector value loc))
 	 ((or (char? value)
-	      (integer? value)
+	      (fixnum? value)
 	      (boolean? value)
 	      (real? value)
-	      (cnst? value))
-	  (instantiate::atom (loc loc) (type type) (value value)))
+	      (cnst? value)
+	      (elong? value)
+	      (llong? value)
+	      (int8? value)
+	      (uint8? value)
+	      (int16? value)
+	      (uint16? value)
+	      (int32? value)
+	      (uint32? value)
+	      (int64? value)
+	      (uint64? value))
+	  (instantiate::literal
+	     (loc loc)
+	     (type (strict-node-type (get-type-atom value) type))
+	     (value value)))
+	 ((bignum? value)
+	  (or (cnst-alloc-bignum value loc)
+	      (instantiate::literal
+		 (loc loc)
+		 (type (strict-node-type (get-type-atom value) type))
+		 (value value))))
+	 ((struct? value)
+	  (cnst-alloc-struct value loc))
 	 (else
 	  (internal-error "cnst-quote" "Illegal expression" (shape node))))))
 
@@ -98,6 +130,16 @@
       (cnst*! nodes)
       node))
 
+;*---------------------------------------------------------------------*/
+;*    cnst! ::sync ...                                                 */
+;*---------------------------------------------------------------------*/
+(define-method (cnst! node::sync)
+   (with-access::sync node (body mutex prelock)
+      (set! mutex (cnst! mutex))
+      (set! prelock (cnst! prelock))
+      (set! body (cnst! body))
+      node))
+
 ;*---------------------------------------------------------------------*/
 ;*    cnst! ::extern ...                                               */
 ;*---------------------------------------------------------------------*/
@@ -143,10 +185,10 @@
       node))
 
 ;*---------------------------------------------------------------------*/
-;*    cnst! ::select ...                                               */
+;*    cnst! ::switch ...                                               */
 ;*---------------------------------------------------------------------*/
-(define-method (cnst! node::select)
-   (with-access::select node (clauses test)
+(define-method (cnst! node::switch)
+   (with-access::switch node (clauses test)
       (set! test (cnst! test))
       (for-each (lambda (clause)
 		   (set-cdr! clause (cnst! (cdr clause))))
@@ -184,8 +226,9 @@
 ;*    cnst! ::set-ex-it ...                                            */
 ;*---------------------------------------------------------------------*/
 (define-method (cnst! node::set-ex-it)
-   (with-access::set-ex-it node (var body)
+   (with-access::set-ex-it node (var body onexit)
       (set! body (cnst! body))
+      (set! onexit (cnst! onexit))
       node))
 
 ;*---------------------------------------------------------------------*/
@@ -260,8 +303,8 @@
       ;; then we make special cases depending on the called function
       (if (null? args)
 	  node
-	  (let* ((fun          (var-variable (app-fun node)))
-		 (actual       (car args))
+	  (let* ((fun (var-variable (app-fun node)))
+		 (actual (car args))
 		 (actual-value (get-node-atom-value actual)))
 	     (cond
 		((eq? fun *string->bstring*)
@@ -273,11 +316,11 @@
 		((eq? fun *bool->bbool*)
 		 (if (boolean? actual-value)
 		     (if actual-value
-			 (instantiate::var
+			 (instantiate::ref
 			    (loc loc)
 			    (type *bbool*)
 			    (variable *btrue*))
-			 (instantiate::var
+			 (instantiate::ref
 			    (loc loc)
 			    (type *bbool*)
 			    (variable *bfalse*)))
@@ -285,13 +328,47 @@
 		((or (eq? fun *make-fx-procedure*)
 		     (eq? fun *make-va-procedure*))
 		 (let ((size-value (get-node-atom-value (caddr args))))
-		    (if (and (integer? size-value) (=fx size-value 0))
+		    (if (and (fixnum? size-value) (=fx size-value 0))
 			(cnst-alloc-procedure node loc)
 			node)))
+		((eq? fun *make-l-procedure*)
+		 (let ((size-value (get-node-atom-value (cadr args))))
+		    (if (and (fixnum? size-value) (=fx size-value 0))
+			(cnst-alloc-l-procedure node loc)
+			node)))
 		((eq? fun *double->real*)
 		 (if (real? actual-value)
 		     (cnst-alloc-real actual-value loc)
 		     node))
+		((eq? fun *elong->belong*)
+		 (if (elong? actual-value)
+		     (cnst-alloc-elong actual-value loc)
+		     node))
+		((eq? fun *llong->bllong*)
+		 (if (llong? actual-value)
+		     (cnst-alloc-llong actual-value loc)
+		     node))
+		((eq? fun *int32->bint32*)
+		 (if (int32? actual-value)
+		     (cnst-alloc-int32 actual-value loc)
+		     node))
+		((eq? fun *uint32->buint32*)
+		 (if (uint32? actual-value)
+		     (cnst-alloc-uint32 actual-value loc)
+		     node))
+		((eq? fun *int64->bint64*)
+		 (if (int64? actual-value)
+		     (cnst-alloc-int64 actual-value loc)
+		     node))
+		((eq? fun *uint64->buint64*)
+		 (if (uint64? actual-value)
+		     (cnst-alloc-uint64 actual-value loc)
+		     node))
+		((eq? fun *fixnum->bignum*)
+		 (if (fixnum? actual-value)
+		     (or (cnst-alloc-bignum (fixnum->bignum actual-value) loc)
+			 node)
+		     node))
 		(else
 		 node))))))
 
