@@ -3,8 +3,8 @@
 ;*    -------------------------------------------------------------    */
 ;*    Author      :  Manuel Serrano                                    */
 ;*    Creation    :  Tue Nov  9 16:41:17 1993                          */
-;*    Last change :  Fri Dec 29 18:56:36 2000 (serrano)                */
-;*    Copyright   :  1993-2000 Manuel Serrano, see LICENSE file        */
+;*    Last change :  Tue Nov 18 10:37:37 2014 (serrano)                */
+;*    Copyright   :  1993-2014 Manuel Serrano, see LICENSE file        */
 ;*    -------------------------------------------------------------    */
 ;*    L'expansion des formes `assert'                                  */
 ;*=====================================================================*/
@@ -15,8 +15,8 @@
 (module expand_assert
    (include "Tools/location.sch")
    (import  engine_param
+	    backend_backend
 	    tools_misc
-	    tools_progn
 	    tools_location)
    (export  (expand-assert ::obj ::procedure)))
 
@@ -31,8 +31,12 @@
 	  (replace! x new)
 	  (e x e)))
       ((?- (and ?vars (? list?)) . ?pred)
-       (if (or (and (integer? *compiler-debug*) (>=fx *compiler-debug* 1))
-	       (and (integer? *bdb-debug*) (>=fx *bdb-debug* 1)))
+       (if (or (and (fixnum? *compiler-debug*)
+		    (>=fx *compiler-debug* 1)
+		    (not *bmem-profiling*))
+	       (and (memq 'bdb (backend-debug-support (the-backend)))
+		    (fixnum? *bdb-debug*)
+		    (>=fx *bdb-debug* 1)))
 	   (let ((new (make-one-assert e x vars pred)))
 	      (replace! x new))
 	   #unspecified))
@@ -47,7 +51,7 @@
 		    (list #unspecified)
 		    pred))
 	  (old-pred (dup pred)))
-      `(if ,(e (normalize-progn pred) e)
+      `(if ,(e (expand-progn pred) e)
 	   #unspecified
 	   (begin
 	      ;; we send all vars to the interpreters
