@@ -3,8 +3,8 @@
 ;*    -------------------------------------------------------------    */
 ;*    Author      :  Manuel Serrano                                    */
 ;*    Creation    :  Wed Dec 28 15:44:53 1994                          */
-;*    Last change :  Sat Jun 16 09:01:56 2001 (serrano)                */
-;*    Copyright   :  1994-2001 Manuel Serrano, see LICENSE file        */
+;*    Last change :  Mon Oct 18 08:37:41 2010 (serrano)                */
+;*    Copyright   :  1994-2010 Manuel Serrano, see LICENSE file        */
 ;*    -------------------------------------------------------------    */
 ;*    The lambda macro-expansion.                                      */
 ;*=====================================================================*/
@@ -22,7 +22,8 @@
 	   engine_param
 	   (find-location tools_location))
    (export internal-definition?
-	   (expand-lambda                      ::obj ::procedure)
+	   (expand-args ::obj ::procedure)
+	   (expand-lambda ::obj ::procedure)
 	   (internal-begin-expander::procedure ::procedure)))
 
 ;*---------------------------------------------------------------------*/
@@ -30,6 +31,26 @@
 ;*---------------------------------------------------------------------*/
 (define internal-definition? #f)
 
+;*---------------------------------------------------------------------*/
+;*    expand-args ...                                                  */
+;*---------------------------------------------------------------------*/
+(define (expand-args args e)
+   (let loop ((args args))
+      (cond
+	 ((null? args)
+	  '())
+	 ((symbol? args)
+	  args)
+	 ((not (pair? args))
+	  (error "expand" "Illegal argument" args))
+	 ((not (and (pair? (car args))
+		    (pair? (cdr (car args)))
+		    (null? (cddr (car args)))))
+	  (cons (car args) (loop (cdr args))))
+	 (else
+	  (cons (list (car (car args)) (e (cadr (car args)) e))
+		(loop (cdr args)))))))
+       
 ;*---------------------------------------------------------------------*/
 ;*    expand-lambda ...                                                */
 ;*---------------------------------------------------------------------*/
@@ -43,36 +64,48 @@
 				   (find-location x)
 				   (lambda ()
 				      (let ((e (internal-begin-expander e)))
-					 `(,lam ,args
-					     ,(e (normalize-progn body) e))))))
+					 `(,lam ,(expand-args args e)
+					     ,(e (expand-progn body) e))))))
 		    (else
 		     (error #f "Illegal `lambda' form" x)))))
 	 (set! internal-definition? old-internal)
 	 (replace! x res))))
-  
+
 ;*---------------------------------------------------------------------*/
 ;*    internal-begin-expander ...                                      */
 ;*---------------------------------------------------------------------*/
 (define (internal-begin-expander old-expander)
    (lambda (expr expander)
-      (let ((res (if (and (pair? expr) (eq? (car expr) 'begin))
-		     (if (not (null? (cdr expr)))
-			 (let ((rest (with-lexical
-				      (begin-bindings (cdr expr))
-				      '_
-				      (find-location expr)
-				      (lambda ()
-					 (lambda-defines
-					  (emap (lambda (x) (expander x expander))
-						(cdr expr)))))))
-			    (if (epair? expr)
-				(econs 'begin rest (cer expr))
-				(cons 'begin rest)))
-			 #unspecified)
-		     (old-expander expr expander))))
-	 (if (pair? res)
-	     (replace! expr res)
-	     res))))
+      (match-case expr
+	 ((begin)
+	  #unspecified)
+	 ((begin . ?rest)
+	  (if (not (list? rest))
+	      (error 'begin "Illegal `begin' form" expr)
+	      (with-lexical
+	       (begin-bindings rest)
+	       '_
+	       (find-location expr)
+	       (lambda ()
+		  (lambda-defines
+		   (emap (lambda (x) (old-expander x old-expander)) rest))))))
+	 (else
+	  (let ((nexpr (old-expander expr old-expander)))
+	     (match-case nexpr
+		((begin)
+		 #unspecified)
+		((begin . ?rest)
+		 (if (not (list? rest))
+		     (error 'begin "Illegal `begin' form" expr)
+		     (with-lexical
+		      (begin-bindings rest)
+		      '_
+		      (find-location expr)
+		      (lambda ()
+			 (lambda-defines
+			  (emap (lambda (x) (old-expander x old-expander)) rest))))))
+		(else
+		 nexpr)))))))
 
 ;*---------------------------------------------------------------------*/
 ;*    begin-bindings ...                                               */
@@ -86,22 +119,17 @@
 	  (let ((form (car oldforms)))
 	     (match-case form
 		((define (?var . ?-) . ?-)
-		 (loop (cdr oldforms)
-		       (cons var vars)))
+		 (loop (cdr oldforms) (cons var vars)))
 		((define ?var . ?-)
-		 (loop (cdr oldforms)
-		       (cons var vars)))
+		 (loop (cdr oldforms) (cons var vars)))
 		(else
-		 (loop (cdr oldforms)
-		       vars))))
+		 (loop (cdr oldforms) vars))))
 	  vars)))
 
 ;*---------------------------------------------------------------------*/
 ;*    lambda-defines ...                                               */
 ;*---------------------------------------------------------------------*/
-(define (lambda-defines body)
-   (trace expand "lambda-defines: " body
-	  #\Newline)
+(define (lambda-defines body::pair-nil)
    (let loop ((oldforms  body)
 	      (newforms '())
 	      (vars     '())
@@ -113,8 +141,12 @@
 		 (loop (cdr oldforms)
 		       newforms
 		       (cons var vars)
-		       (cons `(,var ,val)
-			     decls)))
+		       (cons `(,var ,val) decls)))
+		((begin . ?body)
+		 (loop (append body (cdr oldforms))
+		       newforms
+		       vars
+		       decls))
 		(else
 		 (loop (cdr oldforms)
 		       (cons form newforms)
@@ -122,9 +154,7 @@
 		       decls))))
 	  (cond
 	     ((not (null? vars))
-	      (let ((nf (if (null? newforms)
-			    #unspecified
-			    (normalize-progn (reverse newforms)))))
-		 `((letrec ,decls ,nf))))
+	      `(letrec* ,(reverse! decls) ,(expand-progn (reverse newforms))))
 	     (else
-	      body)))))
+	      (expand-progn body))))))
+
