@@ -1,41 +1,10 @@
-;*---------------------------------------------------------------------*/
-;*   A pratical implementation for the Scheme programming language     */
-;*                                                                     */
-;*                                    ,--^,                            */
-;*                              _ ___/ /|/                             */
-;*                          ,;'( )__, ) '                              */
-;*                         ;;  //   L__.                               */
-;*                         '   \\   /  '                               */
-;*                              ^   ^                                  */
-;*                                                                     */
-;*   Copyright (c) 1992-1999 Manuel Serrano                            */
-;*                                                                     */
-;*     Bug descriptions, use reports, comments or suggestions are      */
-;*     welcome. Send them to                                           */
-;*       bigloo-request@kaolin.unice.fr                                */
-;*       http://kaolin.unice.fr/bigloo                                 */
-;*                                                                     */
-;*   This program is free software; you can redistribute it            */
-;*   and/or modify it under the terms of the GNU General Public        */
-;*   License as published by the Free Software Foundation; either      */
-;*   version 2 of the License, or (at your option) any later version.  */
-;*                                                                     */
-;*   This program is distributed in the hope that it will be useful,   */
-;*   but WITHOUT ANY WARRANTY; without even the implied warranty of    */
-;*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     */
-;*   GNU General Public License for more details.                      */
-;*                                                                     */
-;*   You should have received a copy of the GNU General Public         */
-;*   License along with this program; if not, write to the Free        */
-;*   Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,   */
-;*   MA 02111-1307, USA.                                               */
-;*---------------------------------------------------------------------*/
 ;*=====================================================================*/
-;*    serrano/prgm/project/bigloo/comptime/Globalize/free.scm          */
+;*    .../prgm/project/bigloo/bigloo/comptime/Globalize/free.scm       */
 ;*    -------------------------------------------------------------    */
 ;*    Author      :  Manuel Serrano                                    */
 ;*    Creation    :  Fri Jan 27 14:20:15 1995                          */
-;*    Last change :  Thu May 28 19:20:52 1998 (serrano)                */
+;*    Last change :  Thu Jul  8 11:29:16 2021 (serrano)                */
+;*    Copyright   :  1995-2021 Manuel Serrano, see LICENSE file        */
 ;*    -------------------------------------------------------------    */
 ;*    The search of free variables.                                    */
 ;*=====================================================================*/
@@ -54,6 +23,7 @@
 	    ast_node
 	    ast_sexp
 	    ast_glo-def
+	    ast_env
 	    globalize_ginfo
 	    globalize_node
 	    engine_param
@@ -93,18 +63,19 @@
 ;*    free-variable? ...                                               */
 ;*---------------------------------------------------------------------*/
 (define (free-variable? local)
-   (let ((info (local-value local)))
-      (cond
-	 ((svar/Ginfo? info)
-	  (not (eq? (svar/Ginfo-free-mark info) *round*)))
-	 ((sfun/Ginfo? info)
-	  (not (eq? (sfun/Ginfo-free-mark info) *round*)))
-	 ((sexit/Ginfo? info)
-	  (not (eq? (sexit/Ginfo-free-mark info) *round*)))
-	 (else
-	  (error "free-variable?"
-		 "Unknown variable type"
-		 (cons local (shape local)))))))
+   (when (local? local)
+      (let ((info (local-value local)))
+	 (cond
+	    ((svar/Ginfo? info)
+	     (not (eq? (svar/Ginfo-free-mark info) *round*)))
+	    ((sfun/Ginfo? info)
+	     (not (eq? (sfun/Ginfo-free-mark info) *round*)))
+	    ((sexit/Ginfo? info)
+	     (not (eq? (sexit/Ginfo-free-mark info) *round*)))
+	    (else
+	     (error "free-variable?"
+		(format "Unknown variable type (~a)" (shape local))
+		local))))))
        
 ;*---------------------------------------------------------------------*/
 ;*    get-free-vars ...                                                */
@@ -113,7 +84,7 @@
 ;*    variables.                                                       */
 ;*---------------------------------------------------------------------*/
 (define (get-free-vars node::node integrator::local)
-   (trace (globalize 2) "get-free-vars [" (shape integrator) "] : "
+   (trace (globalize 5) "get-free-vars [" (shape integrator) "] : "
 	  #\Newline)
    (let ((free (sfun/Ginfo-free (local-value integrator))))
       (if (or (null? free) (pair? free))
@@ -134,13 +105,15 @@
 ;*    it maintains `bound' variables list.                             */
 ;*---------------------------------------------------------------------*/
 (define (internal-get-free-vars! node::node integrator::local)
+   (trace (globalize 5) "internal-get-free-vars [" (shape integrator)
+      "] : node=" (shape node) #\Newline)
    (set! *round* (+fx *round* 1))
    ;; we mark integrator and its formals
    (set! *integrator* integrator)
    (bind-variable! integrator integrator)
    (bind-variable! (the-closure integrator #f) integrator)
    (for-each (lambda (l) (bind-variable! l integrator))
-	     (sfun-args (local-value integrator)))
+      (sfun-args (local-value integrator)))
    ;; we can now walk across the body
    (node-free node '()))
 
@@ -155,6 +128,9 @@
 ;*---------------------------------------------------------------------*/
 (define-method (node-free node::var free)
    (with-access::var node (variable)
+      (trace (globalize 5) "node-free ::var var=" (shape node)
+	 " global=" (global? variable) " free="
+	 (free-variable? variable) #\Newline)
       (cond
 	 ((global? variable)
 	  free)
@@ -186,6 +162,13 @@
    (with-access::sequence node (nodes)
       (node-free* nodes free)))
 
+;*---------------------------------------------------------------------*/
+;*    node-free ::sync ...                                             */
+;*---------------------------------------------------------------------*/
+(define-method (node-free node::sync free)
+   (with-access::sync node (body mutex prelock)
+      (node-free body (node-free prelock (node-free mutex free)))))
+
 ;*---------------------------------------------------------------------*/
 ;*    node-free ::app ...                                              */
 ;*---------------------------------------------------------------------*/
@@ -203,7 +186,7 @@
 	 (node-free* args free))))
  
 ;*---------------------------------------------------------------------*/
-;*    node-free ::app-ly ...                                            */
+;*    node-free ::app-ly ...                                           */
 ;*---------------------------------------------------------------------*/
 (define-method (node-free node::app-ly free)
    (with-access::app-ly node (fun arg)
@@ -217,11 +200,11 @@
       (node-free fun (node-free* args free))))
 
 ;*---------------------------------------------------------------------*/
-;*    node-free ::pragma ...                                           */
+;*    node-free ::extern ...                                           */
 ;*---------------------------------------------------------------------*/
-(define-method (node-free node::pragma free)
-   (with-access::pragma node (args)
-      (node-free* args free)))
+(define-method (node-free node::extern free)
+   (with-access::extern node (expr*)
+      (node-free* expr* free)))
 
 ;*---------------------------------------------------------------------*/
 ;*    node-free ::cast ...                                             */
@@ -252,10 +235,10 @@
       (node-free proc (node-free msg (node-free obj free)))))
 
 ;*---------------------------------------------------------------------*/
-;*    node-free ::select ...                                           */
+;*    node-free ::switch ...                                           */
 ;*---------------------------------------------------------------------*/
-(define-method (node-free node::select free)
-   (with-access::select node (clauses test)
+(define-method (node-free node::switch free)
+   (with-access::switch node (clauses test)
       (let loop ((clauses clauses)
 		 (free    free))
 	 (if (null? clauses)
@@ -305,9 +288,9 @@
 ;*    node-free ::set-ex-it ...                                        */
 ;*---------------------------------------------------------------------*/
 (define-method (node-free node::set-ex-it free)
-   (with-access::set-ex-it node (var body)
+   (with-access::set-ex-it node (var body onexit)
       (bind-variable! (var-variable var) *integrator*)
-      (node-free body free)))
+      (node-free body (node-free onexit free))))
 
 ;*---------------------------------------------------------------------*/
 ;*    node-free ::jump-ex-it ...                                       */
@@ -360,24 +343,39 @@
 ;*    the-global-closure ...                                           */
 ;*---------------------------------------------------------------------*/
 (define (the-global-closure global::global loc)
+
+   (define (global-alias-closure g)
+      (with-access::global g (alias id module)
+	 (when alias
+	    (let ((ag (find-global alias module)))
+	       (when (global? ag)
+		  (the-global-closure ag loc))))))
+   
    (let ((closure (fun-the-closure (global-value global))))
-      (if (global? closure)
-	  closure
+      (cond
+	 ((global? closure)
+	  closure)
+	 ((global-alias-closure global)
+	  =>
+	  (lambda (g)
+	     (fun-the-closure-set! (global-value global) g)
+	     g))
+	 (else
 	  (let* ((gloclo   (make-global-closure global))
 		 (arity    (fun-arity (global-value global)))
 		 (make-clo (if (<fx arity 0)
 			       'make-va-procedure
 			       'make-fx-procedure))
 		 (node     (sexp->node `(,make-clo
-					 ,(instantiate::var
+					 ,(instantiate::ref
 					     (loc loc)
 					     (type *_*)
 					     (variable gloclo))
-					 ,(instantiate::atom
+					 ,(instantiate::literal
 					     (loc loc)
 					     (type *_*)
 					     (value arity))
-					 ,(instantiate::atom
+					 ,(instantiate::literal
 					     (loc loc)
 					     (type *_*)
 					     (value 0)))
@@ -385,8 +383,10 @@
 				       loc
 				       'value))
 		 (closure  (def-global-scnst! (symbol-append
-					       (global-id global)
-					       '-env::procedure)
+						 (if (global-alias global)
+						     (global-alias global)
+						     (global-id global))
+						 '-env::procedure)
 			      (global-module global)
 			      node
 			      (if (sfun? (global-value global))
@@ -395,18 +395,20 @@
 				      'sgfun)
 				     (else
 				      'sfun))
-				  'sfun))))
+				  'sfun)
+			      loc)))	     
+	     (global-library-set! closure (global-library global))
 	     (global-import-set! closure (global-import global))
 	     (fun-the-closure-set! (global-value global) closure)
 	     ;; this next setting is require by the Cfa pass
 	     ;; (file Cfa/closure.scm) when light and extra-light
 	     ;; closures are built.
 	     (fun-the-closure-set! (global-value gloclo) closure)
-	     (trace (globalize 2) "J'ai pluge pour la globale: "
+	     (trace (globalize 5) "J'ai pluge pour la globale: "
 		    (shape closure) " l'ast: "
 		    (shape (scnst-node (global-value closure)))
 		    #\Newline)
-	     closure))))
+	     closure)))))
 
 ;*---------------------------------------------------------------------*/
 ;*    the-local-closure ...                                            */
@@ -429,7 +431,7 @@
    (set! *round* (+fx *round* 1))
    ;; we re-mark all bound variables.
    (let ((finfo (local-value integrator)))
-      (trace (globalize 2)
+      (trace (globalize 5)
 	     "   bound(" (shape integrator) ") : "
 	     (shape (sfun/Ginfo-bound finfo))
 	     #\Newline
