@@ -3,8 +3,8 @@
 ;*    -------------------------------------------------------------    */
 ;*    Author      :  Manuel Serrano                                    */
 ;*    Creation    :  Fri Jun 21 09:03:24 1996                          */
-;*    Last change :  Mon May 15 07:50:25 2000 (serrano)                */
-;*    Copyright   :  1996-2000 Manuel Serrano, see LICENSE file        */
+;*    Last change :  Tue Feb  9 07:20:18 2016 (serrano)                */
+;*    Copyright   :  1996-2020 Manuel Serrano, see LICENSE file        */
 ;*    -------------------------------------------------------------    */
 ;*    The definition of the info structures for the globalization      */
 ;*=====================================================================*/
@@ -14,7 +14,8 @@
 ;*---------------------------------------------------------------------*/
 (module globalize_ginfo
    
-   (include "Ast/node.sch")
+   (include "Ast/node.sch"
+	    "Globalize/ginfo.sch")
    
    (export  (wide-class sfun/Ginfo::sfun
 	       ;; is a function globalized
@@ -28,9 +29,13 @@
 	       ;; transitive closure of cto
 	       (cto* (default #f))
 	       ;; the list of E functions
-	       (cfunction (default '()))
+	       (efunctions (default '()))
 	       ;; a place to be integrated
-	       (integrator (default '()))
+	       (integrator (default #unspecified))
+	       ;; a place to be integrated (see globalize_integration)
+	       (imark (default '()))
+	       ;; the function this function belongs to
+	       (owner (default #f))
 	       ;; a list a integrated functions
 	       (integrated (default '()))
 	       ;; where is it plugged
@@ -61,8 +66,10 @@
 	       (free-mark::long (default -10))
 	       ;; a mark
 	       (mark::long (default -10))
-	       ;; celled ?
-	       (celled?::bool (default #f)))
+	       ;; celled?
+	       (celled?::bool (default #f))
+	       ;; if a box is to be created, can it be stack allocated?
+	       (stackable::bool (default #t)))
 
 	    (wide-class sexit/Ginfo::sexit
 	       ;; is a function globalized
@@ -76,7 +83,9 @@
 
 	    (wide-class local/Ginfo::local
 	       ;; is the local function escaping ?
-	       (escape?::bool (default #f)))
+	       (escape?::bool (default #f))
+	       ;; is the local globalized? (only used by globalize_integration)
+	       (globalized?::bool (default #f)))
 	    
 	    (wide-class global/Ginfo::global
 	       ;; is the global function escaping ?
