@@ -1,10 +1,10 @@
 ;*=====================================================================*/
-;*    serrano/prgm/project/bigloo/comptime/Globalize/gn.scm            */
+;*    serrano/prgm/project/bigloo/bigloo/comptime/Globalize/gn.scm     */
 ;*    -------------------------------------------------------------    */
 ;*    Author      :  Manuel Serrano                                    */
 ;*    Creation    :  Thu Jan 26 14:54:22 1995                          */
-;*    Last change :  Thu Jul  5 18:18:08 2001 (serrano)                */
-;*    Copyright   :  1995-2001 Manuel Serrano, see LICENSE file        */
+;*    Last change :  Wed Jun 16 16:02:00 2021 (serrano)                */
+;*    Copyright   :  1995-2021 Manuel Serrano, see LICENSE file        */
 ;*    -------------------------------------------------------------    */
 ;*    We compute the G0 and G1 properties which is defined as follow:  */
 ;*                                                                     */
@@ -94,6 +94,14 @@
 (define-method (E node::sequence caller g)
    (E* (sequence-nodes node) caller g))
 
+;*---------------------------------------------------------------------*/
+;*    E ::sync ...                                                     */
+;*---------------------------------------------------------------------*/
+(define-method (E node::sync caller g)
+   (E (sync-body node) caller
+      (E (sync-prelock node) caller
+	 (E (sync-mutex node) caller g))))
+
 ;*---------------------------------------------------------------------*/
 ;*    E ::app ...                                                      */
 ;*---------------------------------------------------------------------*/
@@ -103,7 +111,7 @@
       (E* args caller g)))
  
 ;*---------------------------------------------------------------------*/
-;*    E ::app-ly ...                                                    */
+;*    E ::app-ly ...                                                   */
 ;*---------------------------------------------------------------------*/
 (define-method (E node::app-ly caller g)
    (with-access::app-ly node (fun arg)
@@ -152,10 +160,10 @@
       (E proc caller (E msg caller (E obj caller g)))))
 
 ;*---------------------------------------------------------------------*/
-;*    E ::select ...                                                   */
+;*    E ::switch ...                                                   */
 ;*---------------------------------------------------------------------*/
-(define-method (E node::select caller g)
-   (with-access::select node (clauses test)
+(define-method (E node::switch caller g)
+   (with-access::switch node (clauses test)
       (let loop ((clauses clauses)
 		 (g       g))
 	 (if (null? clauses)
@@ -192,8 +200,8 @@
 ;*    E ::set-ex-it ...                                                */
 ;*---------------------------------------------------------------------*/
 (define-method (E node::set-ex-it caller g)
-   (with-access::set-ex-it node (body)
-      (E body caller g)))
+   (with-access::set-ex-it node (body onexit)
+      (E body caller (E onexit caller g))))
 
 ;*---------------------------------------------------------------------*/
 ;*    E ::jump-ex-it ...                                               */
@@ -246,12 +254,11 @@
 		 (sfun/Ginfo-cfrom-set! callee-info
 					(cons caller
 					      (sfun/Ginfo-cfrom callee-info)))
-		 (if (local? caller)
-		     (let ((caller-info (local-value caller)))
-			(sfun/Ginfo-cto-set! caller-info
-					     (cons callee
-						   (sfun/Ginfo-cto
-						    caller-info)))))))
+		 (let ((caller-info (variable-value caller)))
+		    (sfun/Ginfo-cto-set! caller-info
+					 (cons callee
+					       (sfun/Ginfo-cto
+						caller-info))))))
 	  'done)))
 	  
 ;*---------------------------------------------------------------------*/
@@ -262,13 +269,11 @@
        'done
        (let ((caller-info (local-value caller)))
 	  (trace (globalize 3) "save-fun!: "
-		 (shape caller) " " (shape callee)
-		 #\Newline)
-	  (if (not (memq callee (sfun/Ginfo-cfunction caller-info)))
-	      (sfun/Ginfo-cfunction-set! caller-info
-					 (cons callee
-					       (sfun/Ginfo-cfunction
-						caller-info))))
+	     (shape caller) " " (shape callee)
+	     #\Newline)
+	  (if (not (memq callee (sfun/Ginfo-efunctions caller-info)))
+	      (sfun/Ginfo-efunctions-set! caller-info
+		 (cons callee (sfun/Ginfo-efunctions caller-info))))
 	  'done)))
 	  
 ;*---------------------------------------------------------------------*/
