@@ -1,10 +1,10 @@
 ;*=====================================================================*/
-;*    serrano/prgm/project/bigloo2.3/comptime/Globalize/node.scm       */
+;*    .../prgm/project/bigloo/bigloo/comptime/Globalize/node.scm       */
 ;*    -------------------------------------------------------------    */
 ;*    Author      :  Manuel Serrano                                    */
 ;*    Creation    :  Fri Jan 27 14:12:58 1995                          */
-;*    Last change :  Thu Jul 13 11:11:28 2000 (serrano)                */
-;*    Copyright   :  1995-2000 Manuel Serrano, see LICENSE file        */
+;*    Last change :  Thu Jul  8 11:29:07 2021 (serrano)                */
+;*    Copyright   :  1995-2021 Manuel Serrano, see LICENSE file        */
 ;*    -------------------------------------------------------------    */
 ;*    We transforme the ast in order to fix the free variables, to     */
 ;*    remove the useless local functions (globalized or integrated     */
@@ -19,6 +19,7 @@
    (import  tools_shape
 	    type_type
 	    type_cache
+	    type_typeof
 	    ast_var
 	    ast_node
 	    globalize_ginfo
@@ -26,7 +27,8 @@
 	    ast_env
 	    ast_local
 	    globalize_free
-	    globalize_local->global)
+	    globalize_local->global
+	    tools_error)
    (export  (node-globalize!::node ::node ::variable <variablexvariable>*)))
 
 ;*---------------------------------------------------------------------*/
@@ -36,19 +38,20 @@
 ;*    returns a free variables list.                                   */
 ;*---------------------------------------------------------------------*/
 (define (node-globalize! node integrator what/by*)
+   (trace (globalize 5) "node-globalize!=" (shape node) #\Newline)
    ;; for each celled variable, we declare a new local
    ;; variable
-   (let* ((fun      (variable-value integrator))
-	  (celled   (celled-bindings (sfun-args fun)))
+   (let* ((fun (variable-value integrator))
+	  (celled (celled-bindings (sfun-args fun)))
 	  (what/by* (append celled what/by*)))
       ;; we set alpha-fast slot 
-      (for-each (lambda (w.b)
-		   (local-fast-alpha-set! (car w.b) (cdr w.b)))
+      (for-each (lambda (w-b)
+		   (local-fast-alpha-set! (car w-b) (cdr w-b)))
 		what/by*)
       (let ((res (cell-formals celled (glo! node integrator))))
 	 ;; we remove alpha-fast slots
-	 (for-each (lambda (w.b)
-		      (local-fast-alpha-set! (car w.b) #unspecified))
+	 (for-each (lambda (w-b)
+		      (local-fast-alpha-set! (car w-b) #unspecified))
 		   what/by*)
 	 res)))
 
@@ -64,15 +67,22 @@
 	 ((not (celled? (car formals)))
 	  (loop celled (cdr formals)))
 	 (else
-	  (let* ((var (make-local-svar (local-id (car formals)) *obj*))
-		 (o.n (cons (car formals) var)))
-	     (local-access-set! var 'cell-globalize)
-	     (local-user?-set! var (local-user? (car formals)))
-	     (widen!::svar/Ginfo (local-value var)
-		(celled? #t)
-		(kaptured? #t))
-	     (loop (cons o.n celled) (cdr formals)))))))
+	  (let* ((ovar (car formals))
+		 (nvar (cell-variable ovar)))
+	     (loop (cons (cons ovar nvar) celled) (cdr formals)))))))
 
+;*---------------------------------------------------------------------*/
+;*    cell-variable ...                                                */
+;*---------------------------------------------------------------------*/
+(define (cell-variable::local local::local)
+   (let ((var (make-local-svar (local-id local) *cell*)))
+      (local-access-set! var 'cell-globalize)
+      (local-user?-set! var (local-user? local))
+      (widen!::svar/Ginfo (local-value var)
+	 (celled? #t)
+	 (kaptured? #t))
+      var))
+   
 ;*---------------------------------------------------------------------*/
 ;*    cell-formals ...                                                 */
 ;*---------------------------------------------------------------------*/
@@ -83,15 +93,16 @@
 	  (instantiate::let-var
 	     (loc loc)
 	     (body body)
-	     (type *_*)
-	     (bindings (map (lambda (o.n)
-			       (cons (cdr o.n)
-				     (a-make-cell (instantiate::var
-						     (type *_*)
-						     (loc loc)
-						     (variable (car o.n)))
-						  (car o.n))))
-			    celled))))))
+	     (type (node-type body))
+	     (bindings (map (lambda (o-n)
+			       (cons (cdr o-n)
+				  (a-make-cell
+				     (instantiate::ref
+					(type (variable-type (car o-n)))
+					(loc loc)
+					(variable (car o-n)))
+				     (car o-n))))
+			  celled))))))
 
 ;*---------------------------------------------------------------------*/
 ;*    a-make-cell ...                                                  */
@@ -100,7 +111,12 @@
    (with-access::node node (loc)
       (local-access-set! variable 'cell-globalize)
       (svar/Ginfo-celled?-set! (variable-value variable) #t)
-      (instantiate::make-box (type *_*) (loc loc) (value node))))
+      (instantiate::make-box
+	 (type *cell*)
+	 (vtype (get-bigloo-defined-type (variable-type variable)))
+	 (loc loc)
+	 (value node)
+	 (stackable (svar/Ginfo-stackable (variable-value variable))))))
    
 ;*---------------------------------------------------------------------*/
 ;*    celled? ...                                                      */
@@ -133,29 +149,39 @@
 ;*    glo! ...                                                         */
 ;*---------------------------------------------------------------------*/
 (define-method (glo! node::var integrator)
-   (let* ((var   (var-variable node))
-	  (alpha (variable-fast-alpha var)))
-      (cond
-	 ((local? alpha)
-	  (var-variable-set! node alpha)
-	  (glo! node integrator))
-	 ((global? var)
-	  node)
-	 ((celled? var)
-	  (local-access-set! var 'cell-globalize)
-	  (instantiate::box-ref
-	     (loc (node-loc node))
-	     (type (node-type node))
-	     (var node)))
-	 (else
-	  node))))
+   (let* ((variable (var-variable node))
+	  (vtype (variable-type variable)))
+      (let loop ((variable variable))
+	 (let ((alpha (variable-fast-alpha variable)))
+	    (cond
+	       ((local? alpha)
+		(var-variable-set! node alpha)
+		(node-type-set! node (get-bigloo-defined-type vtype))
+		(loop alpha))
+	       ((global? variable)
+		node)
+	       ((celled? variable)
+		;; (local-access-set! variable 'cell-globalize)
+		(let ((vtype (get-bigloo-defined-type vtype))
+		      (ntype (get-bigloo-defined-type (node-type node))))
+		   ;; (node-type-set! node (get-bigloo-defined-type vtype))
+		   (node-type-set! node *cell*)
+		   (instantiate::box-ref
+		      (loc (node-loc node))
+		      ;; (vtype (get-bigloo-defined-type vtype))
+		      ;; (type (get-bigloo-defined-type (node-type node)))
+		      (vtype vtype)
+		      (type ntype)
+		      (var node))))
+	       (else
+		node))))))
 
 ;*---------------------------------------------------------------------*/
 ;*    glo! ::closure ...                                               */
 ;*---------------------------------------------------------------------*/
 (define-method (glo! node::closure integrator)
    (with-access::closure node (loc variable)
-      (glo! (instantiate::var
+      (glo! (instantiate::ref
 	       (loc loc)
 	       (type *procedure*)
 	       (variable (the-closure variable loc)))
@@ -169,6 +195,16 @@
       (glo*! nodes integrator)
       node))
 
+;*---------------------------------------------------------------------*/
+;*    glo! ::sync ...                                                  */
+;*---------------------------------------------------------------------*/
+(define-method (glo! node::sync integrator)
+   (with-access::sync node (body mutex prelock)
+      (set! mutex (glo! mutex integrator))
+      (set! prelock (glo! prelock integrator))
+      (set! body (glo! body integrator))
+      node))
+
 ;*---------------------------------------------------------------------*/
 ;*    glo! ::app ...                                                   */
 ;*---------------------------------------------------------------------*/
@@ -189,7 +225,7 @@
 		      (not (local/Ginfo-escape? fun)))
 		  (sfun/Ginfo-G? info))
 	     (app-fun-set! node
-			   (instantiate::var
+			   (instantiate::ref
 			      (loc loc)
 			      (type type)
 			      (variable (the-global fun)))))
@@ -210,9 +246,9 @@
 	    ((local/Ginfo-escape? fun)
 	     ;; this is a direct call to an escaping call,
 	     ;; we add its environement if it is a local function ...
-	     (set! args (cons (glo! (instantiate::var
+	     (set! args (cons (glo! (instantiate::ref
 				       (loc loc)
-				       (type *_*)
+				       (type (variable-type (the-closure fun loc)))
 				       (variable (the-closure fun loc)))
 				    integrator)
 			      args)))
@@ -226,9 +262,9 @@
 		    (let* ((kap   (car kaptured))
 			   (alpha (local-fast-alpha kap))
 			   (var   (if (local? alpha) alpha kap)))
-		       (loop (cons (instantiate::var
+		       (loop (cons (instantiate::ref
 				      (loc loc)
-				      (type *_*)
+				      (type (variable-type var))
 				      (variable var))
 				   new-actuals)
 			     (cdr kaptured)))))))
@@ -274,7 +310,8 @@
 (define-method (glo! node::setq integrator)
    (with-access::setq node (value)
       (set! value (glo! value integrator))
-      (let ((var (var-variable (setq-var node))))
+      (let* ((var (var-variable (setq-var node)))
+	     (vtype (variable-type var)))
 	 (let loop ((var   var)
 		    (alpha (variable-fast-alpha var)))
 	    (if (local? alpha)
@@ -287,19 +324,21 @@
 			     (loc   (node-loc node)))
 			  (local-access-set! var 'cell-globalize)
 			  (local-user?-set! a-var (local-user? var))
+			  (node-type-set! (setq-var node) *obj*)
 			  (widen!::svar/Ginfo (local-value a-var)
 			     (kaptured? #f))
 			  (instantiate::let-var
 			     (loc loc)
-			     (type *_*)
+			     (type *unspec*)
 			     (bindings (list (cons a-var value)))
 			     (body (instantiate::box-set!
 				      (loc loc)
-				      (type *_*)
+				      (type *unspec*)
+				      (vtype (get-bigloo-defined-type vtype))
 				      (var (setq-var node))
-				      (value (instantiate::var
+				      (value (instantiate::ref
 						(loc loc)
-						(type *_*)
+						(type (variable-type a-var))
 						(variable a-var)))))))
 		       node)))))))
 
@@ -324,10 +363,10 @@
       node))
 
 ;*---------------------------------------------------------------------*/
-;*    glo! ::select ...                                                */
+;*    glo! ::switch ...                                                */
 ;*---------------------------------------------------------------------*/
-(define-method (glo! node::select integrator)
-   (with-access::select node (clauses test)
+(define-method (glo! node::switch integrator)
+   (with-access::switch node (clauses test)
       (set! test (glo! test integrator))
       (for-each (lambda (clause)
 		   (set-cdr! clause (glo! (cdr clause) integrator)))
@@ -345,14 +384,15 @@
 		 (ebindings '()))
 	 (if (not (null? obindings))
 	     (trace (globalize 4)
-		    "glo!(let-fun): " (shape (car obindings)) ": "
-		    "  integrator: " (shape integrator)
-		    "  escape: " (local/Ginfo-escape? (car obindings))
+		    "glo!(let-fun): " (shape (car obindings))
+		    ", integrator=" (shape integrator)
+		    ", escape=" (local/Ginfo-escape? (car obindings))
+		    ", eq=" (eq? integrator (car obindings))
 		    #\Newline))
 	 (cond
 	    ((null? obindings)
 	     (set! locals nbindings)
-	     (if (null? ebindings)
+	     (if (null? ebindings) 
 		 node
 		 (make-escaping-bindings ebindings node integrator)))
 	    ((and (not (eq? (car obindings) integrator))
@@ -360,6 +400,14 @@
 	     (liip (cdr obindings)
 		   nbindings
 		   (cons (car obindings) ebindings)))
+	    ((eq? (car obindings) integrator)
+	     ;; MS: 3 oct 2006
+	     (internal-error 'glo!
+			     "Shoud not be here (because of the integration"
+			     (shape (car obindings)))
+	     (liip (cdr obindings)
+		   nbindings
+		   ebindings))
 	    (else
 	     (let ((local (car obindings)))
 		(globalize-local-fun! local integrator)
@@ -376,28 +424,36 @@
 		   (let ((var (car binding))
 			 (val (cdr binding)))
 		      (set-cdr! binding (glo! val integrator))
-		      (if (celled? var)
-			  (begin
-			     (local-type-set! var *obj*)
-			     (set-cdr! binding (a-make-cell (cdr binding)
-							    var))))))
-		bindings)
+		      (when (celled? var)
+			 (let ((nvar (cell-variable var)))
+			    (local-fast-alpha-set! var nvar)
+			    (set-cdr! binding
+			       (a-make-cell (cdr binding) var))))))
+	 bindings)
       (set! body (glo! body integrator))
+      (for-each (lambda (binding)
+		   (let ((var (car binding)))
+		      (when (celled? var)
+			 (let ((nvar (cell-variable var)))
+			    (set-car! binding (variable-fast-alpha var))
+			    (local-fast-alpha-set! var #unspecified)))))
+	 bindings)
       node))
 
 ;*---------------------------------------------------------------------*/
 ;*    glo! ::set-ex-it ...                                             */
 ;*---------------------------------------------------------------------*/
 (define-method (glo! node::set-ex-it integrator)
-   (with-access::set-ex-it node (var body)
+   (with-access::set-ex-it node (var body onexit)
       (let ((hdlg (sexit-handler (local-value (var-variable var)))))
 	 (trace (globalize 2)
 		"# glo!   :" (shape node) #\Newline
 		"# handler: " (shape hdlg)
 		#\Newline)
-	 (if (sfun/Ginfo-G? (local-value hdlg))
-	     (sexit-detached?-set! (local-value (var-variable var)) #t))
+	 (when (sfun/Ginfo-G? (local-value hdlg))
+	    (sexit-detached?-set! (local-value (var-variable var)) #t))
 	 (set! body (glo! body integrator))
+	 (set! onexit (glo! onexit integrator))
 	 node)))
 
 ;*---------------------------------------------------------------------*/
@@ -450,23 +506,21 @@
 ;*---------------------------------------------------------------------*/
 (define (make-escaping-bindings ebindings node::node integrator::variable)
    (trace (globalize 2)
-	  "make-escaping-bindings: " (shape node)
-	  #\Newline
-	  "             ebindings: " (shape ebindings)
-	  #\Newline)
+	  "make-escaping-bindings: " (shape node) #\Newline
+	  "             ebindings: " (shape ebindings) #\Newline)
    (let loop ((ebindings ebindings)
 	      (bindings  '())
 	      (sets      '()))
       (if (null? ebindings)
 	  (instantiate::let-var
 	     (loc (node-loc node))
-	     (type *_*)
+	     (type (node-type node))
 	     (bindings bindings)
 	     (body (if (null? sets)
 		       node
 		       (instantiate::sequence
 			  (loc (node-loc node))
-			  (type *_*)
+			  (type (node-type node))
 			  (nodes (append sets (list node)))))))
 	  (let* ((local (car ebindings))
 		 (new   (the-closure local (node-loc node)))
@@ -485,18 +539,20 @@
 ;*    globalize-local-fun! ...                                         */
 ;*---------------------------------------------------------------------*/
 (define (globalize-local-fun! local::local integrator::variable)
+   (trace (globalize 4) "globalize-local-fun: local=" (shape local)
+	  " integrator=" (shape integrator) #\Newline)
    (let* ((fun   (local-value local))
 	  (obody (sfun-body fun)))
       (if (eq? local integrator)
 	  (sfun-body-set! fun (glo! obody integrator))
 	  (let ((celled (celled-bindings (sfun-args fun))))
-	     (for-each (lambda (w.b)
-			  (local-fast-alpha-set! (car w.b) (cdr w.b)))
+	     (for-each (lambda (w-b)
+			  (local-fast-alpha-set! (car w-b) (cdr w-b)))
 		       celled)
 	     (let* ((nbody1 (glo! obody integrator))
 		    (nbody2 (cell-formals celled nbody1)))
-		(for-each (lambda (w.b)
-			     (local-fast-alpha-set! (car w.b) #unspecified))
+		(for-each (lambda (w-b)
+			     (local-fast-alpha-set! (car w-b) #unspecified))
 			  celled)
 		(sfun-body-set! fun nbody2))))))
 
@@ -504,32 +560,35 @@
 ;*    a-make-procedure ...                                             */
 ;*---------------------------------------------------------------------*/
 (define (a-make-procedure local::local)
-   (let* ((fun      (local-value local))
-	  (arity    (sfun-arity fun))
+   (let* ((fun (local-value local))
+	  (arity (sfun-arity fun))
 	  (kaptured (sfun/Ginfo-kaptured (local-value local)))
-;* 	  (loc      (sfun-loc fun))                                    */
-	  (loc      #unspecified)
-;*  ici, je ne sais pas ce qu'il faut mettre....                       */
-	  (make-p   (if (>=fx arity 0) 'make-fx-procedure 'make-va-procedure)))
+	  (loc #unspecified)
+	  (make-p (if (<fx arity 0) 'make-va-procedure 'make-fx-procedure))
+	  (v (find-global/module make-p 'foreign)))
       (instantiate::app
 	 (loc loc)
-	 (type *_*)
-	 (fun (instantiate::var
+	 (type (variable-type v))
+	 (stackable (when (eq? (sfun-stackable fun) #t)
+		       (let ((gv (global-value v)))
+			  (when (fun? gv)
+			     (fun-stack-allocator gv)))))
+	 (fun (instantiate::ref
 		 (loc loc)
-		 (type *_*)
-		 (variable (find-global/module make-p 'foreign))))
-	 (args (list (instantiate::var
+		 (type (variable-type v))
+		 (variable v)))
+	 (args (list (instantiate::ref
 			(loc loc)
-			(type *_*)
+			(type (variable-type (the-global local)))
 			(variable (the-global local)))
-		     (instantiate::atom
-			(loc loc)
-			(type *_*)
-			(value arity))
-		     (instantiate::atom
-			(loc loc)
-			(type *_*)
-			(value (length kaptured))))))))
+		  (instantiate::literal
+		     (loc loc)
+		     (type (get-type-atom arity))
+		     (value arity))
+		  (instantiate::literal
+		     (loc loc)
+		     (type (get-type-atom 1))
+		     (value (length kaptured))))))))
 
 ;*---------------------------------------------------------------------*/
 ;*    make-sets ...                                                    */
@@ -537,22 +596,22 @@
 (define (make-sets new loc kaptured integrator)
    (if (null? kaptured)
        '()
-       (instantiate::sequence
-	  (loc loc)
-	  (type *_*)
-	  (nodes (let loop ((kaptured kaptured)
-			    (indice   0)
-			    (sets     '()))
-		    (if (null? kaptured)
-			(reverse! sets)
-			(loop (cdr kaptured)
-			      (+fx indice 1)
-			      (cons (a-procedure-set! loc
-						      new
-						      indice
-						      (car kaptured)
-						      integrator)
-				    sets))))))))
+       (let loop ((kaptured kaptured)
+		  (indice   0)
+		  (sets     '()))
+	  (if (null? kaptured)
+	      (instantiate::sequence
+		 (loc loc)
+		 (type (node-type (car sets)))
+		 (nodes (reverse! sets)))
+	      (loop (cdr kaptured)
+		    (+fx indice 1)
+		    (cons (a-procedure-set! loc
+					    new
+					    indice
+					    (car kaptured)
+					    integrator)
+			  sets))))))
 
 ;*---------------------------------------------------------------------*/
 ;*    a-procedure-set! ...                                             */
@@ -563,25 +622,28 @@
 	 (if (local? alpha)
 	     (alpha-convert alpha)
 	     var)))
-   (instantiate::app
-      (loc loc)
-      (type *_*)
-      (fun (instantiate::var
-	      (loc loc)
-	      (type *_*)
-	      (variable (find-global/module 'procedure-set! 'foreign))))
-      (args (list (instantiate::var
-		     (loc loc)
-		     (type *_*)
-		     (variable new))
-		  (instantiate::atom
-		     (loc loc)
-		     (type *_*)
-		     (value indice))
-		  (instantiate::var
-		     (loc loc)
-		     (type *_*)
-		     (variable (alpha-convert kaptured)))))))
+   (let ((vf (find-global/module 'procedure-set! 'foreign))
+	 (va (alpha-convert kaptured)))
+      (instantiate::app
+	 (loc loc)
+	 (type (variable-type vf))
+	 (fun (instantiate::ref
+		 (loc loc)
+		 (type (variable-type vf))
+		 (variable vf)))
+	 (args (list (instantiate::ref
+			(loc loc)
+			(type (variable-type new))
+			(variable new))
+		     (instantiate::literal
+			(loc loc)
+			(type (get-type-atom indice))
+			(value indice))
+		     (let ()
+			(instantiate::ref
+			   (loc loc)
+			   (type (variable-type va))
+			   (variable va))))))))
 						       
 							     
 			    
