@@ -1,10 +1,10 @@
 ;*=====================================================================*/
-;*    serrano/prgm/project/bigloo/comptime/Globalize/walk.scm          */
+;*    .../prgm/project/bigloo/bigloo/comptime/Globalize/walk.scm       */
 ;*    -------------------------------------------------------------    */
 ;*    Author      :  Manuel Serrano                                    */
 ;*    Creation    :  Thu Jan 26 14:25:07 1995                          */
-;*    Last change :  Mon May 15 07:51:36 2000 (serrano)                */
-;*    Copyright   :  1995-2000 Manuel Serrano, see LICENSE file        */
+;*    Last change :  Wed Jan  8 13:49:53 2020 (serrano)                */
+;*    Copyright   :  1995-2020 Manuel Serrano, see LICENSE file        */
 ;*    -------------------------------------------------------------    */
 ;*    The `globalization' stage                                        */
 ;*=====================================================================*/
@@ -14,8 +14,10 @@
 ;*---------------------------------------------------------------------*/
 (module globalize_walk
    (include "Engine/pass.sch"
-	    "Ast/node.sch")
+	    "Ast/node.sch"
+	    "Tools/trace.sch")
    (import  tools_error
+	    tools_shape
 	    globalize_escape
 	    globalize_globalize
 	    (foreign-closures globalize_global-closure)
@@ -26,17 +28,19 @@
 ;*    globalize-walk! ...                                              */
 ;*---------------------------------------------------------------------*/
 (define (globalize-walk! globals remove)
-   (pass-prelude "Globalization")
-   ;; we compute the escape! property for all function (local and global)
+   (pass-prelude "Closure (globalize)")
+   ;; compute the escape! property for all functions (locals and globals)
    (for-each escape-fun! globals)
    ;; we perform the globalization
-   (let loop ((globals     globals)
+   (let loop ((globals globals)
 	      (new-globals '()))
+      (trace (globalize 3) "globalize-walk!, new-globals="
+	 (map shape new-globals) #\Newline)
       (if (null? globals)
-	  (pass-postlude (remove-var remove
-				     (append new-globals (foreign-closures))))
+	  (pass-postlude
+	     (remove-var remove (append new-globals (foreign-closures))))
 	  (loop (cdr globals)
-		(append (globalize! (car globals)) new-globals)))))
+	     (append (globalize! (car globals)) new-globals)))))
 
 
 
