@@ -1,12 +1,14 @@
 ;*=====================================================================*/
-;*    serrano/prgm/project/bigloo/comptime/Integrate/ctn.scm           */
+;*    .../prgm/project/bigloo/bigloo/comptime/Integrate/ctn.scm        */
 ;*    -------------------------------------------------------------    */
 ;*    Author      :  Manuel Serrano                                    */
 ;*    Creation    :  Wed Mar 15 14:10:09 1995                          */
-;*    Last change :  Mon May 15 07:54:16 2000 (serrano)                */
-;*    Copyright   :  1995-2000 Manuel Serrano, see LICENSE file        */
+;*    Last change :  Wed Oct 13 11:37:31 2021 (serrano)                */
+;*    Copyright   :  1995-2021 Manuel Serrano, see LICENSE file        */
 ;*    -------------------------------------------------------------    */
 ;*    The computation of `Cn' and `Ct'.                                */
+;*       Cn(f,g) === exists k, A(f,g,k)                                */
+;*       Ct(f,g) === A(f,g,t)                                          */
 ;*=====================================================================*/
 
 ;*---------------------------------------------------------------------*/
@@ -25,35 +27,37 @@
 ;*---------------------------------------------------------------------*/
 ;*    Cn&Ct! ...                                                       */
 ;*    -------------------------------------------------------------    */
-;*    This function returns the list of the globalized functions due   */
+;*    This function returns the list of globalized functions due       */
 ;*    to the Cn property.                                              */
 ;*---------------------------------------------------------------------*/
 (define (Cn&Ct! A)
-   (let loop ((As    A)
+   (let loop ((As A)
 	      (G/cn '()))
       (if (null? As)
 	  (begin
 	     (trace-ctn)
 	     G/cn)
-	  (let* ((A  (car As))
-		 (f  (car A))
-		 (g  (cadr A))
-		 (k  (caddr A))
+	  (let* ((A (car As))
+		 (f (car A))
+		 (g (cadr A))
+		 (k (caddr A))
 		 (fi (variable-value f))
 		 (gi (variable-value g)))
 	     (cond
 		((global? g)
 		 (loop (cdr As) G/cn))
+		((sfun/Iinfo-forceG? gi)
+		 (if (not (sfun/Iinfo-G? gi))
+		     (loop (cdr As) (append (globalize! g) G/cn))
+		     (loop (cdr As) G/cn)))
 		((eq? k 'tail)
 		 (sfun/Iinfo-Ct-set! fi (cons g (sfun/Iinfo-Ct fi)))
-		 (if (and (not (eq? f g))
-			  (not (memq g (sfun/Iinfo-kont fi))))
-		     (sfun/Iinfo-kont-set! fi (cons g (sfun/Iinfo-kont fi))))
+		 (when (and (not (eq? f g))
+			    (not (memq g (sfun/Iinfo-kont fi))))
+		    (sfun/Iinfo-kont-set! fi (cons g (sfun/Iinfo-kont fi))))
 		 (loop (cdr As) G/cn))
 		((eq? k 'escape)
-		 (if (not (memq g (sfun/Iinfo-kont fi)))
-		     (sfun/Iinfo-kont-set! fi (cons g (sfun/Iinfo-kont fi))))
-		 (loop (cdr As) G/cn))
+		 (error "Cn&Ct!" "Should not be here" A))
 		((sfun/Iinfo-U gi)
 		 (sfun/Iinfo-Ct-set! fi (cons g (sfun/Iinfo-Ct fi)))
 		 (if (not (memq g (sfun/Iinfo-kont fi)))
@@ -62,11 +66,25 @@
 		(else
 		 (sfun/Iinfo-Cn-set! fi (cons g (sfun/Iinfo-Cn fi)))
 		 (if (not (sfun/Iinfo-G? gi))
-		     (begin
-			(sfun/Iinfo-G?-set! gi #t)
-			(loop (cdr As) (cons g G/cn)))
+		     (loop (cdr As) (append (globalize! g) G/cn))
 		     (loop (cdr As) G/cn))))))))
 
+;*---------------------------------------------------------------------*/
+;*    globalize! ...                                                   */
+;*---------------------------------------------------------------------*/
+(define (globalize!::pair l::local)
+   (let ((i (local-value l)))
+      (sfun/Iinfo-G?-set! i #t)
+      (let ((xg (append-map (lambda (x)
+			       (let ((xi (local-value x)))
+				  (if (sfun/Iinfo-G? xi)
+				      '()
+				      (begin
+					 (sfun/Iinfo-G?-set! xi #t)
+					 (list x)))))
+		   (sfun/Iinfo-xhdls i))))
+	 (cons l xg))))
+
 ;*---------------------------------------------------------------------*/
 ;*    trace-ctn ...                                                    */
 ;*---------------------------------------------------------------------*/
@@ -78,7 +96,7 @@
 	     (for-each (lambda (p)
 			  (let ((ifun (variable-value p)))
 			     (fprint *trace-port*
-				     " --> " (shape p) #\: #\Newline
+				     " " (shape p) #\: #\Newline
 				     "   Cn        : "
 				     (shape (sfun/Iinfo-Cn ifun))
 				     #\Newline
