@@ -1,10 +1,10 @@
 ;*=====================================================================*/
-;*    serrano/prgm/project/bigloo/comptime/Integrate/iinfo.scm         */
+;*    .../prgm/project/bigloo/bigloo/comptime/Integrate/iinfo.scm      */
 ;*    -------------------------------------------------------------    */
 ;*    Author      :  Manuel Serrano                                    */
 ;*    Creation    :  Fri Jun 28 14:06:46 1996                          */
-;*    Last change :  Mon May 15 07:54:54 2000 (serrano)                */
-;*    Copyright   :  1996-2000 Manuel Serrano, see LICENSE file        */
+;*    Last change :  Wed Oct 13 11:28:44 2021 (serrano)                */
+;*    Copyright   :  1996-2021 Manuel Serrano, see LICENSE file        */
 ;*    -------------------------------------------------------------    */
 ;*    The extension for the integrate pass.                            */
 ;*=====================================================================*/
@@ -17,6 +17,8 @@
    (import  type_type
 	    ast_var
 	    ast_node)
+
+   (include "Integrate/iinfo.sch")
    
    (export (wide-class svar/Iinfo::svar
 	      ;; a stamp to compute free variables
@@ -26,7 +28,9 @@
 	      ;; is a local kaptured ?
 	      (kaptured?::bool (default #f))
 	      ;; celled?
-	      (celled?::bool (default #f)))
+	      (celled?::bool (default #f))
+	      ;; set-exit handler
+	      (xhdl::obj (default #f)))
 
 	   (wide-class sexit/Iinfo::sexit
 	      ;; a stamp to compute free variables
@@ -48,7 +52,7 @@
 	      ;; the list of variables which calls the fun
 	      (cfrom (default '()))
 	      ;; the list of variables which are called.
-	      (cto (default '()))
+	      (cto (default #unspecified))
 	      ;; the K property
 	      (K (default '()))
 	      ;; the K* property
@@ -61,8 +65,10 @@
 	      (Ct (default '()))
 	      ;; the variable from which self is the kont.
 	      (kont (default '()))
-	      ;; is this function globalized ?
+	      ;; is this function globalized?
 	      G?::bool
+	      ;; is this function forced to be globalized?
+	      (forceG?::bool (default #f))
 	      ;; the integrator (if not G?)
 	      (L (default #unspecified))
 	      ;; the integrated functions (if G?).
@@ -72,6 +78,12 @@
 	      ;; a pointer to the globalized function.
 	      (global (default #unspecified))
 	      ;; the list of the kaptured variables
-	      (kaptured (default #unspecified)))))
+	      (kaptured (default #unspecified))
+	      ;; tail-coercion
+	      (tail-coercion::obj (default #unspecified))
+	      ;; is it an exit handler
+	      (xhdl?::bool (default #f))
+	      ;; the list of exit handlers the function calls
+	      (xhdls::pair-nil (default '())))))
 	      
 	   
