@@ -1,10 +1,10 @@
 ;*=====================================================================*/
-;*    serrano/prgm/project/bigloo2.3/comptime/Module/checksum.scm      */
+;*    .../prgm/project/bigloo/bigloo/comptime/Module/checksum.scm      */
 ;*    -------------------------------------------------------------    */
 ;*    Author      :  SERRANO Manuel                                    */
 ;*    Creation    :  Thu Aug 21 08:38:45 1997                          */
-;*    Last change :  Wed Sep  6 07:47:56 2000 (serrano)                */
-;*    Copyright   :  1997-2000 Manuel Serrano, see LICENSE file        */
+;*    Last change :  Wed Apr 28 09:37:22 2021 (serrano)                */
+;*    Copyright   :  1997-2021 Manuel Serrano, see LICENSE file        */
 ;*    -------------------------------------------------------------    */
 ;*    We compute checksum for modules in order to be able to check,    */
 ;*    at module initialization time, that modules are coherent. Only   */
@@ -16,63 +16,62 @@
 ;*    The module                                                       */
 ;*---------------------------------------------------------------------*/
 (module module_checksum
-   (import engine_param)
-   (export (module-checksum::long mclause::pair)))
+   (export (module-checksum::long mclause::pair ::pair-nil)))
 
 ;*---------------------------------------------------------------------*/
 ;*    module-checksum ...                                              */
 ;*---------------------------------------------------------------------*/
-(define (module-checksum module)
-   (clause-checksum (symbol->number (cadr module)) (cddr module)))
+(define (module-checksum module include-path)
+   (clause-checksum (symbol->number (cadr module)) (cddr module) include-path))
 
 ;*---------------------------------------------------------------------*/
 ;*    include-checksum ...                                             */
 ;*---------------------------------------------------------------------*/
-(define (include-checksum include checksum)
-   (define directives?
-      (match-lambda
+(define (include-checksum include checksum include-path)
+   (define (directives? exp)
+      (match-case exp
 	 ((directives . ?-)
 	  #t)
 	 (else
 	  #f)))
-   (if (file-exists? include)
-       (let* ((port (open-input-file include))
-	      (dir  (unwind-protect (let ((exp (read port)))
-				       (if (directives? exp)
-					   exp
-					   #f))
-				    (close-input-port port))))
-	  (if dir
-	      (clause-checksum (atom->number checksum include) (cdr dir))
-	      checksum))
-       checksum))
+   (let ((fi (find-file/path include *load-path*)))
+      (if (and (string? fi) (file-exists? fi))
+	  (let* ((port (open-input-file fi))
+		 (dir  (unwind-protect (let ((exp (read port)))
+					  (if (directives? exp)
+					      exp
+					      #f))
+				       (close-input-port port))))
+	     (if dir
+		 (clause-checksum (atom->number checksum include)
+				  (cdr dir)
+				  include-path)
+		 checksum))
+	  (error 'include-checksum "Can't find include file" include))))
 	      
 ;*---------------------------------------------------------------------*/
 ;*    clause-checksum ...                                              */
 ;*---------------------------------------------------------------------*/
-(define (clause-checksum checksum clauses)
+(define (clause-checksum checksum clauses include-path)
    (let loop ((clauses  clauses)
 	      (checksum checksum))
       (match-case clauses
 	 (()
 	  checksum)
 	 (((export . ?export) . ?rest)
-          (loop rest
-                (list->number checksum export)))
+          (loop rest (list->number checksum export)))
 	 ((((or foreign extern java) . ?export) . ?rest)
-	  (loop rest
-		(extern-clause-checksum checksum export)))
+	  (loop rest (extern-clause-checksum checksum export include-path)))
 	 (((include . ?files) . ?rest)
 	  (let laap ((files    files)
 		     (checksum checksum))
 	     (if (null? files)
-		 (loop rest
-		       checksum)
-		 (laap (cdr files)
-		       (include-checksum (car files) checksum)))))
+		 (loop rest checksum)
+		 (laap (cdr files) (include-checksum (car files)
+						     checksum
+						     include-path)))))
 	 (else
-	  (loop (cdr clauses)
-		checksum)))))
+	  (loop (cdr clauses) checksum)))))
 
 ;*---------------------------------------------------------------------*/
 ;*    symbol->number ...                                               */
@@ -82,7 +81,7 @@
       (let ((len (-fx (string-length str) 1)))
 	 (let loop ((i 0)
 		    (armed? #f))
-
+	    
 	    (cond
 	       ((=fx i len)
 		str)
@@ -92,8 +91,7 @@
 		    (loop (+fx i 1) #t)))
 	       (else
 		(loop (+fx i 1) #f))))))
-   (string->0..2^x-1 (type-component (symbol->string symbol)) 16))
-
+   (get-hashnumber (type-component (symbol->string symbol))))
 
 ;*---------------------------------------------------------------------*/
 ;*    keyword->number ...                                              */
@@ -107,21 +105,47 @@
 (define (atom->number checksum clause)
    (cond
       ((fixnum? clause)
-       (bit-xor (+fx 5 checksum) clause))
+       (bit-xor checksum clause))
       ((flonum? clause)
        (atom->number checksum (real->string clause)))
       ((char? clause)
-       (bit-xor (+fx 23 checksum) (char->integer clause)))
+       (bit-xor checksum (+fx 23 (char->integer clause))))
       ((cnst? clause)
-       (bit-xor (+fx 90 checksum) (cnst->integer clause)))
+       (bit-xor checksum (+fx 90 (cnst->integer clause))))
       ((string? clause)
-       (bit-xor (+fx 4 checksum) (string->0..2^x-1 clause 16)))
+       (bit-xor checksum (+fx 4 (get-hashnumber clause))))
       ((symbol? clause)
-       (bit-xor (+fx 150 checksum) (symbol->number clause)))
+       (bit-xor checksum (+fx 150 (symbol->number clause))))
       ((keyword? clause)
-       (bit-xor (+fx 150 checksum) (keyword->number clause)))
+       (bit-xor checksum (+fx 151 (keyword->number clause))))
       ((pair? clause)
        (list->number checksum clause))
+      ((vector? clause)
+       (vector->number checksum clause))
+      ((elong? clause)
+       (bit-xor checksum (elong->fixnum clause)))
+      ((llong? clause)
+       (bit-xor checksum
+		(bit-xorllong (bit-rshllong clause 32)
+			      (bit-rshllong (bit-lshllong clause 32) 32))))
+      ((int8? clause)
+       (bit-xor checksum (int8->fixnum clause)))
+      ((uint8? clause)
+       (bit-xor checksum (uint8->fixnum clause)))
+      ((int16? clause)
+       (bit-xor checksum (int16->fixnum clause)))
+      ((uint16? clause)
+       (bit-xor checksum (uint16->fixnum clause)))
+      ((int32? clause)
+       (bit-xor checksum (int32->fixnum clause)))
+      ((uint32? clause)
+       (bit-xor checksum (uint32->fixnum clause)))
+      ((int64? clause)
+       (bit-xor checksum (int64->fixnum clause)))
+      ((uint64? clause)
+       (bit-xor checksum (uint64->fixnum clause)))
+      ((int32? clause)
+       (bit-xor checksum (int32->fixnum clause)))
       (else
        (warning "module checksum:Unknown clause" clause " -- "
 		(find-runtime-type clause))
@@ -138,14 +162,19 @@
        0)
       (else
        (if (pair? clause)
-	   (list->number (atom->number checksum (car clause))
-			 (cdr clause))
+	   (list->number (atom->number checksum (car clause)) (cdr clause))
 	   (atom->number checksum clause)))))
 	 
+;*---------------------------------------------------------------------*/
+;*    vector->number ...                                               */
+;*---------------------------------------------------------------------*/
+(define (vector->number checksum clause)
+   (list->number (bit-xor 73854 checksum) (vector->list clause)))
+	 
 ;*---------------------------------------------------------------------*/
 ;*    extern-clause-checksum ...                                       */
 ;*---------------------------------------------------------------------*/
-(define (extern-clause-checksum checksum clauses)
+(define (extern-clause-checksum checksum clauses include-path)
    (let loop ((clauses  clauses)
 	      (checksum checksum))
       (match-case clauses
@@ -154,9 +183,11 @@
 	 ;; checksum value needs to change if timestamp of C include files
 	 ;; changes
 	 (((include ?file) . ?rest)
-	  (let* ((fullname (find-file/path file *mco-include-path*))
+	  (let* ((fullname (find-file/path file include-path))
 		 (time (if (and (string? fullname) (file-exists? fullname))
-			   (file-modification-time fullname)
+			   (flonum->fixnum
+			    (elong->flonum
+			     (file-modification-time fullname)))
 			   0)))
 	     (loop (cdr clauses)
 		   (atom->number checksum time))))
