@@ -3,8 +3,8 @@
 ;*    -------------------------------------------------------------    */
 ;*    Author      :  Manuel Serrano                                    */
 ;*    Creation    :  Tue Jun  4 16:28:03 1996                          */
-;*    Last change :  Sun Jun 10 14:06:18 2001 (serrano)                */
-;*    Copyright   :  1996-2001 Manuel Serrano, see LICENSE file        */
+;*    Last change :  Thu Apr 11 13:55:49 2013 (serrano)                */
+;*    Copyright   :  1996-2013 Manuel Serrano, see LICENSE file        */
 ;*    -------------------------------------------------------------    */
 ;*    The foreign and extern clauses compilation. Foreign and extern   */
 ;*    clauses only differs by their syntax. They play the same role    */
@@ -21,6 +21,7 @@
    (import  module_module
 	    module_checksum
 	    engine_param
+	    backend_backend
 	    ast_glo-decl
 	    tools_error
 	    tools_shape
@@ -42,27 +43,29 @@
 ;*    make-foreign-compiler ...                                        */
 ;*---------------------------------------------------------------------*/
 (define (make-foreign-compiler)
-   (instantiate::ccomp (id 'foreign)
-		       (producer foreign-producer)
-		       (consumer (lambda (m c) (foreign-producer c)))
-		       (finalizer foreign-finalizer)))
+   (instantiate::ccomp
+      (id 'foreign)
+      (producer foreign-producer)
+      (consumer (lambda (m c) (foreign-producer c)))
+      (finalizer foreign-finalizer)))
 
 ;*---------------------------------------------------------------------*/
 ;*    make-extern-compiler ...                                         */
 ;*---------------------------------------------------------------------*/
 (define (make-extern-compiler)
-   (instantiate::ccomp (id 'extern)
-		       (producer extern-producer)
-		       (consumer (lambda (m c) (extern-producer c)))))
+   (instantiate::ccomp
+      (id 'extern)
+      (producer (lambda (c) (extern-producer c #t)))
+      (consumer (lambda (m c) (extern-producer c #f)))))
 
 ;*---------------------------------------------------------------------*/
 ;*    foreign-producer ...                                             */
 ;*---------------------------------------------------------------------*/
 (define (foreign-producer clause)
-   (if (not (eq? *target-language* 'jvm))
+   (if (memq 'foreign (backend-foreign-clause-support (the-backend)))
        (match-case clause
 	  ((?- . ?protos)
-	   (for-each foreign-parser protos)
+	   (for-each (lambda (p) (foreign-parser p #t)) protos)
 	   '())
 	  (else
 	   (user-error "Parse error"
@@ -74,11 +77,11 @@
 ;*---------------------------------------------------------------------*/
 ;*    extern-producer ...                                              */
 ;*---------------------------------------------------------------------*/
-(define (extern-producer clause)
-   (if (not (eq? *target-language* 'jvm))
+(define (extern-producer clause exportp)
+   (if (memq 'extern (backend-foreign-clause-support (the-backend)))
        (match-case clause
 	  ((?- . ?protos)
-	   (for-each extern-parser protos)
+	   (for-each (lambda (p) (extern-parser p exportp)) protos)
 	   '())
 	  (else
 	   (user-error "Parse error"
@@ -107,7 +110,7 @@
 ;*---------------------------------------------------------------------*/
 ;*    foreign-parser ...                                               */
 ;*---------------------------------------------------------------------*/
-(define (foreign-parser foreign)
+(define (foreign-parser foreign exportp)
    (trace (ast 2) "foreign parser: " foreign #\Newline)
    (match-case foreign
       ((include ?string)
@@ -118,7 +121,8 @@
       ((type . ?-)
        (parse-c-foreign-type foreign))
       ((export (and (? symbol?) ?bname) (and (? string?) ?cname))
-       (set! *foreign-exported* (cons foreign *foreign-exported*)))
+       (set! *foreign-exported*
+	  (cons (cons foreign exportp) *foreign-exported*)))
       (((or export type include) . ?-)
        ;; an error
        (user-error "Parse error" "Illegal foreign form" foreign '()))
@@ -129,20 +133,14 @@
 		     (symbol? l-name)
 		     (check-c-args? proto)))
 	   (user-error "Parse error" "Illegal `macro' form" foreign '())
-	   (declare-global-cfun! l-name
-				 c-name
-				 type
-				 proto
-				 (eq? (car foreign) 'infix)
-				 #t
-				 foreign
-				 foreign)))
+	   (declare-global-cfun! l-name #f 'foreign c-name
+	      type proto (eq? (car foreign) 'infix) #t foreign foreign)))
       ((macro ?type ?l-name ?c-name)
        (if (not (and (string? c-name)
 		     (symbol? type)
 		     (symbol? l-name)))
 	   (user-error "Parse error" "Illegal `macro' form" foreign '())
-	   (declare-global-cvar! l-name c-name type #t foreign foreign)))
+	   (declare-global-cvar! l-name #f c-name type #t foreign foreign)))
       ((macro . ?-)
        (user-error "Parse error" "Illegal foreign form" foreign '()))
       ((?type ?l-name ?proto ?c-name)
@@ -151,34 +149,28 @@
 		     (symbol? l-name)
 		     (check-c-args? proto)))
 	   (user-error "Parse error" "Illegal `function' form" foreign '())
-	   (declare-global-cfun! l-name
-				 c-name
-				 type
-				 proto
-				 #f
-				 #f
-				 foreign
-				 foreign)))
+	   (declare-global-cfun! l-name #f 'foreign c-name
+	      type proto #f #f foreign foreign)))
       ((?type ?l-name ?c-name)
        (if (not (and (string? c-name)
 		     (symbol? type)
 		     (symbol? l-name)))
 	   (user-error "Parse error" "Illegal `variable' form" foreign '())
-	   (declare-global-cvar! l-name c-name type #f foreign #f)))
+	   (declare-global-cvar! l-name #f c-name type #f foreign #f)))
       (else
        (user-error "Parse error" "Illegal foreign form" foreign '()))))
 
 ;*---------------------------------------------------------------------*/
 ;*    extern-parser ...                                                */
 ;*---------------------------------------------------------------------*/
-(define (extern-parser extern)
+(define (extern-parser extern exportp)
    (trace (ast 2) "extern parser: " extern #\Newline)
    (match-case extern
       ((type . ?-)
        ;; type clauses
        (parse-c-extern-type extern))
       (((or export include) . ?-)
-       (foreign-parser extern))
+       (foreign-parser extern exportp))
       ((or (macro (and (? symbol?) ?id) ?proto (and (? string?) ?cn))
 	   (infix macro (and (? symbol?) ?id) ?proto (and (? string?) ?cn)))
        ;; macro function definitions
@@ -189,7 +181,8 @@
 		  (not (check-c-args? proto)))
 	      (user-error "Parse error" "Illegal extern form" extern '())
 	      (let ((infix? (eq? (car extern) 'infix)))
-		 (declare-global-cfun! ln cn type proto infix? #t extern #f)))))
+		 (declare-global-cfun! ln #f 'foreign
+		    cn type proto infix? #t extern #f)))))
       ((macro (and (? symbol?) ?id) (and (? string?) ?c-name))
        ;; macro variable definitions
        (let* ((pid    (parse-id id (find-location extern)))
@@ -197,7 +190,7 @@
 	      (type   (type-id (default-c-type (cdr pid) extern))))
 	  (if (not (check-id pid extern))
 	      (user-error "Parse error" "Illegal extern form" extern '())
-	      (declare-global-cvar! l-name c-name type #t extern #f))))
+	      (declare-global-cvar! l-name #f c-name type #t extern #f))))
       ((macro . ?-)
        (user-error "Parse error" "Illegal extern form" extern '()))
       (((and (? symbol?) ?id) ?proto (and (? string?) ?cn))
@@ -208,7 +201,8 @@
 	  (if (or (not (check-id pid extern))
 		  (not (check-c-args? proto)))
 	      (user-error "Parse error" "Illegal extern form" extern '())
-	      (declare-global-cfun! ln cn type proto #f #f extern #f))))
+	      (declare-global-cfun! ln #f 'foreign
+		 cn type proto #f #f extern #f))))
       (((and (? symbol?) ?id) (and (? string?) ?c-name))
        ;; variable definitions
        (let* ((pid    (parse-id id (find-location extern)))
@@ -216,7 +210,7 @@
 	      (type   (type-id (default-c-type (cdr pid) extern))))
 	  (if (not (check-id pid extern))
 	      (user-error "Parse error" "Illegal extern form" extern '())
-	      (declare-global-cvar! l-name c-name type #f extern #f))))
+	      (declare-global-cvar! l-name #f c-name type #f extern #f))))
       (else
        (user-error "Parse error" "Illegal extern form" extern '()))))
 
@@ -372,16 +366,20 @@
 (define (foreign-finalizer)
    ;; we patch bigloo foreign exported variables
    (for-each (lambda (foreign)
-		(let ((global (find-global (cadr foreign)))
-		      (name   (caddr foreign)))
+		(let* ((fo (car foreign))
+		       (ex (cdr foreign))
+		       (global (find-global (cadr fo)))
+		       (name (caddr fo)))
 		   (cond
 		      ((not (global? global))
-		       (if (not (or (eq? *pass* 'make-add-heap)
-				    (eq? *pass* 'make-heap)))
-			   (user-error "Foreign"
-				       "Unbound global variable"
-				       foreign
-				       '())))
+		       (when ex
+			  (if (not (or (eq? *pass* 'make-add-heap)
+				       (eq? *pass* 'make-heap)))
+			      (user-error "Foreign"
+				 (format "Unbound global variable \"~a\""
+				    (cadr fo))
+				 foreign
+				 '()))))
 		      ((string? (global-name global))
 		       (user-warning
 			"Foreign"
@@ -396,7 +394,7 @@
        'void
        (let ((accesses (reverse! *foreign-accesses*)))
 	  (set! *foreign-accesses* '())
-	  (list (unit 'foreign 48 accesses #t)))))
+	  (list (unit 'foreign 48 accesses #t #f)))))
 	  
 ;*---------------------------------------------------------------------*/
 ;*    default-c-type ...                                               */
