@@ -3,8 +3,8 @@
 ;*    -------------------------------------------------------------    */
 ;*    Author      :  Manuel Serrano                                    */
 ;*    Creation    :  Thu Jul 20 16:05:33 2000                          */
-;*    Last change :  Sun Jun 24 19:29:06 2001 (serrano)                */
-;*    Copyright   :  2000-01 Manuel Serrano                            */
+;*    Last change :  Wed Nov 21 07:30:08 2012 (serrano)                */
+;*    Copyright   :  2000-12 Manuel Serrano                            */
 ;*    -------------------------------------------------------------    */
 ;*    The Java module clause handling.                                 */
 ;*=====================================================================*/
@@ -14,13 +14,15 @@
 ;*---------------------------------------------------------------------*/
 (module module_java
    (include "Ast/unit.sch"
-	    "Tools/trace.sch")
+	    "Tools/trace.sch"
+	    "Module/java.sch")
    (import  module_module
 	    module_checksum
 	    module_class
 	    module_prototype
 	    module_foreign
 	    engine_param
+	    backend_backend
 	    tools_error
 	    tools_shape
 	    tools_misc
@@ -39,7 +41,10 @@
 	    foreign_jtype
 	    foreign_access)
    (export  (make-java-compiler)
-	    (find-java-class class::symbol))
+	    (find-java-class ::symbol)
+	    ;; heap-add-jclass is untyped other it force the module
+	    ;; object-module to be imported in too many places.
+	    (heap-add-jclass! jclass))
    (static  (class jklass
 	       (bind-jklass!)
 	       (src::pair read-only)
@@ -51,42 +56,43 @@
 	       (fields::pair-nil (default '()))
 	       (methods::pair-nil (default '()))
 	       (constructors::pair-nil (default '()))
-	       (abstract?::bool (default #f)))
+	       (abstract?::bool (default #f))
+	       (module (default #unspecified)))
 	    (class jmethod
 	       (src::pair read-only)
 	       (id::symbol read-only)
 	       (args::pair-nil read-only)
 	       (jname::bstring read-only)
-	       (modifiers::pair-nil read-only (default '())))))
+	       (modifiers::pair-nil read-only (default '())))
+	    (class jconstructor::jmethod)))
 
 ;*---------------------------------------------------------------------*/
 ;*    make-java-compiler ...                                           */
 ;*---------------------------------------------------------------------*/
 (define (make-java-compiler)
-   (instantiate::ccomp (id 'java)
-		       (producer java-producer)
-		       (consumer (lambda (m c) (java-producer c)))
-		       (finalizer java-finalizer)))
+   (instantiate::ccomp
+      (id 'java)
+      (producer (lambda (c) (java-producer *module* c)))
+      (consumer (lambda (m c) (java-producer m c)))
+      (finalizer java-finalizer)))
 
 ;*---------------------------------------------------------------------*/
 ;*    java-error ...                                                   */
 ;*---------------------------------------------------------------------*/
 (define (java-error java . msg)
    (user-error "Parse error"
-	       (if (pair? msg)
-		   (car msg)
-		   "Illegal java variable")
+	       (if (pair? msg) (car msg) "Illegal java variable")
 	       java
 	       '()))
 
 ;*---------------------------------------------------------------------*/
 ;*    java-producer ...                                                */
 ;*---------------------------------------------------------------------*/
-(define (java-producer clause)
-   (if (eq? *target-language* 'jvm)
+(define (java-producer module clause)
+   (if (memq 'java (backend-foreign-clause-support (the-backend)))
        (match-case clause
 	  ((?- . ?protos)
-	   (for-each java-parser protos)
+	   (for-each (lambda (p) (java-parser p module)) protos)
 	   '())
 	  (else
 	   (java-error clause "Illegal `java' clause")))
@@ -95,20 +101,20 @@
 ;*---------------------------------------------------------------------*/
 ;*    java-parser ...                                                  */
 ;*---------------------------------------------------------------------*/
-(define (java-parser java)
-   (trace (ast 2) "java parser: " java #\Newline)
+(define (java-parser java module)
+   (trace (ast 2) "java parser: " java " " module #\Newline)
    (match-case java
       ;; export clauses
       ((export (and (? symbol?) ?bname) (and (? string?) ?cname))
-       (set! *jexported* (cons java *jexported*)))
+       (set! *jexported* (cons (cons java module) *jexported*)))
       ((export . ?-)
        (java-error java "Illegal java export form"))
       ;; a java class
       ((class ?ident . ?rest)
-       (java-parse-class java ident rest #f))
+       (java-parse-class java ident rest #f module))
       ;; an abstract java class
       ((abstract-class ?ident . ?rest)
-       (java-parse-class java ident rest #t))
+       (java-parse-class java ident rest #t module))
       ((array (and (? symbol?) ?ident) (and (? symbol?) ?of))
        (java-declare-array java ident of))
       (else
@@ -197,13 +203,16 @@
 		 *jklasses* (reverse! *jclasses*)))
    (set! *jclasses* '())
    ;; we patch bigloo java exported variables name
-   (for-each (lambda (java)
-		(let ((global (find-global (cadr java)))
-		      (name   (caddr java)))
+   (for-each (lambda (jmod)
+		(let* ((java (car jmod))
+		       (module (cdr jmod))
+		       (global (find-global (cadr java)))
+		       (name (caddr java)))
 		   (cond
 		      ((not (global? global))
-		       (if (not (or (eq? *pass* 'make-add-heap)
-				    (eq? *pass* 'make-heap)))
+		       (if (and (not (or (eq? *pass* 'make-add-heap)
+					 (eq? *pass* 'make-heap)))
+				(eq? module *module*))
 			   (java-error java
 				       "Unbound (or static) global variable")))
 		      ((string? (global-name global))
@@ -221,23 +230,22 @@
 ;*---------------------------------------------------------------------*/
 ;*    java-parse-class ...                                             */
 ;*---------------------------------------------------------------------*/
-(define (java-parse-class java ident rest abstract?)
+(define (java-parse-class java ident rest abstract? module)
    (let* ((tser (reverse rest))
-	  (jname (if (pair? tser)
-		     (car tser)
-		     #f)))
+	  (jname (if (pair? tser) (car tser) #f)))
       (cond
 	 ((not (symbol? ident))
 	  (java-error java "Illegal java class"))
 	 ((string? jname)
-	  (java-declare-class java ident jname (cdr tser) abstract?))
+	  (java-declare-class java ident jname (cdr tser) abstract? module))
 	 (else
-	  (java-refine-class java ident rest)))))
+	  (java-refine-class java ident rest module)))))
 
 ;*---------------------------------------------------------------------*/
 ;*    java-declare-class ...                                           */
 ;*---------------------------------------------------------------------*/
-(define (java-declare-class j id::symbol jname::bstring comp::pair-nil a::bool)
+(define (java-declare-class j id::symbol jname::bstring comp::pair-nil a::bool
+			    module)
    (let* ((loc (find-location j))
 	  (jklass (let ((jklass (find-jklass id)))
 		    (cond
@@ -248,7 +256,8 @@
 			   (id id)
 			   (idd (fast-id-of-id id loc))
 			   (jname jname)
-			   (abstract? a)))
+			   (abstract? a)
+			   (module module)))
 		       ((not (eq? (jklass-abstract? jklass) a))
 			(java-error j "Illegal Java class redefinition"))
 		       ((not (string? (jklass-jname jklass)))
@@ -265,7 +274,7 @@
 ;*    This function is used when someone refine the declaration        */
 ;*    of a Java class.                                                 */
 ;*---------------------------------------------------------------------*/
-(define (java-refine-class j ident::symbol comp::pair-nil)
+(define (java-refine-class j ident::symbol comp::pair-nil module)
    (let ((jklass (let ((jklass (find-jklass ident)))
 		    (if (jklass? jklass)
 			jklass
@@ -273,14 +282,15 @@
 			   (src j)
 			   (loc (find-location j))
 			   (idd (fast-id-of-id ident (find-location j)))
-			   (id ident))))))
+			   (id ident)
+			   (module module))))))
       (for-each (lambda (c) (java-declare-component j jklass c)) comp)))
 
 ;*---------------------------------------------------------------------*/
 ;*    java-declare-component ...                                       */
 ;*---------------------------------------------------------------------*/
 (define (java-declare-component j jklass::jklass component)
-   (define (every? pred? lst)
+   (define (every pred? lst)
       (let loop ((lst lst))
 	 (cond
 	    ((null? lst)
@@ -292,13 +302,13 @@
 	    (else
 	     #f))))
    (define (arg-list? lst)
-      (every? (lambda (s) (and (symbol? s) (type-ident? s))) lst))
+      (every (lambda (s) (and (symbol? s) (type-ident? s))) lst))
    (define (modifier-list? lst)
-      (every? (lambda (s)
-		 (and (symbol? s)
-		      (memq s '(public private protected
-				       static final synchronized
-				       abstract))))
+      (every (lambda (s)
+		(and (symbol? s)
+		     (memq s '(public private protected
+			       static final synchronized
+			       abstract))))
 	      lst))
    (match-case component
       ((field . ?rest)
@@ -331,13 +341,14 @@
 		     (set! methods (cons jmet methods))))))
 	  (else
 	   (java-error component "Illegal Java method"))))
-      ((constructor (and (? symbol?) ?id) (and (? arg-list?) ?args))
+      ((or (constructor public (and (? symbol?) ?id) (and (? arg-list?) ?args))
+	   (constructor (and (? symbol?) ?id) (and (? arg-list?) ?args)))
        (with-access::jklass jklass (constructors methods idd)
-	  (let ((jconstr (instantiate::jmethod
+	  (let ((jconstr (instantiate::jconstructor
 			    (src component)
 			    (id (make-typed-ident (symbol-append '%% idd '- id)
-						  'void))
-			    (args (cons (make-typed-formal idd) args))
+						  idd))
+			    (args args)
 			    (jname "<init>"))))
 	     (set! methods (cons jconstr methods))
 	     (set! constructors (cons (cons id args) constructors)))))
@@ -359,18 +370,19 @@
    (with-access::jklass jklass (id jname constructors methods src loc)
       (define (declare-java-static-method jmet)
 	 (with-access::jmethod jmet (id args jname src modifiers)
-	    (let ((g (declare-java-method! id jname args modifiers
-					   (jklass-jname jklass)
-					   src)))
-	       (global-module-set! g (jklass-id jklass))
-	       g)))
+	    (declare-java-method! id (jklass-id jklass)
+				  jname args modifiers
+				  (jklass-jname jklass)
+				  src)))
       (define (declare-java-virtual-method jmet)
 	 (with-access::jmethod jmet (id args jname src modifiers)
-	    (if (not (and (pair? args)
-			  (eq? (type-id (type-of-id (car args) loc))
-			       (jklass-idd jklass))))
+	    (if (and (not (jconstructor? jmet))
+		     (not (and (pair? args)
+			       (eq? (type-id (type-of-id (car args) loc))
+				    (jklass-idd jklass)))))
 		(java-error src "Illegal first argument of virtual method")
-		(declare-java-method! id jname args modifiers
+		(declare-java-method! id (jklass-id jklass)
+				      jname args modifiers
 				      (jklass-jname jklass)
 				      src))))
       (define (declare-java-method jmet::jmethod)
@@ -399,11 +411,11 @@
 ;*---------------------------------------------------------------------*/
 ;*    declare-java-method! ...                                         */
 ;*---------------------------------------------------------------------*/
-(define (declare-java-method! id jname args modifiers kname src)
+(define (declare-java-method! id module jname args modifiers kname src)
    (let* ((pid (parse-id id (find-location src)))
 	  (ln (car pid))
 	  (tid (type-id (cdr pid))))
-      (let ((g (declare-global-cfun! ln jname tid args #f #f src #f)))
+      (let ((g (declare-global-cfun! ln #f module jname tid args #f #f src #f)))
 	 (cfun-method-set! (global-value g) modifiers)
 	 (global-jvm-type-name-set! g kname)
 	 g)))
@@ -414,7 +426,8 @@
 (define (declare-java-class!::jclass jklass::jklass)
    (with-access::jklass jklass (src id jname package loc
 				    fields constructors
-				    abstract?)
+				    abstract?
+				    module)
       (let* ((pid (parse-id id loc))
 	     (jid (car pid))
 	     (super (cdr pid)))
@@ -423,7 +436,7 @@
 	 ;; and we create a type for this class
 	 (let ((jclass (declare-java-class-type! jid super jname package src)))
 	    ;; some paranoid checking
-	    [assert (jclass) (jclass? jclass)]
+	    (assert (jclass) (jclass? jclass))
 	    ;; we store the src-import location in order to print a nice error
 	    ;; message if that tclass is not defined
 	    (type-import-location-set! jclass loc)
@@ -435,6 +448,7 @@
 		     constructors
 		     jclass
 		     abstract?
+		     module
 		     src)))
 	    ;; we are done
 	    jclass))))
@@ -450,6 +464,13 @@
 (define (register-java-class! class::symbol qualified-name::bstring)
    (set! *java-classes* (cons (cons class qualified-name) *java-classes*)))
 
+;*---------------------------------------------------------------------*/
+;*    heap-add-jclass! ...                                             */
+;*---------------------------------------------------------------------*/
+(define (heap-add-jclass! new)
+   (assert (new) (jclass? new))
+   (register-java-class! (jclass-id new) (jclass-name new)))
+
 ;*---------------------------------------------------------------------*/
 ;*    find-java-class ...                                              */
 ;*---------------------------------------------------------------------*/
