@@ -3,8 +3,8 @@
 ;*    -------------------------------------------------------------    */
 ;*    Author      :  Manuel Serrano                                    */
 ;*    Creation    :  Tue Jun  4 14:27:58 1996                          */
-;*    Last change :  Fri Nov  2 12:14:48 2001 (serrano)                */
-;*    Copyright   :  1996-2001 Manuel Serrano, see LICENSE file        */
+;*    Last change :  Sat Mar  3 10:01:29 2007 (serrano)                */
+;*    Copyright   :  1996-2007 Manuel Serrano, see LICENSE file        */
 ;*    -------------------------------------------------------------    */
 ;*    The prototype management                                         */
 ;*=====================================================================*/
@@ -18,25 +18,38 @@
 	   tools_misc
 	   type_type
 	   ast_ident
+	   module_eval
 	   (find-location/loc tools_location)
-	   (find-location tools_location))
+	   (find-location tools_location)
+	   type_cache)
    (export (parse-prototype prototype)))
 
 ;*---------------------------------------------------------------------*/
 ;*    parse-prototype ...                                              */
 ;*---------------------------------------------------------------------*/
-(define (parse-prototype prototype)
-   (match-case prototype
+(define (parse-prototype proto)
+   (match-case proto
       (((and ?class (or class final-class wide-class abstract-class)) . ?-)
-       (parse-class class (cdr prototype)))
+       (parse-class class (cdr proto)))
       ((generic . ?-)
-       (parse-function-prototype (cdr prototype) 'sgfun))
+       (parse-function-prototype (cdr proto) 'sgfun))
       ((inline . ?-)
-       (parse-function-prototype (cdr prototype) 'sifun))
-      ((?- . ?-)
-       (parse-function-prototype prototype 'sfun))
+       (parse-function-prototype (cdr proto) 'sifun))
+      ((macro . ?-)
+       (parse-macro (cdr proto)))
+      ((syntax . ?-)
+       (parse-syntax (cdr proto)))
+      ((expander . ?-)
+       (parse-expander (cdr proto)))
+      ((?id . ?-)
+       (if (or (not *all-export-mutable?*)
+	       (memq id '(main module-initialization))
+	       (not (eq? (get-default-type)
+			 (type-of-id id (find-location proto)))))
+	   (parse-function-prototype proto 'sfun)
+	   (parse-variable-prototype id)))
       (else
-       (parse-variable-prototype prototype))))
+       (parse-variable-prototype proto))))
 
 ;*---------------------------------------------------------------------*/
 ;*    parse-function-prototype ...                                     */
@@ -44,18 +57,20 @@
 (define (parse-function-prototype proto class)
    (match-case proto
       (((and ?id (? symbol?)) . ?the-args)
-       (let loop ((args the-args))
-	  (cond
-	     ((null? args)
-	      (list class id (dsssl-formals-skeleton the-args)))
-	     ((symbol? args)
-	      (list class id the-args))
-	     ((and (pair? args)
-		   (or (symbol? (car args))
-		       (dsssl-named-constant? (car args))))
-	      (loop (cdr args)))
-	     (else
-	      #f))))
+       (if (dsssl-check-prototype? the-args)
+	   (list class id the-args)
+	   (let loop ((args the-args))
+	      (cond
+		 ((null? args)
+		  (list class id the-args))
+		 ((symbol? args)
+		  (list class id the-args))
+		 ((not (pair? args))
+		  #f)
+		 ((not (symbol? (car args)))
+		  #f)
+		 (else
+		  (loop (cdr args)))))))
       (else
        #f)))
 
@@ -98,6 +113,62 @@
 	 (else
 	  #f))))
 
+;*---------------------------------------------------------------------*/
+;*    parse-macro ...                                                  */
+;*---------------------------------------------------------------------*/
+(define (parse-macro macro-def)
+   (match-case macro-def
+      (((and ?id (? symbol?)) . ?rest)
+       (let loop ((args rest))
+	  (cond
+	     ((null? args)
+	      (cons 'macro macro-def))
+	     ((symbol? args)
+	      (cons 'macro macro-def))
+	     ((not (pair? args))
+	      (user-error "Parse error" "Illegal macro definition" macro-def))
+	     ((not (symbol? (car args)))
+	      (user-error "Parse error" "Illegal macro definition" macro-def))
+	     (else
+	      (loop (cdr args))))))
+      ((((and ?id (? symbol?)) . ?the-args) . ?body)
+       (let loop ((args the-args))
+	  (cond
+	     ((null? body)
+	      (user-error "Parse error" "Illegal macro definition" macro-def))
+	     ((null? args)
+	      (cons 'define-macro macro-def))
+	     ((symbol? args)
+	      (cons 'define-macro macro-def))
+	     ((not (pair? args))
+	      (user-error "Parse error" "Illegal macro definition" macro-def))
+	     ((not (symbol? (car args)))
+	      (user-error "Parse error" "Illegal macro definition" macro-def))
+	     (else
+	      (loop (cdr args))))))
+      (else
+       (user-error "Parse error" "Illegal macro definition" macro-def))))
+
+;*---------------------------------------------------------------------*/
+;*    parse-syntax ...                                                 */
+;*---------------------------------------------------------------------*/
+(define (parse-syntax syntax-def)
+   (match-case syntax-def
+      (((and ?id (? symbol?)))
+	(cons 'syntax syntax-def))
+      (else
+       (user-error "Parse error" "Illegal syntax definition" syntax-def))))
+
+;*---------------------------------------------------------------------*/
+;*    parse-expander ...                                               */
+;*---------------------------------------------------------------------*/
+(define (parse-expander expander-def)
+   (match-case expander-def
+      (((and ?id (? symbol?)))
+       (cons 'expander expander-def))
+      (else
+       (user-error "Parse error" "Illegal expander definition" expander-def))))
+
 ;*---------------------------------------------------------------------*/
 ;*    parse-class-slot ...                                             */
 ;*---------------------------------------------------------------------*/
