@@ -1,10 +1,10 @@
 ;*=====================================================================*/
-;*    serrano/prgm/project/bigloo/comptime/Object/class.scm            */
+;*    serrano/prgm/project/bigloo/bigloo/comptime/Object/class.scm     */
 ;*    -------------------------------------------------------------    */
 ;*    Author      :  Manuel Serrano                                    */
 ;*    Creation    :  Thu May 30 16:46:40 1996                          */
-;*    Last change :  Tue Nov 27 10:28:15 2001 (serrano)                */
-;*    Copyright   :  1996-2001 Manuel Serrano, see LICENSE file        */
+;*    Last change :  Sun Nov 14 06:36:49 2021 (serrano)                */
+;*    Copyright   :  1996-2021 Manuel Serrano, see LICENSE file        */
 ;*    -------------------------------------------------------------    */
 ;*    The class definition                                             */
 ;*=====================================================================*/
@@ -18,8 +18,10 @@
 	    type_type
 	    type_cache
 	    type_env
+	    type_coercion
 	    object_tools
 	    object_slots
+	    object_coercion
 	    module_module
 	    engine_param
 	    foreign_jtype
@@ -27,6 +29,8 @@
 	    ast_ident
 	    (find-location tools_location))
 
+   (include "Object/class.sch")
+
    (export  (wide-class tclass::type
 	       ;; the `super' field
 	       its-super
@@ -45,30 +49,41 @@
 	       ;; the number of virtual slots this class implements
 	       (virtual-slots-number (default 0))
 	       ;; abstract class
-	       (abstract?::bool read-only (default #f)))
+	       (abstract?::bool read-only (default #f))
+	       ;; the true wide type associated with the wide classes
+	       (wide-type (default #f))
+	       ;; the list of subclasses
+	       (subclasses::pair-nil (default '())))
 
 	    (wide-class jclass::type
 	       ;; the `super' field
 	       (its-super (default #unspecified))
 	       ;; the slots of the class
 	       (slots (default #unspecified))
-;* 	       ;; final                                                */
-;* 	       (final?::bool read-only (default #f))                   */
 	       ;; package
 	       (package::bstring read-only (default "")))
+
+	    (wide-class wclass::type
+	       ;; the plain class that uses this wide chunk
+	       (its-class (default #unspecified)))
 	    
 	    (get-class-list::pair-nil)
 	    (heap-add-class! ::tclass)
+	    (wide-chunk-class-id::symbol ::symbol)
+	    (type-class-name::bstring ::type)
 	    (declare-class-type!::type ::obj ::global ::obj ::bool ::bool ::obj)
 	    (declare-java-class-type!::type ::symbol ::obj ::bstring ::bstring ::pair)
-	    (dump-class-types ::output-port)
-	    (emit-class-types ::output-port)
-	    (emit-bdb-class-types ::output-port)
 	    (final-class?::bool ::obj)
 	    (wide-class?::bool ::obj)
-	    (find-class-constructors ::tclass)
+	    (find-class-constructor ::tclass)
+	    (find-common-super-class ::tclass ::tclass)
 	    (type-subclass?::bool ::type ::type)
-	    (tclass-all-slots::pair-nil ::tclass)))
+	    (class-make::obj ::tclass)
+	    (class-fill::obj ::tclass)
+	    (class-predicate::symbol ::tclass)
+	    (class-allocate::symbol ::tclass)
+	    (check-class-declaration?::bool ::tclass ::obj)
+	    (set-class-slots! ::tclass ::pair-nil ::pair-nil)))
 
 ;*---------------------------------------------------------------------*/
 ;*    *class-type-list* ...                                            */
@@ -89,7 +104,29 @@
 ;*---------------------------------------------------------------------*/
 (define (heap-add-class! type::tclass)
    (set! *class-type-list* (cons type *class-type-list*)))
-   
+
+;*---------------------------------------------------------------------*/
+;*    wide-chunk-class-id ...                                          */
+;*    -------------------------------------------------------------    */
+;*    This function construct type name of the wide component          */
+;*    of a wide class. The idea is to generate a private name that the */
+;*    user cannot specify himself in his programs.                     */
+;*---------------------------------------------------------------------*/
+(define (wide-chunk-class-id class-id)
+   (string->symbol (string-append "#!" (symbol->string class-id))))
+
+;*---------------------------------------------------------------------*/
+;*    type-class-name ...                                              */
+;*---------------------------------------------------------------------*/
+(define (type-class-name class)
+   (cond
+      ((not (tclass? class))
+       (type-name class))
+      ((and (tclass-widening class) *saw*)
+       (type-name (tclass-wide-type class)))
+      (else
+       (type-name class))))
+
 ;*---------------------------------------------------------------------*/
 ;*    declare-class-type! ...                                          */
 ;*    -------------------------------------------------------------    */
@@ -118,21 +155,36 @@
 	  (type        (declare-type! class-id t-name 'bigloo)))
       ;; we mark that the holder is a read-only variable
       (global-set-read-only! class-holder)
+      (global-evaluable?-set! class-holder #t)
+      (global-type-set! class-holder (get-class-type))
       ;; By now we make the assumption that super is a correct class.
       ;; Super will be checked in `make-class-accesses!' (see module
       ;; object_access).
       (widen!::tclass type
 	 (its-super   super)
-	 (depth       (if (not (tclass? super))
-			  0
-			  (+fx (tclass-depth super) 1)))
+	 (depth       0)
 	 (holder      class-holder)
 	 (widening    widening)
 	 (final?      final?)
 	 (abstract?   abstract?)
 	 (constructor (cadr class-def)))
-      ;; we set the sizeof field
-      (type-size-set!  type sizeof)
+      (when (isa? super tclass)
+	 (with-access::tclass super (subclasses)
+	    (set! subclasses (cons type subclasses))))
+      ;; wide classes creates a new type denoting the wide chunk of the
+      ;; wide class. In addition, the type name of a wide classes is the
+      ;; type name of its super class.
+      (if (eq? widening 'widening)
+	  (let* ((wtid (wide-chunk-class-id class-id))
+		 (wt (widen!::wclass (declare-type! wtid t-name 'bigloo)
+			(its-class type))))
+	     (wclass-size-set! wt sizeof)
+	     (tclass-wide-type-set! type wt)
+	     (type-name-set! type (type-name super))
+	     (type-size-set! type (type-size super))
+	     (gen-coercion-clause! type wtid super #f)
+	     (gen-class-coercers! wt super))
+	  (type-size-set! type sizeof))
       ;; we add the class for the C type emission
       (set! *class-type-list* (cons type *class-type-list*))
       ;; we are done
@@ -162,97 +214,6 @@
       ;; we are done
       type))
 
-;*---------------------------------------------------------------------*/
-;*    cross-name ...                                                   */
-;*    -------------------------------------------------------------    */
-;*    This function return the typedef name for non class objects      */
-;*    and the `struct ??? *' name for classes. We need this function   */
-;*    because C does not support cross typedefed references.           */
-;*---------------------------------------------------------------------*/
-(define (cross-name type)
-   (if (tclass? type)
-       (string-append (type-size type) " *")
-       (type-name type)))
-
-;*---------------------------------------------------------------------*/
-;*    dump-class-types ...                                             */
-;*---------------------------------------------------------------------*/
-(define (dump-class-types oport)
-   (define (dump-slot slot)
-      (with-access::slot slot (virtual-num indexed name)
-	 (let ((cname (cross-name (slot-type slot))))
-	    (cond
-	       ((>=fx virtual-num 0)
-		#unspecified)
-	       (indexed
-		(fprint oport "   (" cname " *" name ")"))
-	       (else
-		(fprint oport "   (" cname " " name ")"))))))
-   (for-each (lambda (class)
-		(if (not (eq? class (get-object-type)))
-		    (begin
-		       (fprint oport "(define-class " (type-name class))
-		       (fprint oport "   (constructor ...)")
-		       (fprint oport "   (obj_t widening)")
-		       (for-each dump-slot (tclass-slots class))
-		       (fprint oport  #")\n"))))
-	     (reverse! *class-type-list*)))
-
-;*---------------------------------------------------------------------*/
-;*    emit-class-types ...                                             */
-;*---------------------------------------------------------------------*/
-(define (emit-class-types oport)
-   (define (emit-slot slot)
-      (with-access::slot slot (type virtual-num indexed name)
-	 (let ((cname (cross-name (slot-type slot))))
-	    (cond
-	       ((>=fx virtual-num 0)
-		#unspecified)
-	       (indexed
-		(fprint oport "   " cname " *" name ";"))
-	       (else
-		(fprint oport "   " cname " " name ";"))))))
-   (if (pair? *class-type-list*)
-       (fprint oport #\Newline "/* Object type definitions */"))
-   (for-each (lambda (class)
-		(if (not (eq? class (get-object-type)))
-		    (begin
-		       (fprint oport "typedef " (type-size class) " {")
-		       (if (not (tclass-widening class))
-			   (begin
-			      (fprint oport "   header_t header;")
-			      (fprint oport "   obj_t widening;"))
-			   (cond
-			      ((and (fixnum? *heap-debug*)
-				    (>=fx *heap-debug* 1))
-			       (fprint oport "   header_t header;"))
-			      ((null? (tclass-slots class))
-			       ;; this is an empty object (with no fields)
-			       ;; and some ISO C compilers (is it in the
-			       ;; definition ?) does not support empty types.
-			       ;; Hence, we generate a dummy field as
-			       ;; small as possible.
-			       (fprint oport "   char dummy;"))))
-		       (for-each emit-slot (tclass-slots class))
-		       (fprint oport "} *" (type-name class) #";\n"))))
-	     (reverse! *class-type-list*))
-   (if (pair? *class-type-list*)
-       (newline oport)))
-
-;*---------------------------------------------------------------------*/
-;*    emit-bdb-class-types ...                                         */
-;*---------------------------------------------------------------------*/
-(define (emit-bdb-class-types oport)
-   (if (pair? *class-type-list*)
-       (fprint oport "   /* Bigloo classes */"))
-   (for-each (lambda (class)
-		(with-access::tclass class (holder)
-		   (if (eq? (global-module holder) *module*)
-		       (fprint oport
-			       "   {\"" (type-size class)
-			       " *\", 0 },"))))
-	     *class-type-list*))
-
 ;*---------------------------------------------------------------------*/
 ;*    final-class? ...                                                 */
 ;*    -------------------------------------------------------------    */
@@ -300,30 +261,216 @@
        #f)))
 
 ;*---------------------------------------------------------------------*/
-;*    find-class-constructors ...                                      */
-;*    -------------------------------------------------------------    */
-;*    I just don't know what to do here. i) Shall we invoke the        */
-;*    all constructors (a la C++). ii) Shall we call the first         */
-;*    constructor defined? iii) Shall we call the constructor          */
-;*    if it exists? For now I have chosen ii) because it fits the need */
-;*    for all the code I have currently that make use of constructors. */
+;*    find-class-constructor ...                                       */
 ;*---------------------------------------------------------------------*/
-(define (find-class-constructors class::tclass)
+(define (find-class-constructor class::tclass)
    (let loop ((class class))
-      (with-access::tclass class (constructor its-super)
-	 (cond
-	    ((or (not (tclass? class)) (eq? class its-super))
-	     '())
-	    (constructor
-	     (list constructor))
-	    (else
-	     (loop its-super))))))
+      (when (tclass? class)
+	 (with-access::tclass class (constructor its-super)
+	    (cond
+	       ((eq? class its-super) #f)
+	       (constructor constructor)
+	       (else (loop its-super)))))))
 
 ;*---------------------------------------------------------------------*/
-;*    tclass-all-slots ...                                             */
+;*    find-common-super-class ...                                      */
 ;*---------------------------------------------------------------------*/
-(define (tclass-all-slots::pair-nil class::tclass)
-   (if (not (tclass-widening class))
-       (tclass-slots class)
-       (append (tclass-slots (tclass-its-super class))
-	       (tclass-slots class))))
+(define (find-common-super-class c1::tclass c2::tclass)
+   (define (tclass-super* c)
+      (with-access::tclass c (its-super)
+	 (if (or (not its-super) (eq? its-super c))
+	     '()
+	     (cons c (tclass-super* its-super)))))
+   (cond
+      ((type-subclass? c1 c2)
+       c2)
+      ((type-subclass? c2 c1)
+       c1)
+      (else
+       (let ((l1 (tclass-super* c1))
+	     (l2 (tclass-super* c2)))
+	  (let loop ((l l1))
+	     (cond
+		((null? l)
+		 #f)
+		((memq (car l) l2)
+		 (car l))
+		(else
+		 (loop (cdr l)))))))))
+      
+;*---------------------------------------------------------------------*/
+;*    class-make ...                                                   */
+;*    -------------------------------------------------------------    */
+;*    The name of the constructor                                      */
+;*---------------------------------------------------------------------*/
+(define (class-make t::tclass)
+   (if (tclass-abstract? t)
+       #f
+       (symbol-append 'make- (type-id t))))
+
+;*---------------------------------------------------------------------*/
+;*    class-fill ...                                                   */
+;*    -------------------------------------------------------------    */
+;*    The name of the filler                                           */
+;*---------------------------------------------------------------------*/
+(define (class-fill t::tclass)
+   (if (tclass-abstract? t)
+       #f
+       (symbol-append 'fill- (type-id t) '!)))
+
+;*---------------------------------------------------------------------*/
+;*    class-predicate ...                                              */
+;*    -------------------------------------------------------------    */
+;*    The name of the predicate                                        */
+;*---------------------------------------------------------------------*/
+(define (class-predicate::symbol t::tclass)
+   (symbol-append (type-id t) '?))
+
+;*---------------------------------------------------------------------*/
+;*    class-allocate ...                                               */
+;*    -------------------------------------------------------------    */
+;*    The name of the allocator                                        */
+;*---------------------------------------------------------------------*/
+(define (class-allocate::symbol t::tclass)
+   (symbol-append '%allocate- (type-id t)))
+
+;*---------------------------------------------------------------------*/
+;*    type-occurrence-increment! ::tclass ...                          */
+;*---------------------------------------------------------------------*/
+(define-method (type-occurrence-increment! t::tclass)
+   (call-next-method)
+   (with-access::tclass t (widening its-super)
+      (when widening (type-occurrence-increment! its-super))))
+
+;*---------------------------------------------------------------------*/
+;*    check-class-declaration? ...                                     */
+;*    -------------------------------------------------------------    */
+;*    This function checks if the super class is conform to the        */
+;*    class declaration.                                               */
+;*    -------------------------------------------------------------    */
+;*    Called when registering a class.                                 */
+;*    See @ref ../Module/class.scm:register-class@                     */
+;*---------------------------------------------------------------------*/
+(define (check-class-declaration? class src-def)
+   (if (wide-class? class)
+       (check-wide-class-declaration? class src-def)
+       (check-plain-class-declaration? class src-def)))
+
+;*---------------------------------------------------------------------*/
+;*    check-plain-class-declaration? ...                               */
+;*    -------------------------------------------------------------    */
+;*    This function checks that the super class is conform to the      */
+;*    class. That is, the class is not a wide class and the super      */
+;*    class is not final.                                              */
+;*---------------------------------------------------------------------*/
+(define (check-plain-class-declaration? class src-def)
+   (let ((super (tclass-its-super class))
+	 (class-id (tclass-id class)))
+      ;; Now that the class is defined we check the super (is it or
+      ;; not a class).
+      (cond
+	 ((and (type? super) (not (tclass? super)))
+	  (user-error (symbol->string (type-id super))
+	     (format "super of \"~a\" is not a class" class-id)
+	     src-def
+	     type)
+	  #f)
+	 ((wide-class? class)
+	  ;; internal error because wide classes must be processed
+	  ;; by make-wide-class-accesses
+	  (internal-error "check-plain-class-declation?"
+	     "Should not be able to see a wide class here"
+	     src-def)
+	  #f)
+	 ((wide-class? super)
+	  ;; no one can inherite of a wide class
+	  (user-error (symbol->string (type-id super))
+	     (format "super of \"~a\" is a wide class" class-id)
+	     src-def
+	     type)
+	  #f)
+	 ((final-class? super)
+	  ;; only wide class can inherit of final classes
+	  (user-error (symbol->string (type-id super))
+	     "Only wide classes can inherit of final classes" 
+	     src-def
+	     type)
+	  #f)
+	 (else
+	  #t))))
+
+;*---------------------------------------------------------------------*/
+;*    check-wide-class-declaration? ...                                */
+;*    -------------------------------------------------------------    */
+;*    This function checks that the super class is conform to the      */
+;*    class. That is, the class is a wide class and the super          */
+;*    class is final.                                                  */
+;*---------------------------------------------------------------------*/
+(define (check-wide-class-declaration? class src-def)
+   (let* ((super (tclass-its-super class))
+	  (class-id (tclass-id class)))
+      ;; Now that the class is defined we check the super (is it or
+      ;; not a class).
+      (cond
+	 ((and (type? super) (not (tclass? super)))
+	  (user-error (symbol->string (type-id super))
+	     (format "super of \"~a\" is not a class" class-id)
+	     src-def
+	     type))
+	 ((not (wide-class? class))
+	  ;; internal error because wide classes must be processed
+	  ;; by make-class-accesses
+	  (internal-error "check-wide-class-declaration?"
+	     "Should not be able to see a plain class here"
+	     src-def))
+	 ((wide-class? super)
+	  ;; no one can inherite of a wide class
+	  (user-error (symbol->string (type-id super))
+	     (format "super of \"~a\" is a wide class" class-id)
+	     src-def
+	     type))
+	 ((not (final-class? super))
+	  ;; wide class can only inherit of final classes
+	  (user-error (symbol->string (type-id super))
+	     (format "super of wide class \"~a\" is not a final class"
+		class-id)
+	     src-def
+	     type))
+	 ((final-class? class)
+	  ;; a class can't be final and wide
+	  (user-error class-id
+	     "A class can't be \"wide\" and \"final\""
+	     src-def
+	     type))
+	 (else
+	  #t))))
+
+;*---------------------------------------------------------------------*/
+;*    set-class-slots! ...                                             */
+;*    -------------------------------------------------------------    */
+;*    Parse the class slots and store inside the class structure       */
+;*    how many slots are virtual.                                      */
+;*    -------------------------------------------------------------    */
+;*    Called when registering a class.                                 */
+;*    See @ref ../Module/class.scm:register-class@                     */
+;*---------------------------------------------------------------------*/
+(define (set-class-slots! class class-def src-def)
+   (let* ((super (let ((super (tclass-its-super class)))
+		    (if (eq? super class)
+			#f
+			super)))
+	  (super-vnum (if (tclass? super)
+			  (tclass-virtual-slots-number super)
+			  0))
+	  (slots (cddr class-def)))
+      ;; we store inside the class the result of the parsing
+      ;; we pre-parse the class slot and we return the data structure
+      ;; describing the result of this parsing
+      (let* ((cslots (make-class-slots class slots super super-vnum src-def))
+	     (local-vnum (get-local-virtual-slots-number class cslots)))
+	 ;; we set the number of virtual slots for the current class
+	 (tclass-virtual-slots-number-set! class local-vnum)
+	 ;; and we store the slots
+	 (tclass-slots-set! class cslots))))
+
+       
