@@ -3,8 +3,8 @@
 ;*    -------------------------------------------------------------    */
 ;*    Author      :  Manuel Serrano                                    */
 ;*    Creation    :  Tue Jun 18 12:48:07 1996                          */
-;*    Last change :  Tue Nov 27 10:28:26 2001 (serrano)                */
-;*    Copyright   :  1996-2001 Manuel Serrano, see LICENSE file        */
+;*    Last change :  Wed Aug  2 17:16:26 2017 (serrano)                */
+;*    Copyright   :  1996-2017 Manuel Serrano, see LICENSE file        */
 ;*    -------------------------------------------------------------    */
 ;*    We build the class slots                                         */
 ;*=====================================================================*/
@@ -13,9 +13,13 @@
 ;*    The module                                                       */
 ;*---------------------------------------------------------------------*/
 (module object_slots
-   (include "Tools/trace.sch")
+   
+   (include "Tools/trace.sch"
+	    "Object/slots.sch")
+   
    (import  tools_error
 	    tools_location
+	    tools_shape
 	    type_type
 	    type_cache
 	    type_env
@@ -25,35 +29,39 @@
 	    ast_var
 	    ast_ident
 	    ast_glo-decl)
+   
    (export  (class slot
-	       (id::symbol read-only)
 	       ;; the slot identifier
+	       (id::symbol read-only)
+	       ;; the slot name
 	       (name::bstring read-only)
 	       ;; the source of that field (for reporting errors)
 	       (src read-only)
 	       ;; the class that slot belongs to
 	       (class-owner read-only)
-	       ;; the slot name
+	       ;; index in the class
+	       (index::long read-only)
+	       ;; the slot type (i.e., the type of values)
 	       (type read-only)
-	       ;; the slot type
+	       ;; access property
 	       (read-only?::bool read-only (default #f))
 	       ;; the slot default value
-	       (default-value read-only (default (slot-no-default-value-mark)))
+	       (default-value read-only (default (slot-no-default-value)))
 	       ;; virtual slot number (> 0 if virtual)
 	       (virtual-num (default -1))
+	       ;; virtual override
+	       (virtual-override::bool read-only (default #f))
 	       ;; the virtual slot getter
 	       (getter (default #f))
 	       ;; the virtual slot setter
 	       (setter (default #f))
-	       ;; for an indexed slot, indexed holds a tvec structure
-	       ;; that can be used to generate JVM accessors
-	       (indexed read-only (default #f))
 	       ;; some user information associated to that slot
 	       (user-info read-only (default #unspecified)))
-	    
-	    (slot-no-default-value-mark)
+
 	    (slot-default?::bool ::slot)
 	    (slot-virtual?::bool ::slot)
+	    (slot-inline-default?::bool ::slot)
+	    (slot-default-inline-value::obj ::slot)
 	    (make-class-slots ::tclass ::obj ::obj ::int ::obj)
 	    (make-java-class-slots ::jclass ::obj ::obj ::obj)
 	    (get-local-virtual-slots-number ::tclass ::pair-nil)
@@ -61,23 +69,32 @@
 	    (make-class-make-typed-formals ::pair-nil ::pair-nil)))
 
 ;*---------------------------------------------------------------------*/
-;*    *slot-no-default-value-mark* ...                                 */
+;*    slot-no-default-value ...                                        */
 ;*---------------------------------------------------------------------*/
-(define *slot-no-default-value-mark*
-   ;; this value can't be gensymed because it has to traverse libraries
-   'slot-no-default-value__17_5_1996)
+(define (slot-no-default-value)
+   (cons 1 2))
 
 ;*---------------------------------------------------------------------*/
-;*    slot-no-default-value-mark ...                                   */
+;*    slot-inline-mark ...                                             */
 ;*---------------------------------------------------------------------*/
-(define (slot-no-default-value-mark)
-   *slot-no-default-value-mark*)
+(define slot-inline-mark
+   (cons 3 4))
+
+;*---------------------------------------------------------------------*/
+;*    shape ::slot ...                                                 */
+;*---------------------------------------------------------------------*/
+(define-method (shape s::slot)
+   (with-access::slot s (id name class-owner type default-value)
+      (string-append "(" (symbol->string id) " (" name ")"
+	 " owner: " (shape class-owner)
+	 " type: " (shape type)
+	 ")")))
 
 ;*---------------------------------------------------------------------*/
 ;*    slot-default? ...                                                */
 ;*---------------------------------------------------------------------*/
 (define (slot-default? slot)
-   (not (eq? (slot-default-value slot) (slot-no-default-value-mark))))
+   (not (equal? (slot-default-value slot) (slot-no-default-value))))
 
 ;*---------------------------------------------------------------------*/
 ;*    slot-virtual? ...                                                */
@@ -85,6 +102,19 @@
 (define (slot-virtual? slot)
    (>=fx (slot-virtual-num slot) 0))
 
+;*---------------------------------------------------------------------*/
+;*    slot-inline-default? ...                                         */
+;*---------------------------------------------------------------------*/
+(define (slot-inline-default? slot)
+   (and (pair? (slot-default-value slot))
+	(eq? (car (slot-default-value slot)) slot-inline-mark)))
+
+;*---------------------------------------------------------------------*/
+;*    slot-default-inline-value ...                                    */
+;*---------------------------------------------------------------------*/
+(define (slot-default-inline-value slot)
+   (cdr (slot-default-value slot)))
+
 ;*---------------------------------------------------------------------*/
 ;*    ensure-type-defined! ...                                         */
 ;*---------------------------------------------------------------------*/
@@ -95,26 +125,30 @@
 ;*---------------------------------------------------------------------*/
 ;*    make-class-slots ...                                             */
 ;*---------------------------------------------------------------------*/
-(define (make-class-slots class slots super vnum src)
-   (trace (ast 2) "make-class-slots: " slots " " vnum #\Newline)
+(define (make-class-slots class clauses super vnum src)
+   
    (define (find-default-attr attr)
       ;; seek for the default attribute of slots
       (if (not (pair? attr))
-	  (slot-no-default-value-mark)
+	  (slot-no-default-value)
 	  (match-case (car attr)
 	     ((default ?value)
 	      value)
+	     ((inline-default ?value)
+	      (cons slot-inline-mark value))
 	     (else
 	      (find-default-attr (cdr attr))))))
+   
    (define (find-info-attr attr)
       ;; seek for the slot user info attribute
       (if (not (pair? attr))
 	  #f
 	  (match-case (car attr)
-	     ((info ?-)
-	      (car attr))
+	     ((info ?val)
+	      val)
 	     (else
 	      (find-info-attr (cdr attr))))))
+   
    (define (find-virtual-attr attr)
       ;; seek for the virtual getter and setter definitions
       (let loop ((attr attr)
@@ -130,120 +164,45 @@
 		(else
 		 (loop (cdr attr) get set))))))
    
-   (define (slot-member? slot slot-list)
+   (define (slot-member? id slot-list)
       ;; this function returns true if there is a slot with the same name
       ;; in the list
-      (let ((id (slot-id slot)))
-	 (let loop ((slot-list slot-list))
-	    (cond
-	       ((null? slot-list)
-		#f)
-	       ((eq? (slot-id (car slot-list)) id)
-		(car slot-list))
-	       (else
-		(loop (cdr slot-list)))))))
-   (define (unique-virtual-slots slots)
-      ;; this function removes overriden duplicated virtual slots
-      (let loop ((slots slots)
-		 (res   '()))
+      (let loop ((slot-list slot-list))
 	 (cond
-	    ((null? slots)
-	     res)
-	    ((null? (car slots))
-	     ;; this is an error, skip because the compilation will be
-	     ;; stopped soon
-	     (loop (cdr slots) '()))
-	    ((slot-virtual? (car slots))
-	     (let ((other (slot-member? (car slots) res)))
-		(if (slot? other)
-		    (let ((loc (find-location src))
-			  (id  (slot-id (car slots))))
-		       (if (eq? (slot-class-owner (car slots))
-				(slot-class-owner other))
-			   (user-error/location
-			    loc
-			    "Class declarator"
-			    "Illegal duplicated virtual slot"
-			    id)
-			   (let ((new-num (slot-virtual-num (car slots))))
-			      ;; we overriding a virtual slot we have to
-			      ;; ajust the slot numbering
-			      (slot-virtual-num-set! other new-num)
-			      ;; ...and we emit a warning
-			      (if (and (number? *warning*)
-				       (>= *warning* 2)
-				       *warning-overriden-slots*)
-				  (user-warning/location
-				   loc
-				   "Class declarator"
-				   "Overriden virtual slot"
-				   id))
-			      (loop (cdr slots) res))))
-		    (loop (cdr slots) (cons (car slots) res)))))
+	    ((null? slot-list)
+	     #f)
+	    ((eq? (slot-id (car slot-list)) id)
+	     (car slot-list))
 	    (else
-	     (loop (cdr slots) (cons (car slots) res))))))
-   (define (make-indexed-slots s slot-id attr)
-      ;; indexed slot.
-      (multiple-value-bind (vget vset)
-	 (find-virtual-attr attr)
-	 (if (or vget vset)
-	     (user-error/location "Parse error"
-				  "virtual slot can't be indexed"
-				  (car slot-id)
-				  '())))
-      (let* ((ftype (find-slot-type slot-id src))
-	     (iid (gensym))
-	     (itype (declare-subtype! iid
-				      (symbol->string iid)
-				      (list 'obj)
-				      'bigloo))
-	     (sval (instantiate::slot
-		      (id (car slot-id))
-		      (name (scheme-symbol->c-string (car slot-id)))
-		      (class-owner class)
-		      (src s)
-		      (type ftype)
-		      (read-only? (memq 'read-only attr))
-		      (default-value (find-default-attr attr))
-		      (indexed (widen!::tvec itype
-				  (item-type ftype)))
-		      (user-info (find-info-attr attr))))
-	     (slen (let ((len-id (symbol-append (car slot-id) '-len)))
-		      (instantiate::slot
-			 (id len-id)
-			 (name (scheme-symbol->c-string len-id))
-			 (type *long*)
-			 (src s)
-			 (class-owner class)
-			 (read-only? #t)))))
-	 (type-init?-set! itype #t)
-	 (values sval slen)))
-   (define (make-attribute-slot s slot-id attr vget vset vnum)
+	     (loop (cdr slot-list))))))
+   
+   (define (make-attribute-slot s slot-id attr vget vset vnum vover index)
       ;; direct slot with attribute. Because of the presence of
       ;; the attributes, this slot may be virtual
       (let ((reado? (memq 'read-only attr)))
 	 (cond
 	    ((and vset (not vget))
 	     (user-error/location (find-location s)
-				  "Parse error"
-				  "Illegal virtual slot (missing getter)"
-				  (car slot-id)
-				  '()))
+		(type-id class)
+		"Illegal virtual slot (missing getter)"
+		(car slot-id)
+		'()))
 	    ((and vset reado?)
 	     (user-error/location (find-location s)
-				  "Parse error"
-				  "Illegal virtual slot (read-only)"
-				  (car slot-id)
-				  '()))
+		(type-id class)
+		"Illegal virtual slot (read-only)"
+		(car slot-id)
+		'()))
 	    ((and vget (not vset) (not reado?))
 	     (user-error/location (find-location s)
-				  "Parse error"
-				  "Illegal virtual slot (missing setter)"
-				  (car slot-id)
-				  '()))
+		(type-id class)
+		"Illegal virtual slot (missing setter)"
+		(car slot-id)
+		'()))
 	    (else
 	     (instantiate::slot
 		(id (car slot-id))
+		(index index)
 		(name (scheme-symbol->c-string (car slot-id)))
 		(src s)
 		(class-owner class)
@@ -251,77 +210,132 @@
 		(read-only? reado?)
 		(default-value (find-default-attr attr))
 		(virtual-num (if vget vnum -1))
+		(virtual-override vover)
 		(getter vget)
 		(setter vset)
 		(user-info (find-info-attr attr)))))))
-   (define (make-direct-slot slot-id)
+   
+   (define (check-super-slot nslot sslots class)
+      (unless (slot-virtual? nslot)
+	 (let ((id (slot-id nslot)))
+	    (when (any (lambda (ss) (eq? (slot-id ss) id)) sslots)
+	       (user-error/location (find-location (slot-src nslot))
+		  (tclass-id class)
+		  "slot already defined in super class"
+		  id)))))
+   
+   (define (check-super-slots nslots sslots class)
+      (for-each (lambda (slot)
+		   (check-super-slot slot sslots class))
+	 nslots))
+   
+   (define (replace new old lst)
+      (let loop ((lst lst))
+	 (cond
+	    ((null? lst)
+	     lst)
+	    ((eq? (car lst) old)
+	     (cons new (cdr lst)))
+	    (else
+	     (cons (car lst) (loop (cdr lst)))))))
+   
+   (define (make-direct-slot slot-id index)
       ;; plain direct slot
       (instantiate::slot
 	 (id (car slot-id))
+	 (index index)
 	 (name (scheme-symbol->c-string (car slot-id)))
 	 (src slot-id)
 	 (class-owner class)
+	 (user-info #f)
 	 (type (find-slot-type slot-id src))))
-   (let loop ((slots slots)
-	      (res   (cond
-			((not (type? super))
-			 '())
-			((not (tclass? super))
-			 '())
-			(else
-			 ;; take care to the double reverse. Here we can't
-			 ;; do in in-place reverse otherwise will be destroying
-			 ;; super classes fields. This reverse allocates
-			 ;; a fresh slots list.
-			 (reverse (tclass-slots super)))))
-	      (vnum  vnum))
-      (if (null? slots)
-	  ;; this is mandatory that unique-virtual-slots is called
-	  ;; on a reversed list. This function fixs up virtual slots
-	  ;; number. If the list is the wrong direction, numbers will get
-	  ;; mixed up.
-	  (unique-virtual-slots res)
-	  (let ((s (car slots)))
-	     (match-case s
-		((* (id ?id) . ?attr)
-		 (multiple-value-bind (sval slen)
-		    (make-indexed-slots s id attr)
-		    (loop (cdr slots)
-			  (cons* sval slen res)
-			  vnum)))
-		(((id ?id) . ?attr)
-		 (multiple-value-bind (vget vset)
-		    (find-virtual-attr attr)
-		    (loop (cdr slots)
-			  (cons (make-attribute-slot s id attr vget vset vnum)
-				res)
-			  (if (or vget vset) (+ vnum 1) vnum))))
-		((id ?id)
-		 (loop (cdr slots)
-		       (cons (make-direct-slot id) res)
-		       vnum))
-		(else
-		 (user-error "Parse error"
-			     "Unknown class slot type"
-			     s
-			     '())))))))
+   
+   (trace (ast 2) "make-class-slots: " clauses " " vnum #\Newline)
+   (let ((sslots (cond
+		    ((not (type? super))
+		     '())
+		    ((not (tclass? super))
+		     '())
+		    (else
+		     (tclass-slots super)))))
+      (let loop ((clauses clauses)
+		 (nslots '())
+		 (sslots sslots)
+		 (vnum  vnum)
+		 (index (length sslots)))
+	 (if (null? clauses)
+	     (begin
+		;; check that this class does not re-define a super class slot
+		(check-super-slots nslots sslots class)
+		(append sslots (reverse nslots)))
+	     (let ((s (car clauses)))
+		(match-case s
+		   (((id ?id) . ?attr)
+		    (multiple-value-bind (vget vset)
+		       (find-virtual-attr attr)
+		       (cond
+			  ((and vget (slot-member? (car id) nslots))
+			   (user-error/location (find-location s)
+			      (car id)
+			      "Illegal duplicated virtual slot"
+			      (car id)))
+			  ((and vget (slot-member? (car id) sslots))
+			   =>
+			   (lambda (slot)
+			      (when (and (>= (bigloo-warning) 2)
+					 *warning-overriden-slots*)
+				 (user-warning/location (find-location s)
+				    (car id)
+				    (format "Overriden \"~a\" virtual slot"
+				       (type-id (slot-class-owner slot)))
+				    (car id)))
+			      (let ((vn (slot-virtual-num slot))
+				    (vi (slot-index slot)))
+				 (loop (cdr clauses)
+				    nslots
+				    (replace
+				       (make-attribute-slot s id attr vget vset vn #t vi)
+				       slot sslots)
+				    vnum
+				    index))))
+			  (else
+			   (loop (cdr clauses)
+			      (cons (make-attribute-slot s id attr vget vset vnum #f index)
+				 nslots)
+			      sslots
+			      (if (or vget vset) (+ vnum 1) vnum)
+			      (+fx index 1))))))
+		   ((id ?id)
+		    (loop (cdr clauses)
+		       (cons (make-direct-slot id index) nslots)
+		       sslots
+		       vnum
+		       (+fx index 1)))
+		   (else
+		    (user-error/location (find-location s)
+		       (tclass-id class)
+		       "Illegal slot"
+		       s))))))))
 
 ;*---------------------------------------------------------------------*/
 ;*    make-java-class-slots ...                                        */
 ;*---------------------------------------------------------------------*/
-(define (make-java-class-slots class slots super src)
-   (trace (ast 2) "make-java-class-slots: " slots " " 0 #\Newline)
+(define (make-java-class-slots class clauses super src)
+   (trace (ast 2) "make-java-class-slots: " clauses " " 0 #\Newline)
    (let ((loc (find-location src)))
-      (define (make-java-class-slot ident jname reado? s)
+      
+      (define (make-java-class-slot ident jname reado? s index)
 	 (let* ((loc (find-location/loc s loc))
 		(slot-id (parse-id ident loc)))
 	    (instantiate::slot
 	       (id (car slot-id))
+	       (index index)
 	       (name jname)
 	       (type (find-slot-type slot-id src))
 	       (read-only? reado?)
 	       (class-owner class)
 	       (src s))))
+      
       (define (declare-static-field! id jname reado? src)
 	 (let* ((loc (find-location/loc src loc))
 		(slot-id (parse-id id loc))
@@ -333,25 +347,25 @@
 			  (cdr slot-id)))
 		(qc-id jname))
 	    (ensure-type-defined! t-id src)
-	    (let ((g (declare-global-cvar! l-id qc-id
-					   (type-id t-id) reado? src #f)))
+	    (let ((g (declare-global-cvar! l-id #f qc-id
+			(type-id t-id) reado? src #f)))
 	       (global-module-set! g (jclass-id class))
+	       (global-jvm-type-name-set! g (jclass-name class))
 	       g)))
-      (let loop ((slots slots)
-		 (res   (cond
-			   ((not (type? super))
-			    '())
-			   ((not (jclass? super))
-			    '())
-			   (else
-			    (reverse (jclass-slots super))))))
-	 (if (null? slots)
-	     ;; this is mandatory that unique-virtual-slots is called
-	     ;; on a reversed list. This function fixs up virtual slots
-	     ;; number. If the list is the wrong direction, numbers will get
-	     ;; mixed up.
+
+      (let ((sslots (cond
+		       ((not (type? super))
+			'())
+		       ((not (jclass? super))
+			'())
+		       (else
+			(reverse (jclass-slots super))))))
+      (let loop ((clauses clauses)
+		 (res sslots)
+		 (index (length sslots)))
+	 (if (null? clauses)
 	     res
-	     (let* ((s (car slots))
+	     (let* ((s (car clauses))
 		    (src (car s))
 		    (id (cadr s))
 		    (jname (caddr s))
@@ -360,10 +374,11 @@
 		(if (memq 'static mod)
 		    (begin
 		       (declare-static-field! id jname reado? src)
-		       (loop (cdr slots) res))
-		    (loop (cdr slots)
-			  (cons (make-java-class-slot id jname reado? src)
-				res))))))))
+		       (loop (cdr clauses) res index))
+		    (loop (cdr clauses)
+		       (cons (make-java-class-slot id jname reado? src index)
+			  res)
+		       (+fx 1 index)))))))))
 
 ;*---------------------------------------------------------------------*/
 ;*    find-slot-type ...                                               */
@@ -443,3 +458,4 @@
 					(type-id (slot-type (car slots))))
 		      formals))))))
 	  
+      
