@@ -1,10 +1,10 @@
 ;*=====================================================================*/
-;*    serrano/prgm/project/bigloo/comptime/Reduce/1occ.scm             */
+;*    serrano/prgm/project/bigloo/bigloo/comptime/Reduce/1occ.scm      */
 ;*    -------------------------------------------------------------    */
 ;*    Author      :  Manuel Serrano                                    */
 ;*    Creation    :  Thu Jul 13 10:29:17 1995                          */
-;*    Last change :  Thu Nov  9 15:31:05 2000 (serrano)                */
-;*    Copyright   :  1995-2000 Manuel Serrano, see LICENSE file        */
+;*    Last change :  Sat Aug 27 16:36:01 2022 (serrano)                */
+;*    Copyright   :  1995-2022 Manuel Serrano, see LICENSE file        */
 ;*    -------------------------------------------------------------    */
 ;*    The removal of the local variables appearing just once.          */
 ;*    The only goal of this pass is to prune the ast.                  */
@@ -20,20 +20,22 @@
 	    tools_error
 	    type_type
 	    type_cache
-	    coerce_typeof
+	    type_typeof
+	    type_misc
 	    coerce_coerce
 	    effect_effect
 	    ast_var
 	    ast_node
 	    ast_lvtype
-	    ast_occur)
+	    ast_occur
+	    ast_dump)
    (export  (reduce-1occ! globals)))
 
 ;*---------------------------------------------------------------------*/
 ;*    reduce-1occ! ...                                                 */
 ;*---------------------------------------------------------------------*/
 (define (reduce-1occ! globals)
-   (verbose 2 #"      single occurrence      ")
+   (verbose 2 #"      single occurrence")
    ;; we have to recompute the occurrences because both `copy-propagation
    ;; and `cse' have changed the number of occurrence (in two directions).
    (occur-var globals)
@@ -41,13 +43,13 @@
    (set! *variable-removed* 0)
    (for-each (lambda (global)
 		(let* ((fun  (global-value global))
-		       (node (sfun-body fun))) 
+		       (node (sfun-body fun)))
 		   (sfun-body-set! fun (multiple-value-bind (_ node)
 					  (node-1occ! node '())
 					  node))
 		   #unspecified))
 	     globals)
-   (verbose 2 "(removed : " *variable-removed* #\) #\newline)
+   (verbose 2 "      (removed: " *variable-removed* #\) #\newline)
    globals)
 
 ;*---------------------------------------------------------------------*/
@@ -102,6 +104,19 @@
    (with-access::sequence node (nodes)
       (values (node-1occ*! nodes 1-exp*) node)))
 
+;*---------------------------------------------------------------------*/
+;*    node-1occ! ::sync ...                                            */
+;*---------------------------------------------------------------------*/
+(define-method (node-1occ! node::sync 1-exp*)
+   (with-access::sync node (body mutex prelock)
+      (multiple-value-bind (resetm nmutex)
+	 (node-1occ! mutex 1-exp*)
+	 (set! mutex nmutex)
+	 (multiple-value-bind (resetp nprelock)
+	    (node-1occ! prelock 1-exp*)
+	    (set! prelock nprelock)
+	    (values (or (node-1occ! body 1-exp*) resetm resetp) node)))))
+
 ;*---------------------------------------------------------------------*/
 ;*    node-1occ! ::app-ly ...                                          */
 ;*---------------------------------------------------------------------*/
@@ -132,6 +147,13 @@
 (define-method (node-1occ! node::extern 1-exp*)
    (values (side-effect? node) node))
 
+;*---------------------------------------------------------------------*/
+;*    node-1occ! ::private ...                                         */
+;*---------------------------------------------------------------------*/
+(define-method (node-1occ! node::private 1-exp*)
+   (with-access::private node (expr*)
+      (values (node-1occ*! expr* 1-exp*) node)))
+
 ;*---------------------------------------------------------------------*/
 ;*    node-1occ! ::cast ...                                            */
 ;*---------------------------------------------------------------------*/
@@ -188,10 +210,10 @@
 		     (values (or reset reset' reset'') node))))))))
 
 ;*---------------------------------------------------------------------*/
-;*    node-1occ! ::select ...                                          */
+;*    node-1occ! ::switch ...                                          */
 ;*---------------------------------------------------------------------*/
-(define-method (node-1occ! node::select 1-exp*)
-   (with-access::select node (clauses test)
+(define-method (node-1occ! node::switch 1-exp*)
+   (with-access::switch node (clauses test)
       (multiple-value-bind (reset ntest)
 	 (node-1occ! test 1-exp*)
 	 (set! test ntest)
@@ -229,12 +251,38 @@
 ;*    node-1occ! ::let-var ...                                         */
 ;*---------------------------------------------------------------------*/
 (define-method (node-1occ! node::let-var 1-exp*)
+   (with-access::let-var node (body bindings loc type removable?)
+      ;; we first treat the very special case
+      ;;    (let ((var expr)) (if var ... ...))
+      ;; this special case is important in order to get good
+      ;; compile-type error messages
+      (if (and (pair? bindings)
+	       (null? (cdr bindings))
+	       (not (eq? removable? 'never))
+	       (let ((var (caar bindings)))
+		  (and (=fx (local-occurrence var) 1)
+		       (eq? (variable-type var) *bool*)
+		       (conditional? body)
+		       (with-access::conditional body (test)
+			  (and (var? test)
+			       (eq? (var-variable test) var))))))
+	  ;; yes
+	  (with-access::conditional body (test)
+	     (set! test (cdar bindings))
+	     (set! bindings '())
+	     (node-1occ! body 1-exp*))
+	  (node-1occ-let-var! node 1-exp*))))
+
+;*---------------------------------------------------------------------*/
+;*    node-1occ-let-var! ...                                           */
+;*---------------------------------------------------------------------*/
+(define (node-1occ-let-var! node::let-var 1-exp*)
    (with-access::let-var node (body bindings loc type removable?)
       (let loop ((obindings bindings)
-		 (reset     #f)
-		 (extend    '()))
+		 (reset #f)
+		 (extend '()))
 	 (if (null? obindings)
-	     (if reset
+	     (if (or reset (side-effect? body))
 		 (multiple-value-bind (reset' nbody)
 		    (node-1occ! body '())
 		    (set! body nbody)
@@ -251,10 +299,9 @@
 			     ((null? obindings)
 			      (if (and removable? (null? nbindings))
 				  (begin
-				     (trace (reduce 3)
-					    "***1occ: removing bindings: "
-					    (shape body)
-					    #\Newline)
+				     (trace (reduce 3) "***1occ: remove bindings: "
+					(shape body)
+					#\Newline)
 				     (values reset' body))
 				  (begin
 				     (set! bindings (reverse! nbindings))
@@ -263,14 +310,13 @@
 				    (val (cdr (car obindings))))
 				 (and (=fx (local-occurrence var) 0)
 				      (not (side-effect? val))))
-			      (trace (reduce 3)
-				     "***1occ: removing: "
-				     (shape (car obindings))
-				     #\Newline)
+			      (trace (reduce 3) "***1occ: remove: "
+				 (shape (car obindings))
+				 #\Newline)
 			      (loop (cdr obindings) nbindings))
 			     (else
 			      (loop (cdr obindings)
-				    (cons (car obindings) nbindings))))))))
+				 (cons (car obindings) nbindings))))))))
 	     (let ((binding (car obindings)))
 		(let ((var (car binding))
 		      (val (cdr binding)))
@@ -279,48 +325,32 @@
 		      (set-cdr! binding nval)
 		      (cond
 			 ((or reset reset')
-			  (loop (cdr obindings)
-				#t
-				'()))
+			  (loop (cdr obindings) #t '()))
 			 ((not (eq? (local-access var) 'read))
-			  (loop (cdr obindings)
-				#f
-				extend))
+			  (loop (cdr obindings) #f extend))
 			 ((and (=fx (local-occurrence var) 1)
 			       (eq? (local-access var) 'read)
-			       (not (side-effect? val)))
-			  (trace (reduce 3)
-				 "***1occ: applying: "
-				 (shape var)
-				 " "
-				 (shape val)
-				 #\Newline)
-			  (loop (cdr obindings)
-				#f
-				(cons binding extend)))
+			       (not (side-effect? val))
+			       (type-less-specific? (local-type var)
+				  (get-type val #f)))
+			  (trace (reduce 3) "***1occ: apply: "
+			     (shape var) " " (shape val) #\Newline)
+			  (loop (cdr obindings) #f (cons binding extend)))
 			 (else
-			  (if (and (=fx (local-occurrence var) 1)
-				   (eq? (local-access var) 'read)
-				   (side-effect? val))
-			      (trace (reduce 3)
-				     "***1occ: NOT applying: "
-				     (shape var)
-				     " "
-				     (shape val)
-				     #\Newline))
-			  (loop (cdr obindings)
-				#f
-				extend))))))))))
+			  (loop (cdr obindings) #f extend))))))))))
 
 ;*---------------------------------------------------------------------*/
 ;*    node-1occ! ::set-ex-it ...                                       */
 ;*---------------------------------------------------------------------*/
 (define-method (node-1occ! node::set-ex-it 1-exp*)
-   (with-access::set-ex-it node (var body)
+   (with-access::set-ex-it node (var body onexit)
       (multiple-value-bind (reset nbody)
 	 (node-1occ! body 1-exp*)
 	 (set! body nbody)
-	 (values reset node))))
+	 (multiple-value-bind (reset' nonexit)
+	    (node-1occ! onexit 1-exp*)
+	    (set! onexit nonexit)
+	    (values (or reset reset') node)))))
 
 ;*---------------------------------------------------------------------*/
 ;*    node-1occ! ::jump-ex-it ...                                      */
@@ -336,6 +366,26 @@
 	       (set! value nvalue)
 	       (values (or reset reset') node))))))
 
+;*---------------------------------------------------------------------*/
+;*    node-1occ! ::retblock ...                                        */
+;*---------------------------------------------------------------------*/
+(define-method (node-1occ! node::retblock 1-exp*)
+   (with-access::retblock node (body)
+      (multiple-value-bind (reset nbody)
+	 (node-1occ! body 1-exp*)
+	 (set! body nbody)
+	 (values reset node))))
+
+;*---------------------------------------------------------------------*/
+;*    node-1occ! ::return ...                                          */
+;*---------------------------------------------------------------------*/
+(define-method (node-1occ! node::return 1-exp*)
+   (with-access::return node (value)
+      (multiple-value-bind (reset nvalue)
+	 (node-1occ! value 1-exp*)
+	 (set! value nvalue)
+	 (values reset node))))
+
 ;*---------------------------------------------------------------------*/
 ;*    node-1occ! ::make-box ...                                        */
 ;*---------------------------------------------------------------------*/
@@ -355,7 +405,7 @@
 	 (node-1occ! value 1-exp*)
 	 (set! value nvalue)
 	 (values reset node))))
-
+ 
 ;*---------------------------------------------------------------------*/
 ;*    node-1occ! ::box-ref ...                                         */
 ;*---------------------------------------------------------------------*/
