@@ -1,10 +1,10 @@
 ;*=====================================================================*/
-;*    serrano/prgm/project/bigloo/comptime/Reduce/copy.scm             */
+;*    serrano/prgm/project/bigloo/bigloo/comptime/Reduce/copy.scm      */
 ;*    -------------------------------------------------------------    */
 ;*    Author      :  Manuel Serrano                                    */
 ;*    Creation    :  Thu Jul 13 10:29:17 1995                          */
-;*    Last change :  Thu Dec 28 18:31:53 2000 (serrano)                */
-;*    Copyright   :  1995-2000 Manuel Serrano, see LICENSE file        */
+;*    Last change :  Sat Aug 27 16:44:34 2022 (serrano)                */
+;*    Copyright   :  1995-2022 Manuel Serrano, see LICENSE file        */
 ;*    -------------------------------------------------------------    */
 ;*    The reduction of type checks.                                    */
 ;*=====================================================================*/
@@ -14,12 +14,13 @@
 ;*---------------------------------------------------------------------*/
 (module reduce_copy
    (include "Tools/trace.sch")
-   (import  tools_shape
+   (import  engine_param
+	    tools_shape
 	    tools_speek
 	    tools_error
 	    type_type
 	    type_cache
-	    coerce_typeof
+	    type_misc
 	    coerce_coerce
 	    effect_effect
 	    ast_var
@@ -36,14 +37,13 @@
    (set! *copy-removed* 0)
    (for-each (lambda (global)
 		(let* ((fun  (global-value global))
-		       (node (sfun-body fun))) 
+		       (node (sfun-body fun)))
 		   (sfun-body-set! fun (node-copy! node))
 		   #unspecified))
 	     globals)
-   (verbose 2 "(removed : " *copy-removed* #\) #\newline)
+   (verbose 2 "(removed: " *copy-removed* #\) #\newline)
    globals)
 
-
 ;*---------------------------------------------------------------------*/
 ;*    Statitics ...                                                    */
 ;*---------------------------------------------------------------------*/
@@ -90,6 +90,16 @@
       (node-copy*! nodes)
       node))
 
+;*---------------------------------------------------------------------*/
+;*    node-copy! ::sync ...                                            */
+;*---------------------------------------------------------------------*/
+(define-method (node-copy! node::sync)
+   (with-access::sync node (body mutex prelock)
+      (set! mutex (node-copy! mutex))
+      (set! prelock (node-copy! prelock))
+      (set! body (node-copy! body))
+      node))
+
 ;*---------------------------------------------------------------------*/
 ;*    node-copy! ::app-ly ...                                          */
 ;*---------------------------------------------------------------------*/
@@ -154,10 +164,10 @@
       node))
 
 ;*---------------------------------------------------------------------*/
-;*    node-copy! ::select ...                                          */
+;*    node-copy! ::switch ...                                          */
 ;*---------------------------------------------------------------------*/
-(define-method (node-copy! node::select)
-   (with-access::select node (clauses test)
+(define-method (node-copy! node::switch)
+   (with-access::switch node (clauses test)
       (set! test (node-copy! test))
       (for-each (lambda (clause)
 		   (set-cdr! clause (node-copy! (cdr clause))))
@@ -176,6 +186,76 @@
       (set! body (node-copy! body))
       node))
 
+;*---------------------------------------------------------------------*/
+;*    copyable? ...                                                    */
+;*    -------------------------------------------------------------    */
+;*    Is an expression "copyable" in the copy optimization?            */
+;*---------------------------------------------------------------------*/
+(define-generic (copyable? node v)
+   #f)
+
+;*---------------------------------------------------------------------*/
+;*    copyable? ::atom ...                                             */
+;*---------------------------------------------------------------------*/
+(define-method (copyable? node::atom v)
+   #t)
+
+;*---------------------------------------------------------------------*/
+;*    copyable? ::var ...                                              */
+;*---------------------------------------------------------------------*/
+(define-method (copyable? node::var v)
+   (eq? (variable-access (var-variable node)) 'read))
+
+;*---------------------------------------------------------------------*/
+;*    copyable? ::sequence ...                                         */
+;*---------------------------------------------------------------------*/
+(define-method (copyable? node::sequence v)
+   (with-access::sequence node (nodes)
+      (every (lambda (n) (copyable? n v)) nodes)))
+
+;*---------------------------------------------------------------------*/
+;*    copyable? ::vlength ...                                          */
+;*---------------------------------------------------------------------*/
+(define-method (copyable? node::vlength v)
+   (with-access::vlength node (expr*)
+      (every (lambda (n) (copyable? n v)) expr*)))
+
+;*---------------------------------------------------------------------*/
+;*    copyable? ::cast ...                                             */
+;*---------------------------------------------------------------------*/
+(define-method (copyable? node::cast v)
+   (with-access::cast node (arg)
+      (copyable? arg v)))
+
+;*---------------------------------------------------------------------*/
+;*    copyable? ::cast-null ...                                        */
+;*---------------------------------------------------------------------*/
+(define-method (copyable? node::cast-null v)
+   (with-access::cast-null node (expr*)
+      (every (lambda (n) (copyable? n v)) expr*)))
+
+;*---------------------------------------------------------------------*/
+;*    copyable? ::instanceof ...                                       */
+;*---------------------------------------------------------------------*/
+(define-method (copyable? node::instanceof v)
+   (with-access::instanceof node (expr*)
+      (every (lambda (n) (copyable? n v)) expr*)))
+
+;*---------------------------------------------------------------------*/
+;*    copyable? ::conditional ...                                      */
+;*---------------------------------------------------------------------*/
+(define-method (copyable? node::conditional v)
+   (with-access::conditional node (test true false)
+      (when (<=fx (variable-occurrence v) 1)
+	 (and (copyable? test v) (copyable? true v) (copyable? false v)))))
+
+;*---------------------------------------------------------------------*/
+;*    copyable? ::fail ...                                             */
+;*---------------------------------------------------------------------*/
+(define-method (copyable? node::fail v)
+   (with-access::fail node (proc msg obj)
+      (and (copyable? proc v) (copyable? msg v) (copyable? obj v))))
+
 ;*---------------------------------------------------------------------*/
 ;*    node-copy! ::let-var ...                                         */
 ;*---------------------------------------------------------------------*/
@@ -191,37 +271,59 @@
 		    (set! body (node-copy! body))
 		    node))
 	     (let* ((binding (car obindings))
-		    (var     (car binding))
-		    (val     (node-copy! (cdr binding))))
+		    (var (car binding))
+		    (val (node-copy! (cdr binding))))
 		(set-cdr! binding val)
 		;; we test a bit more than exact copy propagation
 		;; because we check if val is also an atom.
-		(trace (reduce 3)
-		       "copy-propagation: var:" (shape var) " ["
-		       (variable-access var)
-		       "]  val: " (shape val) " "
-		       (if (var? val) (variable-access (var-variable val)) "?")
-		       "]" #\Newline)
-		(if (and (or (atom? val)
-			     (and (var? val)
-				  (eq? (variable-access (var-variable val))
-				       'read)))
-			 (eq? (variable-access var) 'read))
-		    (begin
+		(trace (reduce reduce+ 3)
+		   "copy-propagation: var:" (shape var) " ["
+		   (variable-access var)
+		   "] val=" (shape val) " "
+		   (if (var? val) (variable-access (var-variable val)) "?")
+		   "] copyable=" (copyable? val var) " type="
+		   (cell-type-less-specific?
+		      (node-type val)
+		      (variable-type var))
+		   " tval=" (shape (node-type val))
+		   " tvar=" (shape (variable-type var))
+		   #\Newline)
+		(if (and removable?
+			 (eq? (variable-access var) 'read)
+			 (copyable? val var)
+			 (cell-type-less-specific?
+			    (node-type val)
+			    (variable-type var)))
+ 		    (begin
 		       ;; we propagate the copy
-		       (trace (reduce 3) "copy: Je reduis..." #\Newline)
+		       (trace (reduce reduce+ 3) "copy: reducing: "
+			  (shape val) " -> " (shape var) #\Newline)
 		       (set! *copy-removed* (+fx *copy-removed* 1))
-		       (variable-fast-alpha-set! var val)
+		       (variable-fast-alpha-set! var
+			  (if (eq? (node-type val) (variable-type var))
+			      val
+			      (instantiate::cast
+				 (loc (node-loc val))
+				 (type (variable-type var))
+				 (arg val))))
 		       (loop (cdr obindings) nbindings))
 		    (loop (cdr obindings) (cons binding nbindings))))))))
 
+;*---------------------------------------------------------------------*/
+;*    cell-type-less-specific? ...                                     */
+;*---------------------------------------------------------------------*/
+(define (cell-type-less-specific? ty1 ty2)
+   (or (type-less-specific? ty1 ty2)
+       (and (eq? ty2 *obj*) (eq? ty1 *cell*))))
+
 ;*---------------------------------------------------------------------*/
 ;*    node-copy! ::set-ex-it ...                                       */
 ;*---------------------------------------------------------------------*/
 (define-method (node-copy! node::set-ex-it)
-   (with-access::set-ex-it node (var body)
+   (with-access::set-ex-it node (var body onexit)
       (set! body (node-copy! body))
       (set! var (node-copy! var))
+      (set! onexit (node-copy! onexit))
       node))
 
 ;*---------------------------------------------------------------------*/
@@ -233,6 +335,22 @@
       (set! value (node-copy! value))
       node))
 
+;*---------------------------------------------------------------------*/
+;*    node-copy! ::retblock ...                                        */
+;*---------------------------------------------------------------------*/
+(define-method (node-copy! node::retblock)
+   (with-access::retblock node (body)
+      (set! body (node-copy! body))
+      node))
+
+;*---------------------------------------------------------------------*/
+;*    node-copy! ::return ...                                          */
+;*---------------------------------------------------------------------*/
+(define-method (node-copy! node::return)
+   (with-access::return node (value)
+      (set! value (node-copy! value))
+      node))
+
 ;*---------------------------------------------------------------------*/
 ;*    node-copy! ::make-box ...                                        */
 ;*---------------------------------------------------------------------*/
@@ -245,18 +363,32 @@
 ;*    node-copy! ::box-set! ...                                        */
 ;*---------------------------------------------------------------------*/
 (define-method (node-copy! node::box-set!)
-   (with-access::box-set! node (var value)
-      (set! var (node-copy! var))
+   (with-access::box-set! node (var value) 
       (set! value (node-copy! value))
-      node))
+      (let ((cp (node-copy! var)))
+	 (if (isa? cp cast)
+	     (with-access::cast cp (arg)
+		(set! var arg)
+		(set! arg node)
+		cp)
+	     (begin
+		(set! var cp)
+		node)))))
 
 ;*---------------------------------------------------------------------*/
 ;*    node-copy! ::box-ref ...                                         */
 ;*---------------------------------------------------------------------*/
 (define-method (node-copy! node::box-ref)
    (with-access::box-ref node (var)
-      (set! var (node-copy! var))
-      node))
+      (let ((cp (node-copy! var)))
+	 (if (isa? cp cast)
+	     (with-access::cast cp (arg)
+		(set! var arg)
+		(set! arg node)
+		cp)
+	     (begin
+		(set! var cp)
+		node)))))
 
 ;*---------------------------------------------------------------------*/
 ;*    node-copy! ::app ...                                             */
