@@ -1,12 +1,12 @@
 ;*=====================================================================*/
-;*    serrano/prgm/project/bigloo/comptime/Reduce/cse.scm              */
+;*    serrano/prgm/project/bigloo/bigloo/comptime/Reduce/cse.scm       */
 ;*    -------------------------------------------------------------    */
 ;*    Author      :  Manuel Serrano                                    */
 ;*    Creation    :  Thu Jul 13 10:29:17 1995                          */
-;*    Last change :  Sun Jan 21 03:53:47 2001 (serrano)                */
-;*    Copyright   :  1995-2001 Manuel Serrano, see LICENSE file        */
+;*    Last change :  Thu Jul  8 11:31:48 2021 (serrano)                */
+;*    Copyright   :  1995-2021 Manuel Serrano, see LICENSE file        */
 ;*    -------------------------------------------------------------    */
-;*    The reduction of type checks.                                    */
+;*    Common sub-expression elimination.                               */
 ;*=====================================================================*/
 
 ;*---------------------------------------------------------------------*/
@@ -19,7 +19,6 @@
 	    tools_error
 	    type_type
 	    type_cache
-	    coerce_typeof
 	    coerce_coerce
 	    effect_effect
 	    ast_var
@@ -43,7 +42,7 @@
 					   node))
 		   #unspecified))
 	     globals)
-   (verbose 2 "(removed : " *cse-removed* #\) #\newline)
+   (verbose 2 "(removed: " *cse-removed* #\) #\newline)
    globals)
 
 ;*---------------------------------------------------------------------*/
@@ -89,6 +88,19 @@
    (with-access::sequence node (nodes)
       (values (node-cse*! nodes stack) node)))
 
+;*---------------------------------------------------------------------*/
+;*    node-cse! ::sync ...                                             */
+;*---------------------------------------------------------------------*/
+(define-method (node-cse! node::sync stack)
+   (with-access::sync node (body mutex prelock)
+      (multiple-value-bind (resetm nmutex)
+	 (node-cse! mutex stack)
+	 (set! mutex nmutex)
+	 (multiple-value-bind (resetp nprelock)
+	    (node-cse! prelock stack)
+	    (set! prelock nprelock)
+	    (values (or (node-cse! body stack) resetm resetp) node)))))
+
 ;*---------------------------------------------------------------------*/
 ;*    node-cse! ::app-ly ...                                           */
 ;*---------------------------------------------------------------------*/
@@ -174,10 +186,10 @@
 		     (values (or reset reset' reset'') node))))))))
    
 ;*---------------------------------------------------------------------*/
-;*    node-cse! ::select ...                                           */
+;*    node-cse! ::switch ...                                           */
 ;*---------------------------------------------------------------------*/
-(define-method (node-cse! node::select stack)
-   (with-access::select node (clauses test)
+(define-method (node-cse! node::switch stack)
+   (with-access::switch node (clauses test)
       (multiple-value-bind (reset ntest)
 	 (node-cse! test stack)
 	 (set! test ntest)
@@ -215,11 +227,14 @@
 ;*    node-cse! ::set-ex-it ...                                        */
 ;*---------------------------------------------------------------------*/
 (define-method (node-cse! node::set-ex-it stack)
-   (with-access::set-ex-it node (var body)
+   (with-access::set-ex-it node (var body onexit)
       (multiple-value-bind (reset nbody)
 	 (node-cse! body '())
 	 (set! body nbody)
-	 (values reset node))))
+	 (multiple-value-bind (reset' nonexit)
+	    (node-cse! onexit '())
+	    (set! onexit nonexit)
+	    (values (or reset reset') node)))))
 
 ;*---------------------------------------------------------------------*/
 ;*    node-cse! ::jump-ex-it ...                                       */
@@ -235,6 +250,26 @@
 	       (set! value nvalue)
 	       (values (or reset reset') node))))))
 
+;*---------------------------------------------------------------------*/
+;*    node-cse! ::retblock ...                                         */
+;*---------------------------------------------------------------------*/
+(define-method (node-cse! node::retblock stack)
+   (with-access::retblock node (body)
+      (multiple-value-bind (reset nbody)
+	 (node-cse! body stack)
+	 (set! body nbody)
+	 (values reset node))))
+
+;*---------------------------------------------------------------------*/
+;*    node-cse! ::return ...                                           */
+;*---------------------------------------------------------------------*/
+(define-method (node-cse! node::return stack)
+   (with-access::return node (value)
+      (multiple-value-bind (reset nvalue)
+	 (node-cse! value stack)
+	 (set! value nvalue)
+	 (values reset node))))
+
 ;*---------------------------------------------------------------------*/
 ;*    node-cse! ::make-box ...                                         */
 ;*---------------------------------------------------------------------*/
@@ -277,7 +312,7 @@
 		   ((variable? previous)
 		    (set! *cse-removed* (+fx 1 *cse-removed*))
 		    (trace (reduce 2) "***cse: " (shape node) #\Newline)
-		    (values stack (instantiate::var
+		    (values stack (instantiate::ref
 				     (loc loc)
 				     (type type)
 				     (variable previous))))
@@ -312,7 +347,7 @@
 			      (set! *cse-removed* (+fx 1 *cse-removed*))
 			      (trace (reduce 2) "***cse: " (shape node)
 				     #\Newline)
-			      (values stack' (instantiate::var
+			      (values stack' (instantiate::ref
 						(loc loc)
 						(type type)
 						(variable previous))))
