@@ -1,10 +1,10 @@
 ;*=====================================================================*/
-;*    serrano/prgm/project/bigloo/comptime/Reduce/typec.scm            */
+;*    serrano/prgm/project/bigloo/bigloo/comptime/Reduce/typec.scm     */
 ;*    -------------------------------------------------------------    */
 ;*    Author      :  Manuel Serrano                                    */
 ;*    Creation    :  Thu Jul 13 10:29:17 1995                          */
-;*    Last change :  Tue Apr 17 14:30:34 2001 (serrano)                */
-;*    Copyright   :  1995-2001 Manuel Serrano, see LICENSE file        */
+;*    Last change :  Thu Nov  3 11:06:44 2022 (serrano)                */
+;*    Copyright   :  1995-2022 Manuel Serrano, see LICENSE file        */
 ;*    -------------------------------------------------------------    */
 ;*    The reduction of type checks.                                    */
 ;*=====================================================================*/
@@ -19,30 +19,43 @@
 	    tools_error
 	    type_type
 	    type_cache
-	    coerce_typeof
+	    type_typeof
+	    type_misc
+	    type_coercion
 	    coerce_coerce
 	    effect_effect
 	    ast_var
 	    ast_node
 	    ast_lvtype
+	    ast_env
+	    ast_dump
 	    object_class)
-   (export  (reduce-type-check! globals)))
+   (export  (reduce-type-check! globals::pair-nil)))
 
 ;*---------------------------------------------------------------------*/
 ;*    reduce-type-check! ...                                           */
 ;*---------------------------------------------------------------------*/
 (define (reduce-type-check! globals)
    (verbose 2 #"      type check             ")
+   (set! *pair?* (find-global/module '$pair? 'foreign))
+   (set! *null?* (find-global/module '$null? 'foreign))
    (for-each (lambda (global)
 		(let* ((fun  (global-value global))
 		       (node (sfun-body fun))) 
 		   (sfun-body-set! fun (node-typec! node))
 		   #unspecified))
 	     globals)
-   (verbose 2 "(removed : " *type-checks-removed* #")\t")
-   (verbose 2 "(remaining : " *type-checks-remaining* #\) #\newline)
+   (verbose 2 "(removed: " *type-checks-removed*
+	    ", remaining: " *type-checks-remaining* #")\n")
+   (set! *pair?* #f)
+   (set! *null?* #f)
    globals)
 
+;*---------------------------------------------------------------------*/
+;*    *pair?* ...                                                      */
+;*---------------------------------------------------------------------*/
+(define *pair?* #f)
+(define *null?* #f)
 
 ;*---------------------------------------------------------------------*/
 ;*    Statitics ...                                                    */
@@ -87,6 +100,16 @@
       (node-typec*! nodes)
       node))
 
+;*---------------------------------------------------------------------*/
+;*    node-typec! ::sync ...                                           */
+;*---------------------------------------------------------------------*/
+(define-method (node-typec! node::sync)
+   (with-access::sync node (body mutex prelock)
+      (set! mutex (node-typec! mutex))
+      (set! prelock (node-typec! prelock))
+      (set! body (node-typec! body))
+      node))
+
 ;*---------------------------------------------------------------------*/
 ;*    node-typec! ::app-ly ...                                         */
 ;*---------------------------------------------------------------------*/
@@ -135,10 +158,24 @@
 ;*---------------------------------------------------------------------*/
 (define-method (node-typec! node::conditional)
    (with-access::conditional node (test true false)
-       (set! test (node-typec! test))
-       (set! true (node-typec! true))
-       (set! false (node-typec! false))
-       node))
+      (set! test (node-typec! test))
+      (set! true (node-typec! true))
+      (set! false (node-typec! false))
+      (when (pair-of-pair-nil? test)
+	 ;; an expression
+	 ;;     (if (pair? x) e1 e2)
+	 ;; is replaced with
+	 ;;     (if (null? x) e2 e1)
+	 ;; if the compiler knows that x is ::pair-nil because testing null?
+	 ;; is faster than testing pair?
+	 (set-null-test! test)
+	 (let ((otrue true))
+	    (set! true false)
+	    (set! false otrue)
+	    (trace (reduce 3) "typec: inverting pair?/null? test: "
+		   (shape node)
+		   #\Newline)))
+      node))
 
 ;*---------------------------------------------------------------------*/
 ;*    node-typec! ::fail ...                                           */
@@ -151,10 +188,10 @@
       node))
 
 ;*---------------------------------------------------------------------*/
-;*    node-typec! ::select ...                                         */
+;*    node-typec! ::switch ...                                         */
 ;*---------------------------------------------------------------------*/
-(define-method (node-typec! node::select)
-   (with-access::select node (clauses test)
+(define-method (node-typec! node::switch)
+   (with-access::switch node (clauses test)
       (set! test (node-typec! test))
       (for-each (lambda (clause)
 		   (set-cdr! clause (node-typec! (cdr clause))))
@@ -188,8 +225,9 @@
 ;*    node-typec! ::set-ex-it ...                                      */
 ;*---------------------------------------------------------------------*/
 (define-method (node-typec! node::set-ex-it)
-   (with-access::set-ex-it node (var body)
+   (with-access::set-ex-it node (var body onexit)
       (set! body (node-typec! body))
+      (set! onexit (node-typec! onexit))
       (set! var (node-typec! var))
       node))
 
@@ -202,6 +240,22 @@
       (set! value (node-typec! value))
       node))
 
+;*---------------------------------------------------------------------*/
+;*    node-typec! ::retblock ...                                       */
+;*---------------------------------------------------------------------*/
+(define-method (node-typec! node::retblock)
+   (with-access::retblock node (body)
+      (set! body (node-typec! body))
+      node))
+
+;*---------------------------------------------------------------------*/
+;*    node-typec! ::return ...                                         */
+;*---------------------------------------------------------------------*/
+(define-method (node-typec! node::return)
+   (with-access::return node (value)
+      (set! value (node-typec! value))
+      node))
+
 ;*---------------------------------------------------------------------*/
 ;*    node-typec! ::make-box ...                                       */
 ;*---------------------------------------------------------------------*/
@@ -247,53 +301,94 @@
 ;*    @ref ../../runtime/Llib/type.scm:pair-nil subtyping@             */
 ;*---------------------------------------------------------------------*/
 (define-method (node-typec! node::app)
-   (define (compatible-type? typec typep)
+
+   (define (type-check-disjoint? typec typea)
       (cond
-	 ((eq? typec typep)
-	  #t)
-	 ((type-subclass? typep typec)
-	  #t)
-	 ((and (or (eq? typec *pair*) (eq? typec *epair*) (eq? typec *bnil*))
-	       (eq? typep *pair-nil*))
-	  #t)
+	 ((eq? typec typea)
+	  #f)
+	 ((eq? typea *_*)
+	  #f)
+	 ((and (bigloo-type? typec) (not (bigloo-type? typea)))
+	  (not (find-coercer typea typec)))
+	 ((or (type-less-specific? typec typea)
+	      (type-less-specific? typea typec))
+	  #f)
+	 ((or (find-coercer typec typea) (find-coercer typea typec))
+	  #f)
+	 ((and (foreign-type? typec) (foreign-type? typea))
+	  #f)
+	 ((and (eq? typec *foreign*) (or (jclass? typea) (foreign-type? typea)))
+	  #f)
 	 (else
-	  #f)))
-   (with-access::app node (fun args loc)
+	  #t)))
+   
+   (define (check-type node typec typea)
+      (with-access::app node (loc)
+	 (let ((type  (get-type node #f)))
+	    (cond
+	       ((type-less-specific? typec typea)
+		(set! *type-checks-removed* (+fx 1 *type-checks-removed*))
+		(trace (reduce 3) "typec: reducing: "
+		   (shape node) " => #t" #\Newline)
+		(let ((node (coerce! (instantiate::literal
+					(loc loc)
+					(type type)
+					(value #t))
+			       #unspecified
+			       type
+			       #f)))
+		   node))
+	       ((type-check-disjoint? typec typea)
+		(set! *type-checks-removed* (+fx 1 *type-checks-removed*))
+		(trace (reduce 3) "typec: reducing: "
+		   (shape node) " => #f (typec="
+		   (shape typec) " " (foreign-type? typec)
+		   " typea=" (shape typea) " " (jclass? typea) ")"
+		   #\Newline)
+		(let ((node (coerce! (instantiate::literal
+					(loc loc)
+					(type type)
+					(value #f))
+			       #unspecified
+			       type
+			       #f)))
+		   node))
+	       (else
+		(set! *type-checks-remaining* (+fx 1 *type-checks-remaining*))
+		node)))))
+   
+   (with-access::app node (fun args)
       (node-typec*! args)
       (let* ((var   (var-variable fun))
-	     (typec (fun-predicate-of (variable-value var)))
-	     (type  (typeof node)))
-	 (if (and (pair? args)
+	     (typec (fun-predicate-of (variable-value var))))
+	 (cond
+	    ((and (pair? args)
 		  (null? (cdr args))
 		  (type? typec)
-		  (not (side-effect? (car args))))
-	     (let ((typep (typeof (car args))))
-		(cond
-		   ((or (eq? typep *obj*) (type-subclass? typec typep)
-			(and (or (eq? typep *pair*)
-				 (eq? typep *epair*)
-				 (eq? typep *bnil*))
-			     (eq? typec *pair-nil*))
-			(eq? typep *pair-nil*))
-		    (set! *type-checks-remaining*
-			  (+fx 1 *type-checks-remaining*))
-		    node)
-		   (else
-		    (begin
-		       (trace (reduce 2)
-			      "Je reduis: " (shape node) " en: " (shape type)
-			      " car: typep: " (shape typep) " typec: "
-			      (shape typec) #\Newline)
-		       (set! *type-checks-removed*
-			     (+fx 1 *type-checks-removed*))
-		       (let ((check-val (compatible-type? typec typep)))
-			  (let ((node (coerce! (instantiate::atom
-						  (loc loc)
-						  (type type)
-						  (value check-val))
-					       #unspecified
-					       type)))
-			     (lvtype-node! node)
-			     node))))))
-	     node))))
+		  (or (not (side-effect? (car args))) (var? (car args))))
+	     (check-type node typec (get-type (car args) #f)))
+	    ((isa-of node)
+	     =>
+	     (lambda (typec)
+		(check-type node typec (get-type (car args) #f))))
+	    (else
+	     node)))))
 
+;*---------------------------------------------------------------------*/
+;*    pair-of-pair-nil? ...                                            */
+;*---------------------------------------------------------------------*/
+(define (pair-of-pair-nil? node)
+   (when (app? node)
+      (with-access::app node (fun args loc)
+	 (and (eq? (var-variable fun) *pair?*)
+	      (pair? args)
+	      (null? (cdr args))
+	      (eq? (get-type (car args) #f) *pair-nil*)))))
+
+;*---------------------------------------------------------------------*/
+;*    set-null-test! ...                                               */
+;*---------------------------------------------------------------------*/
+(define (set-null-test! node::app)
+   (with-access::app node (fun)
+      (with-access::var fun (variable)
+	 (set! variable *null?*))))
