@@ -1,10 +1,10 @@
 ;*=====================================================================*/
-;*    serrano/prgm/project/bigloo/comptime/Tools/error.scm             */
+;*    serrano/prgm/project/bigloo/bigloo/comptime/Tools/error.scm      */
 ;*    -------------------------------------------------------------    */
 ;*    Author      :  Manuel Serrano                                    */
 ;*    Creation    :  Sun Dec 25 10:47:51 1994                          */
-;*    Last change :  Mon May 15 08:02:25 2000 (serrano)                */
-;*    Copyright   :  1994-2000 Manuel Serrano, see LICENSE file        */
+;*    Last change :  Fri Apr 12 11:04:42 2019 (serrano)                */
+;*    Copyright   :  1994-2019 Manuel Serrano, see LICENSE file        */
 ;*    -------------------------------------------------------------    */
 ;*    Error utilities                                                  */
 ;*=====================================================================*/
@@ -20,18 +20,20 @@
 	    engine_param
 	    tools_location
 	    tools_trace
+	    tools_shape
 	    init_main)
    (export  *nb-error-on-pass*
-	    (internal-error        <obj> <obj> <obj>)
-	    (user-warning          <obj> <obj> <obj>)
+	    (internal-error <obj> <obj> <obj>)
+	    (user-warning <obj> <obj> <obj>)
 	    (user-warning/location ::obj <obj> <obj> <obj>)
-	    (user-error            <obj> <obj> <obj> . <obj>)
-	    (user-error/location   ::obj <obj> <obj> <obj> . <obj>)
+	    (user-error-notify ::obj ::symbol)
+	    (user-error <obj> <obj> <obj> . <obj>)
+	    (user-error/location ::obj <obj> <obj> <obj> . <obj>)
 	    (current-function)
-	    (enter-function        ::symbol)
+	    (enter-function ::symbol)
 	    (leave-function)
 	    (no-warning thunk)))
-
+ 
 ;*---------------------------------------------------------------------*/
 ;*    *nb-error-on-pass* ...                                           */
 ;*---------------------------------------------------------------------*/
@@ -50,20 +52,32 @@
 	   #\Newline
 	   *bigloo-author* " [" *bigloo-email* "], thank you.)")
    (error proc mes obj)
-   (exit-bigloo -1))
+   (compiler-exit 1))
 
+;*---------------------------------------------------------------------*/
+;*    with-dump-stack ...                                              */
+;*---------------------------------------------------------------------*/
+(define (with-dump-stack thunk)
+   (if *compiler-stack-debug?*
+       (thunk)
+       (let ((st (bigloo-trace-stack-depth)))
+	  (bigloo-trace-stack-depth-set! 0)
+	  (unwind-protect
+	     (thunk)
+	     (bigloo-trace-stack-depth-set! st)))))
+   
 ;*---------------------------------------------------------------------*/
 ;*    user-warning/location ...                                        */
 ;*---------------------------------------------------------------------*/
 (define (user-warning/location loc proc mes obj)
-   (if (not (location? loc))
-       (warning proc mes " -- " obj)
-       (warning/location (location-fname loc)
-			 (location-pos loc)
-			 proc
-			 mes
-			 " -- "
-			 obj)))
+   (when (>fx (bigloo-warning) 0)
+      (with-dump-stack
+	 (lambda ()
+	    (if (not (location? loc))
+		(warning proc mes " -- " obj)
+		(warning/location (location-full-fname loc)
+		   (location-pos loc) proc
+		   mes " -- " obj))))))
    
 ;*---------------------------------------------------------------------*/
 ;*    user-warning ...                                                 */
@@ -75,66 +89,80 @@
 ;*    no-warning ...                                                   */
 ;*---------------------------------------------------------------------*/
 (define (no-warning thunk)
-   (let ((warning *warning*))
-      (set! *warning* #f)
+   (let ((warning (bigloo-warning)))
+      (bigloo-warning-set! 0)
       (let ((val (thunk)))
-	 (set! *warning* warning)
+	 (bigloo-warning-set! warning)
 	 val)))
-	 
+
+;*---------------------------------------------------------------------*/
+;*    user-error-notify ...                                            */
+;*---------------------------------------------------------------------*/
+(define (user-error-notify e proc)
+   (when (isa? e &error)
+      (with-dump-stack
+	 (lambda ()
+	    (with-access::&error e (obj (p proc))
+	       (let ((loc (find-location obj)))
+		  (set! *nb-error-on-pass* (+fx *nb-error-on-pass* 1))
+		  (if (location? loc)
+		      (error-notify (duplicate::&error e
+				       (proc (or p proc))
+				       (fname (location-full-fname loc))
+				       (location (location-pos loc))))
+		      (begin
+			 (unless p (set! p proc))
+			 (error-notify e)))))))))
+
 ;*---------------------------------------------------------------------*/
 ;*    user-error ...                                                   */
 ;*---------------------------------------------------------------------*/
 (define (user-error proc mes obj . continue)
    (if (pair? continue)
-       (user-error/location (find-location obj) proc mes obj
+       (user-error/location (find-location obj) proc mes (shape obj)
 			    (car continue))
-       (user-error/location (find-location obj) proc mes obj)))
+       (user-error/location (find-location obj) proc mes (shape obj))))
 
 ;*---------------------------------------------------------------------*/
 ;*    user-error/location ...                                          */
 ;*---------------------------------------------------------------------*/
-(define (user-error/location loc proc mes obj . continue)
-   (if (output-port? *trace-port*)
-       (fprint *trace-port*  "*** ERROR:" proc ":" mes ":" obj))
-   (set! *nb-error-on-pass* (+fx *nb-error-on-pass* 1))
-   (let* ((proc-string (cond
-			  ((string? proc)
-			   proc)
-			  ((symbol? proc)
-			   (symbol->string proc))
-			  (else
-			   #f)))
-	  (fun-string  (symbol->string (current-function)))
-	  (proc        (if (and (string? proc-string)
-				(not (string=? proc-string fun-string)))
-			   (string-append fun-string ":" proc-string)
-			   fun-string)))
-      (let ((obj-prn  (let ((port (open-output-string)))
-			 (display obj port)
-			 (let ((string (close-output-port port)))
-			    (if (>fx (string-length string) 45)
-				(string-append (substring string 0 44) " ...")
-				string))))
-	    (handler  (lambda (escape proc mes obj)
-			 (notify-error proc mes obj)
-			 (escape (car continue)))))
-	 (if (location? loc)
-	     (if (pair? continue)
-		 (try (error/location proc
-				      mes
-				      obj-prn
-				      (location-fname loc)
-				      (location-pos   loc))
-		      handler) 
-		 (error/location proc
-				 mes
-				 obj-prn
-				 (location-fname loc)
-				 (location-pos   loc)))
-	     (if (pair? continue)
-		 (try (error proc mes obj)
-		      handler)
-		 (error proc mes obj))))))
+(define (user-error/location loc proc msg obj . continue)
+   (with-dump-stack
+      (lambda ()
+	 (if (output-port? *trace-port*)
+	     (fprint *trace-port*  "*** ERROR:" proc ":" msg ":" obj))
+	 (set! *nb-error-on-pass* (+fx *nb-error-on-pass* 1))
+	 (let* ((proc-string (cond
+				((string? proc)
+				 proc)
+				((symbol? proc)
+				 (symbol->string proc))
+				(else
+				 #f)))
+		(fun-string (symbol->string (current-function)))
+		(proc (if (and (string? proc-string)
+			       (not (string=? proc-string fun-string)))
+			  (string-append fun-string ":" proc-string)
+			  fun-string))
+		(objprn (let ((port (open-output-string)))
+			   (display-circle obj port)
+			   (let ((string (close-output-port port)))
+			      (if (>fx (string-length string) 45)
+				  (string-append (substring string 0 44) " ...")
+				  string)))))
+	    (bind-exit (skip)
+	       (with-exception-handler
+		  (lambda (e)
+		     (error-notify e)
+		     (if (pair? continue)
+			 (skip (car continue))
+			 (exit 1)))
+		  (lambda ()
+		     (if (location? loc)
+			 (let ((fname (location-full-fname loc))
+			       (pos (location-pos loc)))
+			    (error/location proc msg objprn fname pos))
+			 (error proc msg objprn)))))))))
 		 
 ;*---------------------------------------------------------------------*/
 ;*    *sfun-stack*                                                     */
