@@ -3,8 +3,8 @@
 ;*    -------------------------------------------------------------    */
 ;*    Author      :  Manuel Serrano                                    */
 ;*    Creation    :  Tue Dec 27 17:02:04 1994                          */
-;*    Last change :  Thu Oct 18 10:42:41 2001 (serrano)                */
-;*    Copyright   :  1994-2001 Manuel Serrano, see LICENSE file        */
+;*    Last change :  Thu Jun  1 19:01:11 2017 (serrano)                */
+;*    Copyright   :  1994-2021 Manuel Serrano, see LICENSE file        */
 ;*    -------------------------------------------------------------    */
 ;*    In order to print human readable messages, we designed this      */
 ;*    tool.                                                            */
@@ -27,6 +27,8 @@
 ;*---------------------------------------------------------------------*/
 (define-generic (shape exp::obj)
    (cond
+      ((class? exp)
+       (format "#<class:~a>" (class-name exp)))
       ((pair? exp)
        (let loop ((exp exp))
 	  (cond
@@ -70,8 +72,9 @@
 ;*    shape ::global                                                   */
 ;*---------------------------------------------------------------------*/
 (define-method (shape var::global)
-   (let* ((str-id (symbol->string (global-id var)))
-	  (id     (string->symbol str-id))
+   (let* ((str-id (if (not *name-shape?*)
+		      (symbol->string (global-id var))
+		      (format "~a<<~a>>" (global-id var) (global-name var))))
 	  (module (symbol->string (global-module var)))
 	  (type   (global-type var))
 	  (tshape (if (not *type-shape?*)
@@ -86,32 +89,24 @@
 		     ((not *access-shape?*)
 		      "")
 		     (else
-		      (string-append "{" (symbol->string (global-access var))
-				     "}")))))
+		      (string-append
+			 "{" (symbol->string (global-access var)) "}")))))
       (cond
 	 (*module-shape?*
-	  (string->symbol (string-append str-id
-					 "@"
-					 module
-					 tshape
-					 ushape
-					 ashape)))
+	  (string->symbol
+	   (string-append str-id "@" module tshape ushape ashape)))
 	 (else
 	  (case (global-bucket-position (global-id var) (global-module var))
 	     ((-1)
-	      (internal-error "global-shape"
-			      "Can't find global any more"
-			      `(@ ,id ,(global-module var))))
+	      (warning "global-shape:"
+		       "Can't find global anymore -- "
+		       `(@ ,(global-id var) ,(global-module var)))
+	      (string->symbol (string-append str-id tshape ushape ashape)))
 	     ((0)
-	      (symbol-append id
-			     (string->symbol tshape)
-			     (string->symbol ushape)
-			     (string->symbol ashape)))
+	      (string->symbol (string-append str-id tshape ushape ashape)))
 	     (else
-	      (let ((sym (symbol-append id
-					(string->symbol tshape)
-					(string->symbol ushape)
-					(string->symbol ashape))))
+	      (let ((sym (string->symbol
+			  (string-append str-id tshape ushape ashape))))
 		 `(@ ,sym ,(string->symbol module)))))))))
 
 ;*---------------------------------------------------------------------*/
@@ -120,9 +115,9 @@
 (define-method (shape var::local)
    (let* ((sym    (if *key-shape?*
 		      (symbol-append (local-id var)
-				     '_
-				     (string->symbol
-				      (integer->string (local-key var))))
+			 '_
+			 (string->symbol
+			    (integer->string (local-key var))))
 		      (local-id var)))
 	  (sym    (symbol->string sym))
 	  (type   (local-type var))
@@ -139,26 +134,36 @@
 		      "")
 		     (else
 		      (string-append "{" (symbol->string (local-access var))
-				     "}")))))
-      (string->symbol (string-append sym tshape ushape ashape))))
+			 "}"))))
+	  (xshape (if *access-shape?*
+		      (with-access::local var (val-noescape)
+			 (cond
+			    ((eq? val-noescape #t) "<noescape>")
+			    ((eq? val-noescape #f) "<escape>")
+			    (else "<?>")))
+		      "")))
+      (string->symbol (string-append sym tshape ushape ashape xshape))))
    
 ;*---------------------------------------------------------------------*/
 ;*    shape ::type ...                                                 */
 ;*---------------------------------------------------------------------*/
 (define-method (shape type::type)
-   (let ((sp (symbol->string (type-id type))))
-      (if *key-shape?*
-	  (string-append
-	   sp
-	   "-"
-	   (cond-expand
-	      (bigloo-c (integer->string (pragma::long "(long)($1)" type) 16))
-	      (bigloo-jvm "")))
-	  sp)))
+   (let* ((sp (symbol->string (type-id type)))
+	  (s (if *key-shape?*
+		 (string-append
+		  sp
+		  "-"
+		  (cond-expand
+		     (bigloo-c
+		      (integer->string (pragma::long "(long)($1)" type) 16))
+		     (else "")))
+		 sp)))
+      (if *typename-shape?*
+	  (string-append s "\"" (type-name type) "\"")
+	  s)))
 
 ;*---------------------------------------------------------------------*/
 ;*    shape ::node ...                                                 */
 ;*---------------------------------------------------------------------*/
 (define-method (shape node::node)
    (node->sexp node))
-   
