@@ -1,10 +1,10 @@
 ;*=====================================================================*/
-;*    serrano/prgm/project/bigloo/comptime/Trace/walk.scm              */
+;*    serrano/prgm/project/bigloo/bigloo/comptime/Trace/walk.scm       */
 ;*    -------------------------------------------------------------    */
 ;*    Author      :  Manuel Serrano                                    */
 ;*    Creation    :  Thu Apr 13 13:53:58 1995                          */
-;*    Last change :  Mon Dec 25 16:29:25 2000 (serrano)                */
-;*    Copyright   :  1995-2000 Manuel Serrano, see LICENSE file        */
+;*    Last change :  Tue Jul 19 13:56:36 2022 (serrano)                */
+;*    Copyright   :  1995-2022 Manuel Serrano, see LICENSE file        */
 ;*    -------------------------------------------------------------    */
 ;*    The introduction of trace in debugging mode.                     */
 ;*=====================================================================*/
@@ -16,15 +16,19 @@
    (include "Engine/pass.sch"
 	    "Ast/node.sch"
 	    "Tools/location.sch")
-   (import  tools_shape
+   (import  ast_let
+	    tools_shape
 	    tools_error
 	    tools_misc
+	    tools_location
 	    type_env
+	    backend_backend
 	    ast_sexp
 	    ast_ident
 	    module_module
 	    engine_param
 	    (mark-symbol-non-user! ast_ident)
+	    trace_isloop
 	    (find-global ast_env)
 	    (find-location tools_location))
    (export  (trace-walk! tree)))
@@ -35,17 +39,17 @@
 (define (trace-walk! globals)
    (pass-prelude "Trace")
    ;; We make some extra tracing for top level set! expression. Top level
-   ;; set! expression that change a global variable are traced. This enables
+   ;; set! expressions that change global variables are traced. This enables
    ;; tracing of toplevel defined closures. This transformation is applied
    ;; before regular tracing because it only scans [begin] top level forms.
    ;; It stops before any nested expression.
-   (let* ((id   'toplevel-init)
-	  (glo  (find-global id *module*)))
+   (let* ((id 'toplevel-init)
+	  (glo (find-global id *module*)))
       (if (global? glo)
 	  (with-access::sfun (global-value glo) (body)
 	     (set! body (toplevel-trace-node body)))))
-   ;; then, we trace all functions (including the toplevel one)
-   (for-each trace-fun! globals)
+   ;; then, trace all functions (including the toplevel one)
+   (for-each (lambda (v) (trace-fun! v '() *compiler-debug-trace*)) globals)
    (pass-postlude globals))
 
 ;*---------------------------------------------------------------------*/
@@ -57,47 +61,57 @@
 ;*    rather anonymous [toplevel-init] ident, we use the name of       */
 ;*    the module.                                                      */
 ;*---------------------------------------------------------------------*/
-(define (trace-id variable)
-   (if (and (global? variable) (eq? (global-id variable) 'toplevel-init))
-       (symbol-append '@ (global-module variable))
-       (variable-id variable)))
+(define (trace-id v)
+   (cond
+      ((and (global? v) (eq? (global-id v) 'toplevel-init))
+       (symbol-append (string->symbol "%toplevel@") (global-module v)))
+      ((and (global? v) (eq? (global-id v) 'imported-modules-init))
+       (symbol-append (string->symbol "%import@") (global-module v)))
+      ((global? v)
+       (symbol-append (global-id v) '@ (global-module v)))
+      (else
+       (variable-id v))))
 
 ;*---------------------------------------------------------------------*/
 ;*    trace-fun! ...                                                   */
 ;*    -------------------------------------------------------------    */
-;*    We don't trace predicates. It is useless and make the code       */
+;*    We don't trace predicates. It is useless and makes the code      */
 ;*    much bigger in safe modes.                                       */
 ;*---------------------------------------------------------------------*/
-(define (trace-fun! var)
+(define (trace-fun! var stack level)
    (let* ((fun  (variable-value var))
 	  (body (sfun-body fun))
-	  (type (variable-type var))
-	  (lloc (if (global? var)
-		    (find-location (find-last-sexp (global-src var)))
-		    (node-loc (find-last-node body)))))
-      (if (and (not (fun-predicate-of fun))
-	       (not (memq 'no-trace (sfun-property fun))))
-	  (begin
-	     (enter-function (trace-id var))
-	     (let* ((new-body  (if (or (>=fx *compiler-debug* 2)
-				       (and (global? var)
-					    (eq? (global-id var)
-						 'toplevel-init)))
-				   ;; we always goes trough the first level
-				   ;; (i.e. not the nested local functions)
-				   ;; of the toplevel-init function even
-				   ;; if [*compiler-debug* < 2]. That way
-				   ;; we are sure that global closures will
-				   ;; be correctly traced and not labeled
-				   ;; [toplevel-init].
-				   (trace-node body)
-				   body))
-		    (new2-body (make-traced-node new-body
-						 type
-						 (trace-id var)
-						 lloc)))
-		(sfun-body-set! fun new2-body)
-		(leave-function))))))
+;* 	  (lloc (if (global? var)                                      */
+;* 		    (find-location (find-last-sexp (global-src var)))  */
+;* 		    (node-loc (find-last-node body))))                 */
+	  (lloc (sfun-loc fun)))
+      (when (and (not (fun-predicate-of fun))
+		 (not (memq 'no-trace (sfun-property fun)))
+		 (user-symbol? (variable-id var)))
+	 (enter-function (trace-id var))
+	 (let* ((bd (if (or (>fx level 1)
+			    (and (global? var)
+				 (or (eq? (global-id var) 'toplevel-init)
+				     (eq? (global-id var) 'method-init)
+				     (eq? (global-id var) 'generic-init))))
+			;; we always goes trough the first level
+			;; (i.e. not the nested local functions)
+			;; of the toplevel-init function even
+			;; if [*compiler-debug-trace* < 2]. That way
+			;; we are sure that global closures will
+			;; be correctly traced and not labeled
+			;; [toplevel-init].
+			(if (or (local? var)
+				(eq? (global-id var) 'method-init)
+			        (eq? (global-id var) 'generic-init))
+			    (trace-node body stack (-fx level 1))
+			    (trace-node body (cons var stack) (-fx level 1)))
+			body))
+		(t (strict-node-type (node-type body) (variable-type var)))
+		(id (trace-id var))
+		(nbody (make-traced-node bd t id lloc stack)))
+	    (sfun-body-set! fun nbody)
+	    (leave-function)))))
 
 ;*---------------------------------------------------------------------*/
 ;*    find-last-sexp ...                                               */
@@ -135,6 +149,13 @@
 	  (find-last-node (car (last-pair nodes)))
 	  node)))
 
+;*---------------------------------------------------------------------*/
+;*    find-last-node ::sync ...                                        */
+;*---------------------------------------------------------------------*/
+(define-method (find-last-node node::sync)
+   (with-access::sync node (mutex prelock body)
+      (find-last-node body)))
+
 ;*---------------------------------------------------------------------*/
 ;*    find-last-node ::app ...                                         */
 ;*---------------------------------------------------------------------*/
@@ -187,10 +208,10 @@
    (find-last-node (fail-obj node)))
 
 ;*---------------------------------------------------------------------*/
-;*    find-last-node ::select ...                                      */
+;*    find-last-node ::switch ...                                      */
 ;*---------------------------------------------------------------------*/
-(define-method (find-last-node node::select)
-   (with-access::select node (clauses test)
+(define-method (find-last-node node::switch)
+   (with-access::switch node (clauses test)
       (if (pair? clauses)
 	  (find-last-node (cdr (last-pair clauses)))
 	  (find-last-sexp test))))
@@ -222,189 +243,215 @@
 ;*---------------------------------------------------------------------*/
 ;*    make-traced-node ...                                             */
 ;*---------------------------------------------------------------------*/
-(define (make-traced-node::let-var node::node type::type symbol::symbol lloc)
+(define (make-traced-node::let-var node::node type::type symbol lloc stack)
+
+   (define (fqname sym stack)
+      (if (null? stack)
+	  sym
+	  (symbol-append sym ': (fqname (trace-id (car stack)) (cdr stack)))))
+   
    (let* ((loc  (node-loc node))
 	  (aux  (mark-symbol-non-user! (gensym 'aux)))
 	  (taux (make-typed-ident aux (type-id type)))
-	  (sym  (mark-symbol-non-user! (gensym 'symbol)))
-	  (exp `(let ((,sym ',symbol))
+	  (tmp1 (mark-symbol-non-user! (gensym 'name)))
+	  (tmp2 (mark-symbol-non-user! (gensym 'loc)))
+	  (tmp3 (mark-symbol-non-user! (gensym 'env)))
+	  (sym  (if (and (pair? stack) (variable? (car stack)))
+		    (fqname symbol stack)
+		    ;;(symbol-append symbol ': (variable-id (car stack)))
+		    symbol))
+	  (l    (when (location? loc)
+		   `(at ,(location-full-fname loc) ,(location-pos loc))))
+	  (exp `(let ((,tmp1 ',sym)
+		      (,tmp2 ',l)
+		      (,(make-typed-ident tmp3 'dynamic-env) (current-dynamic-env)))
 		   (let ()
-		      (c-push-trace ,sym)
-		      ;; this dummy `let' and `set!' and used to normalize
-		      ;; the code and disable the reduction optimizations
-		      ;; on `push-trace' nodes (at least it wont produce
-		      ;; any code and then, has no cost).
-		      (set! ,sym ,sym)
+		      ($env-push-trace ,tmp3 ,tmp1 ,tmp2)
 		      (let ((,taux ,node))
 			 ,(if (location? lloc)
-			      (econs 'c-pop-trace '() lloc)
-			      '(c-pop-trace))
+			      (econs '$env-pop-trace
+				     (econs tmp3 '() lloc)
+				     lloc)
+			      `($env-pop-trace ,tmp3))
 			 ,aux))))
-	  (node (sexp->node exp '() loc 'value)))
-      (if (not (eq? *target-language* 'jvm))
-	  ;; with the JVM back-end there is no constraint of
-	  ;; block declarations
-	  (let-var-removable?-set! node #f))
-      node))
+	  (nnode (sexp->node exp '() loc 'value)))
+      (let-var-removable?-set! nnode (backend-remove-empty-let (the-backend)))
+      nnode))
 
 ;*---------------------------------------------------------------------*/
 ;*    trace-node ...                                                   */
 ;*---------------------------------------------------------------------*/
-(define-generic (trace-node::node node::node)
+(define-generic (trace-node::node node::node stack level)
    node)
 
 ;*---------------------------------------------------------------------*/
 ;*    trace-node ::sequence ...                                        */
 ;*---------------------------------------------------------------------*/
-(define-method (trace-node node::sequence)
-   (trace-node*! (sequence-nodes node))
+(define-method (trace-node node::sequence stack level)
+   (trace-node*! (sequence-nodes node) stack level)
+   node)
+
+;*---------------------------------------------------------------------*/
+;*    trace-node ::sync ...                                            */
+;*---------------------------------------------------------------------*/
+(define-method (trace-node node::sync stack level)
+   (with-access::sync node (mutex prelock body)
+      (set! mutex (trace-node mutex stack level))
+      (set! prelock (trace-node prelock stack level))
+      (set! body (trace-node body stack level)))
    node)
 
 ;*---------------------------------------------------------------------*/
 ;*    trace-node ::app ...                                             */
 ;*---------------------------------------------------------------------*/
-(define-method (trace-node node::app)
-   (trace-node*! (app-args node))
+(define-method (trace-node node::app stack level)
+   (trace-node*! (app-args node) stack level)
    node)
  
 ;*---------------------------------------------------------------------*/
 ;*    trace-node ::app-ly ...                                          */
 ;*---------------------------------------------------------------------*/
-(define-method (trace-node node::app-ly)
+(define-method (trace-node node::app-ly stack level)
    (with-access::app-ly node (fun arg)
-      (set! fun (trace-node fun))
-      (set! arg (trace-node arg))
+      (set! fun (trace-node fun stack level))
+      (set! arg (trace-node arg stack level))
       node))
 
 ;*---------------------------------------------------------------------*/
 ;*    trace-node ::funcall ...                                         */
 ;*---------------------------------------------------------------------*/
-(define-method (trace-node node::funcall)
+(define-method (trace-node node::funcall stack level)
    (with-access::funcall node (fun args)
-      (set! fun (trace-node fun))
-      (trace-node*! args)
+      (set! fun (trace-node fun stack level))
+      (trace-node*! args stack level)
       node))
 
 ;*---------------------------------------------------------------------*/
 ;*    trace-node ::extern ...                                          */
 ;*---------------------------------------------------------------------*/
-(define-method (trace-node node::extern)
-   (trace-node*! (extern-expr* node))
+(define-method (trace-node node::extern stack level)
+   (trace-node*! (extern-expr* node) stack level)
    node)
 
 ;*---------------------------------------------------------------------*/
 ;*    trace-node ::cast ...                                            */
 ;*---------------------------------------------------------------------*/
-(define-method (trace-node node::cast)
-   (trace-node (cast-arg node))
+(define-method (trace-node node::cast stack level)
+   (cast-arg-set! node (trace-node (cast-arg node) stack level))
    node)
 
 ;*---------------------------------------------------------------------*/
 ;*    trace-node ::setq ...                                            */
 ;*---------------------------------------------------------------------*/
-(define-method (trace-node node::setq)
-   (setq-value-set! node (trace-node (setq-value node)))
+(define-method (trace-node node::setq stack level)
+   (setq-value-set! node (trace-node (setq-value node) stack level))
    node)
 
 ;*---------------------------------------------------------------------*/
 ;*    trace-node ::conditional ...                                     */
 ;*---------------------------------------------------------------------*/
-(define-method (trace-node node::conditional)
+(define-method (trace-node node::conditional stack level)
    (with-access::conditional node (test true false)
-       (set! test (trace-node test))
-       (set! true (trace-node true))
-       (set! false (trace-node false))
+       (set! test (trace-node test stack level))
+       (set! true (trace-node true stack level))
+       (set! false (trace-node false stack level))
        node))
 
 ;*---------------------------------------------------------------------*/
 ;*    trace-node ::fail ...                                            */
 ;*---------------------------------------------------------------------*/
-(define-method (trace-node node::fail)
+(define-method (trace-node node::fail stack level)
    (with-access::fail node (proc msg obj)
-      (set! proc (trace-node proc))
-      (set! msg (trace-node msg))
-      (set! obj (trace-node obj))
+      (set! proc (trace-node proc stack level))
+      (set! msg (trace-node msg stack level))
+      (set! obj (trace-node obj stack level))
       node))
 
 ;*---------------------------------------------------------------------*/
-;*    trace-node ::select ...                                          */
+;*    trace-node ::switch ...                                          */
 ;*---------------------------------------------------------------------*/
-(define-method (trace-node node::select)
-   (with-access::select node (clauses test)
-      (set! test (trace-node test))
+(define-method (trace-node node::switch stack level)
+   (with-access::switch node (clauses test)
+      (set! test (trace-node test stack level))
       (for-each (lambda (clause)
-		   (set-cdr! clause (trace-node (cdr clause))))
+		   (set-cdr! clause (trace-node (cdr clause) stack level)))
 		clauses)
       node))
 
 ;*---------------------------------------------------------------------*/
 ;*    trace-node ::let-fun ...                                         */
 ;*---------------------------------------------------------------------*/
-(define-method (trace-node node::let-fun)
-   (with-access::let-fun node (body locals)
-      (for-each trace-fun! locals)
-      (set! body (trace-node body))
-      node))
+(define-method (trace-node node::let-fun stack level)
+   (if (isloop? node)
+       (with-access::let-fun node (body locals loc)
+	  (let* ((var (car locals))
+		 (t (strict-node-type (node-type body) (variable-type var)))
+		 (id (trace-id var)))
+	     (set! body (make-traced-node body t id loc '())))
+	  node)
+       (with-access::let-fun node (body locals)
+	  (for-each (lambda (v) (trace-fun! v stack level)) locals)
+	  (set! body (trace-node body stack level))
+	  node)))
 
 ;*---------------------------------------------------------------------*/
 ;*    trace-node ::let-var ...                                         */
 ;*---------------------------------------------------------------------*/
-(define-method (trace-node node::let-var)
+(define-method (trace-node node::let-var stack level)
    (with-access::let-var node (body bindings)
       (for-each (lambda (binding)
-		   (set-cdr! binding (trace-node (cdr binding))))
+		   (set-cdr! binding (trace-node (cdr binding) stack level)))
 		bindings)
-      (set! body (trace-node body))
+      (set! body (trace-node body stack level))
       node))
 
 ;*---------------------------------------------------------------------*/
 ;*    trace-node ::set-ex-it ...                                       */
 ;*---------------------------------------------------------------------*/
-(define-method (trace-node node::set-ex-it)
-   (set-ex-it-body-set! node (trace-node (set-ex-it-body node)))
+(define-method (trace-node node::set-ex-it stack level)
+   (set-ex-it-body-set! node (trace-node (set-ex-it-body node) stack level))
+   (set-ex-it-onexit-set! node (trace-node (set-ex-it-onexit node) stack level))
    node)
 
 ;*---------------------------------------------------------------------*/
 ;*    trace-node ::jump-ex-it ...                                      */
 ;*---------------------------------------------------------------------*/
-(define-method (trace-node node::jump-ex-it)
+(define-method (trace-node node::jump-ex-it stack level)
    (with-access::jump-ex-it node (exit value)
-      (set! exit (trace-node exit)) 
-      (set! value (trace-node value))
+      (set! exit (trace-node exit stack level)) 
+      (set! value (trace-node value stack level))
       node))
 
 ;*---------------------------------------------------------------------*/
 ;*    trace-node ::make-box ...                                        */
 ;*---------------------------------------------------------------------*/
-(define-method (trace-node node::make-box)
-   (make-box-value-set! node (trace-node (make-box-value node)))
+(define-method (trace-node node::make-box stack level)
+   (make-box-value-set! node (trace-node (make-box-value node) stack level))
    node)
 
 ;*---------------------------------------------------------------------*/
 ;*    trace-node ::box-ref ...                                         */
 ;*---------------------------------------------------------------------*/
-(define-method (trace-node node::box-ref)
-   (box-ref-var-set! node (trace-node (box-ref-var node)))
+(define-method (trace-node node::box-ref stack level)
+   (box-ref-var-set! node (trace-node (box-ref-var node) stack level))
    node)
 
 ;*---------------------------------------------------------------------*/
 ;*    trace-node ::box-set! ...                                        */
 ;*---------------------------------------------------------------------*/
-(define-method (trace-node node::box-set!)
+(define-method (trace-node node::box-set! stack level)
    (with-access::box-set! node (var value)
-      (set! var (trace-node var))
-      (set! value (trace-node value))
+      (set! var (trace-node var stack level))
+      (set! value (trace-node value stack level))
       node))
 
 ;*---------------------------------------------------------------------*/
 ;*    trace-node*! ...                                                 */
 ;*---------------------------------------------------------------------*/
-(define (trace-node*! node*)
-   (if (null? node*)
-       'done
-       (begin
-	  (set-car! node* (trace-node (car node*)))
-	  (trace-node*! (cdr node*)))))
+(define (trace-node*! node* stack level)
+   (unless (null? node*)
+      (set-car! node* (trace-node (car node*) stack level))
+      (trace-node*! (cdr node*) stack level)))
    
 ;*---------------------------------------------------------------------*/
 ;*    toplevel-trace-node ...                                          */
@@ -419,37 +466,43 @@
    (toplevel-trace-node*! (sequence-nodes node))
    node)
 
+;*---------------------------------------------------------------------*/
+;*    toplevel-trace-node ::sync ...                                   */
+;*---------------------------------------------------------------------*/
+(define-method (toplevel-trace-node node::sync)
+   (sync-mutex-set! node (toplevel-trace-node (sync-mutex node)))
+   (sync-prelock-set! node (toplevel-trace-node (sync-prelock node)))
+   (sync-body-set! node (toplevel-trace-node (sync-body node)))
+   node)
+
 ;*---------------------------------------------------------------------*/
 ;*    toplevel-trace-node ::setq ...                                   */
 ;*    -------------------------------------------------------------    */
 ;*    This method instruments global variable affections provided      */
 ;*    those affections are:                                            */
 ;*      - top level                                                    */
-;*      - they set composed values (i.e. non trivial values).          */
+;*      - they assign non trivial values (i.e., non literal values).   */
 ;*---------------------------------------------------------------------*/
 (define-method (toplevel-trace-node node::setq)
    (with-access::setq node (var value loc)
       (with-access::var var (variable)
-	 (if (and (global? variable)
-		  (not (or (atom? value)
-			   (var? value)
-			   (kwote? value)
-			   (pragma? value))))
-	     (let* ((id    (global-id variable))
-		    (type  (global-type variable))
-		    (trace (make-traced-node value type id loc)))
-		(set! value trace)))))
+	 (when (and (global? variable)
+		    (not (or (atom? value)
+			     (var? value)
+			     (kwote? value)
+			     (pragma? value))))
+	    (let* ((t (strict-node-type (node-type value) (global-type variable)))
+		   (trace (make-traced-node value t (trace-id variable) loc '())))
+	       (set! value trace)))))
    node)
 
 ;*---------------------------------------------------------------------*/
 ;*    toplevel-trace-node*! ...                                        */
 ;*---------------------------------------------------------------------*/
 (define (toplevel-trace-node*! node*)
-   (if (null? node*)
-       'done
-       (begin
-	  (set-car! node* (toplevel-trace-node (car node*)))
-	  (toplevel-trace-node*! (cdr node*)))))
+   (unless (null? node*)
+      (set-car! node* (toplevel-trace-node (car node*)))
+      (toplevel-trace-node*! (cdr node*))))
    
 
 
