@@ -3,8 +3,8 @@
 ;*    -------------------------------------------------------------    */
 ;*    Author      :  Manuel Serrano                                    */
 ;*    Creation    :  Mon Mar 27 13:33:40 1995                          */
-;*    Last change :  Sat Jul  7 07:46:03 2001 (serrano)                */
-;*    Copyright   :  1995-2001 Manuel Serrano, see LICENSE file        */
+;*    Last change :  Thu Nov 10 07:09:46 2011 (serrano)                */
+;*    Copyright   :  1995-2011 Manuel Serrano, see LICENSE file        */
 ;*    -------------------------------------------------------------    */
 ;*    We install all the coercer and accessor for `tvector' types.     */
 ;*=====================================================================*/
@@ -17,86 +17,83 @@
 	    type_type
 	    type_env
 	    engine_param
+	    backend_backend
 	    tvector_tvector
 	    module_module
 	    tools_error
+	    tools_shape
 	    ast_ident
 	    ast_private)
-   (export  (make-tvector-accesses tvector::tvec ::obj)))
+   (export  (make-tvector-accesses tvector::tvec ::obj ::bool)))
 
 ;*---------------------------------------------------------------------*/
 ;*    make-tvector-accesses ...                                        */
 ;*---------------------------------------------------------------------*/
-(define (make-tvector-accesses tv::tvec src)
-   (let* ((tv-id             (tvec-id tv))
-	  (tv-name           (tvec-name tv))
-	  (obj               (find-type 'obj))
-	  (item-type         (tvec-item-type tv))
-	  (item-id           (type-id item-type))
-	  (item-name         (type-name item-type))
-	  (descr-id          (symbol-append tv-id '-descriptor))
-	  (tv-make-id        (symbol-append 'make- tv-id))
-	  (tv-alloc-id       (symbol-append 'allocate- tv-id))
-	  (tv-ref-id         (symbol-append tv-id '-ref))
-	  (tv-set!-id        (symbol-append tv-id '-set!))
-	  (tv?-id            (symbol-append tv-id '?))
-	  (tv->vector-id     (symbol-append tv-id '->vector))
-	  (vector->tv-id     (symbol-append 'vector-> tv-id))
-	  (tv->list          (symbol-append tv-id '->list))
-	  (tv-length-id      (symbol-append tv-id '-length)))
+(define (make-tvector-accesses tv::tvec src import)
+   (let* ((tv-id         (tvec-id tv))
+	  (tv-name       (tvec-name tv))
+	  (obj           (find-type 'obj))
+	  (item-type     (tvec-item-type tv))
+	  (item-id       (type-id item-type))
+	  (item-name     (type-name item-type))
+	  (mitem-name    (if (string? item-name)
+			     (bigloo-mangle item-name)
+			     "obj_t"))
+	  (descr-id      (symbol-append tv-id '-descriptor))
+	  (tv-make-id    (symbol-append 'make- tv-id))
+	  (tv-alloc-id   (symbol-append 'allocate- tv-id))
+	  (tv-ref-id     (symbol-append tv-id '-ref))
+	  (tv-set!-id    (symbol-append tv-id '-set!))
+	  (tv?-id        (symbol-append tv-id '?))
+	  (tv->vector-id (symbol-append tv-id '->vector))
+	  (vector->tv-id (symbol-append 'vector-> tv-id))
+	  (tv->list      (symbol-append tv-id '->list))
+	  (tv-length-id  (symbol-append tv-id '-length)))
       
       (define (make-descr)
-	 `(define ,(symbol-append descr-id '::obj)
-	     ((@ declare-tvector! __tvector)
-	      ,(symbol->string tv-id)
-	      ,tv-alloc-id
-	      ,tv-ref-id
-	      ,tv-set!-id)))
+	 (if import
+	     `(define ,(symbol-append descr-id '::obj)
+		 ((@ get-tvector-descriptor __tvector) ',tv-id))
+	     `(define ,(symbol-append descr-id '::obj)
+		 ((@ declare-tvector! __tvector)
+		  ,(symbol->string tv-id)
+		  ,tv-alloc-id
+		  ,tv-ref-id
+		  ,tv-set!-id))))
       
       (define (make-c-tv?)
 	 `(define-inline (,(symbol-append tv?-id '::bool) o::obj)
 	     (if (tvector? o)
-		 (eq? (tvector-descr o) ,descr-id)
+		 (eq? ($tvector-descr o) ,descr-id)
 		 #f)))
 
       (define (make-jvm-tv?)
 	 `(define-inline (,(symbol-append tv?-id '::bool) o::obj)
-	     ,(make-private-sexp 'isa tv-id 'o)))
+	     ,(make-private-sexp 'instanceof tv-id 'o)))
        
       (define (make-tv?)
-	 (if (eq? *target-language* 'jvm)
-	     (make-jvm-tv?)
-	     (make-c-tv?)))
+	 (if (backend-tvector-descr-support (the-backend))
+	     (make-c-tv?)
+	     (make-jvm-tv?)))
       
       (define (make-tv-ref)
-	 (let* ((pfmt (string-append "TVECTOR_REF( " item-name ",$1,$2 )"))
+	 (let* ((pfmt (string-append "TVECTOR_REF( " mitem-name ",$1,$2 )"))
 		(exp (make-private-sexp 'vref tv-id item-id 'int
 					pfmt 'tv 'o)))
 	    `(define-inline (,(make-typed-ident tv-ref-id item-id)
 			     ,(make-typed-ident 'tv tv-id)
 			     o::int)
-		,(if *unsafe-range*
-		     exp
-		     `(if (vector-bound-check? o (tvector-length tv))
-			  ,exp
-			  (error ,tv-ref-id "Index out of bounds" o))))))
+		,exp)))
       
       (define (make-tv-set!)
-	 (let* ((pfmt (string-append "TVECTOR_SET( " item-name ",$1,$2,$3 )"))
+	 (let* ((pfmt (string-append "TVECTOR_SET( " mitem-name ",$1,$2,$3 )"))
 		(exp (make-private-sexp 'vset! tv-id item-id 'int
 					pfmt 'tv 'o 'v)))
 	    `(define-inline (,(symbol-append tv-set!-id '::obj)
 			     ,(make-typed-ident 'tv tv-id)
 			     o::int
 			     ,(make-typed-ident 'v item-id))
-		,(if *unsafe-range*
-		     exp
-		     `(if (vector-bound-check? o (tvector-length tv))
-			  ,exp
-			  (error ,(string-append (symbol->string tv-id)
-						 "-set!")
-				 "Index out of bounds"
-				 o))))))
+		,exp)))
       
       (define (make-tv)
 	 `(define-inline (,(make-typed-ident tv-make-id tv-id)
@@ -120,9 +117,11 @@
 	 `(define-inline (,(make-typed-ident tv-alloc-id tv-id) len::int)
 	     ,(make-private-sexp 'valloc tv-id item-id 'int
 				 (string-append (allocate-tvector)
+						mitem-name ", "
 						item-name
 						", $1, $2 )")
 				 (string-append "ALLOCATE_S_TVECTOR( "
+						mitem-name ", "
 						item-name
 						", $1, $2 )")
 				 #f
@@ -130,22 +129,25 @@
       
       (define (make-jvm-alloc-tv)
 	 `(define-inline (,(make-typed-ident tv-alloc-id tv-id) len::int)
-	     (let ((o ,(make-private-sexp 'valloc tv-id item-id 'int
+	     (let ((,(make-typed-ident 'v tv-id)
+		    ,(make-private-sexp 'valloc tv-id item-id 'int
 					  (string-append (allocate-tvector)
+							 mitem-name ", "
 							 item-name
 							 ", $1, $2 )")
 					  (string-append "ALLOCATE_S_TVECTOR( "
+							 mitem-name ", "
 							 item-name
 							 ", $1, $2 )")
 					  #f
 					  'len)))
-		(tvector-descr-set! o ,descr-id)
-		o)))
+		($tvector-descr-set! v ,descr-id)
+		v)))
 
       (define (make-alloc-tv)
-	 (if (eq? *target-language* 'jvm)
-	     (make-jvm-alloc-tv)
-	     (make-c-alloc-tv)))
+	 (if (backend-tvector-descr-support (the-backend))
+	     (make-c-alloc-tv)
+	     (make-jvm-alloc-tv)))
 	     
       (define (make-tv->vector)
 	 `(define-inline (,(symbol-append tv->vector-id '::vector)
