@@ -3,8 +3,8 @@
 ;*    -------------------------------------------------------------    */
 ;*    Author      :  Manuel Serrano                                    */
 ;*    Creation    :  Tue Dec 27 18:43:04 1994                          */
-;*    Last change :  Mon May 15 08:07:06 2000 (serrano)                */
-;*    Copyright   :  1994-2000 Manuel Serrano, see LICENSE file        */
+;*    Last change :  Fri Nov 18 07:14:29 2011 (serrano)                */
+;*    Copyright   :  1994-2011 Manuel Serrano, see LICENSE file        */
 ;*    -------------------------------------------------------------    */
 ;*    The coercion management                                          */
 ;*=====================================================================*/
@@ -19,8 +19,8 @@
 	    tools_shape
 	    type_type
 	    engine_param)
-   (export  (add-coercion!         ::type ::type ::obj ::obj)
-	    (find-coercer::obj     ::type ::type)
+   (export  (add-coercion! ::type ::type ::obj ::obj)
+	    (find-coercer::obj ::type ::type)
 	    (coercer-exists?::bool ::type ::type)))
 
 ;*---------------------------------------------------------------------*/
@@ -30,7 +30,7 @@
 ;*---------------------------------------------------------------------*/
 (define (find-coercer from::type to::type)
    (let ((from (get-aliased-type from))
-	 (to   (get-aliased-type to)))
+	 (to (get-aliased-type to)))
       (let loop ((coercer (type-coerce-to from)))
 	 (cond
 	    ((null? coercer)
@@ -75,18 +75,18 @@
 ;*    `obj' and `foreign' since this link already exists.              */
 ;*---------------------------------------------------------------------*/
 (define (add-coercion! from to check coerce)
-   [assert (check coerce) (check-coercion? check coerce)]
-   (trace (ast 2) "add-coercion!:" (shape from) " -> " (shape to)
-	  " check: " check " coerce: " coerce #\Newline)
-   (if (null? check) (set! check '(())))
-   (if (null? coerce) (set! coerce '(())))
+   (assert (check coerce) (check-coercion? check coerce))
+   (trace (ast 2) "add-coercion!: " (shape from) " -> " (shape to)
+	  " check: " (shape check) " coerce: " (shape coerce) #\Newline)
+;*    (if (null? check) (set! check '(())))                            */
+;*    (if (null? coerce) (set! coerce '(())))                          */
    (let ((from (get-aliased-type from))
-	 (to   (get-aliased-type to)))
+	 (to (get-aliased-type to)))
       (if (coercer? (find-coercer from to))
-	  (if (not *lib-mode*)
-	      (warning "add-coercion!"
-		       "Type coercion redefinition -- "
-		       (shape (list from to check coerce))))
+	  (unless *lib-mode*
+	     (warning "add-coercion!"
+		      "Type coercion redefinition -- "
+		      (shape (list from to check coerce))))
 	  (begin
 	     ;; we set the coercion between `from' and `to'
 	     (let ((new (coercer from to check coerce)))
@@ -100,9 +100,12 @@
 		     (let ((coercer-p (find-coercer to parent)))
 			(if (not (coercer? coercer-p))
 			    (user-error "Can't find coercion"
-					(shape to)
-					(shape parent))
-			    (let ((check-p  (coercer-check-op coercer-p))
+					(format "~a -> ~a"
+						(shape to) (shape parent))
+					(format "while adding: ~a -> ~a"
+						(shape from)
+						(shape to)))
+			    (let ((check-p (coercer-check-op coercer-p))
 				  (coerce-p (coercer-coerce-op coercer-p)))
 			       (add-coercion! from
 					      parent
@@ -118,9 +121,11 @@
 		     (let ((coercer-p (find-coercer parent from)))
 			(if (not (coercer? coercer-p))
 			    (user-error "Can't find coercion"
-					(shape parent)
-					(shape from))
-			    (let ((check-p  (coercer-check-op coercer-p))
+					(format "~a -> ~a"
+						(shape parent) (shape from))
+					(format "while adding: ~a -> ~a"
+						(shape from) (shape to)))
+			    (let ((check-p (coercer-check-op coercer-p))
 				  (coerce-p (coercer-coerce-op coercer-p)))
 			       (add-coercion! parent
 					      to
@@ -140,18 +145,20 @@
 		((null? coerce)
 		 #t)
 		((match-case (car coerce)
-		    ((? symbol?)
-		     #f)
-		    (()
-		     #f)
-		    ((lambda (?-) . ?-)
-		     #f)
-		    (else
-		     #t))
+		    (((? symbol?) . ?-) #f)
+		    (((@ (? symbol?) (? symbol?)) . ?-) #f)
+		    ((#t . ?-) #f)
+		    (((lambda (?-) . ?-) . ?-) #f)
+		    (else #t))
 		 #f)
 		(else
 		 (loop (cdr coerce))))))
-	 ((and (not (symbol? (car check))) (not (null? (car check))))
+	 ((and (not (symbol? (caar check)))
+	       (not (eq? (caar check) #t))
+	       (not (match-case (caar check)
+		       ((@ (? symbol?) (? symbol?)) #t)
+		       ((lambda (?-) . ?-) #t)
+		       (else #f))))
 	  #f)
 	 (else
 	  (loop (cdr check))))))
