@@ -3,8 +3,8 @@
 ;*    -------------------------------------------------------------    */
 ;*    Author      :  Manuel Serrano                                    */
 ;*    Creation    :  Sat Dec 31 07:29:03 1994                          */
-;*    Last change :  Tue Nov 27 10:28:43 2001 (serrano)                */
-;*    Copyright   :  1994-2001 Manuel Serrano, see LICENSE file        */
+;*    Last change :  Fri Mar 18 11:43:48 2011 (serrano)                */
+;*    Copyright   :  1994-2020 Manuel Serrano, see LICENSE file        */
 ;*    -------------------------------------------------------------    */
 ;*    The ast pretty-printer                                           */
 ;*=====================================================================*/
@@ -27,56 +27,52 @@
 ;*    write-ast ...                                                    */
 ;*---------------------------------------------------------------------*/
 (define (write-ast globals)
-   (profile
-    write
-    (let* ((output-name (if (string? *dest*)
-			    *dest*
-			    (if (and (pair? *src-files*)
-				     (string? (car *src-files*)))
-				(string-append (prefix (car *src-files*))
-					       ".ast")
-				#f)))
-	   (port        (if (string? output-name)
-			    (open-output-file output-name)
-			    (current-output-port))))
-       (if (not (output-port? port))
-	   (error "write-ast" "Can't open output file" output-name)
-	   (let ((handler (lambda (escape proc mes obj)
-			     (notify-error proc mes obj)
-			     (close-output-port port)
-			     (exit-bigloo -8))))
-	      (try (begin
-		      (if (not *ast-case-sensitive*)
-			  (set! *pp-case* 'lower))
-		      (write-scheme-file-header port
-						(string-append "The AST ("
-							       *current-pass*
-							       ")"))
-		      (for-each
-		       (lambda (g)
-			  (let ((fun (global-value g)))
-			     (write-scheme-comment port (shape g))
-			     (write-scheme-comment port
-						   (function-type->string g))
-			     (write-scheme-comment port (make-sfun-sinfo g))
-			     (pp `(,(case (sfun-class fun)
-				       ((sgfun)
-					'define-generic)
-				       ((sifun)
-					'define-inline)
-				       ((smfun)
-					'define-method)
-				       (else
-					'define))
-				   ,(cons (shape g)
-					  (args-list->args*
-					   (map shape (sfun-args fun))
-					   (sfun-arity fun)))
-				   ,(shape (sfun-body fun)))
-				 port)))
-		       globals)
-		      (close-output-port port))
-		   handler))))))
+   (profile write
+      (let* ((output-name (if (string? *dest*)
+			      *dest*
+			      (if (and (pair? *src-files*)
+				       (string? (car *src-files*)))
+				  (string-append (prefix (car *src-files*))
+				     "." (symbol->string *pass*)
+				     ".ast")
+				  #f)))
+	     (port (if (string? output-name)
+		       (open-output-file output-name)
+		       (current-output-port))))
+	 (if (not (output-port? port))
+	     (error "write-ast" "Can't open output file" output-name)
+	     (unwind-protect
+		(begin
+		   (if (not *ast-case-sensitive*)
+		       (set! *pp-case* 'lower))
+		   (write-scheme-file-header port
+					     (string-append "The AST ("
+							    *current-pass*
+							    ")"))
+		   (for-each
+		    (lambda (g)
+		       (let ((fun (global-value g)))
+			  (write-scheme-comment port (shape g))
+			  (write-scheme-comment port
+						(function-type->string g))
+			  (write-scheme-comment port (make-sfun-sinfo g))
+			  (pp `(,(case (sfun-class fun)
+				    ((sgfun)
+				     'define-generic)
+				    ((sifun)
+				     'define-inline)
+				    ((smfun)
+				     'define-method)
+				    (else
+				     'define))
+				,(cons (shape g)
+				       (args-list->args*
+					(map shape (sfun-args fun))
+					(sfun-arity fun)))
+				,(shape (sfun-body fun)))
+			      port)))
+		    globals))
+		(close-output-port port))))))
 
 ;*---------------------------------------------------------------------*/
 ;*    make-sfun-sinfo ...                                              */
@@ -109,7 +105,7 @@
 		     (if *ast-case-sensitive*
 			 (symbol->string (global-import g))
 			 (string-downcase (symbol->string (global-import g))))
-		     "  side-effect: " (atom->string (sfun-side-effect? sfun))
+		     "  side-effect: " (atom->string (sfun-side-effect sfun))
 		     (let ((t (sfun-predicate-of sfun)))
 			(if (type? t)
 			    (string-append "  predicate-of: "
@@ -123,4 +119,7 @@
 		     (if (global-user? g)
 			 "  user?: #t"
 			 "  user?: #f")
+		     " removable: " (with-output-to-string
+				       (lambda ()
+					  (display (global-removable g))))
 		     "]")))
