@@ -3,8 +3,8 @@
 ;*    -------------------------------------------------------------    */
 ;*    Author      :  Manuel Serrano                                    */
 ;*    Creation    :  Sun Mar 12 14:03:51 1995                          */
-;*    Last change :  Sat Apr  7 08:14:01 2001 (serrano)                */
-;*    Copyright   :  1995-2001 Manuel Serrano, see LICENSE file        */
+;*    Last change :  Tue Jun 20 14:16:45 2017 (serrano)                */
+;*    Copyright   :  1995-2017 Manuel Serrano, see LICENSE file        */
 ;*    -------------------------------------------------------------    */
 ;*    La version de Bigloo.                                            */
 ;*=====================================================================*/
@@ -43,15 +43,8 @@
 	      (substring *bigloo-date* 1 (string-length *bigloo-date*))
 	      *bigloo-date*)
 	  *bigloo-author*
-	  "email:"
-	  *bigloo-email*)
-   (if (>=fx *verbose* 3)
-       (begin
-	  (display-to-column " " 78 #\-)
-	  (newline)
-	  (verbose 3 " Ce travail est dedi  Nelly Lebas, ma grand-mere, dcde le 10 Mars 1995," #\Newline)
-	  (verbose 3 "  Marcel Lebas, mon grand-pere, dcd le 29 Octobre 1998" #\Newline)
-	  (verbose 3 " et maintenant,  Carlos, dcd le 12 Mars 2001 au matin." #\Newline)))
+	  (string-append "email: " *bigloo-email*)
+	  (string-append "url: " *bigloo-url*))
    (display-to-column "" 79 #\-)
    (newline)
    (newline))
