@@ -1,11 +1,10 @@
 import java.io.*;
 import bigloo.*;
-import main;
 
 public class Point implements Intf {
     public int x;
     public int y;
-    private static int point_num = 0;
+    public static int point_num = 0;
     
     public Point( int _x, int _y ) {
 	point_num++;
@@ -24,7 +23,7 @@ public class Point implements Intf {
     }
 
     public void show() {
-	System.out.print( "x= " + x + " y= " + y );
+       System.out.print( "x= " + x + " y= " + y );
     }
 
     public static int PointStatistics() {
