@@ -1,11 +1,10 @@
 import java.io.*;
-import extern;
 
 public class Point implements Intf {
     public int x;
     public int y;
-    private static int point_num = 0;
-    
+    public static int point_num = 0;
+
     public Point( int _x, int _y ) {
 	point_num++;
 	x = _x;
@@ -27,6 +26,6 @@ public class Point implements Intf {
     }
 
     public static int PointStatistics() {
-	return point_num + extern.callback( 10 );
+	return point_num + external.callback( 10 );
     }
 }
