@@ -3,7 +3,7 @@
 ;*                                                                     */
 ;*    Author      :  Manuel Serrano                                    */
 ;*    Creation    :  Tue Nov  3 13:57:49 1992                          */
-;*    Last change :  Fri Jul  6 09:40:06 2001 (serrano)                */
+;*    Last change :  Mon Nov 11 08:36:25 2013 (serrano)                */
 ;*                                                                     */
 ;*    On test differents call/cc                                       */
 ;*---------------------------------------------------------------------*/
@@ -32,12 +32,12 @@
       (set-exit
        (an-exit1013)
        (let ()
-	  (push-exit! an-exit1013 #t)
+	  (push-exit! an-exit1013 1)
 	  (let ((exp (let ((body1018 (lambda () 9)))
 			(set-exit
 			 (an-exit1025)
 			 (let ()
-			    (push-exit! an-exit1025 #f)
+			    (push-exit! an-exit1025 0)
 			    (body1018))))))
 	     (set! exp #unspecified))))
       9))
@@ -59,15 +59,14 @@
        (lambda (return)
 	  (letrec ((r (lambda (obj)
 			 (cond ((null? obj) 0)
-			       ((pair? obj)
-				(+ (r (cdr obj)) 1))
+			       ((pair? obj) (+ (r (cdr obj)) 1))
 			       (else (return #f))))))
 	     (r obj))))))
 
 ;*---------------------------------------------------------------------*/
-;*    test1 ...                                                        */
+;*    test-set ...                                                     */
 ;*---------------------------------------------------------------------*/
-(define (test1)
+(define (test-set)
    (let (v)
       (let ((k (call/cc (lambda (x) (set! v x)))))
 	 (if k
@@ -95,7 +94,7 @@
 ;*---------------------------------------------------------------------*/
 ;*    test3 ...                                                        */
 ;*    -------------------------------------------------------------    */
-;*    Cette fonction test les interferences possibles entre le GC et   */
+;*    Cette fonction teste les interferences possibles entre le GC et  */
 ;*    call/cc                                                          */
 ;*---------------------------------------------------------------------*/
 (define (test3) 
@@ -103,16 +102,16 @@
       (eval '(define resume (lambda (v) (display "not initialized"))))
       (eval '(define (callcc-foo n)
 		(bind-exit (stop)
-			   (define (loop f)
-			      (call/cc (lambda (k)
-					  (set! resume k)
-					  (f)))
-			      (loop f))
-			   (loop (lambda () (begin
-					       (set! n (- n 1))
-					       (if (= n 0)
-						   (stop n)
-						   (resume n))))))))
+		   (define (loop f)
+		      (call/cc (lambda (k)
+				  (set! resume k)
+				  (f)))
+		      (loop f))
+		   (loop (lambda () (begin
+				       (set! n (- n 1))
+				       (if (= n 0)
+					   (stop n)
+					   (resume n))))))))
       (eval '(callcc-foo 100))))
 
 ;*---------------------------------------------------------------------*/
@@ -160,37 +159,78 @@
 ;*---------------------------------------------------------------------*/
 (define (test-callcc)
    (test-module "callcc" "callcc.scm")
-   (test "call/cc(r4)" (call/cc (lambda (exit)
+   (test "call/cc(r4).1" (call/cc (lambda (exit)
 				   (for-each (lambda (x)
 						(if (negative? x)
 						    (exit x)))
 					     '(54 0 37 -3 245 19))
 				   #t))
 	 -3)
-   (test "call/cc(r4)" (list-length '(1 2 3 4)) 4)
-   (test "call/cc(r4)" (list-length '(a b . c)) #f)
-   (test "call/cc(simple)" (call/cc (lambda (k) 'a)) 'a)
-   (test "call/cc(simple)" (call/cc (lambda (k) (k 'a))) 'a)
-   (test "call/cc(simple)" (call/cc (lambda (k) (cons (k 'a) 1))) 'a)
+   (test "call/cc(r4).2" (list-length '(1 2 3 4)) 4)
+   (test "call/cc(r4).3" (list-length '(a b . c)) #f)
+   (test "call/cc(simple).1" (call/cc (lambda (k) 'a)) 'a)
+   (test "call/cc(simple).2" (call/cc (lambda (k) (k 'a))) 'a)
+   (test "call/cc(simple).3" (call/cc (lambda (k) (cons (k 'a) 1))) 'a)
    (when-call/cc (test-callcc2)))
 
+;*---------------------------------------------------------------------*/
+;*    test-stack-traces ...                                            */
+;*---------------------------------------------------------------------*/
+(define-expander lbind-exit
+   (lambda (x e)
+      (match-case x
+	 ((lbind-exit (?exit) . ?body)
+	  (e `(call/cc (lambda (,exit)
+			  ,@body))
+	     e)))))
+
+(define (test-stack-traces-main x)
+   (car x)
+   (lbind-exit (exit)
+      (test-stack-traces-foo x exit))
+   (caddr (cddr x)))
+
+(define (test-stack-traces-foo x exit)
+   (cadr x)
+   (lbind-exit (stop)
+      (test-stack-traces-bar x stop))
+   (caddr (cdr x))
+   (exit 1))
+
+(define (test-stack-traces-bar x stop)
+   (caddr x)
+   (stop 2))
+
+(define (test-stack-traces)
+   (let ((l '(() (1) (1 2) (1 2 3) (1 2 3 4) (1 2 3 4 5)))
+	 (old (bigloo-debug)))
+      (bigloo-debug-set! 10)
+      (let ((r (= (apply + (map (lambda (v) 
+				   (try (begin (test-stack-traces-main v) 0)
+					(lambda (a b c d)
+					   (a 1))))
+				l))
+		  5)))
+	 (bigloo-debug-set! old)
+	 r)))
+
 ;*---------------------------------------------------------------------*/
 ;*    test-callcc2 ...                                                 */
 ;*---------------------------------------------------------------------*/
 (define (test-callcc2)
-   (test "call/cc(set!)" ((test1) #f) #f)
-   (test "call/cc(set!)" ((lambda (a)
+   (test "call/cc(set!).1" ((test-set) #f) #f)
+   (test "call/cc(set!).2" ((lambda (a)
 			       (begin (set! a (call/cc (lambda (k) k)))
 				      (a (lambda (x) 1))))
 			    2)
 	 1)
-   (test "call/cc(set!)" ((lambda (a b)
+   (test "call/cc(set!).3" ((lambda (a b)
 			       (begin (set! a (call/cc (lambda (k) k)))
 				      (begin (set! b (+ 1 b))
 					     (a (lambda (x) b)))))
 			    3 0)
 	 2)
-   (test "call/cc(set!)"
+   (test "call/cc(set!).4"
 	 ((lambda (foo)
 	     (begin
 		(set! foo ((lambda (z)
@@ -211,7 +251,7 @@
 					      '())))))))
 	  'foo)
 	 '(z a d ee d))
-   (test "call/cc(set!)"
+   (test "call/cc(set!).5"
 	 ((lambda (foo)
 	     (begin
 		(set! foo ((lambda (z)
@@ -234,8 +274,8 @@
 					(cons-gd (foo 'f 5) '())))))))
 	  'foo)
 	 '(z a d ee d))
-   (test "call/cc(set!)" (test2 'toto) #f)
-   (test "call/cc(set!)" (test2 "TOTO") 'TOTO0)
+   (test "call/cc(set!).6" (test2 'toto) #f)
+   (test "call/cc(set!).7" (test2 "TOTO") 'TOTO0)
    (test "call/cc(gc)" (test3) 0)
    (let ((box1 (make-box 33)))
       (test "box" (box1 'get) 33)
@@ -279,4 +319,5 @@
 						 (call/cc (lambda (cont)
 							     (cont 'x 'y))))
 					      cons)
-	 '(x . y)))
+	 '(x . y))
+   (test "stack traces" (test-stack-traces) #t))
