@@ -3,7 +3,7 @@
 ;*                                                                     */
 ;*    Author      :  Manuel Serrano                                    */
 ;*    Creation    :  Tue Nov  3 11:30:18 1992                          */
-;*    Last change :  Thu Jul  5 08:34:57 2001 (serrano)                */
+;*    Last change :  Mon Oct 17 10:04:27 2016 (serrano)                */
 ;*                                                                     */
 ;*    Des tests qui globalisent des fonctions locales par la passe Glo */
 ;*---------------------------------------------------------------------*/
@@ -23,8 +23,7 @@
 	    (compile x)
 	    (plante-5 x)
 	    (plante-6 x e)
-	    (plante-7)
-	    (plante-8))
+	    (plante-7))
    (option  (set! *inlining?* #f)))
 
 ;*---------------------------------------------------------------------*/
@@ -68,8 +67,8 @@
 ;*---------------------------------------------------------------------*/
 (define (test5 a b . c)
    (labels ((foo (x y) (set! c y) y)
-	    (bar (z)   (foo 2 1))
-	    (hux (t)   (set! a (foo t t)) c))
+	    (bar (z) (foo 2 1))
+	    (hux (t) (set! a (foo t t)) c))
       hux))
 
 ;*---------------------------------------------------------------------*/
@@ -243,12 +242,12 @@
       (set-exit
        (an-exit1013)
        (let ()
-	  (push-exit! an-exit1013 #t)
+	  (push-exit! an-exit1013 1)
 	  (let ((exp (let ((body1018 (lambda () 9)))
 			(set-exit
 			 (an-exit1025)
 			 (let ()
-			    (push-exit! an-exit1025 #f)
+			    (push-exit! an-exit1025 0)
 			    (body1018))))))
 	     (set! exp #unspecified)
 	     (labels ((foo () (print exp)))
@@ -258,30 +257,6 @@
 (define (plante-7-bar f)
    (print (f)))
 
-;*---------------------------------------------------------------------*/
-;*    plante-8 ...                                                     */
-;*---------------------------------------------------------------------*/
-(define (plante-8)
-   (let ((exp (let ((handler1018
-		     (lambda (escape proc obj msg)
-			9)))
-		 (set-exit
-		  (an-exit1019)
-		  (dynamic-wind
-		     (lambda ()
-			(add-error-handler! handler1018 #f))
-		     (lambda () #f)
-		     (lambda () #f))))))
-      (set-exit
-       (an-exit1025)
-       (dynamic-wind
-	  (lambda ()
-	     (add-error-handler! #f #f))
-	  (lambda ()
-	     (eval exp))
-	  (lambda () #f)))
-      (set! exp #unspecified)))
-
 ;*---------------------------------------------------------------------*/
 ;*    test8 ...                                                        */
 ;*---------------------------------------------------------------------*/
@@ -298,17 +273,26 @@
 	 (p (open-input-string str)))
       (read/rp g p)))
 
+;*---------------------------------------------------------------------*/
+;*    static-application ...                                           */
+;*---------------------------------------------------------------------*/
+(define (static-application)
+   (((lambda (a b) b)
+     3
+     (lambda (x) x)) 3))
+
 ;*---------------------------------------------------------------------*/
 ;*    test-globalisation ...                                           */
 ;*---------------------------------------------------------------------*/
 (define (test-globalisation)
    (test-module "globalisation" "globalisation.scm")
-   (test "globalisation" ((test1)) 0)
-   (test "kapture" ((test2 1 2 3)) 1)
-   (test "kapture" (test3) 1)
-   (test "cell" ((test4 1 2) 3) 2)
-   (test "cell" ((test5 1 2 3 4) 2) 2)
-   (test "cell" ((test6 1 2 3 4) 2) 2)
-   (test "globalisation" (((test7 5) 6) 7) 5)
+   (test "globalisation.1" ((test1)) 0)
+   (test "globalisation.2" (((test7 5) 6) 7) 5)
+   (test "kapture.1" ((test2 1 2 3)) 1)
+   (test "kapture.2" (test3) 1)
+   (test "cell.1" ((test4 1 2) 3) 2)
+   (test "cell.2" ((test5 1 2 3 4) 2) 2)
+   (test "cell.3" ((test6 1 2 3 4) 2) 2)
    (test "rgc" (test8 "(foo(bar(gee)))") '(foo bar gee))
-   (test "_" (_plante-4 5 6) 11))
+   (test "_" (_plante-4 5 6) 11)
+   (test "ast" (static-application) 3))
