@@ -3,7 +3,7 @@
 ;*                                                                     */
 ;*    Author      :  Manuel Serrano                                    */
 ;*    Creation    :  Mon Nov 23 15:13:19 1992                          */
-;*    Last change :  Fri Feb  9 18:17:52 2001 (serrano)                */
+;*    Last change :  Fri Oct 22 16:25:28 2010 (serrano)                */
 ;*                                                                     */
 ;*    Des tests d'expanseurs.                                          */
 ;*---------------------------------------------------------------------*/
@@ -77,6 +77,77 @@
       ((2) 2)
       (else 3)))
 
+;*---------------------------------------------------------------------*/
+;*    Nested defines                                                   */
+;*---------------------------------------------------------------------*/
+(define-macro (define-glop a v)
+   `(begin
+       (current-date)
+       (define ,a ,v)))
+
+(define-macro (define-node sig v)
+      `(define ,sig ,v))
+
+(define (nested-define)
+   (define x 1)
+   (current-seconds)
+   (define-node t 10)
+   (define-glop y 20)
+   (current-seconds)
+   (+ x t y))
+
+;*---------------------------------------------------------------------*/
+;*    nested define 2                                                  */
+;*---------------------------------------------------------------------*/
+(define-macro (insert-code)
+   '(begin (define tag3 (list 'foo))))
+
+(define nested-define2 #f)
+
+(insert-code)
+   
+(let ((x tag3))
+   (begin
+      (insert-code)
+      (set! nested-define2 (eq? x tag3))))
+
+;*---------------------------------------------------------------------*/
+;*    nested define 3                                                  */
+;*---------------------------------------------------------------------*/
+(begin
+   (define tag2 (list 'foo)))
+
+(define nested-define3 #f)
+
+(let ((x tag2))
+   (begin
+     (define tag2 (list 'foo)))
+   (set! nested-define3 (eq? x tag2)))
+
+;*---------------------------------------------------------------------*/
+;*    nested define 4 ...                                              */
+;*---------------------------------------------------------------------*/
+(define tag1 (list 'foo))
+
+(define nested-define4 #f)
+
+(let ((x tag1))
+
+   (define tag1 (list 'foo))
+
+   (set! nested-define4 (eq? x tag1)))
+
+(define-expander cond-expandi
+   (lambda (x e)
+      (match-case x
+         ((?- ?- ((kwote else) . ?body))
+          (e `(begin ,@body) e)))))
+
+(define (nested-define5 x)
+   (cond-expandi
+      (toto '(print 'coucou))
+      (else (define (foo x) (not x)) (foo x))))
+
 ;*---------------------------------------------------------------------*/
 ;*    test-macro ...                                                   */
 ;*---------------------------------------------------------------------*/
@@ -86,6 +157,11 @@
    (test "O-macro" (integer->char) 1)
    (test "O-macro" (bar (lambda () 1)) 1)
    (test "eval" eval-macro 10)
-   (test "case-macro" (case-macro 2) 2))
+   (test "case-macro" (case-macro 2) 2)
+   (test "nested define" (nested-define) 31)
+   (test "nested define.2" nested-define2 #f)
+   (test "nested define.3" nested-define3 #f)
+   (test "nested define.4" nested-define4 #f)
+   (test "nested define.5" (nested-define5 #t) #f))
 
  
