@@ -1,9 +1,9 @@
 ;*=====================================================================*/
-;*    serrano/prgm/project/bigloo/recette/main.scm                     */
+;*    serrano/prgm/project/bigloo/bigloo/recette/main.scm              */
 ;*    -------------------------------------------------------------    */
 ;*    Author      :  Manuel Serrano                                    */
 ;*    Creation    :  Mon Nov  2 17:24:13 1992                          */
-;*    Last change :  Wed Nov 21 13:31:21 2001 (serrano)                */
+;*    Last change :  Mon Jul  5 10:53:18 2021 (serrano)                */
 ;*    -------------------------------------------------------------    */
 ;*    The recette entry point                                          */
 ;*=====================================================================*/
@@ -21,6 +21,7 @@
 	   bool
 	   list
 	   vector
+	   srfi4
 	   struct
 	   print
 	   bchar
@@ -38,14 +39,16 @@
 	   rgc-trap
 	   rgc-jm
 	   rgc-eval
+	   rgc-insert
 	   rgc
+	   lalr
 	   input-port
+	   mmap
 	   read
 	   callcc
 	   fringe
 	   tail
-	   extern
-	   big-file
+	   external
 	   sqic
 	   reval
 	   inline
@@ -53,6 +56,7 @@
 	   macro
 	   flonum
 	   number
+	   bignum
 	   define
 	   cse
 	   error
@@ -65,20 +69,29 @@
 	   import1
 	   import2
 	   object
-	   object4
-	   hygien
+	   object-sans
+	   object5
+	   object5-sans
+	   hygiene
 	   wind
 	   dsssl
 	   peek
 	   unicode
 	   optim
-	   pregexp)
-   
+	   pregexp
+	   system
+	   date
+           process
+           weakptr
+	   crypto
+	   crc)
    (export (do-test name thunk good?)
 	   (test-module name file)
-	   *recette-port*)
+	   *recette-port*
+	   *bigloo-path*
+	   *silent*)
    
-   (option (set! *debug* #f)))
+   (option (bigloo-debug-set! 0)))
    
 ;*---------------------------------------------------------------------*/
 ;*    Des variables statiques                                          */
@@ -93,6 +106,10 @@
 (define *tick-number*   -1)
 (define *callcc?*       #t)
 (define *dumping*       #f)
+(define *modules*       '())
+(define *bigloo-path*   (if (string=? (os-class) "unix")
+                            "../bin/bigloo.sh"
+                            "..\\bin\\bigloo.exe"))
 
 ;*---------------------------------------------------------------------*/
 ;*    tick ...                                                         */
@@ -109,14 +126,11 @@
       ((3) (write-char #\\))))
 
 ;*---------------------------------------------------------------------*/
-;*    recette-port ...                                                 */
+;*    if-module ...                                                    */
 ;*---------------------------------------------------------------------*/
-(define recette-port
-   (let ((port (open-output-file "recette.log")))
-      (if (not (output-port? port))
-	  (error "recette-port" "Can't open output-file" "recette.log")
-	  (lambda ()
-	     port))))
+(define (if-module sym thunk)
+   (if (or (null? *modules*) (memq sym *modules*))
+       (thunk)))
 
 ;*---------------------------------------------------------------------*/
 ;*    do-test ...                                                      */
@@ -129,40 +143,47 @@
 	  (and (flonum? result)
 	       (flonum? wanted)
 	       (<fl (absfl (-fl result wanted)) 0.00001))))
-   (try (let ((result (thunk)))
-	   (if (correct? result wanted)
-	       (begin
-		  (if (not *silent*)
+   (let ((err (cons 1 2)))
+      (define (test-fail result)
+	 (set! *nb-err* (+ 1 *nb-err*))
+	 (display* *test-number* #\. *module-name* "(" name ") ")
+	 (if (eq? result err)
+	     (display "abort: ")
+	     (begin
+		(display* "fail: ")
+		(display "provided [" )
+		(write-circle result)
+		(display "], ")))
+	 (display "wanted [")
+	 (write-circle wanted)
+	 (print "]"))
+      (bind-exit (esc)
+	 (with-exception-handler
+	    (lambda (e)
+	       (newline (current-error-port))
+	       (test-fail err)
+	       (flush-output-port (current-output-port))
+	       (flush-output-port (current-error-port))
+	       (error-notify e)
+	       (newline (current-error-port))
+	       (esc #f))
+	    (lambda ()
+	       (let ((result (thunk)))
+		  (if (correct? result wanted)
 		      (begin
-			 (display* *test-number* #\. *module-name* " : "
-				   name " --> ")
-			 (display "ok.")))
-		  (if *verbose*
-		      (begin
-			 (display " [")
-			 (write-circle result)
-			 (print "]"))
-		      (if (not *silent*)
-			  (newline))))
-	       (begin
-		  (set! *nb-err* (+ 1 *nb-err*))
-		  (display* *test-number* #\. *module-name* " : " name )
-		  (let ((p (open-output-string))
-			(w (open-output-string)))
-		     (display "provided [" p)
-		     (write result p)
-		     (display "]" p)
-		     (display "wanted [" w)
-		     (write wanted w)
-		     (display "]" w)
-		     (error name
-			    (close-output-port p)
-			    (close-output-port w))))))
-	(lambda (escape proc obj msg)
-	   (flush-output-port (current-output-port))
-	   (notify-error proc obj msg)
-	   (flush-output-port (current-error-port))
-	   (escape #f))))
+			 (if (not *silent*)
+			     (begin
+				(display* *test-number* #\. *module-name* " : "
+					  name " --> ")
+				(display "ok.")))
+			 (if *verbose*
+			     (begin
+				(display " [")
+				(write-circle result)
+				(print "]"))
+			     (if (not *silent*)
+				 (newline))))
+		      (test-fail result))))))))
 
 ;*---------------------------------------------------------------------*/
 ;*    test-module ...                                                  */
@@ -174,13 +195,30 @@
        (newline))
    (print file-name ":"))
 
+;*---------------------------------------------------------------------*/
+;*    recette-resume ...                                               */
+;*---------------------------------------------------------------------*/
+(define (recette-resume inc)
+   (set! *nb-err* (+fx inc *nb-err*))
+   (close-output-port *recette-port*)
+   (print #\Newline "------------------------------")
+   (if (> *nb-err* 0)
+       (begin
+	  (fprint (current-error-port) "recette failed: "
+	     *nb-err* " error(s) occurred.")
+	  -1)
+       (begin
+	  (print "recette succeeded: the " *nb-test*
+	     " tests are clear.")
+	  0)))
+
 ;*---------------------------------------------------------------------*/
 ;*    recette ...                                                      */
 ;*---------------------------------------------------------------------*/
 (define (recette argv)
    (args-parse (cdr argv)
       (section "Misc")
-      (("-help" (help "This help message"))
+      ((("-help" "--help") (help "This help message"))
        (args-parse-usage #f)
        (exit 0))
       (section "Verbosity")
@@ -199,7 +237,7 @@
 	   (set! *silent* #f)
 	   (set! *verbose* #t))
 	  (else
-	   (args-parse-error the-remaining-args))))
+	   (error 'recette "Illegal `--verbose' argument" level))))
       (("-args-parse" ?dum1 ?dum2 (help "Use this option to check ags-parse"))
        (print "dummy1=" dum1 " dummy2=" dum2))
       (("--args-parse" ?dum1 ?dum2 (help "--args-parse dummy1 dummy2"
@@ -213,89 +251,96 @@
        (set! *callcc?* #f))
       (("--dump" ?fname (help "Don't test but dump a binary structure"))
        (set! *dumping* fname))
+      ((("-m" "--module") ?mod (help "Module to be tested"))
+       (set! *modules* (cons (string->symbol mod) *modules*)))
+      (("--bigloo" ?bigloo-path (help "Path to the Bigloo compiler"))
+       (set! *bigloo-path* bigloo-path))
       (else
-       (args-parse-error else)))
+       (error 'recette "Illegal argument" else)))
    (if (string? *dumping*)
        (begin
 	  (dump-obj *dumping*)
 	  (print #\Newline "------------------------------")
 	  (print (cond-expand
 		    (bigloo-c "C dump done...")
-		    (bigloo-jvm "JVM dump done..."))))
+		    (bigloo-jvm "JVM dump done...")
+		    (bigloo-.net ".NET dump done..."))))
        (begin
 	  (set! *recette-port* (open-output-file "recette.log"))
 	  (if (not (output-port? *recette-port*))
 	      (error "recette-port" "Can't open output-file" "recette.log"))
-	  (try (begin
-		  (test-vital)
-		  (test-bps)
-		  (test-cell)
-		  (test-modulel)
-		  (test-hash)
-		  (test-bool)
-		  (test-number)
-		  (test-flonum)
-		  (test-list)
-		  (test-vector)
-		  (test-struct)
-		  (test-print)
-		  (test-char)
-		  (test-string)
-		  (test-kwote)
-		  (test-case)
-		  (test-bind-exit)
-		  (test-vararity)
-		  (test-apply)
-		  (test-globalisation)
-		  (test-kapture)
-		  (test-filtre)
-		  (test-match)
-		  (test-rgc)
-		  (test-rgc-trap)
-		  (test-rgc-jm)
-		  (test-rgc-eval)
-		  (test-input-port)
-		  (test-read)
-		  (if *callcc?*
-		      (begin
-			 (test-callcc)
-			 (test-fringe)
-			 (test-wind)))
-		  (test-dsssl)
-		  (test-tail)
-		  (test-extern)
-		  (test-sqic)
-		  (test-eval)
-		  (test-inline)
-		  (test-letrec)
-		  (test-macro)
-		  (test-define)
-		  (test-cse)
-		  (test-error)
-		  (test-include)
-		  (test-0cfa)
-		  (test-sua)
-		  (test-alias)
-		  (test-object)
-		  (test-object4)
-		  (test-hygien)
-		  (test-peek)
-		  (test-unicode)
-		  (test-optim)
-		  (test-pregexp)
-		  )
-	       (lambda (escape proc mes obj)
-		  (notify-error proc mes obj)
-		  (set! *nb-err* (+ 1 *nb-err*))
-		  (exit -1)))
-	  (close-output-port *recette-port*)
-	  (print #\Newline "------------------------------")
-	  (if (> *nb-err* 0)
+	  (if-module 'vital test-vital)
+	  (if-module 'bps test-bps)
+	  (if-module 'cell test-cell)
+	  (if-module 'modulel test-modulel)
+	  (if-module 'hash test-hash)
+	  (if-module 'bool test-bool)
+	  (if-module 'number test-number)
+	  (if-module 'flonum test-flonum)
+	  (if-module 'bignum test-bignum)
+	  (if-module 'list test-list)
+	  (if-module 'vector test-vector)
+	  (if-module 'srfi4 test-srfi4)
+	  (if-module 'struct test-struct)
+	  (if-module 'print test-print)
+	  (if-module 'char test-char)
+	  (if-module 'string test-string)
+	  (if-module 'kwote test-kwote)
+	  (if-module 'case test-case)
+	  (if-module 'bind-exit test-bind-exit)
+	  (if-module 'vararity test-vararity)
+	  (if-module 'apply test-apply)
+	  (if-module 'globalisation test-globalisation)
+	  (if-module 'kapture test-kapture)
+	  (if-module 'filtre test-filtre)
+	  (if-module 'match test-match)
+	  (if-module 'rgc test-rgc)
+	  (if-module 'rgc-trap test-rgc-trap)
+	  (if-module 'rgc-jm test-rgc-jm)
+	  (if-module 'rgc-eval test-rgc-eval)
+	  (if-module 'rgc-insert test-rgc-insert)
+	  (if-module 'lalr test-lalr)
+	  (if-module 'input-port test-input-port)
+	  (if-module 'mmap test-mmap)
+	  (if-module 'read test-read)
+	  (if *callcc?*
 	      (begin
-		 (notify-error "recette" *nb-err* "error(s) occur")
-		 -1)
-	      (begin
-		 (print "recette done, the " *nb-test* " tests are clear")
-		 0)))))
+		 (if-module 'callcc test-callcc)
+		 (if-module 'fringe test-fringe)
+		 (if-module 'wind test-wind)))
+	  (if-module 'dsssl test-dsssl)
+	  (if-module 'tail test-tail)
+	  (if-module 'external test-external)
+	  (if-module 'sqic test-sqic)
+	  (if-module 'eval test-eval)
+	  (if-module 'inline test-inline)
+	  (if-module 'letrec test-letrec)
+	  (if-module 'macro test-macro)
+	  (if-module 'define test-define)
+	  (if-module 'cse test-cse)
+	  (if-module 'error test-error)
+	  (if-module 'include test-include)
+	  (if-module '0cfa test-0cfa)
+	  (if-module 'sua test-sua)
+	  (if-module 'alias test-alias)
+	  (if-module 'object test-object)
+	  (if-module 'object-sans test-object-sans)
+	  (if-module 'object5 test-object5)
+	  (if-module 'object5-sans test-object5-sans)
+	  (if-module 'hygiene test-hygiene)
+	  (if-module 'peek test-peek)
+	  (if-module 'unicode test-unicode)
+	  (if-module 'optim test-optim)
+	  (if-module 'pregexp test-pregexp)
+	  (if-module 'system test-system)
+	  (if-module 'date test-date)
+	  (if-module 'weakptr test-weakptr)
+	  (if-module 'crc test-crc)
+	  (cond-expand
+	     (bigloo-.net #t)
+	     (else (if-module 'process test-process)))
+	  (if-module 'wind (lambda () (test-wind-sans-handler recette-resume)))
+	  (recette-resume 0))))
+
  
  
