@@ -3,4 +3,8 @@
    (export (class foo2::foo1
 	      y)
 	   (class bar3::bar2
-	      c)))
+	      c)
+	   (final-class wx x)
+	   (wide-class ws::wx y)))
+
+   
