@@ -1,28 +1,21 @@
 /*=====================================================================*/
-/*    serrano/prgm/project/bigloo/runtime/Clib/ceval.c                 */
+/*    serrano/prgm/project/bigloo/bigloo/runtime/Clib/ceval.c          */
 /*    -------------------------------------------------------------    */
 /*    Author      :  Manuel Serrano                                    */
 /*    Creation    :  Fri Sep 24 10:20:30 1993                          */
-/*    Last change :  Sat Feb 17 09:22:17 2001 (serrano)                */
+/*    Last change :  Tue Jul 11 17:05:47 2023 (serrano)                */
 /*    -------------------------------------------------------------    */
 /*    Quelques petites fonctions pour l'interprete (principalement     */
 /*    des fonctions de debug et de trace)                              */
 /*=====================================================================*/
 #include <bigloo.h>
 
-/*---------------------------------------------------------------------*/
-/*    Une recup ...                                                    */
-/*---------------------------------------------------------------------*/
-extern long  bgl_list_length();
-extern obj_t string_to_bstring();
-
 /*---------------------------------------------------------------------*/
 /*    void                                                             */
-/*    init_eval_cnst ...                                               */
+/*    bgl_init_eval_cnst ...                                           */
 /*---------------------------------------------------------------------*/
 void
-init_eval_cnst()
-{
+bgl_init_eval_cnst() {
    ;
 }
 
@@ -31,10 +24,8 @@ init_eval_cnst()
 /*    funcall_0 ...                                                    */
 /*---------------------------------------------------------------------*/
 obj_t
-eval_funcall_0( fun )
-obj_t fun;
-{
-   return PROCEDURE_ENTRY( fun )( fun, BEOA );
+eval_funcall_0(obj_t fun) {
+   return BGL_PROCEDURE_CALL0(fun);
 }
 
 /*---------------------------------------------------------------------*/
@@ -42,10 +33,8 @@ obj_t fun;
 /*    funcall_1 ...                                                    */
 /*---------------------------------------------------------------------*/
 obj_t
-eval_funcall_1( fun, a0 )
-obj_t fun, a0;
-{
-   return PROCEDURE_ENTRY( fun )( fun, a0, BEOA );
+eval_funcall_1(obj_t fun, obj_t a0) {
+   return BGL_PROCEDURE_CALL1(fun, a0);
 }
        
 /*---------------------------------------------------------------------*/
@@ -53,10 +42,8 @@ obj_t fun, a0;
 /*    funcall_2 ...                                                    */
 /*---------------------------------------------------------------------*/
 obj_t
-eval_funcall_2( fun, a0, a1 )
-obj_t fun, a0, a1;
-{
-   return PROCEDURE_ENTRY( fun )( fun, a0, a1, BEOA );
+eval_funcall_2(obj_t fun, obj_t a0, obj_t a1) {
+   return BGL_PROCEDURE_CALL2(fun, a0, a1);
 }
        
 /*---------------------------------------------------------------------*/
@@ -64,10 +51,8 @@ obj_t fun, a0, a1;
 /*    funcall_3 ...                                                    */
 /*---------------------------------------------------------------------*/
 obj_t
-eval_funcall_3( fun, a0, a1, a2 )
-obj_t fun, a0, a1, a2;
-{
-   return PROCEDURE_ENTRY( fun )( fun, a0, a1, a2, BEOA );
+eval_funcall_3(obj_t fun, obj_t a0, obj_t a1, obj_t a2) {
+   return BGL_PROCEDURE_CALL3(fun, a0, a1, a2);
 }
 
 /*---------------------------------------------------------------------*/
@@ -75,25 +60,16 @@ obj_t fun, a0, a1, a2;
 /*    funcall_4 ...                                                    */
 /*---------------------------------------------------------------------*/
 obj_t
-eval_funcall_4( fun, a0, a1, a2, a3 )
-obj_t fun, a0, a1, a2, a3;
-{
-   return PROCEDURE_ENTRY( fun )( fun, a0, a1, a2, a3, BEOA );
+eval_funcall_4(obj_t fun, obj_t a0, obj_t a1, obj_t a2, obj_t a3) {
+   return BGL_PROCEDURE_CALL4(fun, a0, a1, a2, a3);
 }
 
 /*---------------------------------------------------------------------*/
 /*    obj_t                                                            */
 /*    eval_apply ...                                                   */
 /*---------------------------------------------------------------------*/
+BGL_RUNTIME_DEF
 obj_t
-eval_apply( fun, list )
-obj_t fun, list;
-{
-   return apply( fun, list );
+eval_apply(obj_t fun, obj_t list) {
+   return apply(fun, list);
 }
-      
-
-
-
-
-
