@@ -1,39 +1,44 @@
 /*=====================================================================*/
-/*    serrano/prgm/project/bigloo/runtime/Clib/cforeign.c              */
+/*    serrano/prgm/project/bigloo/bigloo/runtime/Clib/cforeign.c       */
 /*    -------------------------------------------------------------    */
 /*    Author      :  Manuel Serrano                                    */
 /*    Creation    :  Tue Oct 19 08:45:22 1993                          */
-/*    Last change :  Sun May  7 09:50:52 2000 (serrano)                */
+/*    Last change :  Tue Apr 17 08:00:01 2018 (serrano)                */
 /*    -------------------------------------------------------------    */
 /*    La gestion de l'interface etrangere                              */
 /*=====================================================================*/
 #include <bigloo.h>
 
+/*---------------------------------------------------------------------*/
+/*    Importations                                                     */
+/*---------------------------------------------------------------------*/
+extern obj_t string_to_symbol( char * );
+
 /*---------------------------------------------------------------------*/
 /*    obj_t                                                            */
 /*    cobj_to_foreign ...                                              */
 /*---------------------------------------------------------------------*/
+BGL_RUNTIME_DEF
 obj_t
-cobj_to_foreign( obj_t id, void *cobj )
-{
+cobj_to_foreign( obj_t id, void *cobj ) {
    obj_t handle;
 
    handle = GC_MALLOC( FOREIGN_SIZE );
-   
-   handle->foreign_t.header = MAKE_HEADER( FOREIGN_TYPE, FOREIGN_SIZE );
-   handle->foreign_t.cobj   = (void *)cobj;
-   handle->foreign_t.id     = id;
 
-   return BGL_HEAP_DEBUG_MARK_OBJ( BREF( handle ) );
+   handle->foreign.header = MAKE_HEADER( FOREIGN_TYPE, FOREIGN_SIZE );
+   handle->foreign.cobj = (void *)cobj;
+   handle->foreign.id = id;
+
+   return BREF( handle );
 }
 
 /*---------------------------------------------------------------------*/
 /*    obj_t                                                            */
 /*    void_star_to_obj ...                                             */
 /*---------------------------------------------------------------------*/
+BGL_EXPORTED_DEF
 obj_t
-void_star_to_obj( void *cobj )
-{
+void_star_to_obj( void *cobj ) {
    static obj_t id = BUNSPEC;
 
    if( !SYMBOLP( id ) )
@@ -43,29 +48,29 @@ void_star_to_obj( void *cobj )
 }
 
 /*---------------------------------------------------------------------*/
-/*    obj_t                                                            */
+/*    long                                                             */
 /*    obj_to_cobj ...                                                  */
 /*---------------------------------------------------------------------*/
-obj_t
-obj_to_cobj( obj_t obj )
-{
+BGL_EXPORTED_DEF
+long
+obj_to_cobj( obj_t obj ) {
    if( INTEGERP( obj ) )
-      return (obj_t)CINT( obj );
+      return (long)CINT( obj );
    if( BOOLEANP( obj ) )
-      return (obj_t)((long)CBOOL( obj ));
+      return (long)((long)CBOOL( obj ));
    if( STRINGP( obj ) )
-      return (obj_t)BSTRING_TO_STRING( obj );
+      return (long)BSTRING_TO_STRING( obj );
    if( CHARP( obj ) )
-      return (obj_t)((long)CCHAR( obj ));
+      return (long)((long)CCHAR( obj ));
    if( FOREIGNP( obj ) )
-      return (obj_t)FOREIGN_TO_COBJ( obj );
+      return (long)FOREIGN_TO_COBJ( obj );
    if( REALP( obj ) )
-      return the_failure( string_to_bstring( "obj->cobj" ),
-			  string_to_bstring( "Can't cast a real to foreign" ),
-			  obj);
+      return (long)the_failure( string_to_bstring( "obj->cobj" ),
+				string_to_bstring( "Can't cast a real to foreign" ),
+				obj);
    else
-      return the_failure( string_to_bstring( "obj->cobj" ),
-			  string_to_bstring( "Illegal object type" ),
-			  obj);
+      return (long)the_failure( string_to_bstring( "obj->cobj" ),
+				string_to_bstring( "Illegal object type" ),
+				obj);
 }
  
