@@ -1,9 +1,9 @@
 /*---------------------------------------------------------------------*/
-/*    serrano/prgm/project/bigloo/runtime/Clib/cmain.c                 */
+/*    serrano/prgm/project/bigloo/bigloo/runtime/Clib/cmain.c          */
 /*                                                                     */
 /*    Author      :  Manuel Serrano                                    */
 /*    Creation    :  Fri Jul 17 09:40:49 1992                          */
-/*    Last change :  Thu Nov 22 11:04:50 2001 (serrano)                */
+/*    Last change :  Sun Sep 29 06:58:30 2019 (serrano)                */
 /*                                                                     */
 /*    Le fichier de main de toute application. Comme je m'y prends     */
 /*    plus intelligement que dans la version 0.8 (si, si :-), je       */
@@ -31,64 +31,103 @@
 #define DEFAULT_HEAP_SIZE 4
 
 /* The initial heap size (in mega byte) */
-long heap_size = DEFAULT_HEAP_SIZE;
+BGL_RUNTIME_DEF long heap_size = DEFAULT_HEAP_SIZE;
 
 /*---------------------------------------------------------------------*/
-/*    Des recuperations externes                                       */
+/*    imports                                                          */
 /*---------------------------------------------------------------------*/
-extern void cref();
-extern void memshow();
-extern char *getenv();
-extern void init_eval_cnst();
-extern obj_t c_constant_string_to_string();
-extern obj_t c_error();
-extern void init_objects();
-extern int bigloo_main();
-extern obj_t bigloo_exit_apply();
+extern obj_t cref( obj_t );
+extern void memshow( char *, char *, long );
+extern void bgl_init_eval_cnst();
+extern obj_t c_constant_string_to_string( char * );
+extern void c_error( char *, char *, int );
+extern void bgl_init_objects();
+extern obj_t bigloo_exit_apply( obj_t );
+extern void *bgl_callcc_get_top_of_stack( void * );
+extern void bgl_end_io();
+extern void GC_set_max_heap_size( long );
+extern void GC_set_all_interior_pointers( int );
+extern obj_t bgl_signal( int, obj_t );
+
+#ifndef _MSC_VER
+#include <stdlib.h>
+#endif
 
 /*---------------------------------------------------------------------*/
-/*    Une variable pour memoriser le bas de la pile                    */
+/*    bottom stack registration                                        */
 /*---------------------------------------------------------------------*/
-char *stack_bottom;
-long  glob_dummy;
+long glob_dummy;
 
-obj_t command_line    = 0L;
+BGL_RUNTIME_DEF obj_t command_line = 0L;
 char *executable_name = 0L;
 char **bgl_envp;
 int bgl_envp_len;
 
 /*---------------------------------------------------------------------*/
-/*    get_top_of_stack ...                                             */
-/*    -------------------------------------------------------------    */
-/*    On ne peut pas chercher le top de la pile dans la fonction       */
-/*    `call_cc' car on ne sait pas si les arguments d'une fonction     */
-/*    sont ranges apres les variables locales ou le contraire. Il me   */
-/*    semble que la solution est donc d'appeller une autre fonction,   */
-/*    prendre une variable locale et retourner cette adresse car on    */
-/*    est sur qu'elle sera plus grande (ou plus petite si la pile      */
-/*    decroit) que la valeur du frame pointer de `call_cc'.            */
+/*    obj_t                                                            */
+/*    bgl_getenv_all ...                                               */
+/*---------------------------------------------------------------------*/
+obj_t
+bgl_getenv_all() {
+   obj_t alist = BNIL;
+   int i;
+
+   for( i = 0; i < bgl_envp_len; i++ ) {
+      char *sk = bgl_envp[ i ];
+      char *sv = strchr( sk, '=' );
+
+      obj_t k = string_to_bstring_len( sk, sv - sk );
+      obj_t v = string_to_bstring( sv + 1 );
+      alist = MAKE_PAIR( MAKE_PAIR( k, v ), alist );
+   }
+
+   return alist;
+}
+
+/*---------------------------------------------------------------------*/
+/*    obj_t                                                            */
+/*    exit_mutex ...                                                   */
+/*---------------------------------------------------------------------*/
+extern obj_t bgl_exit_mutex();
+
+/*---------------------------------------------------------------------*/
+/*    bgl_get_top_of_stack ...                                         */
 /*    -------------------------------------------------------------    */
 /*    This function can't be included inside callcc.c otherwise it may */
 /*    be incorrectly inlined by optimizing C compilers.                */
 /*---------------------------------------------------------------------*/
-char *
-get_top_of_stack()
-{
-   long *dummy; /* dummy est un long et pas un char car je veux */
-                /* etre sur de ne pas avoir de pbm d'alignement*/
-
-   return (char *)(&(dummy));
+BGL_NOINLINE void *
+bgl_get_top_of_stack() {
+   void *dummy;
+   return bgl_callcc_get_top_of_stack( &dummy );
 }
 
 /*---------------------------------------------------------------------*/
 /*    bigloo_exit ...                                                  */
 /*---------------------------------------------------------------------*/
-obj_t
-bigloo_exit( obj_t val )
-{
+BGL_RUNTIME_DEF obj_t
+bigloo_exit( obj_t val ) {
+   int n;
    val = bigloo_exit_apply( val );
-   FREE_ALLOCATION();
-   exit( (long)INTEGERP( val ) ? (long)CINT( val ) : 0 );
+
+   BGL_MUTEX_LOCK( bgl_exit_mutex() );
+
+   bgl_end_io();
+   
+   n = (long)INTEGERP( val ) ? (long)CINT( val ) : 0;
+   exit( n );
+   
+   BGL_MUTEX_UNLOCK( bgl_exit_mutex() );
+   
+   return val;
+}
+
+/*---------------------------------------------------------------------*/
+/*    bigloo_backend ...                                               */
+/*---------------------------------------------------------------------*/
+BGL_RUNTIME_DEF char *
+bigloo_backend() {
+   return "bigloo-c";
 }
 
 /*---------------------------------------------------------------------*/
@@ -96,72 +135,84 @@ bigloo_exit( obj_t val )
 /*    -------------------------------------------------------------    */
 /*    Le vrai point d'entree d'une application `bigloo'. La fonction   */
 /*    `main' qui est cree par le compilo ne sert a rien. Elle existe   */
-/*    uniquement car je ne suis pas que tous les compilos supporte que */
-/*    la fonction C `main' soit mise en librairie. Comme ca, je suis   */
-/*    tranquile.                                                       */
+/*    uniquement car je ne suis pas sur que tous les compilos supporte */
+/*    que la fonction C `main' soit mise en librairie. Comme ca, je    */
+/*    suis tranquile.                                                  */
 /*---------------------------------------------------------------------*/
-int
-_bigloo_main( int argc, char *argv[], char *envp[] )
-{
+BGL_EXPORTED_DEF int
+_bigloo_main( int argc,
+	      char *argv[],
+	      char *env[],
+	      obj_t (*bigloo_main)(obj_t),
+	      int (*libinit)(int, char *[], char *[]),
+	      long uheapsize ) {
    long  mega_size;
+   long  maxheap_size;
    char *env_size;
    obj_t cons;
    long  i;
-
+   
    /* we store the global environment */
-   bgl_envp = envp;
+   bgl_envp = env;
    bgl_envp_len = 0;
 
-   if( envp ) {
-      char **runner = envp;
+   if( env ) {
+      char **runner = env;
       while( *runner ) bgl_envp_len++, runner++;
    }
-   
-   /* on memorise l'adresse du bas de la pile */
-   stack_bottom = (char *)&argc;
-   
+
    /* on initialise le tas */
-   if( !(env_size = getenv( "BIGLOOHEAP" )) )
-      mega_size = heap_size;
-   else
+   if( !(env_size = getenv( "BIGLOOHEAP" )) ) {
+      mega_size = uheapsize ? uheapsize : DEFAULT_HEAP_SIZE;
+   } else {
       mega_size = atoi( env_size );
+   }
 
-#if( (THE_GC == BOEHM_GC) &&                                     \
-     ( (!defined( BOEHM_GC_VERSION)) ||                          \
-       (BOEHM_GC_VERSION == 2)       ||                          \
-       (BOEHM_GC_VERSION == 3) ) )
-   heap_size = MegTo4K( mega_size );
-#else
-   heap_size = MegToByte( mega_size );
-#endif
+#if( BGL_GC == BGL_BOEHM_GC )
+   if( mega_size > 2048 ) {
+      char mes[ 80 ];
 
-   if( !INIT_ALLOCATION( heap_size ) )
-   {
-      char mes[ 600 ];
-
-      sprintf( mes, "%ld Meg wanted", heap_size / (1024 * 1024) );
-      c_error( "Can't allocate heap", mes, -10 );
+      sprintf( mes, "%ldMB wanted", mega_size );
+      c_error( "Heap size too large (> 2048MB)", mes, -10 );
       return 1;
    }
-   else
-   {
-      /* on initialise les objects pre-alloues (les chars, bool, ...) */
-      init_objects();
+   heap_size = MegToByte( mega_size );
+   if( (env_size = getenv( "BIGLOOMAXHEAP" )) ) {
+      maxheap_size = atoi( env_size );
+      GC_set_max_heap_size(maxheap_size * 1024*  1024);
+   }
+#endif
 
-      /* on initialise les constantes du fichier `Clib/eval.c' */
-      init_eval_cnst();
-	 
-      /* on construit la liste des argv */
-      for( i = argc - 1, cons = BNIL; i >= 0; i-- )
-         cons = MAKE_PAIR( c_constant_string_to_string( argv[ i ] ), cons );
+   if( !INIT_ALLOCATION( heap_size ) ) {
+      char mes[ 80 ];
 
-      /* on sauve le nom de l'executable */
-      executable_name = argv[ 0 ];
+      sprintf( mes, "%ldMB wanted", heap_size );
+      c_error( "Can't allocate heap", mes, -10 );
+      return 1;
+   } else {
+      /* initialize the libraries */
+      libinit( argc, argv, env );
       
-      /* on met a jour la variable `command-line' */
+      /* store the executable name */
+      executable_name = argv[ 0 ];
+
+      /* setup preallocated objects (chars, bool, ...) */
+      bgl_init_objects();
+
+      /* store the stack bottom address */
+      BGL_ENV_STACK_BOTTOM_SET( BGL_CURRENT_DYNAMIC_ENV(), (char *)&mega_size );
+      
+      /* initialize constants */
+      bgl_init_eval_cnst();
+
+      /* build the Bigloo command line list */
+      for( i = argc - 1, cons = BNIL; i >= 0; i-- ) {
+         cons = MAKE_PAIR( c_constant_string_to_string( argv[ i ] ), cons );
+      }
+
       command_line = cons;
 
-      /* initialize the random seed */
+      /* random seed */
       {
 	 time_t taux;
 	 struct tm *tm;
@@ -169,12 +220,21 @@ _bigloo_main( int argc, char *argv[], char *envp[] )
 	 tm = gmtime( &taux );
 	    
 	 srand( (24 * (60 * tm->tm_sec + tm->tm_min)) + tm->tm_hour );
+
+#if( BGL_HAVE_GMP )
+	 /* big numbers init */
+	 gmp_randinit_default( gmp_random_state );
+	 gmp_randseed_ui( gmp_random_state,
+			  (24 * (60 * tm->tm_sec + tm->tm_min)) + tm->tm_hour );
+#endif	 
       }
+
+      /* install sigsegv handler for stack overflow interception */
+      bgl_signal( SIGSEGV, BUNSPEC );
       
-      /* on appelle le main de l'utilisateur */
+      /* jump into the application main */
       bigloo_main( cons );
+
       return 0;
    }
 }
-
-
