@@ -1,52 +1,50 @@
 /*=====================================================================*/
-/*    serrano/prgm/project/bigloo2.3/runtime/Clib/cstruct.c            */
+/*    serrano/prgm/project/bigloo/bigloo/runtime/Clib/cstruct.c        */
 /*    -------------------------------------------------------------    */
 /*    Author      :  Manuel Serrano                                    */
 /*    Creation    :  Tue Jun 16 09:28:53 1992                          */
-/*    Last change :  Fri Aug 25 10:22:44 2000 (serrano)                */
-/*    Copyright   :  2000 Manuel Serrano                               */
+/*    Last change :  Tue Apr 17 07:58:52 2018 (serrano)                */
+/*    Copyright   :  2000-18 Manuel Serrano                            */
 /*    -------------------------------------------------------------    */
 /*    Structures constructions.                                        */
 /*=====================================================================*/
 #include <bigloo.h>
 
 /*---------------------------------------------------------------------*/
-/*    obj_t                                                            */
+/*    static obj_t                                                     */
 /*    set_struct ...                                                   */
 /*---------------------------------------------------------------------*/
-obj_t
-set_struct( obj_t heap, obj_t key, int len )
-{
+static obj_t
+set_struct( obj_t heap, obj_t key, int len ) {
    obj_t structure = (obj_t)heap;
    
 #if( !defined( TAG_STRUCTURE ) )
-   structure->struct_t.header = MAKE_HEADER( STRUCT_TYPE, byte_size );
+   structure->structure.header = MAKE_HEADER( STRUCT_TYPE, 0 );
 #endif	
 
-   structure->struct_t.key    = key;
-   structure->struct_t.length = len;
+   structure->structure.key = key;
+   structure->structure.length = len;
    
-   return BGL_HEAP_DEBUG_MARK_OBJ( BSTRUCTURE( structure ) );
+   return BSTRUCTURE( structure );
 }
    
 /*---------------------------------------------------------------------*/
 /*    create_struct ...                                                */
 /*---------------------------------------------------------------------*/
+BGL_RUNTIME_DEF
 obj_t
-create_struct( obj_t key, int len )
-{
-   int  byte_size = STRUCT_SIZE + ( (len-1) * OBJ_SIZE );
+create_struct( obj_t key, int len ) {
+   int byte_size = STRUCT_SIZE + ( (len-1) * OBJ_SIZE );
    
    return set_struct( (obj_t)GC_MALLOC( byte_size ), key, len );
 }
 
 /*---------------------------------------------------------------------*/
-/*    obj_t                                                            */
+/*    static obj_t                                                     */
 /*    fill_struct ...                                                  */
 /*---------------------------------------------------------------------*/
-obj_t
-fill_struct( obj_t structure, int len, obj_t init )
-{
+static obj_t
+fill_struct( obj_t structure, int len, obj_t init ) {
    int i;
    
    for( i = 0; i < len; i++ )
@@ -58,9 +56,9 @@ fill_struct( obj_t structure, int len, obj_t init )
 /*---------------------------------------------------------------------*/
 /*    make_struct ...                                                  */
 /*---------------------------------------------------------------------*/
+BGL_RUNTIME_DEF
 obj_t
-make_struct( obj_t key, int len, obj_t init )
-{
+make_struct( obj_t key, int len, obj_t init ) {
    obj_t structure;
 
    structure = create_struct( key, len );
