@@ -1,9 +1,9 @@
 ;*=====================================================================*/
-;*    serrano/prgm/project/bigloo/runtime/Eval/expand.scm              */
+;*    serrano/prgm/project/bigloo/bigloo/runtime/Eval/expand.scm       */
 ;*    -------------------------------------------------------------    */
 ;*    Author      :  Manuel Serrano                                    */
 ;*    Creation    :  Thu Nov  3 09:57:39 1994                          */
-;*    Last change :  Mon May  7 18:45:05 2001 (serrano)                */
+;*    Last change :  Sun Aug 25 09:12:58 2019 (serrano)                */
 ;*    -------------------------------------------------------------    */
 ;*    La macro expansion de l'interprete                               */
 ;*=====================================================================*/
@@ -20,35 +20,62 @@
 	    __structure
 	    __tvector
 	    __bexit
+	    __bignum
 	    __os
+	    __param
+	    __object
+	    __thread
+	    __reader
+	    __bit
 	    
 	    __r4_numbers_6_5
 	    __r4_numbers_6_5_fixnum
 	    __r4_numbers_6_5_flonum
+	    __r4_numbers_6_5_flonum_dtoa
 	    __r4_characters_6_6
 	    __r4_equivalence_6_2
 	    __r4_booleans_6_1
 	    __r4_symbols_6_4
 	    __r4_strings_6_7
 	    __r4_pairs_and_lists_6_3
-	    __r4_input_6_10_2
 	    __r4_control_features_6_9
 	    __r4_vectors_6_8
 	    __r4_ports_6_10_1
 	    __r4_output_6_10_3
-
+	    
 	    __evenv
+	    __evutils
 	    __macro)
-
-   (export  (expand             <expression>)
-	    (expand-once        <expression>)
-	    (parse-formal-ident <expression>)))
+   
+   (extern  (macro $lexical-stack::pair-nil ()
+		   "BGL_LEXICAL_STACK")
+	    (macro $lexical-stack-set!::void (::pair-nil)
+		   "BGL_LEXICAL_STACK_SET"))
+	    
+   (java    (class foreign
+	       (method static $lexical-stack::pair-nil ()
+		       "BGL_LEXICAL_STACK")
+	       (method static $lexical-stack-set!::void (::pair-nil)
+		       "BGL_LEXICAL_STACK_SET")))
+   
+   (export  (expand ::obj)
+	    (expand! ::obj)
+	    (expand-once ::obj)
+	    (%lexical-stack::pair-nil)
+	    (%with-lexical ::pair-nil ::obj ::procedure ::obj)
+	    (expand-error proc msg obj)))
 
 ;*---------------------------------------------------------------------*/
 ;*    expand ...                                                       */
 ;*---------------------------------------------------------------------*/
 (define (expand x)
    (initial-expander x initial-expander))
+ 
+;*---------------------------------------------------------------------*/
+;*    expand! ...                                                      */
+;*---------------------------------------------------------------------*/
+(define (expand! x)
+   (initial-expander! x initial-expander!))
 
 ;*---------------------------------------------------------------------*/
 ;*    expand-once ...                                                  */
@@ -60,92 +87,115 @@
 ;*    initial-expander ...                                             */
 ;*---------------------------------------------------------------------*/
 (define (initial-expander x e)
+   (initial-expander/application x e application-eval-expander))
+
+;*---------------------------------------------------------------------*/
+;*    initial-expander! ...                                            */
+;*---------------------------------------------------------------------*/
+(define (initial-expander! x e)
+   (initial-expander/application x e application-eval-expander!))
+
+;*---------------------------------------------------------------------*/
+;*    initial-expander/application ...                                 */
+;*---------------------------------------------------------------------*/
+(define (initial-expander/application x e ae)
    (let ((e1 (cond
 		((symbol? x)
-		 *identifier-eval-expander*)
+		 identifier-eval-expander)
+		((null? x)
+		 (error "expand" "Illegal form" '()))
 		((not (pair? x))
 		 (lambda (x e) x))
 		((symbol? (car x))
-		 (let ((id (car (parse-formal-ident (car x)))))
-		    (let ((b (get-eval-expander id)))
-		       (if b
-			   b
-			   *application-eval-expander*))))
+		 (cond
+		    ((get-eval-expander (car x))
+		     =>
+		     (lambda (x) x))
+		    (else
+		     (let* ((loc (get-source-location x))
+			    (id (car (parse-formal-ident (car x) loc))))
+			(cond
+			   ((pair? (assq id (%lexical-stack)))
+			    ae)
+			   ((get-eval-expander id)
+			    =>
+			    (lambda (x) x))
+			   (else
+			    ae))))))
 		(else
-		 *application-eval-expander*))))
+		 ae))))
       (let ((new (e1 x e)))
-	 (if (and (pair? new)
-		  (not (epair? new))
-		  (epair? x))
+	 (if (and (pair? new) (not (epair? new)) (epair? x))
 	     (econs (car new) (cdr new) (cer x))
 	     new))))
 
 ;*---------------------------------------------------------------------*/
-;*    *identifier-eval-expander* ...                                   */
+;*    identifier-eval-expander ...                                     */
 ;*---------------------------------------------------------------------*/
-(define (*identifier-eval-expander* x e)
+(define (identifier-eval-expander x e)
    x)
 
 ;*---------------------------------------------------------------------*/
-;*    *application-eval-expander* ...                                  */
+;*    application-eval-expander ...                                    */
 ;*---------------------------------------------------------------------*/
-(define (*application-eval-expander* x e)
+(define (application-eval-expander x e)
    (let loop ((x x))
       (cond
 	 ((null? x)
 	  '())
 	 ((not (pair? x))
 	  (error "application" "Illegal form" x))
+	 ((epair? x)
+	  (econs (e (car x) e) (loop (cdr x)) (cer x)))
 	 (else
 	  (cons (e (car x) e) (loop (cdr x)))))))
 
 ;*---------------------------------------------------------------------*/
-;*    parse-formal-ident ...                                           */
+;*    application-eval-expander! ...                                   */
 ;*---------------------------------------------------------------------*/
-(define (parse-formal-ident ident)
-   (cond
-      ((or (eq? ident #!optional)
-	   (eq? ident #!rest)
-	   (eq? ident #!key))
-       (cons (gensym 'dsssl) '()))
-      ((and (pair? ident) (symbol? (car ident)))
-       (cons ident '()))
-      ((not (symbol? ident))
-       (error #f "Illegal formal identifier" ident))
-      (else
-       (let* ((string (symbol->string ident))
-	      (len    (string-length string)))
-	  (let loop ((walker     0)
-		     (id-stop    0)
-		     (type-start 0))
-	     (cond
-		((=fx walker len)
-		 (cond
-		    ((and (=fx id-stop 0)
-			  (>fx type-start 0))
-		     ;; this empty name variable can be usefull to declare
-		     ;; prototype so it is legal.
-		     (cons (string->symbol "")
-			   (string->symbol (substring string type-start len))))
-		    ((=fx id-stop 0)
-		     (cons ident '()))
-		    ((=fx type-start len)
-		     ;; empty type are eroneous
-		     (error #f "Illegal formal identifier" ident))
-		    (else
-		     (cons (string->symbol
-			    (substring string 0 id-stop))
-			   (string->symbol
-			    (substring string type-start len))))))
-		((and (char=? (string-ref string walker) #\:)
-		      (<fx walker (-fx len 1))
-		      (char=? (string-ref string (+fx walker 1)) #\:))
-		 (if (>fx type-start 0)
-		     (error #f "Illegal formal identifier" ident)
-		     (loop (+fx walker 2)
-			   walker
-			   (+fx walker 2))))
-		(else
-		 (loop (+fx walker 1)
-		       id-stop
-		       type-start))))))))
+(define (application-eval-expander! x e)
+   (let loop ((y x))
+      (cond
+	 ((null? y)
+	  x)
+	 ((not (pair? y))
+	  (error "application" "Illegal form" y))
+	 (else
+	  (set-car! y (e (car y) e))
+	  (loop (cdr y))))))
+
+;*---------------------------------------------------------------------*/
+;*    %lexical-stack ...                                               */
+;*---------------------------------------------------------------------*/
+(define (%lexical-stack)
+   ($lexical-stack))
+
+;*---------------------------------------------------------------------*/
+;*    %with-lexical ...                                                */
+;*---------------------------------------------------------------------*/
+(define (%with-lexical new form e key)
+   (let ((old-lexical-stack (%lexical-stack)))
+      ($lexical-stack-set!
+	 (append (map (lambda (n)
+			 (let ((f (parse-formal-ident n
+				     (get-source-location e))))
+			    (if (pair? f)
+				(cons (car f) key)
+				(cons n key))))
+		    new)
+	    old-lexical-stack))
+      (unwind-protect
+	 (e form e)
+	 ($lexical-stack-set! old-lexical-stack))))
+    
+;*---------------------------------------------------------------------*/
+;*    expand-error ...                                                 */
+;*---------------------------------------------------------------------*/
+(define (expand-error proc msg obj)
+   (if (epair? obj)
+       (match-case (cer obj)
+	  ((at ?fname ?loc)
+	   (error/location proc msg obj fname loc))
+	  (else
+	   (error proc msg obj)))
+       (error proc msg obj)))
