@@ -1,9 +1,9 @@
 ;*---------------------------------------------------------------------*/
-;*    serrano/prgm/project/bigloo/runtime/Eval/expd-bool.scm           */
+;*    serrano/prgm/project/bigloo/bigloo/runtime/Eval/expdbool.scm     */
 ;*                                                                     */
 ;*    Author      :  Manuel Serrano                                    */
 ;*    Creation    :  Mon Jan  4 17:12:21 1993                          */
-;*    Last change :  Mon May  7 18:45:43 2001 (serrano)                */
+;*    Last change :  Sun Aug 25 09:13:35 2019 (serrano)                */
 ;*                                                                     */
 ;*    Les expanseurs des formes booleenes.                             */
 ;*---------------------------------------------------------------------*/
@@ -19,36 +19,40 @@
 	    __structure
 	    __tvector
 	    __bexit
+	    __bignum
+	    __param
+	    __object
+	    __thread
 	    
 	    __r4_numbers_6_5
 	    __r4_numbers_6_5_fixnum
 	    __r4_numbers_6_5_flonum
+	    __r4_numbers_6_5_flonum_dtoa
 	    __r4_characters_6_6
 	    __r4_equivalence_6_2
 	    __r4_booleans_6_1
 	    __r4_symbols_6_4
 	    __r4_strings_6_7
 	    __r4_pairs_and_lists_6_3
-	    __r4_input_6_10_2
 	    __r4_control_features_6_9
 	    __r4_vectors_6_8
 	    __r4_ports_6_10_1
 	    __r4_output_6_10_3
 	    
-	    __progn)
+	    __progn
+	    __expand)
 
    (use     __type
-	    __evenv)
+	    __evenv
+	    __bit)
 
-   (export  (expand-or   <expression>)  
-	    (expand-and  <expression>)
-	    (expand-cond <expression>)))
+   (export  (expand-cond <expression>)))
 
 ;*---------------------------------------------------------------------*/
 ;*    get-new-test-name ...                                            */
 ;*---------------------------------------------------------------------*/
 (define (get-new-test-name string)
-   (let ((symbol (gensym (string-append "_" string "test_"))))
+   (let ((symbol (gensym)))
       ;; the non user annotation is used for better bdb code production.
       ;; the non-user property is used by the compiler function
       ;; mark-symbol-non-user! (inside the ast_ident module).
@@ -57,99 +61,68 @@
       symbol))
 
 ;*---------------------------------------------------------------------*/
-;*    expand-or ...                                                    */
+;*    xcons ...                                                        */
 ;*---------------------------------------------------------------------*/
-(define (expand-or exp)
-   (let ((res (let loop ((sor (cdr exp)))
-		 (cond
-		    ((null? sor)
-		     #f)
-		    ((not (pair? sor))
-		     (error "or" "Illegal form" exp))
-		    ((null? (cdr sor))
-		     (car sor))
-		    (else
-		     (let* ((test (get-new-test-name "or"))
-			    (res  `(let ((,test ,(car sor)))
-				      (if ,test ,test ,(loop (cdr sor))))))
-			(cond
-			   ((epair? (car sor))
-			    (econs (car res) (cdr res) (cer (car sor))))
-			   (else
-			    res))))))))
-      (replace! exp res)))
-
-;*---------------------------------------------------------------------*/
-;*    expand-and ...                                                   */
-;*---------------------------------------------------------------------*/
-(define (expand-and exp)
-   (let ((res (let loop ((sand (cdr exp)))
-		 (cond
-		    ((null? sand)
-		     #t)
-		    ((not (pair? sand))
-		     (error "and" "Illegal form" exp))
-		    ((null? (cdr sand))
-		     (car sand))
-		    (else
-		     (let* ((test (get-new-test-name "and"))
-			    (res  `(let ((,test ,(car sand)))
-				      (if ,test ,(loop (cdr sand)) #f))))
-			(cond
-			   ((epair? (car sand))
-			    (econs (car res) (cdr res) (cer (car sand))))
-			   (else
-			    res))))))))
-      (replace! exp res)))
+(define (xcons a d e)
+   (if e
+       (econs a d e)
+       (cons a d)))
 
 ;*---------------------------------------------------------------------*/
 ;*    expand-cond ...                                                  */
 ;*---------------------------------------------------------------------*/
 (define (expand-cond exp)
-   (let* ((clauses  (cdr exp))
-	  (clause1  (if (pair? clauses)
-			(car clauses)
-			'()))
-	  (clause2+ (if (pair? clause1)
-			(cdr clauses)
-			#f)))
+   (let* ((clauses (cdr exp))
+	  (clause1 (if (pair? clauses) (car clauses) '()))
+	  (clause2+ (if (pair? clause1) (cdr clauses) #f)))
       (cond
 	 ((null? clause1)
 	  '#f)
 	 ((or (not (pair? clause1)) (equal? clause1 '(else)))
-	  (error "cond-clause" "Illegal form" exp))
+	  (expand-error "cond" "Illegal form" exp))
 	 ((null? (cdr clause1))
 	  (let ((res `(or ,(car clause1) (cond ,@clause2+))))
 	     (if (epair? (car clause1))
 		 (econs (car res) (cdr res) (cer (car clause1)))
-		 (replace! exp res))))
+		 (evepairify-deep res exp))))
 	 ((and (eq? (cadr clause1) '=>) (=fx (length clause1) 3))
 	  (let* ((aux (get-new-test-name "cd"))
-		 (res `(let ((test-result ,(car clause1))
-			     (thunk2 (lambda (,aux) (,(caddr clause1) ,aux)))
-			     (thunk3 (lambda () (cond ,@clause2+))))
-			  (if test-result (thunk2 test-result) (thunk3)))))
+		 (test (get-new-test-name "test"))
+		 (res `(let ((,test ,(car clause1)))
+			  (if ,test
+			      (let ((,aux ,test))
+				 (,(caddr clause1) ,aux))
+			      (cond ,@clause2+)))))
 	     (if (epair? (car clause1))
 		 (econs (car res) (cdr res) (cer (car clause1)))
-		 (replace! exp res))))
+		 (evepairify-deep res exp))))
 	 ((eq? (car clause1) 'else)
-	  (normalize-body (cdr clause1)))
+	  (when (and (pair? clause2+) (> (bigloo-warning) 0))
+	     (warning "cond" "ignored COND clauses -- " clause2+))
+	  (expand-progn (cdr clause1)))
 	 (else
 	  (let* ((ncond (let ((nc `(cond ,@clause2+)))
 			   (cond
 			      ((and (pair? clause2+) (epair? (car clause2+)))
 			       (econs (car nc) (cdr nc) (cer (car clause2+))))
+			      ((epair? (car clauses))
+			       (econs (car nc) (cdr nc) (cer (car clauses))))
+			      ((epair? clauses)
+			       (econs (car nc) (cdr nc) (cer clauses)))
 			      (else
 			       nc))))
-		 (res `(if ,(car clause1)
-			   ,(normalize-body (cdr clause1))
-			   ,ncond)))
-	     (cond
-		((epair? (car clause1))
-		 (econs (car res) (cdr res) (cer (car clause1))))
-		((epair? clause1)
-		 (econs (car res) (cdr res) (cer clause1)))
-		((and (pair? (cdr clause1)) (epair? (cadr clause1)))
-		 (econs (car res) (cdr res) (cer (cadr clause1))))
-		(else
-		 (replace! exp res))))))))
+		 (loc (when (epair? exp) (cer exp)))
+		 (loc1 (when (epair? clause1) (cer clause1)))
+		 (loc1-car (when (epair? (car clause1)) (cer (car clause1))))
+		 (loc1-cdr (when (epair? (cdr clause1)) (cer (cdr clause1))))
+		 (locn (when (epair? clause2+) (cer clause2+))))
+	     (xcons 'if
+		(xcons (car clause1)
+		   (xcons (expand-progn (cdr clause1))
+		      (xcons
+			 ncond
+			 '()
+			 (or locn loc1 loc))
+		      (or loc1-cdr loc1 loc))
+		   (or loc1-car loc))
+		(or loc1 loc)))))))
