@@ -1,9 +1,9 @@
 ;*---------------------------------------------------------------------*/
-;*    serrano/prgm/project/bigloo/runtime/Eval/expd-case.scm           */
+;*    serrano/prgm/project/bigloo/bigloo/runtime/Eval/expdcase.scm     */
 ;*                                                                     */
 ;*    Author      :  Manuel Serrano                                    */
 ;*    Creation    :  Fri Jul  3 10:13:16 1992                          */
-;*    Last change :  Mon May  7 18:45:55 2001 (serrano)                */
+;*    Last change :  Sun Aug 25 09:14:00 2019 (serrano)                */
 ;*                                                                     */
 ;*    On macro-expanse ce satane-case                                  */
 ;*---------------------------------------------------------------------*/
@@ -19,26 +19,32 @@
 	    __structure
 	    __tvector
 	    __bexit
+	    __bignum
+	    __param
+	    __object
+	    __thread
 	    
 	    __r4_numbers_6_5
 	    __r4_numbers_6_5_fixnum
 	    __r4_numbers_6_5_flonum
+	    __r4_numbers_6_5_flonum_dtoa
 	    __r4_characters_6_6
 	    __r4_equivalence_6_2
 	    __r4_booleans_6_1
 	    __r4_symbols_6_4
 	    __r4_strings_6_7
 	    __r4_pairs_and_lists_6_3
-	    __r4_input_6_10_2
 	    __r4_control_features_6_9
 	    __r4_vectors_6_8
 	    __r4_ports_6_10_1
 	    __r4_output_6_10_3
 	    
-	    __progn)
+	    __progn
+	    __expand)
    
    (use     __type
-	    __evenv)
+	    __evenv
+	    __bit)
    
    (export  (expand-eval-case <expression> <expander>)))
 	   
@@ -50,7 +56,7 @@
       ((?- ?value . ?clauses)
        (generic-case x value clauses e))
       (else
-       (error "case" "Illegal form" x))))
+       (expand-error "case" "Illegal form" x))))
 
 ;*---------------------------------------------------------------------*/
 ;*    generic-case ...                                                 */
@@ -65,22 +71,27 @@
 		     (()
 		      #unspecified)
 		     ((else . ?body)
-		      (if (or (not (null? (cdr clauses)))
-			      (null? body))
-			  (error "case" "Illegal `case' form" x)
-			  (normalize-progn body)))
+		      (if (or (not (null? (cdr clauses))) (null? body))
+			  (expand-error "case" "Illegal `case' form" x)
+			  (expand-progn body)))
 		     (((and ?datums (?- . (?- ???-))) . ?body)
 		      (if (null? body)
-			  (error "case" "Illegal `case' form" x)
-			  `(if (memq case-value ',datums)
-			       ,(normalize-progn body)
-			       ,(loop (cdr clauses)))))
+			  (expand-error "case" "Illegal `case' form" x)
+			  (evepairify
+			     `(if (memv case-value ',datums)
+				  ,(expand-progn body)
+				  ,(loop (cdr clauses)))
+			     (car clauses))))
 		     (((?datums) . ?body)
 		      (if (null? body)
-			  (error "case" "Illegal `case' form" x)
-			  `(if (eq? case-value ',datums)
-			       ,(normalize-progn body)
-			       ,(loop (cdr clauses)))))))))
+			  (expand-error "case" "Illegal `case' form" x)
+			  (evepairify
+			     `(if (eqv? case-value ',datums)
+				  ,(expand-progn body)
+				  ,(loop (cdr clauses)))
+			     (car clauses))))
+		     (else
+		      (expand-error "case" "Illegal `case' form" x))))))
       e))
 
 
