@@ -1,4 +1,5 @@
 package bigloo;
 
-public class cobj {
+public class cobj
+{
 }
