@@ -10,35 +10,43 @@
 
 (module __lalr_gen
    
-	(include "Lalr/lalr.sch")
-
-	(import __error
-		__lalr_global
-		__lalr_rewrite
-	
-		__type
-		__bigloo
-		__tvector
-		__structure
-		__tvector
-
-		__r4_numbers_6_5
-		__r4_numbers_6_5_fixnum
-		__r4_numbers_6_5_flonum
-		__r4_characters_6_6
-		__r4_equivalence_6_2
-		__r4_booleans_6_1
-		__r4_symbols_6_4
-		__r4_strings_6_7
-		__r4_pairs_and_lists_6_3
-		__r4_input_6_10_2
-		__r4_control_features_6_9
-		__r4_vectors_6_8
-		__r4_ports_6_10_1
-		__r4_output_6_10_3
-		__evenv)
-
-	(export (gen-lalr-code)))
+   (include "Lalr/lalr.sch")
+   
+   (import __error
+	   __lalr_global
+	   __lalr_rewrite
+	   
+	   __type
+	   __bigloo
+	   __tvector
+	   __structure
+	   __tvector
+	   __param
+	   __bexit
+	   __bignum
+	   __object
+	   __thread
+	   __rgc
+	   __bit
+	   
+	   __r4_numbers_6_5
+	   __r4_numbers_6_5_fixnum
+	   __r4_numbers_6_5_flonum
+	   __r4_numbers_6_5_flonum_dtoa
+	   __r4_characters_6_6
+	   __r4_equivalence_6_2
+	   __r4_booleans_6_1
+	   __r4_symbols_6_4
+	   __r4_strings_6_7
+	   __r4_pairs_and_lists_6_3
+	   __r4_input_6_10_2
+	   __r4_control_features_6_9
+	   __r4_vectors_6_8
+	   __r4_ports_6_10_1
+	   __r4_output_6_10_3
+	   __evenv)
+   
+   (export (gen-lalr-code)))
 
 (define (gen-lalr-code)
   `(let* (,(gen-action-table)
@@ -70,7 +78,7 @@
 		 (let* ((p (car l)) (x (car p)) (y (cdr p)))
 		   (cons
 		    (cons
-		     (if (integer? x) (vector-ref *symv* (+fx nvars x)) x)
+		     (if (fixnum? x) (vector-ref *symv* (+fx nvars x)) x)
 		     y)
 		    (loop (cdr l))))))
 	   (loop-a (+fx i 1))))))
