@@ -10,87 +10,97 @@
 
 (module __lalr_global
    
-	(include "Lalr/lalr.sch")
-
-	(import __error)
-	
-	(use    __type
-		__bigloo
-		__tvector
-		__structure
-		__tvector
-
-		__r4_numbers_6_5
-		__r4_numbers_6_5_fixnum
-		__r4_numbers_6_5_flonum
-		__r4_characters_6_6
-		__r4_equivalence_6_2
-		__r4_booleans_6_1
-		__r4_symbols_6_4
-		__r4_strings_6_7
-		__r4_pairs_and_lists_6_3
-		__r4_input_6_10_2
-		__r4_control_features_6_9
-		__r4_vectors_6_8
-		__r4_ports_6_10_1
-		__r4_output_6_10_3
-		__evenv)
-
-	(export  grammar
-		 rrhs         
-		 rlhs         
-		 ritem        
-		 nullable     
-		 derives      
-		 fderives     
-		 firsts       
-		 kernel-base  
-		 kernel-end   
-		 shift-symbol 
-		 shift-set    
-		 red-set      
-		 state-table  
-		 acces-symbol 
-		 reduction-table 
-		 shift-table  
-		 consistent   
-		 lookaheads   
-		 LA           
-		 LAruleno     
-		 lookback     
-		 goto-map     
-		 from-state   
-		 to-state     
-		 includes     
-		 F            
-		 action-table 
-		 nitems          
-		 nrules          
-		 nvars           
-		 nterms          
-		 nsyms           
-		 nstates         
-		 first-state     
-		 last-state      
-		 final-state     
-		 first-shift     
-		 last-shift      
-		 first-reduction 
-		 last-reduction  
-		 nshifts         
-		 maxrhs          
-		 ngotos          
-		 token-set-size
-		 terminals
-		 nonterminals
-		 actions
-		 STATE-TABLE-SIZE
-		 (initialize-all)))
+   (include "Lalr/lalr.sch")
+   
+   (import __error
+	   __param)
+   
+   (use    __type
+	   __bigloo
+	   __tvector
+	   __structure
+	   __tvector
+	   __bexit
+	   __bignum
+	   __object
+	   __thread
+	   __rgc
+	   __bit
+	   
+	   __r4_numbers_6_5
+	   __r4_numbers_6_5_fixnum
+	   __r4_numbers_6_5_flonum
+	   __r4_numbers_6_5_flonum_dtoa
+	   __r4_characters_6_6
+	   __r4_equivalence_6_2
+	   __r4_booleans_6_1
+	   __r4_symbols_6_4
+	   __r4_strings_6_7
+	   __r4_pairs_and_lists_6_3
+	   __r4_input_6_10_2
+	   __r4_control_features_6_9
+	   __r4_vectors_6_8
+	   __r4_ports_6_10_1
+	   __r4_output_6_10_3
+	   __evenv)
+   
+   (export  grammar
+	    rrhs         
+	    rlhs         
+	    ritem        
+	    rprec
+	    nullable     
+	    derives      
+	    fderives     
+	    firsts       
+	    kernel-base  
+	    kernel-end   
+	    shift-symbol 
+	    shift-set    
+	    red-set      
+	    state-table  
+	    acces-symbol 
+	    reduction-table 
+	    shift-table  
+	    consistent   
+	    lookaheads   
+	    LA           
+	    LAruleno     
+	    lookback     
+	    goto-map     
+	    from-state   
+	    to-state     
+	    includes     
+	    F            
+	    action-table 
+	    nitems          
+	    nrules          
+	    nvars           
+	    nterms          
+	    nsyms           
+	    nstates         
+	    first-state     
+	    last-state      
+	    final-state     
+	    first-shift     
+	    last-shift      
+	    first-reduction 
+	    last-reduction  
+	    nshifts         
+	    maxrhs          
+	    ngotos          
+	    token-set-size
+	    terminals
+	    nonterminals
+	    actions
+	    STATE-TABLE-SIZE
+	    (initialize-all)))
 
 ;; - Tableaux 
 (define rrhs            #f)
 (define rlhs            #f)
 (define ritem           #f)
+(define rprec		#f)
 (define nullable        #f)
 (define derives         #f)
 (define fderives        #f)
@@ -143,6 +153,7 @@
   (set! rrhs            #f)
   (set! rlhs            #f)
   (set! ritem           #f)
+  (set! rprec		#f)
   (set! nullable        #f)
   (set! derives         #f)
   (set! fderives        #f)
