@@ -19,16 +19,21 @@
 	   __lalr_rewrite
 		
 	   __type
-		
 	   __bigloo
 	   __tvector
 	   __structure
 	   __tvector
 	   __bexit
+	   __bignum
+	   __param
+	   __object
+	   __thread
+	   __rgc
 
 	   __r4_numbers_6_5
 	   __r4_numbers_6_5_fixnum
 	   __r4_numbers_6_5_flonum
+	   __r4_numbers_6_5_flonum_dtoa
 	   __r4_characters_6_6
 	   __r4_equivalence_6_2
 	   __r4_booleans_6_1
@@ -78,10 +83,13 @@
        (error "lalr-grammar" "Illegal form" x))))
 
 ;; --- Encodage compact de la grammaire ... ----------------------------- ;;
+;; Also populates `rprec', which is the precedence for every rule, from 
+;; the precedence of the last terminal in the production.
 (define (pack-grammar)
   (set! rlhs   (make-vector nrules #f))
   (set! rrhs   (make-vector nrules #f))
   (set! ritem  (make-vector (+fx 1 nitems) #f))
+  (set! rprec  (make-vector nrules #f))
   
   (let loop ((p grammar) (item-no 0) (rule-no 1))
     (if (not (null? p))
@@ -98,11 +106,13 @@
 			  (vector-set! ritem it-no3 (negfx rl-no2))
 			  (loop2 (cdr prods) (+fx it-no3 1) (+fx rl-no2 1)))
 			(let* ((sym (car rhs))
-			       (sym-no (getprop (if (pair? sym)
-						    (car sym)
-						    sym)
-						'sym-no)))
-						       
+			       (asym (if (pair? sym)
+				       (car sym)
+				       sym))
+			       (sym-no (getprop asym 'sym-no)))
+
+			  (if (getprop asym 'prec)
+			    (vector-set! rprec rl-no2 (getprop asym 'prec)))
 			  (vector-set! ritem it-no3 sym-no)
 			  (loop3 (cdr rhs) (+fx it-no3 1))))))))))))
 
@@ -130,8 +140,18 @@
 	 (else
 	  (error "lalr-grammar" "Illegal form" x))))
    (for-each (lambda (t)
-		(if (not (symbol? t))
-		    (error "lalr-grammar" "Illegal token" tokens)))
+	       (cond
+		 ((symbol? t)
+		  #t)
+		 ((and (pair? t) (memq (car t) '(left: right: none:)))
+		  (for-each (lambda (t)
+			       (if (not (symbol? t))
+				   (bigloo-type-error 'lalr-grammar
+						      'symbol
+						      t)))
+			    (cdr t)))
+		 (else
+		  (error "lalr-grammar" "Illegal token" t))))
 	     tokens)
    (for-each check-lalr-rule rules))
 
@@ -408,7 +428,7 @@
 	 (n    (length isp))
 	 (key  (let loop ((isp1 isp) (k 0))
 		 (if (null? isp1)
-		     (modulo k STATE-TABLE-SIZE)
+		     (modulofx k STATE-TABLE-SIZE)
 		     (loop (cdr isp1) (+fx k (car isp1))))))
 	 (sp   (vector-ref state-table key)))
     (if (null? sp)
@@ -495,7 +515,7 @@
 ;; --
 
 (define (lalr)
-  (set! token-set-size (+fx 1 (quotient nterms (BITS-PER-WORD))))
+  (set! token-set-size (+fx 1 (quotientfx nterms (BITS-PER-WORD))))
   (set-accessing-symbol)
   (set-shift-table)
   (set-reduction-table)
@@ -644,7 +664,7 @@
 	(begin
 	  (display (list "Error in map-goto" state symbol)) (newline)
 	  0)
-	(let* ((middle (quotient (+fx low high) 2))
+	(let* ((middle (quotientfx (+fx low high) 2))
 	       (s (vector-ref from-state middle)))
 	  (cond
 	   ((= s state)
@@ -852,35 +872,85 @@
 	    (cons (vector-ref *symv* x)
 		  (loop (+fx pos 1)))))))))
 
+(define (get-precedence Sym Act)
+  (if (<=fx Act 0) 
+    (vector-ref rprec (negfx Act))			 ; Reduce (from rule)
+    (getprop (vector-ref *symv* (+fx Sym nvars)) 'prec))); Shift (from terminal)
+
 (define (build-tables)
+
   (define (add-action St Sym Act)
      (let* ((x (vector-ref action-table St))
 	    (y (assv Sym x)))
 	(if y
-	    (if (not (= Act (cdr y)))
-		;; -- there is a conflict 
-		(begin
-		   (if (and (<=fx (cdr y) 0)
-			    (<=fx Act 0))
-		       (begin
-			  (warning "lalr-grammar"
-				   "** Reduce/Reduce conflict: "
-				   #"\n - reduce by rule " (build-rule (negfx Act))
-				   #"\n - reduce by rule " (build-rule (negfx (cdr y)))
-				   #"\non token `" (vector-ref *symv* (+fx nvars Sym))
-				   "'")
-			  (set-cdr! y (max (cdr y) Act)))
-		       (begin
-			  (warning "lalr-grammar"
-				   "** Shift/Reduce conflict: "
-				   #"\n - shift to state "  Act
-				   #"\n - reduce rule " (build-rule (negfx (cdr y)))
-				   #"\non token `"  (vector-ref *symv* (+fx nvars Sym))
-				   "'")
-			  (set-cdr! y Act)))))
-	    (vector-set! action-table St
-			 (cons (cons Sym Act) x)))))
+	  (let ((cur-prec (get-precedence Sym (cdr y)))
+		(new-prec (get-precedence Sym Act)))
+	    (cond
+	      ((=fx Act (cdr y))	  ; ** Same action as in the table.
+	       #t)
+	      ((and (<=fx (cdr y) 0)	  ; ** Reduce/Reduce conflict.
+		    (<=fx Act 0))
+	       (warning "lalr-grammar"
+			"** Reduce/Reduce conflict: "
+			#"\n - reduce by rule " (build-rule (negfx Act))
+			#"\n - reduce by rule " (build-rule (negfx (cdr y)))
+			#"\non token `" (vector-ref *symv* (+fx nvars Sym))
+			"'")
+	       (set-cdr! y (max (cdr y) Act)))
+	      ((or cur-prec new-prec)	  ; ** Resolve with precedence.
+	       (cond
+		 ((not new-prec)	  ; ** Old symbol has precedence.
+		  #t)
+		 ((not cur-prec)	  ; ** New symbol has precedence.
+		  (set-cdr! y Act))
+		 ((=fx (cdr cur-prec) (cdr new-prec))
+		  ; ** Equal precedence, use associativity.
+		  (let ((shift (max (cdr y) Act))
+			(reduce (min (cdr y) Act)))
+		    (case (car cur-prec)
+		      ((left:)
+		       (set-cdr! y reduce))
+		      ((right:)
+		       (set-cdr! y shift))
+		      ((none:)
+		       ;; Dominique Boucher suggested to (set-cdr! y '*error*)
+		       ;; 16 Nov 2006
+		       (set-cdr! y '*error*)))))
+		       ; Remove from action table so we cause an error during
+		       ; parse.  NOTE: If we have another reduce later, we
+		       ; have no record so we can't point out the
+		       ; reduce/reduce conflict.
+;* 		       (vector-set! action-table St                    */
+;* 				    (let loop ((iter x))               */
+;* 				      (cond                            */
+;* 					((null? iter)                  */
+;* 					 iter)                         */
+;* 					((eqv? (car iter) St)          */
+;* 					 (loop (cdr iter)))            */
+;* 					(else                          */
+;* 					 (cons (car iter)              */
+;* 					       (loop (cdr iter)))))))  */
+		 ((>fx (cdr cur-prec) (cdr new-prec))
+		  ; ** New action has higher precedence (lower precedence num).
+		  (set-cdr! y Act))
+		 (else
+		  ; ** Current action has higher precedence.  Do nothing.
+		  #t)))
+	      (else			  ; ** Shift/Reduce conflict.
+	       (warning "lalr-grammar"
+			"** Shift/Reduce conflict: "
+			#"\n - shift to state "  Act
+			#"\n - reduce rule " (build-rule (negfx (cdr y)))
+			#"\non token `"  (vector-ref *symv* (+fx nvars Sym))
+			"'")
+	       (set-cdr! y Act))))
+	  (vector-set! action-table St (cons (cons Sym Act) x)))))
 
+  (define (add-action-for-all-terminals state action)
+     (do ((i 1 (+ i 1)))
+	 ((= i nterms))
+	 (add-action state i action)))
+   
   (set! action-table (make-vector nstates '()))
 
   (do ((i 0 (+fx i 1)))  ; i = state
@@ -888,7 +958,11 @@
     (let ((red (vector-ref reduction-table i)))
       (if (and red (>=fx (red-nreds red) 1))
 	  (if (and (= (red-nreds red) 1) (vector-ref consistent i))
-	      (add-action i *default* (negfx (car (red-rules red))))
+;*---------------------------------------------------------------------*/
+;*    Patch Dominique Boucher, 5 sep 2005.                             */
+;*---------------------------------------------------------------------*/
+;;	      (add-action i *default* (negfx (car (red-rules red))))
+              (add-action-for-all-terminals i (- (car (red-rules red))))
 	      (let ((k (vector-ref lookaheads (+fx i 1))))
 		(let loop ((j (vector-ref lookaheads i)))
 		  (if (<fx j k)
@@ -897,7 +971,7 @@
 			(let loop2 ((token 0) (x (vector-ref lav 0)) (y 1) (z 0))
 			  (if (<fx token nterms)
 			      (begin
-				(let ((in-la-set? (modulo x 2)))
+				(let ((in-la-set? (modulofx x 2)))
 				  (if (= in-la-set? 1)
 				      (add-action i token rule)))
 				(if (= y (BITS-PER-WORD))
@@ -905,7 +979,7 @@
 					   (vector-ref lav (+fx z 1))
 					   1
 					   (+fx z 1))
-				    (loop2 (+fx token 1) (quotient x 2) (+fx y 1) z)))))
+				    (loop2 (+fx token 1) (quotientfx x 2) (+fx y 1) z)))))
 			(loop (+fx j 1)))))))))
 
     (let ((shiftp (vector-ref shift-table i)))
