@@ -11,17 +11,25 @@
 (module __lalr_rewrite
 
    (import __error
-	   __lalr_global)
+	   __lalr_global
+	   __param)
    
    (use    __type
 	   __bigloo
 	   __tvector
 	   __structure
 	   __tvector
+	   __bexit
+	   __bignum
+	   __object
+	   __thread
+	   __rgc
+	   __bit
 
 	   __r4_numbers_6_5
 	   __r4_numbers_6_5_fixnum
 	   __r4_numbers_6_5_flonum
+	   __r4_numbers_6_5_flonum_dtoa
 	   __r4_characters_6_6
 	   __r4_equivalence_6_2
 	   __r4_booleans_6_1
@@ -51,6 +59,8 @@
 	(let ((sym (car l)))
 	  (if (getprop sym 'nt?)
 	      (remprop! sym 'nt?))
+	  (if (getprop sym 'prec)
+	      (remprop! sym 'prec))
 	  (remprop! sym 'sym-no)
 	  (loop (cdr l)))
 	#f)))
@@ -83,7 +93,7 @@
 (define (set-sym-no! sym)
   (let ((x (getprop sym 'sym-no)))
     (if x
-	(error "lalr-grammar" "Grammar symbol already defined" sym)
+	(error 'lalr-grammar "Grammar symbol already defined" sym)
 	(let ((y *max-term*))
 	  (putprop! sym 'sym-no *max-term*) 
 	  (set! *plist* (cons sym *plist*))
@@ -92,7 +102,7 @@
 
 (define (assert-symbol! sym)
   (if (not (getprop sym 'sym-no))
-      (error "lalr-grammar" "Undefined symbol" sym)))
+      (error 'lalr-grammar "Undefined symbol" sym)))
 
 (define (symbol->symbol/binding sym)
   (let loop ((l (string->list (symbol->string sym)))
@@ -118,6 +128,7 @@
   
   (define no-items 0)
   (define no-rules 0)
+  (define prec-level 0)
 
   (set! *plist* '(*eoi* *start*))
   (set! *max-nt* 1)
@@ -130,20 +141,20 @@
 	      (let ((lhs (car prods)))
 		(if (symbol? lhs)
 		    (if (getprop lhs 'nt?)
-			(error "lalr-grammar" 
+			(error 'lalr-grammar
 			       "Non-terminal defined twice"
 			       lhs)
 			(begin
 			  (set-nt-no! lhs)
 			  (loop (cdr l))))
-		    (error "lalr-grammar" 
+		    (error 'lalr-grammar
 			   "LHS must be a symbol"
 			   lhs)))
-	      (error "lalr-grammar" 
+	      (error 'lalr-grammar
 		     "Bad rule specification"
 		     prods)))
 	(if (not (null? l))
-	    (error "lalr-grammar" "Ill-formed grammar" l))))
+	    (error 'lalr-grammar "Ill-formed grammar" l))))
 
   (set! *max-term* (+ *max-nt* 1))
   (putprop! '*eoi* 'sym-no *max-nt*)
@@ -151,18 +162,26 @@
   (putprop! '*start* 'nt? #t)
   
 ;; --- On numrote les terminaux ... ------------------------------------ ;;
-  (let loop ((l terminals))
-    (cond
-     ((pair? l)
-      (let ((term (car l)))
-	(if (symbol? term)
-	    (set-sym-no! term)
-	    (error "lalr-grammar" "Illegal terminal definition" term))
-	(loop (cdr l))))
-     ((null? l)
-      #f)
-     (else
-      (error "lalr-grammar" "Bad terminal definitions" l))))
+;; Also, assign precedence to terminals which have been grouped in a      ;;
+;; (left:/right:/none: ...) precedence grouping.			  ;;
+;; ---------------------------------------------------------------------- ;;
+  (for-each
+    (lambda (t)
+      (cond
+	((symbol? t)
+	 (set-sym-no! t))
+	((and (pair? t) (memq (car t) '(left: right: none:)))
+	 (set! prec-level (+fx prec-level 1))
+	 (for-each
+	   (lambda (inner-t)
+	     (if (not (symbol? inner-t))
+	       (error 'lalr-grammar "Bad terminal" inner-t))
+	     (set-sym-no! inner-t)
+	     (putprop! inner-t 'prec (cons (car t) prec-level)))
+	   (cdr t)))
+	(else
+	 (error 'lalr-grammar "Bad terminal" t))))
+    terminals)
   
 ;; --- On remplace les `foo@bar' par `(foo . bar)' ---------------------- ;;
 ;; --- et on numrote ... ----------------------------------------------- ;;
@@ -180,7 +199,7 @@
 		(if (pair? rhs/action)
 		    (let ((action (cdr rhs/action)))
 		      (if (not (list? action))
-			  (error "lalr-grammar" "Invalid semantic action" rhs/action))
+			  (error 'lalr-grammar "Invalid semantic action" rhs/action))
 		      (if (null? action)
 			  (set-cdr! rhs/action '(#f)))
 		      (let loop-2 ((rhs (car rhs/action)))
@@ -199,14 +218,15 @@
 				  (set-car! rhs sym/var)
 				  (assert-symbol! the-sym)
 				  (loop-2 (cdr rhs)))
-				(error "lalr-grammar" 
+				(error 'lalr-grammar
 				       "Invalid symbol in right-hand side"
 				       rhs))))
 			 (else
-			  (error "lalr-grammar" "Bad right-hand side"
+			  (error 'lalr-grammar
+				 "Bad right-hand side"
 				 rhs))))))))
 	     (else
-	      (error "lalr-grammar" "Bad rule specification" l)))))))
+	      (error 'lalr-grammar "Bad rule specification" l)))))))
   
 ;; --- On ajoute une production  la grammaire et ... ------------------- ;;
 ;; --- on initialise quelques variables globales ... -------------------- ;;
