@@ -1,9 +1,9 @@
 ;*=====================================================================*/
-;*    serrano/prgm/project/bigloo/runtime/Llib/bigloo.scm              */
+;*    serrano/prgm/project/bigloo/bigloo/runtime/Llib/bigloo.scm       */
 ;*    -------------------------------------------------------------    */
 ;*    Author      :  Manuel Serrano                                    */
 ;*    Creation    :  Fri Jan 20 08:24:40 1995                          */
-;*    Last change :  Tue Oct 30 16:01:33 2001 (serrano)                */
+;*    Last change :  Sun May 28 06:51:37 2023 (serrano)                */
 ;*    -------------------------------------------------------------    */
 ;*    The bigloo runtime utility functions                             */
 ;*=====================================================================*/
@@ -13,13 +13,21 @@
 ;*---------------------------------------------------------------------*/
 (module __bigloo
 
-   (import  __error)
+   (import  __error
+	    __configure
+	    __param
+	    __object
+	    __thread
+	    __bexit)
    
    (use     __type
 	    __tvector
 	    __bit
+	    __bignum
 
 	    __r4_numbers_6_5_fixnum
+	    __r4_numbers_6_5_flonum
+	    __r4_numbers_6_5_flonum_dtoa
 	    __r4_equivalence_6_2
 	    __r4_vectors_6_8
 	    __r4_booleans_6_1
@@ -29,32 +37,44 @@
 	    __r4_symbols_6_4
 	    __r4_strings_6_7
 	    __r5_control_features_6_4
-
+	    __r4_output_6_10_3
+	    __r4_ports_6_10_1
+	    
 	    __evenv)
 
    (extern  (macro c-procedure-light?::bool (::obj)
 		   "PROCEDURE_LIGHTP")
 	    (macro va-procedure?::bool (::procedure)
 		   "VA_PROCEDUREP")
-	    (macro procedure-entry::obj (::procedure)
-		   "PROCEDURE_ENTRY")
-	    (macro tprocedure-l-entry::obj (::procedure)
-		   "PROCEDURE_L_ENTRY")
 	    
-	    (macro procedure-arity::int (::procedure)
+	    (macro $procedure-entry->string::bstring (::procedure)
+		   "bgl_procedure_entry_to_string")
+	    (macro $string->procedure-entry::obj (::bstring)
+		   "bgl_string_to_procedure_entry")
+	    
+	    (macro $procedure-arity::int (::procedure)
 		   "PROCEDURE_ARITY")
 	    (macro correct-arity?::bool (::procedure ::int)
 		   "PROCEDURE_CORRECT_ARITYP")
+
+	    (macro $procedure-length::int (::procedure)
+		   "PROCEDURE_LENGTH")
+
+	    (macro $make-procedure::procedure (::obj ::int ::int)
+		   "bgl_make_procedure")
+	    (macro $dup-procedure::procedure (::procedure)
+		   "bgl_dup_procedure")
 	    
 	    (macro make-fx-procedure::procedure (::obj ::int ::int)
-		   "make_fx_procedure")		 
+		   "MAKE_FX_PROCEDURE")		 
 	    (macro make-va-procedure::procedure (::obj ::int ::int)
-		   "make_va_procedure")
+		   "MAKE_VA_PROCEDURE")
 	    
-	    (macro make-el-procedure::procedure (::int)
+	    (macro make-stack-fx-procedure::procedure (::obj ::int ::int)
+		   "BGL_MAKE_STACK_FX_PROCEDURE")		 
+	    
+	    (macro make-el-procedure::procedure-el (::int)
 		   "MAKE_EL_PROCEDURE")	 
-	    (macro make-el-procedure-1::procedure (::int)
-		   "MAKE_EL_PROCEDURE_1")	 
 	    
 	    (macro make-l-procedure::procedure (::obj ::int)
 		   "MAKE_L_PROCEDURE")		 
@@ -64,34 +84,39 @@
 	    (macro procedure-ref::obj (::procedure ::int)
 		   "PROCEDURE_REF")			 
 
+	    (macro $procedure-attr-set!::obj (::procedure ::obj)
+		   "PROCEDURE_ATTR_SET")			 
+	    (macro $procedure-attr::obj (::procedure)
+		   "PROCEDURE_ATTR")			 
+
 	    (macro procedure-l-set!::obj (::obj ::int ::obj)
 		   "PROCEDURE_L_SET")		  
 	    (macro procedure-l-ref::obj (::obj ::int)
 		   "PROCEDURE_L_REF")
 	    
-	    (macro procedure-el-set!::obj (::obj ::int ::obj)
+	    (macro procedure-el-set!::obj (::procedure-el ::int ::obj)
 		   "PROCEDURE_EL_SET")		 
-	    (macro procedure-el-ref::obj (::obj ::int)
+	    (macro procedure-el-ref::obj (::procedure-el ::int)
 		   "PROCEDURE_EL_REF")
 	    
-	    (macro procedure-1-el-set!::obj (::obj ::int ::obj)
-		   "PROCEDURE_1_EL_SET")		 
-	    (macro procedure-1-el-ref::obj (::obj ::int)
-		   "PROCEDURE_1_EL_REF")
-	    
-	    (macro make-cell::obj (::obj)
-		   "MAKE_CELL")
-	    (macro cell-set!::obj (::obj ::obj)
+	    (macro $make-cell::cell (::obj)
+		   "MAKE_YOUNG_CELL")
+	    (macro $make-stack-cell::cell (::obj)
+		   "MAKE_STACK_CELL")
+	    (macro $cell-set!::obj (::cell ::obj)
 		   "CELL_SET")
-	    (macro cell-ref::obj (::obj)
+	    (macro $cell-ref::obj (::cell)
 		   "CELL_REF")
-	    (macro cell?::bool (::obj)
+	    (macro $cell?::bool (::obj)
 		   "CELLP")
 	    
 	    (macro c-cnst?::bool (::obj)
 		   "CNSTP")
+	    
 	    (macro c-opaque?::bool (::obj)
 		   "OPAQUEP")
+	    (macro c-opaque-nil::obj ()
+		   "BGL_OPAQUE_NIL")
 	    
 	    (macro __unspec__::obj "BUNSPEC")
 	    (macro __eoa__::obj "BEOA")
@@ -106,58 +131,67 @@
 	    (macro close-init-string::obj (::obj)
 		   "close_init_string")
 	    
-	    (macro var->root::obj (::obj)
-		   "(obj_t)&")
-	    (macro GC-add-globv!::obj (::obj)
-		   "GC_ADD_GLOBV")
-	    (macro GC-add-roots!::obj (::obj ::obj)
-		   "GC_ADD_ROOTS")
- 
-	    (macro GC-profile-push::long (::string ::obj)
-		    "GC_profile_push")
-	    (macro GC-collect-profile-push::long (::string ::obj)
-		   "GC_collect_profile_push")
-	    (macro GC-profile-pop::long  ()
-		   "GC_profile_pop")
-	    
+	    (macro $GC-collect::void ()
+		   "GC_COLLECT")
+
+	    (macro $GC-finalize::void ()
+		   "GC_invoke_finalizers")
+
 	    (macro %exit::obj (::obj)
 		   "BIGLOO_EXIT")
-	    (macro bigloo-exit::obj (::obj)
-		   "BIGLOO_EXIT")
 
-	    (export bigloo-demangle "bigloo_demangle")
+	    (macro $time::obj (::procedure)
+		   "bgl_time")
+
+	    (macro $null-or-unspecified?::bool (::obj)
+		   "BGL_NULL_OR_UNSPECIFIEDP")
+
+	    ($bgl-bmem-reset::obj () "bgl_bmem_reset")
+	    ($bgl-gc-verbose-set!::void (::bool) "bgl_gc_verbose_set")
+
+	    (export bigloo-mangle "bigloo_mangle")
+	    (export bigloo-module-mangle "bigloo_module_mangle")
 	    (export bigloo-mangled? "bigloo_mangledp")
-	    (export bigloo-exit-apply "bigloo_exit_apply"))
+	    (export bigloo-class-mangled? "bigloo_class_mangledp")
+	    (export bigloo-demangle "bigloo_demangle")
+	    (export bigloo-class-demangle "bigloo_class_demangle")
+	    (export bigloo-module-demangle "bigloo_module_demangle")
+	    (export bigloo-exit-apply "bigloo_exit_apply")
+	    (export bigloo-exit-mutex "bgl_exit_mutex"))
 
    (java    (class foreign
 	       (field static __unspec__::obj "BUNSPEC")
 	       (field static __eoa__::obj "BEOA")
 	       
-	       (method static bigloo-exit::obj (::obj)
-		       "BIGLOO_EXIT")
 	       (method static c-procedure-light?::bool (::obj)
 		       "PROCEDURE_LIGHTP")
 	       (method static va-procedure?::bool (::procedure)
 		       "VA_PROCEDUREP")
-	       (method static procedure-entry::obj (::procedure)
-		       "PROCEDURE_ENTRY")
-	       (method static tprocedure-l-entry::obj (::procedure)
-		       "PROCEDURE_L_ENTRY")
 	       
-	       (method static procedure-arity::int (::procedure)
+	       (method static $procedure-entry->string::bstring (::procedure)
+		       "bgl_procedure_entry_to_string")
+	       (method static $string->procedure-entry::obj (::bstring)
+		       "bgl_string_to_procedure_entry")
+	    
+	       (method static $procedure-arity::int (::procedure)
 		       "PROCEDURE_ARITY")
 	       (method static correct-arity?::bool (::procedure ::int)
 		       "PROCEDURE_CORRECT_ARITYP")
-	       
+
+	       (method static $procedure-length::int (::procedure)
+		       "PROCEDURE_LENGTH")
+
+	       (method static $make-procedure::procedure (::obj ::int ::int)
+		       "bgl_make_procedure")		 
 	       (method static make-fx-procedure::procedure (::obj ::int ::int)
-		       "make_fx_procedure")		 
+		       "make_fx_procedure")
+	       (method static make-stack-fx-procedure::procedure (::obj ::obj ::int)
+		       "make_fx_procedure")
 	       (method static make-va-procedure::procedure (::obj ::int ::int)
 		       "make_va_procedure")
 	       
-	       (method static make-el-procedure::procedure (::int)
+	       (method static make-el-procedure::procedure-el (::int)
 		       "MAKE_EL_PROCEDURE")	 
-	       (method static make-el-procedure-1::procedure (::int)
-		       "MAKE_EL_PROCEDURE_1")	 
 	       
 	       (method static make-l-procedure::procedure (::obj ::int)
 		       "MAKE_L_PROCEDURE")		 
@@ -167,34 +201,38 @@
 	       (method static procedure-ref::obj (::procedure ::int)
 		       "PROCEDURE_REF")			 
 	       
+	       (method static $procedure-attr-set!::obj (::procedure ::obj)
+		       "PROCEDURE_ATTR_SET")			 
+	       (method static $procedure-attr::obj (::procedure)
+		       "PROCEDURE_ATTR")
+	       
 	       (method static procedure-l-set!::obj (::procedure ::int ::obj)
 		       "PROCEDURE_L_SET")		  
 	       (method static procedure-l-ref::obj (::procedure ::int)
 		       "PROCEDURE_L_REF")
 	       
-	       (method static procedure-el-set!::obj (::procedure ::int ::obj)
+	       (method static procedure-el-set!::obj (::procedure-el ::int ::obj)
 		       "PROCEDURE_EL_SET")		 
-	       (method static procedure-el-ref::obj (::procedure ::int)
+	       (method static procedure-el-ref::obj (::procedure-el ::int)
 		       "PROCEDURE_EL_REF")
 	       
-	       (method static procedure-1-el-set!::obj (::procedure ::int ::obj)
-		       "PROCEDURE_1_EL_SET")		 
-	       (method static procedure-1-el-ref::obj (::procedure ::int)
-		       "PROCEDURE_1_EL_REF")
-	       
-	       (method static make-cell::cell (::obj)
+	       (method static $make-cell::cell (::obj)
 		       "MAKE_CELL")
-	       (method static cell-set!::obj (::cell ::obj)
+	       (method static $make-stack-cell::cell (::obj)
+		       "MAKE_CELL")
+	       (method static $cell-set!::obj (::cell ::obj)
 		       "CELL_SET")
-	       (method static cell-ref::obj (::cell)
+	       (method static $cell-ref::obj (::cell)
 		       "CELL_REF")
-	       (method static cell?::bool (::obj)
+	       (method static $cell?::bool (::obj)
 		       "CELLP")
 	       
 	       (method static c-cnst?::bool (::obj)
 		       "CNSTP")
 	       (method static c-opaque?::bool (::obj)
 		       "OPAQUEP")
+	       (method static c-opaque-nil::obj ()
+		       "BGL_OPAQUE_NIL")
 	       
 	       (method static declare-cnst-table::obj (::obj)
 		       "DECLARE_CNST_TABLE")
@@ -206,72 +244,91 @@
 	       (method static close-init-string::obj (::obj)
 		       "close_init_string")
 	       
-	       (method static var->root::obj (::obj)
-		       "VAR_ROOT")
-	       (method static GC-add-globv!::obj (::obj)
-		       "GC_ADD_GLOBV")
-	       (method static GC-add-roots!::obj (::obj ::obj)
-		       "GC_ADD_ROOTS")
-	       
-	       (method static GC-profile-push::long (::string ::obj)
-		       "GC_profile_push")
-	       (method static GC-collect-profile-push::long (::string ::obj)
-		       "GC_collect_profile_push")
-	       (method static GC-profile-pop::long  ()
-		       "GC_profile_pop")
-	       
 	       (method static %exit::obj (::obj)
-		       "BIGLOO_EXIT"))
+		       "BIGLOO_EXIT")
+
+	       (method static $time::obj (::procedure)
+		       "bgl_time"))
 	    
-	    (export bigloo-demangle "bigloo_demangle")
+	    (export bigloo-mangle "bigloo_mangle")
+	    (export bigloo-module-mangle "bigloo_module_mangle")
 	    (export bigloo-mangled? "bigloo_mangledp")
+	    (export bigloo-demangle "bigloo_demangle")
+	    (export bigloo-class-mangled? "bigloo_class_mangledp")
+	    (export bigloo-class-demangle "bigloo_class_demangle")
+	    (export bigloo-module-demangle "bigloo_module_demangle")
 	    (export bigloo-exit-apply "bigloo_exit_apply"))
 
-   (export  *bigloo-strict-r5rs-strings*
-	    (check-version! ::obj ::string ::obj)
+   (export  (check-version! ::obj ::string ::obj)
 	    (inline cnst?::bool ::obj)
 	    (inline opaque?::bool ::obj)
-	    (inline closure-arity::int ::procedure)
+	    (inline opaque-nil::obj)
+	    (inline procedure-arity::int ::procedure)
+	    (inline procedure-length::int ::procedure)
+	    (inline procedure-attr::obj ::procedure)
+	    (inline procedure-attr-set!::obj ::procedure ::obj)
 	    (inline unspecified::unspecified)
-	    (bigloo-mangle::bstring ::bstring)
-	    (bigloo-module-mangle::bstring ::bstring ::bstring)
+	    (inline null-or-unspecified?::bool ::obj)
 	    (bigloo-mangled?::bool ::bstring)
 	    (bigloo-need-mangling?::bool ::bstring)
+	    (bigloo-class-mangled?::bool ::bstring)
+	    (bigloo-mangle::bstring ::bstring)
+	    (bigloo-module-mangle::bstring ::bstring ::bstring)
 	    (bigloo-demangle ::bstring)
+	    (bigloo-class-demangle::bstring ::bstring)
+	    (bigloo-module-demangle::bstring ::bstring)
 	    (register-exit-function! ::procedure)
-	    (bigloo-exit-apply::obj ::obj))
+	    (unregister-exit-function! ::procedure)
+	    (bigloo-exit-apply::obj ::obj)
+ 	    (bigloo-exit-mutex::obj)
 
-   (pragma  (c-procedure-light? nesting)
-	    (va-procedure? nesting)
-	    (procedure-entry nesting)
-	    (tprocedure-l-entry nesting)
-	    (procedure-arity nesting)
-	    (correct-arity? nesting)
-	    (make-fx-procedure no-cfa-top nesting)
-	    (make-va-procedure no-cfa-top nesting)
-	    (procedure-set! no-cfa-top nesting)
-	    (procedure-ref no-cfa-top side-effect-free nesting)
-	    (procedure-l-set! nesting)
-	    (procedure-l-ref nesting)
-	    (procedure-el-set! nesting)
-	    (procedure-el-ref nesting)
-	    (procedure-1-el-set! nesting)
-	    (procedure-1-el-ref nesting)
-	    (cell? (predicate-of cell) nesting)
-	    (cell-set! nesting)
-	    (cell-ref nesting)
-	    (c-cnst? (predicate-of cnst) nesting)
-	    (c-opaque? (predicate-of opaque) nesting))
+	    (bmem-reset!)
+	    
+	    (time::obj ::procedure)
+	    (gc-verbose-set! ::bool)
+	    (gc #!key (finalize #t))
+	    (inline gc-finalize)
+
+	    (inline make-cell::cell ::obj)
+	    (inline cell? ::obj)
+	    (inline cell-ref ::cell)
+	    (inline cell-set! ::cell ::obj))
+
+   (pragma  (cnst-table-ref no-alloc fail-safe)
+	    (c-procedure-light? nesting fail-safe)
+	    (va-procedure? nesting fail-safe)
+	    (procedure-arity nesting args-safe fail-safe)
+	    ($procedure-arity nesting args-safe fail-safe)
+	    (procedure-length nesting args-safe fail-safe)
+	    ($procedure-length nesting args-safe fail-safe)
+	    (correct-arity? nesting args-safe fail-safe)
+	    (make-fx-procedure no-cfa-top nesting args-safe fail-safe
+	       (stack-allocator "char ~a[ BGL_PROCEDURE_BYTE_SIZE( $3 ) ]" "BGL_MAKE_FX_PROCEDURE_STACK"))
+	    (make-va-procedure no-cfa-top nesting args-safe fail-safe)
+	    (make-stack-fx-procedure no-cfa-top nesting args-safe fail-safe)
+	    (procedure-set! no-cfa-top nesting args-safe fail-safe)
+	    (procedure-ref no-cfa-top side-effect-free nesting args-safe fail-safe)
+	    (procedure-l-set! nesting args-safe fail-safe)
+	    (procedure-l-ref nesting args-safe fail-safe)
+	    (procedure-el-set! nesting args-safe fail-safe)
+	    (procedure-el-ref nesting args-safe fail-safe)
+	    ($make-cell args-safe fail-safe (args-retescape)
+	       (stack-allocator "struct bgl_cell ~a" "BGL_MAKE_CELL_STACK"))
+	    ($make-stack-cell args-safe fail-safe (args-retescape)
+	       (stack-allocator "struct bgl_cell ~a" "BGL_MAKE_CELL_STACK"))
+	    (make-cell args-safe fail-safe (args-retescape)
+	       (stack-allocator "struct bgl_cell ~a" "BGL_MAKE_CELL_STACK"))
+	    ($cell? (predicate-of cell) nesting fail-safe)
+	    ($cell-set! nesting args-safe fail-safe (args-noescape 1))
+	    ($cell-ref nesting args-safe fail-safe (args-noescape))
+	    (cell? (predicate-of cell) nesting fail-safe)
+	    (cell-set! nesting args-safe fail-safe (args-noescape 1))
+	    (cell-ref nesting  args-safe fail-safe (args-noescape))
+	    (c-cnst? (predicate-of cnst) nesting fail-safe)
+	    (c-opaque? (predicate-of opaque) nesting fail-safe))
 
    (option  (set! *unsafe-version* #t)))
 
-;*---------------------------------------------------------------------*/
-;*    Set to #t for a pure r5rs interpretation of escape String        */
-;*    sequences. Set to #f to evaluates sequences such a \t, \n        */
-;*    as C strings.                                                    */
-;*---------------------------------------------------------------------*/
-(define *bigloo-strict-r5rs-strings* #f)
-
 ;*---------------------------------------------------------------------*/
 ;*    check-version! ...                                               */
 ;*    -------------------------------------------------------------    */
@@ -291,17 +348,17 @@
 	      (not (string=? (substring release 0 min)
 			     (substring *release* 0 min))))
 	   (and (char? level) (char? *level*) (not (char=? *level* level))))
-       (define (release-name release level)
-	  (if (char? level)
-	      (let ((s (string-copy " (level 0)")))
-		 (string-set! s 8 level)
-		 (string-append release s))
-	      release))
-       (error (string-append "Some modules have been compiled by: "
-			     (release-name *release* *level*))
-	      (string-append "and other by: "
-			     (release-name release level))
-	      (cons module *modules*)))
+       (let ((release-name (lambda (release level)
+			      (if (char? level)
+				  (let ((s (string-copy " (level 0)")))
+				     (string-set! s 8 level)
+				     (string-append release s))
+				  release))))
+	  (error (string-append "Some modules have been compiled by: "
+				(release-name *release* *level*))
+		 (string-append "and other by: "
+				(release-name release level))
+		 (cons module *modules*))))
       (else
        (set! *modules* (cons module *modules*)))))
 
@@ -313,10 +370,29 @@
 (define *modules* '())
    
 ;*---------------------------------------------------------------------*/
-;*    closure-arity ...                                                */
+;*    procedure-arity ...                                              */
 ;*---------------------------------------------------------------------*/
-(define-inline (closure-arity proc)
-   (procedure-arity proc))
+(define-inline (procedure-arity proc)
+   ($procedure-arity proc))
+
+;*---------------------------------------------------------------------*/
+;*    procedure-length ...                                             */
+;*---------------------------------------------------------------------*/
+(define-inline (procedure-length proc)
+   ($procedure-length proc))
+
+;*---------------------------------------------------------------------*/
+;*    procedure-attr ...                                               */
+;*---------------------------------------------------------------------*/
+(define-inline (procedure-attr proc)
+   ($procedure-attr proc))
+
+;*---------------------------------------------------------------------*/
+;*    procedure-attr-set! ...                                          */
+;*---------------------------------------------------------------------*/
+(define-inline (procedure-attr-set! proc obj)
+   ($procedure-attr-set! proc obj)
+   obj)
 
 ;*---------------------------------------------------------------------*/
 ;*    unspecified ...                                                  */
@@ -324,6 +400,18 @@
 (define-inline (unspecified)
    __unspec__)
 
+;*---------------------------------------------------------------------*/
+;*    null-or-unspecified? ...                                         */
+;*---------------------------------------------------------------------*/
+(define-inline (null-or-unspecified? obj)
+   (cond-expand
+      (bigloo-c
+       ;; this stupid trick is only used to improve JavaScript performance
+       ;; don't use that in real code!
+       ($null-or-unspecified? obj))
+      (else
+       (or (null? obj) (eq? obj #unspecified)))))
+
 ;*---------------------------------------------------------------------*/
 ;*    cnst? ...                                                        */
 ;*---------------------------------------------------------------------*/
@@ -336,6 +424,12 @@
 (define-inline (opaque? obj)
    (c-opaque? obj))
 
+;*---------------------------------------------------------------------*/
+;*    opaque-nil ...                                                   */
+;*---------------------------------------------------------------------*/
+(define-inline (opaque-nil)
+   (c-opaque-nil))
+
 ;*---------------------------------------------------------------------*/
 ;*    4bits->char ...                                                  */
 ;*---------------------------------------------------------------------*/
@@ -401,7 +495,7 @@
 ;*---------------------------------------------------------------------*/
 (define (bigloo-module-mangle id module)
    (let* ((len (+fx (string-length id) (string-length module)))
-	  (new (make-string (+fx (*fx len 3) 9))))
+	  (new (make-string (+fx (*fx len 3) 12))))
       (if (=fx len 0)
 	  (error "bigloo-mangle-string" "Can't mangle empty string" string)
 	  (let ((mod-start (mangle-at! new id (string-length id) 4)))
@@ -412,7 +506,7 @@
 				     (+fx mod-start 2))))
 		(blit-string! "BGl_" 0 new 0 4)
 		(substring new 0 stop))))))
-	  
+
 ;*---------------------------------------------------------------------*/
 ;*    bigloo-mangled? ...                                              */
 ;*---------------------------------------------------------------------*/
@@ -500,7 +594,7 @@
 	       (values id module))))
       (cond
 	 ((<fx len 8)
-	  (error "bigloo-demangle" "Not a Bigloo mangled identifier" string))
+	  string)
 	 ((substring=? string "BgL_" 4)
 	  (bigloo-demangle-simple))
 	 ((substring=? string "BGl_" 4)
@@ -508,6 +602,48 @@
 	 (else
 	  string))))
 
+;*---------------------------------------------------------------------*/
+;*    bigloo-module-demangle ...                                       */
+;*---------------------------------------------------------------------*/
+(define (bigloo-module-demangle string)
+   (multiple-value-bind (id module)
+      (bigloo-demangle string)
+      (if (string? module)
+	  (string-append id "@" module)
+	  id)))
+
+;*---------------------------------------------------------------------*/
+;*    bigloo-class-mangled? ...                                        */
+;*---------------------------------------------------------------------*/
+(define (bigloo-class-mangled? string)
+   (let ((len (string-length string)))
+      (and (>fx len 8)
+	   (char=? (string-ref string (-fx len 1)) #\t)
+	   (char=? (string-ref string (-fx len 2)) #\l)
+	   (char=? (string-ref string (-fx len 3)) #\g)
+	   (char=? (string-ref string (-fx len 4)) #\b)
+	   (char=? (string-ref string (-fx len 5)) #\_)
+	   (bigloo-mangled? (substring string 0 (-fx len 5))))))
+   
+;*---------------------------------------------------------------------*/
+;*    bigloo-class-demangle ...                                        */
+;*---------------------------------------------------------------------*/
+(define (bigloo-class-demangle string)
+   (string-append (bigloo-demangle
+		   (substring string 0 (-fx (string-length string) 5)))
+		  "_bglt"))
+
+;*---------------------------------------------------------------------*/
+;*    *exit-mutex* ...                                                 */
+;*---------------------------------------------------------------------*/
+(define *exit-mutex* (make-mutex 'bigloo-exit))
+
+;*---------------------------------------------------------------------*/
+;*    bigloo-exit-mutex ...                                            */
+;*---------------------------------------------------------------------*/
+(define (bigloo-exit-mutex)
+   *exit-mutex*)
+
 ;*---------------------------------------------------------------------*/
 ;*    *bigloo-exit-functions* ...                                      */
 ;*---------------------------------------------------------------------*/
@@ -517,18 +653,104 @@
 ;*    register-exit-function! ...                                      */
 ;*---------------------------------------------------------------------*/
 (define (register-exit-function! fun)
-   (if (not (correct-arity? fun 1))
-       (error "bigloo-exit-register!"
-	      "Wrong procedure arity"
-	      fun)
-       (set! *bigloo-exit-functions* (cons fun *bigloo-exit-functions*))))
+   (synchronize (bigloo-exit-mutex)
+      (if (not (correct-arity? fun 1))
+	  (error "bigloo-exit-register!"
+	     "Wrong procedure arity"
+	     fun)
+	  (set! *bigloo-exit-functions* (cons fun *bigloo-exit-functions*)))))
+
+;*---------------------------------------------------------------------*/
+;*    unregister-exit-function! ...                                    */
+;*---------------------------------------------------------------------*/
+(define (unregister-exit-function! fun)
+   (synchronize (bigloo-exit-mutex)
+      (set! *bigloo-exit-functions* (remq! fun *bigloo-exit-functions*))))
 
 ;*---------------------------------------------------------------------*/
 ;*    bigloo-exit-apply ...                                            */
 ;*---------------------------------------------------------------------*/
 (define (bigloo-exit-apply val)
-   (if (pair? *bigloo-exit-functions*)
-       (let ((fun (car *bigloo-exit-functions*)))
-	  (set! *bigloo-exit-functions* (cdr *bigloo-exit-functions*))
-	  (bigloo-exit-apply (fun val)))
-       val))
+   (let ((mut (if (mutex? (bigloo-exit-mutex))
+		  (bigloo-exit-mutex)
+		  (make-mutex 'bigloo-exit))))
+      (synchronize mut
+	 (let loop ((val val))
+	    (let ((val (if (integer? val)
+			   val
+			   0)))
+	       (if (pair? *bigloo-exit-functions*)
+		   (let ((fun (car *bigloo-exit-functions*)))
+		      (set! *bigloo-exit-functions*
+			 (cdr *bigloo-exit-functions*))
+		      (let ((nval (fun val)))
+			 (loop (if (integer? nval) nval val))))
+		   val))))))
+
+;*---------------------------------------------------------------------*/
+;*    time ...                                                         */
+;*---------------------------------------------------------------------*/
+(define (time proc)
+   (if (correct-arity? proc 0)
+       ($time proc)
+       (error 'time "Wrong procedure arity" proc)))
+
+;*---------------------------------------------------------------------*/
+;*    bmem-reset! ...                                                  */
+;*    -------------------------------------------------------------    */
+;*    This dummy function is overriden (LD_PRELOADed) by the bmem.so   */
+;*    library. It is used by bmem to reset the allocation statistics   */
+;*    gathering during the execution of a program, on user demand.     */
+;*---------------------------------------------------------------------*/
+(define (bmem-reset!)
+   (cond-expand
+      (bigloo-c ($bgl-bmem-reset))
+      (else #f)))
+
+;*---------------------------------------------------------------------*/
+;*    gc-verbose-set! ...                                              */
+;*---------------------------------------------------------------------*/
+(define (gc-verbose-set! bool)
+   (cond-expand
+      (bigloo-c ($bgl-gc-verbose-set! bool))
+      (else #f)))
+
+;*---------------------------------------------------------------------*/
+;*    gc ...                                                           */
+;*---------------------------------------------------------------------*/
+(define (gc #!key (finalize #t))
+   (cond-expand
+      (bigloo-c (begin ($GC-collect) (when finalize ($GC-finalize))))
+      (else #f)))
+
+;*---------------------------------------------------------------------*/
+;*    gc-finalize ...                                                  */
+;*---------------------------------------------------------------------*/
+(define-inline (gc-finalize)
+   (cond-expand
+      (bigloo-c ($GC-finalize))
+      (else #f)))
+
+;*---------------------------------------------------------------------*/
+;*    make-cell ...                                                    */
+;*---------------------------------------------------------------------*/
+(define-inline (make-cell val)
+   ($make-cell val))
+
+;*---------------------------------------------------------------------*/
+;*    cell? ...                                                        */
+;*---------------------------------------------------------------------*/
+(define-inline (cell? obj)
+   ($cell? obj))
+
+;*---------------------------------------------------------------------*/
+;*    cell-ref ...                                                     */
+;*---------------------------------------------------------------------*/
+(define-inline (cell-ref cell)
+   ($cell-ref cell))
+
+;*---------------------------------------------------------------------*/
+;*    cell-set! ...                                                    */
+;*---------------------------------------------------------------------*/
+(define-inline (cell-set! cell val)
+   ($cell-set! cell val))
