@@ -1,9 +1,9 @@
 ;*=====================================================================*/
-;*    serrano/prgm/project/bigloo/runtime/Llib/process.scm             */
+;*    serrano/prgm/project/bigloo/bigloo/runtime/Llib/process.scm      */
 ;*    -------------------------------------------------------------    */
 ;*    Author      :  Erick Gallesio                                    */
 ;*    Creation    :  Mon Jan 19 17:35:12 1998                          */
-;*    Last change :  Tue Sep 11 08:48:42 2001 (serrano)                */
+;*    Last change :  Sun Aug 25 09:09:19 2019 (serrano)                */
 ;*    -------------------------------------------------------------    */
 ;*    Process handling. This part is mostly compatible with            */
 ;*    STk. This code is extracted from STk by Erick Gallesio.          */
@@ -21,8 +21,14 @@
 	    __tvector
 	    __ucs2
 	    __dsssl
+	    __bexit
+	    __bignum
+	    __object
+	    __thread
+	    __bit
 	    
 	    __r4_numbers_6_5_fixnum
+	    __r4_numbers_6_5_flonum
 	    __r4_equivalence_6_2
 	    __r4_vectors_6_8
 	    __r4_booleans_6_1
@@ -50,11 +56,14 @@
 	    (c-run-process::process (::obj ::obj ::obj ::obj ::obj ::obj ::bstring ::obj ::obj)
 				    "c_run_process")
 	    (c-unregister-process::obj (::process) "c_unregister_process")
-	    (c-process-list::obj () "c_process_list"))
+	    (c-process-list::obj () "c_process_list")
+	    (c-process-nil::process () "bgl_process_nil"))
 
    (java    (class foreign
 	       (method static c-process?::bool (::obj)
 		       "PROCESSP")
+	       (method static c-process-nil::process ()
+		       "bgl_process_nil")
 	       (method static c-process-pid::int (::process)
 		       "PROCESS_PID")
 	       (method static c-process-input-port::obj (::process)
@@ -85,6 +94,7 @@
 		       "c_process_list")))
    
    (export  (inline process?::bool ::obj)
+	    (inline process-nil::process)
 	    (inline process-pid::int ::process)
 	    (inline process-output-port::obj ::process)
 	    (inline process-input-port::obj ::process)
@@ -98,13 +108,14 @@
 	    (inline process-continue::obj ::process)
 	    (inline process-list::obj)
 	    (run-process::process ::bstring . rest)
+	    (close-process-ports ::process)
 	    (inline unregister-process ::process))
 
    (pragma  (c-process? nesting)
-	    (c-process-pid nesting)
-	    (c-process-input-port nesting)
-	    (c-process-output-port nesting)
-	    (c-process-error-port nesting)))
+	    (c-process-pid nesting args-safe)
+	    (c-process-input-port nesting args-safe)
+	    (c-process-output-port nesting args-safe)
+	    (c-process-error-port nesting args-safe)))
 
 ;*---------------------------------------------------------------------*/
 ;*    process? ...                                                     */
@@ -112,6 +123,12 @@
 (define-inline (process? obj)
    (c-process? obj))
 
+;*---------------------------------------------------------------------*/
+;*    process-nil ...                                                  */
+;*---------------------------------------------------------------------*/
+(define-inline (process-nil)
+   (c-process-nil))
+
 ;*---------------------------------------------------------------------*/
 ;*    process-pid ...                                                  */
 ;*---------------------------------------------------------------------*/
@@ -165,7 +182,8 @@
 ;*    process-kill ...                                                 */
 ;*---------------------------------------------------------------------*/
 (define-inline (process-kill proc)
-   (c-process-kill proc))
+   (c-process-kill proc)
+   (close-process-ports proc))
 
 ;*---------------------------------------------------------------------*/
 ;*    process-stop ...                                                 */
@@ -198,6 +216,7 @@
 	 (output #unspecified)
 	 (error  #unspecified)
 	 (host   #unspecified)
+	 (pipes  '(pipe:))
 	 (args   '())
 	 (env    '())
 	 (arg-error (lambda (rest)
@@ -220,15 +239,15 @@
 			(set! fork val)
 			(arg-error rest)))
 		   ((input:)
-		    (if (or (string? val) (eq? val 'pipe:))
+		    (if (or (string? val) (memq val pipes))
 			(set! input val)
 			(arg-error rest)))
 		   ((output:)
-		    (if (or (string? val) (eq? val 'pipe:))
+		    (if (or (string? val) (memq val pipes) (eq? val null:))
 			(set! output val)
 			(arg-error rest)))
 		   ((error:)
-		    (if (or (string? val) (eq? val 'pipe:))
+		    (if (or (string? val) (memq val pipes) (eq? val null:))
 			(set! error val)
 			(arg-error rest)))
 		   ((host:)
@@ -248,9 +267,19 @@
 	    (else
 	     (arg-error rest))))))
 
+;*---------------------------------------------------------------------*/
+;*    close-process-ports ...                                          */
+;*---------------------------------------------------------------------*/
+(define (close-process-ports proc)
+   (if (output-port? (process-input-port proc))
+       (close-output-port (process-input-port proc)))
+   (if (input-port? (process-error-port proc))
+       (close-input-port (process-error-port proc)))
+   (if (input-port? (process-output-port proc))
+       (close-input-port (process-output-port proc))))
+
 ;*---------------------------------------------------------------------*/
 ;*    unregister-process ...                                           */
 ;*---------------------------------------------------------------------*/
 (define-inline (unregister-process proc)
    (c-unregister-process proc))
-   
