@@ -1,9 +1,9 @@
 ;*=====================================================================*/
-;*    serrano/prgm/project/bigloo/runtime/Llib/tvector.scm             */
+;*    serrano/prgm/project/bigloo/bigloo/runtime/Llib/tvector.scm      */
 ;*    -------------------------------------------------------------    */
 ;*    Author      :  Manuel Serrano                                    */
 ;*    Creation    :  Mon Mar 27 09:38:41 1995                          */
-;*    Last change :  Sun Jul  1 08:39:44 2001 (serrano)                */
+;*    Last change :  Sun Aug 25 09:08:27 2019 (serrano)                */
 ;*    -------------------------------------------------------------    */
 ;*    The typed vectors Scheme management.                             */
 ;*=====================================================================*/
@@ -14,45 +14,54 @@
 (module __tvector
 
    (import  __error
-	    __hash)
+	    __hash
+	    __reader
+	    __param
+	    __bexit
+	    __bignum
+	    __object
+	    __thread
+	    __r4_symbols_6_4)
    
    (use     __type
 	    __bigloo
 	    __structure
 	    __r4_numbers_6_5_fixnum
+	    __r4_numbers_6_5_flonum
+	    __r4_numbers_6_5_flonum_dtoa
 	    __r4_equivalence_6_2
 	    __r4_booleans_6_1
 	    __r4_characters_6_6
 	    __r4_vectors_6_8
 	    __r4_pairs_and_lists_6_3
-	    __r4_symbols_6_4
 	    __r4_strings_6_7
 	    __r4_output_6_10_3
+	    __r4_ports_6_10_1
 	    __r4_control_features_6_9
 	    __evenv
-	    __reader)
+	    __bit)
 
-   (extern (macro c-tvector?::bool (::obj)
+   (extern (macro $tvector?::bool (::obj)
 		  "TVECTORP")
-	   (macro c-tvector-length::int (::tvector)
+	   (macro $tvector-length::int (::tvector)
 		  "TVECTOR_LENGTH")
 	   
-	   (macro tvector-descr::obj (::tvector)
+	   (macro $tvector-descr::obj (::tvector)
 		  "TVECTOR_DESCR")
-	   (macro tvector-descr-set!::obj (::tvector ::obj)
+	   (macro $tvector-descr-set!::obj (::tvector ::obj)
 		  "TVECTOR_DESCR_SET")
 	   
 	   (export get-tvector-descriptor "get_tvector_descriptor"))
 
    (java   (class foreign
-	      (method static c-tvector?::bool (::obj)
+	      (method static $tvector?::bool (::obj)
 		      "TVECTORP")
-	      (method static c-tvector-length::int (::obj)
+	      (method static $tvector-length::int (::obj)
 		      "TVECTOR_LENGTH")
 	      
-	      (method static tvector-descr::obj (::obj)
+	      (method static $tvector-descr::obj (::obj)
 		      "TVECTOR_DESCR")
-	      (method static tvector-descr-set!::obj (::obj ::obj)
+	      (method static $tvector-descr-set!::obj (::obj ::obj)
 		      "TVECTOR_DESCR_SET")))
 
    (export  (inline tvector?::bool ::obj)
@@ -61,16 +70,16 @@
 	    (declare-tvector! ::string  ::procedure ::obj ::obj)
 	    (tvector-ref ::tvector)
 	    (tvector-id ::tvector)
-	    (list->tvector ::symbol ::pair)
+	    (list->tvector ::symbol ::pair-nil)
 	    (vector->tvector ::symbol ::vector)
 	    (tvector->vector ::tvector))
 
-   (pragma  (c-tvector? (predicate-of tvector) no-cfa-top nesting)
+   (pragma  ($tvector? (predicate-of tvector) no-cfa-top nesting)
 	    (tvector? side-effect-free no-cfa-top nesting)
-	    (c-tvector-length side-effect-free no-cfa-top nesting)
+	    ($tvector-length side-effect-free no-cfa-top nesting args-safe)
 	    (list->tvector no-cfa-top)
-	    (tvector-descr side-effect-free no-cfa-top nesting)
-	    (tvector-descr-set! nesting)
+	    ($tvector-descr side-effect-free no-cfa-top nesting)
+	    ($tvector-descr-set! nesting)
 	    (get-tvector-descriptor side-effect-free no-cfa-top)
 	    (tvector-ref side-effect-free no-cfa-top)
 	    (tvector-id side-effect-free no-cfa-top)
@@ -81,19 +90,19 @@
 ;*    tvector? ...                                                     */
 ;*---------------------------------------------------------------------*/
 (define-inline (tvector? obj)
-   (c-tvector? obj))
+   ($tvector? obj))
 
 ;*---------------------------------------------------------------------*/
 ;*    tvector-length ...                                               */
 ;*---------------------------------------------------------------------*/
 (define-inline (tvector-length obj)
-   (c-tvector-length obj))
+   ($tvector-length obj))
 
 ;*---------------------------------------------------------------------*/
 ;*    tvector-id ...                                                   */
 ;*---------------------------------------------------------------------*/
 (define (tvector-id tvect)
-   (tvect-descr-id (tvector-descr tvect)))
+   (tvect-descr-id ($tvector-descr tvect)))
 
 ;*---------------------------------------------------------------------*/
 ;*    tvec-descr ...                                                   */
@@ -132,11 +141,11 @@
 (define (declare-tvector! id allocate ref set)
    (let* ((id  (string->symbol (case (bigloo-case-sensitivity)
 				  ((upcase)
-				   (string-upcase (string->bstring id)))
+				   (string-upcase ($string->bstring id)))
 				  ((downcase)
-				   (string-downcase (string->bstring id)))
+				   (string-downcase ($string->bstring id)))
 				  (else
-				   (string->bstring id)))))
+				   ($string->bstring id)))))
 	  (old (get-tvector-descriptor id)))
       (if (not (tvect-descr? old))
 	  (let ((new (tvect-descr id allocate ref set)))
@@ -148,12 +157,12 @@
 ;*    tvector-ref ...                                                  */
 ;*---------------------------------------------------------------------*/
 (define (tvector-ref tvector)
-   (tvect-descr-ref (tvector-descr tvector)))
+   (tvect-descr-ref ($tvector-descr tvector)))
  
 ;*---------------------------------------------------------------------*/
 ;*    list->tvector ...                                                */
 ;*---------------------------------------------------------------------*/
-(define (list->tvector id::symbol l::pair)
+(define (list->tvector id::symbol l::pair-nil)
    (let ((descr (get-tvector-descriptor id)))
       (if (not descr)
 	  (error "list->tvector" "Undeclared tvector" id)
@@ -199,7 +208,7 @@
 ;*    tvector->vector ...                                              */
 ;*---------------------------------------------------------------------*/
 (define (tvector->vector tv::tvector)
-   (let ((descr (tvector-descr tv)))
+   (let ((descr ($tvector-descr tv)))
       (let ((allocate (tvect-descr-allocate descr))
 	    (ref      (tvect-descr-ref descr)))
 	 (if (not (procedure? ref))
@@ -207,7 +216,7 @@
 		    "Unable to convert to such tvector"
 		    (tvector-id tv))
 	     (let* ((len  (tvector-length tv))
-		    (vec  (c-create-vector len)))
+		    (vec  ($create-vector len)))
 		(let loop ((i (-fx len 1)))
 		   (if (=fx i -1)
 		       vec
