@@ -3,7 +3,7 @@
 ;*    -------------------------------------------------------------    */
 ;*    Author      :  Manuel Serrano                                    */
 ;*    Creation    :  Sun May 18 10:24:03 1997                          */
-;*    Last change :  Wed Jun  6 11:09:08 2001 (serrano)                */
+;*    Last change :  Mon Mar  3 08:53:57 2014 (serrano)                */
 ;*    -------------------------------------------------------------    */
 ;*    UCS-2 Characters Scheme management.                              */
 ;*=====================================================================*/
@@ -18,9 +18,11 @@
    (use     __type
 	    __bigloo
 	    __tvector
+	    __bignum
 	    
 	    __r4_equivalence_6_2
 	    __r4_numbers_6_5_fixnum
+	    __r4_numbers_6_5_flonum
 	    __r4_vectors_6_8
 	    __r4_strings_6_7
 	    __r4_booleans_6_1
@@ -31,15 +33,15 @@
 	    __evenv)
   
    (extern  (macro c-ucs2?::bool (::obj)  "UCS2P")
-	    (macro c-ucs2-letter?::bool (::ucs2) "ucs2_letterp")
-	    (macro c-ucs2-digit?::bool (::ucs2) "ucs2_digitp")
-	    (macro c-ucs2-whitespace?::bool (::ucs2) "ucs2_whitespacep")
-	    (macro c-ucs2-upperp::bool (::ucs2) "ucs2_upperp")
-	    (macro c-ucs2-lowerp::bool (::ucs2) "ucs2_lowerp")
-	    (macro c-ucs2-upcase::ucs2 (::ucs2) "ucs2_toupper")
-	    (macro c-ucs2-downcase::ucs2 (::ucs2) "ucs2_tolower")
-	    (macro c-ucs2-defined?::bool (::int) "ucs2_definedp")
-	    (macro c-integer->ucs2::ucs2 (::int) "INT_TO_UCS2")
+	    (c-ucs2-letter?::bool (::ucs2) "ucs2_letterp")
+	    (c-ucs2-digit?::bool (::ucs2) "ucs2_digitp")
+	    (c-ucs2-whitespace?::bool (::ucs2) "ucs2_whitespacep")
+	    (c-ucs2-upperp::bool (::ucs2) "ucs2_upperp")
+	    (c-ucs2-lowerp::bool (::ucs2) "ucs2_lowerp")
+	    (c-ucs2-upcase::ucs2 (::ucs2) "ucs2_toupper")
+	    (c-ucs2-downcase::ucs2 (::ucs2) "ucs2_tolower")
+	    (c-ucs2-defined?::bool (::int) "ucs2_definedp")
+	    (macro c-integer->ucs2::ucs2 (::int) "BGL_INT_TO_UCS2")
 	    (macro c-ucs2->integer::int (::bucs2) "CUCS2")
 	    (infix macro c-ucs2=?::bool (::ucs2 ::ucs2) "==")
 	    (infix macro c-ucs2<?::bool (::ucs2 ::ucs2) "<")
@@ -67,7 +69,7 @@
 	       (method static c-ucs2-defined?::bool (::int)
 		       "ucs2_definedp")
 	       (method static c-integer->ucs2::ucs2 (::int)
-		       "INT_TO_UCS2")
+		       "BGL_INT_TO_UCS2")
 	       (method static c-ucs2->integer::int (::ucs2)
 		       "CUCS2")
 	       (method static c-ucs2=?::bool (::ucs2 ::ucs2)
@@ -99,15 +101,21 @@
 	    (inline ucs2-lower-case?::bool ::ucs2)
 	    (inline ucs2->integer::int ::ucs2)
 	    (integer->ucs2::ucs2 ::int)
+	    (inline integer->ucs2-ur::ucs2 ::int)
 	    (ucs2->char::char ::ucs2)
 	    (inline char->ucs2::ucs2 ::char)
 	    (inline ucs2-upcase::ucs2 ::ucs2)
 	    (inline ucs2-downcase::ucs2 ::ucs2))
    
-   (pragma  (c-ucs2? (predicate-of bucs2) no-cfa-top nesting)
+   (pragma  (c-ucs2? (predicate-of bucs2) side-effect-free no-cfa-top nesting)
 	    (ucs2? side-effect-free no-cfa-top nesting)
-	    (c-ucs2-upcase side-effect-free nesting)
-	    (c-ucs2-downcase side-effect-free nesting)
+	    (c-ucs2-upcase side-effect-free nesting args-safe)
+	    (c-ucs2-downcase side-effect-free nesting args-safe)
+	    (c-ucs2=? side-effect-free nesting args-safe)
+	    (c-ucs2<? side-effect-free nesting args-safe)
+	    (c-ucs2>? side-effect-free nesting args-safe)
+	    (c-ucs2<=? side-effect-free nesting args-safe)
+	    (c-ucs2>=? side-effect-free nesting args-safe)
 	    (ucs2=? side-effect-free nesting)
 	    (ucs2<? side-effect-free nesting)
 	    (ucs2>? side-effect-free nesting)
@@ -125,6 +133,7 @@
 	    (ucs2-lower-case? side-effect-free nesting)
 	    (ucs2->integer side-effect-free nesting)
 	    (integer->ucs2 side-effect-free nesting)
+	    (integer->ucs2-ur side-effect-free nesting)
 	    (c-integer->ucs2 side-effect-free nesting)
 	    (ucs2->char side-effect-free nesting)
 	    (char->ucs2 side-effect-free nesting)
@@ -165,7 +174,7 @@
 ;*    ucs2>=? ...                                                      */
 ;*---------------------------------------------------------------------*/
 (define-inline  (ucs2>=? ucs2a ucs2b)
-   (c-ucs2>? ucs2a ucs2b))
+   (c-ucs2>=? ucs2a ucs2b))
 
 ;*---------------------------------------------------------------------*/
 ;*    ucs2-ci=? ...                                                    */
@@ -243,12 +252,17 @@
 ;*    integer->ucs2 ...                                                */
 ;*---------------------------------------------------------------------*/
 (define (integer->ucs2::ucs2 int::int)
-   (if (and (>fx int 0)
-	    (<fx int 65536))
+   (if (and (>=fx int 0) (<fx int 65536))
        (if (c-ucs2-defined? int)
 	   (c-integer->ucs2 int)
-	   (error "integer->ucs2" "Undefined UCS-2 character" int))
-       (error "integer->ucs2" "integer out of range or " int)))
+	   (error 'integer->ucs2 "Undefined UCS-2 character" int))
+       (error 'integer->ucs2 "integer out of range or " int)))
+
+;*---------------------------------------------------------------------*/
+;*    integer->ucs2-ur ...                                             */
+;*---------------------------------------------------------------------*/
+(define-inline (integer->ucs2-ur::ucs2 int::int)
+   (c-integer->ucs2 int))
 
 ;*---------------------------------------------------------------------*/
 ;*    ucs2->integer ...                                                */
@@ -260,7 +274,7 @@
 ;*    char->ucs2 ...                                                   */
 ;*---------------------------------------------------------------------*/
 (define-inline (char->ucs2::ucs2 char::char)
-   (integer->ucs2 (char->integer char)))
+   (integer->ucs2-ur (char->integer char)))
 
 ;*---------------------------------------------------------------------*/
 ;*    ucs2->char ...                                                   */
@@ -269,7 +283,7 @@
    (let ((int (ucs2->integer ucs2)))
       (if (<fx int 256)
 	  (integer->char int)
-	  (error "ucs2->char"
+	  (error 'ucs2->char
 		 "UCS-2 character out of ISO-LATIN-1 range"
 		 ucs2))))
 
