@@ -1,9 +1,9 @@
 ;*=====================================================================*/
-;*    serrano/prgm/project/bigloo/runtime/Rgc/rgc-compile.scm          */
+;*    .../prgm/project/bigloo/bigloo/runtime/Rgc/rgccompile.scm        */
 ;*    -------------------------------------------------------------    */
 ;*    Author      :  Manuel Serrano                                    */
 ;*    Creation    :  Sun Sep 13 07:46:39 1998                          */
-;*    Last change :  Mon May  7 18:40:28 2001 (serrano)                */
+;*    Last change :  Sun Aug 25 09:07:40 2019 (serrano)                */
 ;*    -------------------------------------------------------------    */
 ;*    This module implements the DFA compilation. Each state is        */
 ;*    compiled into a lambda expression.                               */
@@ -25,12 +25,19 @@
 	    __tvector
 	    __structure
 	    __os
-	    
 	    __pp
+	    __param
+	    __object
+	    __thread
+	    __bexit
+	    __bignum
+	    __rgc
+	    __bit
 	    
 	    __r4_numbers_6_5
 	    __r4_numbers_6_5_fixnum
 	    __r4_numbers_6_5_flonum
+	    __r4_numbers_6_5_flonum_dtoa
 	    __r4_characters_6_6
 	    __r4_equivalence_6_2
 	    __r4_booleans_6_1
@@ -67,9 +74,7 @@
 ;*---------------------------------------------------------------------*/
 (define *case-threshold*
    (let ((value 80))
-      (if *rgc-optim*
-	  (/fx value 2)
-	  value)))
+      (if *rgc-optim* (/fx value 2) value)))
 
 ;*---------------------------------------------------------------------*/
 ;*    compile-state ...                                                */
@@ -89,9 +94,9 @@
    ;; we start splittint transition in two sets:
    ;;    1. set for special
    ;;    2. set for regular chars
-   `(define (,(state-name state) last-match)
+   `(define (,(state-name state) iport last-match forward bufpos)
        ,(let ((transitions (state-transitions state))
-	      (positions   (state-positions state)))
+	      (positions (state-positions state)))
 	   (if (null? transitions)
 	       'last-match
 	       (multiple-value-bind (special-trans regular-trans)
@@ -99,16 +104,12 @@
 		  (let ((match-body (compile-match special-trans)))
 		     (if match-body
 			 `(let ((new-match ,match-body))
-			     ,(compile-regular submatches
-					       state
-					       regular-trans
-					       'new-match
-					       positions-to-char))
-			 (compile-regular submatches
-					  state
-					  regular-trans
-					  'last-match
-					  positions-to-char))))))))
+			     ,(compile-regular
+				 submatches state
+				 regular-trans 'new-match positions-to-char))
+			 (compile-regular
+			    submatches state
+			    regular-trans 'last-match positions-to-char))))))))
 
 ;*---------------------------------------------------------------------*/
 ;*    split-transitions ...                                            */
@@ -118,8 +119,8 @@
 ;*---------------------------------------------------------------------*/
 (define (split-transitions transitions)
    (let loop ((transitions transitions)
-	      (specials    '())
-	      (regulars    '()))
+	      (specials '())
+	      (regulars '()))
       (cond
 	 ((null? transitions)
 	  (values specials regulars))
@@ -139,58 +140,46 @@
    ;; the first set is to build the <state x transitions>
    ;; association list
    (let ((state-trans (state-transition-list transitions))
-	 (positions   (state-positions current-state)))
+	 (positions (state-positions current-state)))
       (if (null? state-trans)
 	  last-match
-	  ;; if the number of different transition is important
-	  ;; it is better to generate a `case' construction instead
-	  ;; of `cond'. I don't know exactly where the limit.
-	  `(let ((current-char::int (rgc-buffer-get-char input-port)))
-	      ,@(compile-submatches 'current-char
-				    submatches
-				    positions
-				    p->c)
-	      ,(if (<=fx (length state-trans) 12)
-		   (compile-cond-regular current-state
-					 state-trans
-					 last-match)
-		   (compile-case-regular current-state
-					 state-trans
-					 last-match))))))
+	  `(if (=fx forward bufpos)
+	       ;; the buffer is empty
+	       (if (rgc-fill-buffer iport)
+		   ,(compile-jump-to-state current-state 'last-match
+		       '(rgc-buffer-forward iport)
+		       '(rgc-buffer-bufpos iport))
+		   ,last-match)
+	       ;; Generate a `case' construction instead of `cond' when the
+	       ;; the number of transitions exceeds a fixed threshold
+	       (let* ((cur::int (rgc-buffer-get-char iport forward)))
+		  ,@(compile-submatches 'cur submatches positions p->c)
+		  ,(if (<=fx (length state-trans) 12)
+		       (compile-cond-regular
+			  current-state state-trans last-match)
+		       (compile-case-regular
+			  current-state state-trans last-match)))))))
 
 ;*---------------------------------------------------------------------*/
 ;*    compile-jump-to-state ...                                        */
 ;*---------------------------------------------------------------------*/
-(define (compile-jump-to-state state match)
-   `(,(state-name state) ,match))
+(define (compile-jump-to-state state match forward bufpos)
+   `(,(state-name state) iport ,match ,forward ,bufpos))
 
 ;*---------------------------------------------------------------------*/
 ;*    compile-case-regular ...                                         */
 ;*---------------------------------------------------------------------*/
 (define (compile-case-regular current-state state-trans match)
-   (define sentinel-match #f)
-   (define (sentinel-rule)
-      (if (state? sentinel-match)
-	  `((0) (if (rgc-buffer-empty? input-port)
-		    (if (rgc-fill-buffer input-port)
-			,(compile-jump-to-state current-state 'last-match)
-			,match)
-		    ,(compile-jump-to-state sentinel-match match)))
-	  `((0) (if (rgc-fill-buffer-if-empty input-port)
-		    ,(compile-jump-to-state current-state 'last-match)
-		    ,match))))
+   
    (define (compile-case-transition state-trans)
-      (let ((set   (cdr state-trans))
+      (let ((set (cdr state-trans))
 	    (state (car state-trans)))
-	 (if (rgcset-member? set 0)
-	     (begin
-		(set! sentinel-match state)
-		(rgcset-remove! set 0)))
 	 (let ((case-test (rgcset->list set)))
-	    `(,case-test ,(compile-jump-to-state state match)))))
-   `(case current-char
+	    `(,case-test ,(compile-jump-to-state state match
+			     '(+fx 1 forward) 'bufpos)))))
+   
+   `(case cur
        ,@(map compile-case-transition state-trans)
-       ,(sentinel-rule)
        (else
 	,match)))
 
@@ -198,75 +187,60 @@
 ;*    compile-cond-regular ...                                         */
 ;*---------------------------------------------------------------------*/
 (define (compile-cond-regular current-state state-trans match)
-   (define sentinel-match #f)
-   (define (sentinel-rule)
-      (if (state? sentinel-match)
-	  `((=fx current-char 0)
-	    (if (rgc-buffer-empty? input-port)
-		(if (rgc-fill-buffer input-port)
-		    ,(compile-jump-to-state current-state 'last-match)
-		    ,match)
-		,(compile-jump-to-state sentinel-match match)))
-	  `((=fx current-char 0)
-	    (if (rgc-fill-buffer-if-empty input-port)
-		,(compile-jump-to-state current-state 'last-match)
-		,match))))
+   
    (define (compile-cond-transition state-trans prev-test-len)
       (let ((set   (cdr state-trans))
 	    (state (car state-trans)))
-	 (if (rgcset-member? set 0)
-	     (begin
-		(set! sentinel-match state)
-		(rgcset-remove! set 0)))
 	 (multiple-value-bind (cond-test cond-cost)
-	    (compile-cond-test set 'current-char prev-test-len)
-	    (values `(,cond-test ,(compile-jump-to-state state match))
-		    cond-cost))))
-   (let loop ((trans    state-trans)
-	      (tests    '())
-	      (cost     0)
-	      (prev-len 0))
-      (if (null? trans)
-	  (if (>fx cost *case-threshold*)
-	      (compile-case-regular current-state state-trans match)
-	      ;; when compiling for a COND, the sentinel test
-	      ;; have to be inserted before any other test.
-	      `(cond
-		  ,(sentinel-rule)
-		  ,@(reverse! tests)
-		  (else
-		   ,match)))
+	    (compile-cond-test set 'cur prev-test-len)
+	    (values `(,cond-test
+			,(compile-jump-to-state state match
+			    '(+fx 1 forward) 'bufpos))
+	       cond-cost))))
+
+   (let loop ((trans state-trans)
+	      (tests '())
+	      (cost 0)
+	      (prev-len 0)
+	      (elsep #f))
+      (if (or (null? trans) elsep)
+	  (begin
+	     (if (>fx cost *case-threshold*)
+		 (compile-case-regular current-state state-trans match)
+		 `(cond
+		     ,@(reverse! tests)
+		     ,@(if elsep
+			'()
+			`((else ,match))))))
 	  (multiple-value-bind (test c)
 	     (compile-cond-transition (car trans) prev-len)
 	     (loop (cdr trans)
-		   (cons test tests)
-		   (+fx c cost)
-		   (+fx prev-len (rgcset-length (cdr (car trans)))))))))
+		(cons test tests)
+		(+fx c cost)
+		(+fx prev-len (rgcset-length (cdr (car trans))))
+		(and (pair? test) (eq? (car test) 'else)))))))
 
 ;*---------------------------------------------------------------------*/
 ;*    compile-cond-test ...                                            */
 ;*---------------------------------------------------------------------*/
 (define (compile-cond-test set var prev-test-len)
+   
    (define (find-next-member start set)
       (let ((max (rgc-max-char)))
 	 (let loop ((i start))
 	    (cond
-	       ((=fx i max)
-		-1)
-	       ((rgcset-member? set i)
-		i)
-	       (else
-		(loop (+fx i 1)))))))
+	       ((=fx i max) -1)
+	       ((rgcset-member? set i) i)
+	       (else (loop (+fx i 1)))))))
+   
    (define (find-next-non-member start set)
       (let ((max (rgc-max-char)))
 	 (let loop ((i start))
 	    (cond
-	       ((=fx i max)
-		max)
-	       ((rgcset-member? set i)
-		(loop (+fx i 1)))
-	       (else
-		i)))))
+	       ((=fx i max) max)
+	       ((rgcset-member? set i) (loop (+fx i 1)))
+	       (else i)))))
+   
    (define (compile-test start stop set)
       (cond
 	 ((=fx (-fx stop 1) start)
@@ -287,6 +261,7 @@
 	  (values `(<fx ,var ,stop) 1))
 	 (else
 	  (values `(and (>=fx ,var ,start) (<fx ,var ,stop)) 3))))
+   
    (define (compile-range-test set)
       (let loop ((start (find-next-member 1 set))
 		 (tests '())
@@ -299,10 +274,11 @@
 		   (loop (find-next-member stop set)
 			 (cons test tests)
 			 (+fx c cost)))))))
+   
    (let ((max (rgc-max-char))
 	 (len (rgcset-length set)))
       (cond
-	 ((=fx (rgc-max-char) (+fx 1 (+fx len prev-test-len)))
+	 ((=fx (rgc-max-char) (+fx len prev-test-len))
 	  ;; this test, checks for the complementary test of
 	  ;; what have been already tested. in consequence, we may
 	  ;; here generate an ELSE test
@@ -320,17 +296,17 @@
 ;*---------------------------------------------------------------------*/
 (define (state-transition-list transitions)
    (let loop ((transitions transitions)
-	      (res         '()))
+	      (res '()))
       (if (null? transitions)
 	  res
 	  (let* ((transition (car transitions))
-		 (char       (car transition))
-		 (state      (cdr transition)))
+		 (char (car transition))
+		 (state (cdr transition)))
 	     (let ((cell (assq state res)))
 		(if (not (pair? cell))
 		    (let ((set (list->rgcset (list char) (rgc-max-char))))
 		       (loop (cdr transitions)
-			     (cons (cons state set) res)))
+			  (cons (cons state set) res)))
 		    (begin
 		       (rgcset-add! (cdr cell) char)
 		       (loop (cdr transitions) res))))))))
@@ -342,14 +318,10 @@
 ;*---------------------------------------------------------------------*/
 (define (insort el lst)
    (cond
-      ((null? lst)
-       (list el))
-      ((<fx el (car lst))
-       (cons el lst))
-      ((=fx el (car lst))
-       lst)
-      (else
-       (cons (car lst) (insort el (cdr lst))))))
+      ((null? lst) (list el))
+      ((<fx el (car lst)) (cons el lst))
+      ((=fx el (car lst)) lst)
+      (else (cons (car lst) (insort el (cdr lst))))))
 
 ;*---------------------------------------------------------------------*/
 ;*    compile-match ...                                                */
@@ -360,6 +332,7 @@
 ;*    COND construction is inserted.                                   */
 ;*---------------------------------------------------------------------*/
 (define (compile-match transitions)
+   
    ;; we compile a match list into a body that, at runtime, will compute
    ;; we new matching rule. the MATCHES list is sorted.
    (define (compile-matches matches)
@@ -375,19 +348,20 @@
 			;; this rule is conditional
 			`(if (and ,@preds)
 			     (begin
-				(rgc-stop-match! input-port)
+				(rgc-stop-match! iport forward)
 				,match)
 			     ,(loop (cdr matches)))
 			`(begin
-			   (rgc-stop-match! input-port)
+			   (rgc-stop-match! iport forward)
 			   ,match)))))))
+   
    (let loop ((transitions transitions)
-	      (matches     '()))
+	      (matches '()))
       (if (null? transitions)
 	  (compile-matches matches)
 	  (let* ((transition (car transitions))
-		 (char       (car transition))
-		 (state      (cdr transition)))
+		 (char (car transition))
+		 (state (cdr transition)))
 	     (if (special-char-match? char)
 		 (loop (cdr transitions)
 		       (insort (special-match-char->rule-number char) matches))
@@ -420,30 +394,29 @@
 ;*      (65 66 67 70 71 72 73 74) -> ((65 . 67) (70 . 74))             */
 ;*---------------------------------------------------------------------*/
 (define (chars->char-ranges chars)
+   
    (let ((max (rgc-max-char)))
+      
       (define (find-range-start i)
 	 (let loop ((i i))
 	    (cond
-	       ((=fx i max)
-		#f)
-	       ((vector-ref compile-member-vector i)
-		i)
-	       (else
-		(loop (+fx i 1))))))
+	       ((=fx i max) #f)
+	       ((vector-ref compile-member-vector i) i)
+	       (else (loop (+fx i 1))))))
+      
       (define (find-range-stop i)
 	 (let loop ((i i))
 	    (cond
-	       ((=fx i max)
-		max)
-	       ((vector-ref compile-member-vector i)
-		(loop (+fx i 1)))
-	       (else
-		i))))
+	       ((=fx i max) max)
+	       ((vector-ref compile-member-vector i) (loop (+fx i 1)))
+	       (else i))))
+      
       (define (get-next-range i)
 	 (let ((start (find-range-start i)))
 	    (if (not start)
 		max
 		(cons start (-fx (find-range-stop start) 1)))))
+      
       (vector-fill! compile-member-vector #f)
       (for-each (lambda (x) (vector-set! compile-member-vector x #t)) chars)
       (let loop ((i      0)
@@ -452,21 +425,21 @@
 	     (reverse! ranges)
 	     (let ((range (get-next-range i)))
 		(if (pair? range)
-		    (loop (+fx (cdr range) 1)
-			  (cons range ranges))
-		    (loop range
-			  ranges)))))))
+		    (loop (+fx (cdr range) 1) (cons range ranges))
+		    (loop range ranges)))))))
 
 ;*---------------------------------------------------------------------*/
 ;*    char-ranges->test ...                                            */
 ;*---------------------------------------------------------------------*/
 (define (char-ranges->test current ranges)
+   
    (define (char-range->test range)
       (let ((start (car range))
 	    (stop  (cdr range)))
 	 (if (=fx start stop)
 	     `(=fx ,current ,start)
 	     `(and (>=fx ,current ,start) (<=fx ,current ,stop)))))
+   
    `(or ,@(map char-range->test ranges)))
 
 ;*---------------------------------------------------------------------*/
@@ -488,6 +461,7 @@
 ;*    compile-submatches ...                                           */
 ;*---------------------------------------------------------------------*/
 (define (compile-submatches current submatches positions positions-to-char)
+   
    (define (find-same-submatch cell sm)
       (let loop ((sm sm))
 	 (cond
@@ -497,32 +471,34 @@
 	     (car sm))
 	    (else
 	     (loop (cdr sm))))))
+   
    (define (add-to-submatch! char cell)
       (set-car! cell (cons char (car cell)))
       cell)
-   (let loop ((positions       (rgcset->list positions))
+   
+   (let loop ((positions (rgcset->list positions))
 	      (char-submatches '()))
       (if (null? positions)
 	  (map (match-lambda
 		  ((?char () (?m . ?sm))
 		   `(if ,(compile-member-test current char)
-			(rgc-submatch-stop! ,m ,sm)))
+			(rgc-submatch-stop2! ,m ,sm (+fx 1 forward))))
 		  ((?char ((?s ?m ?sm)))
 		   `(if ,(compile-member-test current char)
 			,(if s
 			     ;; this is a submatch at a
 			     ;; nullable position
-			     `(rgc-submatch-start*! ,m ,sm)
-			     `(rgc-submatch-start! ,m ,sm))))
+			     `(rgc-submatch-start*2! ,m ,sm (+fx 1 forward))
+			     `(rgc-submatch-start2! ,m ,sm (+fx 1 forward)))))
 		  ((?char ((?s ?m ?sm)) (?ts . ?tsm))
 		   `(if ,(compile-member-test current char)
 			(begin
 			   ,(if s
 				;; this is a submatch at a
 				;; nullable position
-				`(rgc-submatch-start*! ,m ,sm)
-				`(rgc-submatch-start! ,m ,sm))
-			   (rgc-submatch-stop! ,ts ,tsm))))
+				`(rgc-submatch-start*2! ,m ,sm (+fx 1 forward))
+				`(rgc-submatch-start2! ,m ,sm (+fx 1 forward)))
+			   (rgc-submatch-stop2! ,ts ,tsm (+fx 1 forward)))))
 		  (?else
 		   (error "compile-submatches"
 			  "Illegal char description"
