@@ -1,9 +1,9 @@
 ;*=====================================================================*/
-;*    serrano/prgm/project/bigloo/runtime/Rgc/rgc-rules.scm            */
+;*    serrano/prgm/project/bigloo/bigloo/runtime/Rgc/rgcrules.scm      */
 ;*    -------------------------------------------------------------    */
 ;*    Author      :  Manuel Serrano                                    */
 ;*    Creation    :  Wed Sep  9 09:45:00 1998                          */
-;*    Last change :  Mon May  7 18:42:02 2001 (serrano)                */
+;*    Last change :  Sun Aug 25 09:11:30 2019 (serrano)                */
 ;*    -------------------------------------------------------------    */
 ;*    This module implements the function `rules->tree' that translate */
 ;*    (canonicalize) the user set of clauses into on tree that         */
@@ -32,10 +32,18 @@
 	    __bigloo
 	    __tvector
 	    __structure
+	    __param
+	    __object
+	    __thread
+	    __bexit
+	    __bignum
+	    __rgc
+	    __bit
 	    
 	    __r4_numbers_6_5
 	    __r4_numbers_6_5_fixnum
 	    __r4_numbers_6_5_flonum
+	    __r4_numbers_6_5_flonum_dtoa
 	    __r4_characters_6_6
 	    __r4_equivalence_6_2
 	    __r4_booleans_6_1
@@ -75,15 +83,18 @@
        (error #f "RGC:Illegal clauses" clauses)
        (let ((env  (make-variable-env (append user-env (rgc-env))))
 	     (dflt `(in (0 ,(- (rgc-max-char) 1)))))
-;* 	  (print "--------------------------------------")             */
 	  (let loop ((clauses  clauses)
 		     (match    0)
 		     (branches '())
-		     (actions  '()))
-;* 	     (print "clauses : " clauses)                              */
-;* 	     (print "branches: " branches)                             */
-;* 	     (print "actions : " actions)                              */
+		     (actions  '())
+		     (defs     '()))
 	     (match-case (car clauses)
+		((define . ?-)
+		 (loop (cdr clauses)
+		       match
+		       branches
+		       actions
+		       (cons (car clauses) defs)))
 		((?rule . (and (? pair?) ?acts))
 		 (let ((act `(begin ,@acts)))
 		    (if (null? (cdr clauses))
@@ -93,7 +104,8 @@
 					 ,@branches)
 				    (reverse! (cons act actions))
 				    match
-				    *submatch?*)
+				    *submatch?*
+				    defs)
 			    (values `(or ,(expand-match-rule (+fx match 1)
 							     env
 							     dflt)
@@ -103,7 +115,8 @@
 						     act
 						     actions))
 				    (+fx 1 match)
-				    *submatch?*))
+				    *submatch?*
+				    defs))
 			;; this is not the last clause
 			(if (eq? rule 'else)
 			    (error #f "RGC:Illegal else clause" clauses)
@@ -111,7 +124,8 @@
 			       (loop (cdr clauses)
 				     (+fx match 1)
 				     (cons erule branches)
-				     (cons act actions)))))))
+				     (cons act actions)
+				     defs))))))
 		(else
 		 (error #f "RGC:Illegal clause" clauses)))))))
 
@@ -255,94 +269,40 @@
       (add-special-match-char! special-char match)
       (let loop ((rule rule))
 	 (make-sequence
-	  (list (let loop ((rule rule))
-		   (match-case rule
-		      ((when ?pred ?rule)
-		       (add-predicate-match! match pred)
-		       (loop rule))
-		      ((context ?context ?rule)
-		       (add-predicate-match! match `(rgc-context? ',context))
-		       (loop rule))
-		      ((bol ?rule)
-		       (add-predicate-match! match
-					     '(rgc-buffer-bol? input-port))
-		       (loop rule))
-		      ((eol ?rule)
-		       (add-predicate-match! match
-					     '(rgc-buffer-eol? input-port))
-		       (loop rule))
-		      ((bof ?rule)
-		       (add-predicate-match! match
-					     '(rgc-buffer-bof? input-port))
-		       (loop rule))
-		      ((eof ?rule)
-		       (add-predicate-match! match
-					     '(rgc-buffer-eof? input-port))
-		       (loop rule))
-;* 		      ((keyword . ?kwds)                               */
-;* 		       (print "Je vais re-expanser: "                  */
-;* 			     `(when ,(make-keyword-ci-predicate rule kwds) */
-;* 				 ,(make-keyword-rule kwds)))           */
-;* 		       (loop `(when ,(make-keyword-ci-predicate rule kwds) */
-;* 				 ,(make-keyword-rule kwds))))          */
-;* 		      ((uncase-keyword . ?kwds)                        */
-;* 		       (loop `(when ,(make-keyword-ci-predicate rule kwds) */
-;* 				 ,(make-keyword-rule kwds))))          */
-		      (else
-		       (expand-rule match env rule))))
-		special-char)))))
-
-;* {*---------------------------------------------------------------------*} */
-;* {*    make-keyword-rule ...                                            *} */
-;* {*---------------------------------------------------------------------*} */
-;* (define (make-keyword-rule keywords)                                */
-;*    (let ((vec (make-string (rgc-max-char) #\-)))                    */
-;*       (for-each (lambda (s)                                         */
-;* 		   (let loop ((i (-fx (string-length s) 1)))           */
-;* 		      (string-set! vec (char->integer (string-ref s i)) #\+) */
-;* 		      (if (>=fx i 0)                                   */
-;* 			  (loop (-fx i 1)))))                          */
-;* 		keywords)                                              */
-;*       `(+ (or ,@(let loop ((i (-fx (rgc-max-char) 1))               */
-;* 			   (res '()))                                  */
-;* 		   (cond                                               */
-;* 		      ((<fx i 0)                                       */
-;* 		       res)                                            */
-;* 		      ((char=? (string-ref vec i) '#\+)                */
-;* 		       (loop (-fx i 1) (cons (integer->char i) res)))  */
-;* 		      (else                                            */
-;* 		       (loop (-fx i 1) res))))))))                     */
-;* 			                                               */
-;* {*---------------------------------------------------------------------*} */
-;* {*    make-keyword-predicate ...                                       *} */
-;* {*---------------------------------------------------------------------*} */
-;* (define (make-keyword-predicate rule keywords)                      */
-;*    ;; first, we have to check that all keywords are string          */
-;*    (let loop ((keys keywords))                                      */
-;*       (cond                                                         */
-;* 	 ((null? keys)                                                 */
-;* 	  `(begin (print "Je test: [" (the-symbol) "]")                */
-;* 		  (memq (the-symbol) ',(map string->symbol keywords)))) */
-;* 	 ((and (string? (car keys)) (not (string=? (car keys) "")))    */
-;* 	  (loop (cdr keys)))                                           */
-;* 	 (else                                                         */
-;* 	  (error #f "RGC:Illegal keyword values" rule)))))             */
-;*                                                                     */
-;* {*---------------------------------------------------------------------*} */
-;* {*    make-keyword-ci-predicate ...                                    *} */
-;* {*---------------------------------------------------------------------*} */
-;* (define (make-keyword-ci-predicate rule keywords)                   */
-;*    ;; first, we have to check that all keywords are string          */
-;*    (let loop ((keys keywords))                                      */
-;*       (cond                                                         */
-;* 	 ((null? keys)                                                 */
-;* 	  `(memq (string->symbol (string-upcase (the-string)))         */
-;* 		 ',(map (lambda (x) (string->symbol (string-upcase x))) */
-;* 			keywords)))                                    */
-;* 	 ((and (string? (car keys)) (not (string=? (car keys) "")))    */
-;* 	  (loop (cdr keys)))                                           */
-;* 	 (else                                                         */
-;* 	  (error #f "RGC:Illegal keyword values" rule)))))             */
+	    (list (let loop ((rule rule))
+		     (match-case rule
+			((when ?pred ?rule)
+			 (add-predicate-match! match pred)
+			 (loop rule))
+			((context ?context ?rule)
+			 (add-predicate-match! match
+			    `(eq? the-rgc-context ',context))
+			 (loop rule))
+			((bol ?rule)
+			 (add-predicate-match! match
+			    '(rgc-buffer-bol? iport))
+			 (loop rule))
+			((eol ?rule)
+			 (add-predicate-match! match
+			    '(let ((r (rgc-buffer-eol? iport forward bufpos)))
+			      (set! forward (rgc-buffer-forward iport))
+			      (set! bufpos (rgc-buffer-bufpos iport))
+			      r))
+			 (loop rule))
+			((bof ?rule)
+			 (add-predicate-match! match
+			    '(rgc-buffer-bof? iport))
+			 (loop rule))
+			((eof ?rule)
+			 (add-predicate-match! match
+			    '(let ((r (rgc-buffer-eof2? iport forward bufpos)))
+			      (set! forward (rgc-buffer-forward iport))
+			      (set! bufpos (rgc-buffer-bufpos iport))
+			      r))
+			 (loop rule))
+			(else
+			 (expand-rule match env rule))))
+	       special-char)))))
 
 ;*---------------------------------------------------------------------*/
 ;*    expand-rule ...                                                  */
@@ -350,11 +310,10 @@
 ;*    The general function that expands every constructions.           */
 ;*---------------------------------------------------------------------*/
 (define (expand-rule match env rule)
-;*    (print "expand-rule: " match " " rule)                           */
    (if (not (pair? rule))
        (expand-atom match env rule)
        (match-case rule
-	  ((... ?num ?r)             (expand-... match env num r rule))
+	  ((... ?num ?r)             (expand-dots match env num r rule))
 	  ((uncase ?rule)            (expand-uncase match env rule))
 	  ((* ?rule)                 (expand-* match env rule))
 	  ((+ ?rule)                 (expand-+ match env rule))
@@ -382,7 +341,7 @@
    (cond
       ((char? rule)
        (char->integer rule))
-      ((and (integer? rule)
+      ((and (fixnum? rule)
 	    (>= rule 0)
 	    (< rule (rgc-max-char)))
        (error #f "RGC:illegal atom" rule))
@@ -411,19 +370,19 @@
        (make-sequence (map char->integer (string->list rule)))))
 
 ;*---------------------------------------------------------------------*/
-;*    expand-... ...                                                   */
+;*    expand-dots ...                                                  */
 ;*    -------------------------------------------------------------    */
 ;*    This operator applies to sequences. It constructs a regexp       */
 ;*    that matches, the 1fst, the 1fst and 2nd, the 1fst and 2nd and   */
 ;*    3th, etc... parts of the regexp. For instance                    */
-;*       (expand-... (sequence r1 r2 r3 r4))                           */
+;*       (expand-dots (sequence r1 r2 r3 r4))                          */
 ;*          ->                                                         */
 ;*       (or (sequence r1)                                             */
 ;*           (sequence r1 r2)                                          */
 ;*           (sequence r1 r2 r3)                                       */
 ;*           (sequence r1 r2 r3 r4))                                   */
 ;*---------------------------------------------------------------------*/
-(define (expand-... match env num rule err)
+(define (expand-dots match env num rule err)
    (define (explode-sequence rules)
       (let loop ((rules rules)
 		 (i 0))
@@ -433,7 +392,7 @@
 		(cons (list first)
 		      (map (lambda (r) (cons first r))
 			   (loop (cdr rules) (+fx i 1))))))))
-   (if (not (and (integer? num) (> num 0) (< num *max-rgc-=-num*)))
+   (if (not (and (fixnum? num) (> num 0) (< num *max-rgc-=-num*)))
        (error #f "RGC:Illegal regular range" err)
        (match-case (expand-rule match env rule)
 	  ((sequence . ?rules)
@@ -461,11 +420,8 @@
 ;*    stopping on each leave and checking if it is a char or not.      */
 ;*---------------------------------------------------------------------*/
 (define (expand-uncase match env rule)
-;*    (print "expand-uncase: " rule)                                   */
    (let loop ((rule (expand-rule match env rule))
 	      (res  '()))
-;*       (print "    expand-uncase(rule): " rule)                      */
-;*       (print "    expand-uncase(res) : " res)                       */
       (cond
 	 ((null? rule)
 	  (reverse! res))
@@ -539,7 +495,7 @@
 ;*       (sequence r r r)                                              */
 ;*---------------------------------------------------------------------*/
 (define (expand-= match env num rule err)
-   (if (not (and (integer? num) (> num 0) (< num *max-rgc-=-num*)))
+   (if (not (and (fixnum? num) (> num 0) (< num *max-rgc-=-num*)))
        (error #f "RGC:Illegal regular range" err)
        (let ((erule (expand-rule match env rule)))
 	  (make-sequence (vector->list (make-vector num erule))))))
@@ -553,7 +509,7 @@
 ;*       (sequence r r r (* r))                                        */
 ;*---------------------------------------------------------------------*/
 (define (expand->= match env num rule err)
-   (if (not (and (integer? num) (> num 0) (< num *max-rgc-=-num*)))
+   (if (not (and (fixnum? num) (> num 0) (< num *max-rgc-=-num*)))
        (error #f "RGC:Illegal regular range" err)
        (let ((erule (expand-rule match env rule)))
 	  (make-sequence (append (vector->list (make-vector num erule))
@@ -568,9 +524,9 @@
 ;*       (or (= 3 r) (= 4 r) (= 5 r))                                  */
 ;*---------------------------------------------------------------------*/
 (define (expand-** match env min max rule err)
-   (if (not (and (integer? min)
+   (if (not (and (fixnum? min)
 		 (> min 0)
-		 (integer? max)
+		 (fixnum? max)
 		 (> max min)
 		 (< max *max-rgc-=-num*)))
        (error #f "RGC:Illegal regular range" err)
@@ -596,7 +552,6 @@
 ;*       (in ("ad"))    --> (or #\a #\b #\c #\d)                       */
 ;*---------------------------------------------------------------------*/
 (define (expand-in match env values err)
-;*    (print "expand-in: " values)                                     */
    (define (rgc-char? x)
       ;; this predicate is true if a char or an integer is a legal
       ;; char for rgc (bound checking).
@@ -618,7 +573,7 @@
 	     (error #f "RGC:Illegal range" err)))))
    (define (string-range string)
       (let ((len (string-length string)))
-	 (if (> (remainder len 2) 0)
+	 (if (> (remainderfx len 2) 0)
 	     (error #f "RGC:Illegal range string" err)
 	     (let loop ((i   0)
 			(res '()))
@@ -632,8 +587,6 @@
        'epsilon
        (let loop ((values values)
 		  (chars  '()))
-;*       (print "   values: " values)                                  */
-;*       (print "   chars : " chars)                                   */
 	  (if (null? values)
 	      `(or ,@chars)
 	      (let ((value (car values)))
@@ -748,12 +701,12 @@
 ;*---------------------------------------------------------------------*/
 ;*    make-sequence ...                                                */
 ;*    -------------------------------------------------------------    */
-;*    This function normalize regexp that is:                          */
+;*    This function normalizes regexp. That is:                        */
 ;*       (sequence 1 2 3 (sequence 4 5 6) 7 8)                         */
 ;*    is turned into                                                   */
 ;*       (sequence 1 2 3 4 5 6 7 8)                                    */
 ;*    We _don't_ have to check for deep nesting of sequence            */
-;*    construction because _every_ sequence is added by this           */
+;*    forms because _each_ sequence is added by this                   */
 ;*    function thus, this situation is impossible:                     */
 ;*       (sequence 1 (sequence 2 (sequence 3) ...) ...)                */
 ;*---------------------------------------------------------------------*/
